@@ -18,7 +18,7 @@
 
 /*
  * The functioning of the download architecture, as described by Philip
- * on 28 May 2001:
+ * on 28 May 2001 and updated on 28 June 2001:
  * 
  * When mozilla runs into a file it cannot render internally or that it
  * does not have a plugin for, it calls the
@@ -51,34 +51,35 @@
  * possible mime-actions are:
  * 
  * Save to Disk
- * Save to Disk with External Download Handler
  * Run with Helper App
  * Ask User
- * ( Override mozilla built-in type ) doesn't really work yet. :-) We'll
- * ignore it for now.
  * 
- * The default action is Ask User, and if the mime-type is not in the
+ * The default action is Ask User, and if the mime-type is not in our
  * list, this is what will be assumed.
  * 
  * c) If Ask User is the chosen action, a dialog will be shown to the
- * user allowing the user to choose from the other three possible actions
+ * user allowing the user to choose from the other two possible actions
  * as well as a checkbox to let the user set the default action to the
  * chosen action for the future.
  * 
- * d-1) The "Save to Disk" action.  This action is handled by the mozilla
+ * d-1) The "Save to Disk" action. We first check galeon preferences to
+ * see if the user wants to use the built-in mozilla downloader, gtm or
+ * a command-line executed downloader.
+ *
+ * d-2a) The built-in downloader.  This action is handled by the mozilla
  * backend. Our nsIHelperAppDialog does the same thing that the
  * mozilla-the-browser one does, which is to call the SaveToDisk() method
  * of nsIExternalHelperAppService. This in turn calls the
  * PromptForSaveToFile() method of nsIHelperAppDialog putting the ball
  * back in our court.
  * 
- * d-2) Now, if galeon is configured to always ask for a download
+ * d-2b) Now, if galeon is configured to always ask for a download
  * directory, it will pop up a file selector so that the user can select
  * the directory and filename to save the file to.  Otherwise, it will
  * use galeon's default download directory and proceed without
  * interaction.
  * 
- * d-3) When PromptForSaveToFile() returns, nsIExternalHelperAppService
+ * d-2c) When PromptForSaveToFile() returns, nsIExternalHelperAppService
  * will then call the ShowProgressDialog() method of
  * nsIHelperAppDialog. This progress dialog, obviously, tracks the
  * progress of the download. It is worth noting that mozilla starts the
@@ -94,18 +95,18 @@
  * the download starts early means that the file is actually downloaded
  * to a temp file and only at the end is it moved to it's final location.
  * 
- * e-1) The "Save to Disk with External Handler" action.  This option is
+ * d-3a) The two external downloader options.  These options are
  * handled completely by galeon. The first thing that we do is call the
  * Cancel() method of nsIExternalHelperAppService to cancel the mozilla
  * download. We then pass the url to our own LaunchExternalDownloader()
  * method. This method will ask for a download directory as appropriate
  * as with the "Save to disk" action.
  * 
- * e-2) Finally, depending on whether GTM or a command line handler was
+ * d-3b) Finally, depending on whether GTM or a command line handler was
  * selected in prefs, the external handler will be called with the url
  * passed and the directory selected.
  * 
- * f-1) The "Run with Helper App" action.  This action is currently only
+ * e-1) The "Run with Helper App" action.  This action is currently only
  * working with a minimal implementation.  First, we explicitly call
  * ShowProgressDialog() so the user knows that the file is being
  * downloaded. We also need this so that we only run the helper after the
@@ -114,34 +115,14 @@
  * to call ShowProgressDialog() ourselves because we are not using
  * mozilla's helper mechanism which would usually make the call for us.
  * 
- * f-2) We call out to gnome vfs with the mime-type of the document and
- * attempt to retrieve the default helper app associated with that
- * mime-type in gnome-vfs. If the mime type has a
- * GNOME_VFS_MIME_ACTION_TYPE_APPLICATION action type and it also has a
- * default helper app, then we execute that helper with the file. If the
- * action type is any other type or there is no default helper app, we
- * inform the user of this fact and then Cancel() the download.
+ * e-2) If there is a default helper app in our mime database and alwaysAsk
+ * is false, galeon will run the default helper automatically. Otherwise it
+ * will pop up a helper chooser dialog which lists the helpers that gnome-vfs
+ * knows about as well as providing a GnomeFileEntry to allow the user to
+ * select and arbitrary application. The default value of the GnomeFileEntry
+ * is the helper stored in our database if one exits.
  * 
- * f-3) Once the helper app is launched, galeon no longer has anything to
- * do with the helper or the temp file. We cannot delete the temp file as
- * we don't have a way of telling when the user is finished with
- * it. Mozilla acts in the same way.
- * 
- * g) In the future, I plan to expand helper support so that the user is
- * presented with a list of all the helper apps that gnome-vfs knows of
- * for that mime type. It is often the case that there will be more than
- * one possible helper. eg: pdf files have ggv, gv and xpdf as possible
- * helpers with ggv as default in a normal gnome-vfs setup. currently we
- * always use the default ggv.
- * 
- * It is also likely that we will provide an option to provide a path to
- * an arbirtrary program to use as a helper in case the user doesn't want
- * to use the gnome-vfs helpers or there are no helpers in gnome-vfs.
- * 
- * We will also need to expand the info stored in prefs to note which
- * helper app the user prefers for a given mime-type.
- * 
- * h) General notes.  We cannot use this infrastructure to override
+ * f) General notes.  We cannot use this infrastructure to override
  * native mozilla types. mozilla will attempt to render these types and
  * never call out to us. We are at the end of the chain as the handler of
  * last resort, so native and plugin types will never reach us. This also
@@ -149,12 +130,12 @@
  * text/plain ) will be incorrectly rendered by mozilla. We cannot help
  * this.
  * 
- * We will be expanding the current context menu "download link" option
- * so that it can use the mozilla downloading mechanism as well as the
- * External Handler as is currently possible.  This will be the way to
- * download file with bad mime-types, as is the case in
- * mozilla-the-browser.  We will also implement shift-click to match this
- * behaviour.
+ * Despite the apparent user-side similarity with explicit downloads by
+ * a shift-click or context-menu item, there is actually none at all.
+ * Explicit downloads are handled by the nsIStreamTransfer manager which
+ * we use as is. Currently the progress dialog for the stream transfer
+ * manager is un-overridable, so it appears in XUL. This will change in
+ * due course.
  * 
  * Matt would like the modifiy the progress dialog so each file currently
  * being downloaded becomes a clist entry in a master dialog rather than
@@ -164,9 +145,12 @@
 
 #include "galeon.h"
 #include "mime.h"
-#include "misc.h"
 #include "window.h"
 #include "mozilla.h"
+#include "misc.h"
+#include "glade.h"
+#include "downloader.h"
+#include "dialog.h"
 
 extern "C" {
 #include "libgnomevfs/gnome-vfs-mime-handlers.h"
@@ -185,6 +169,7 @@ extern "C" {
 #include "nsIHelperAppLauncherDialog.h"
 #include "nsIWebProgressListener.h"
 #include "nsIExternalHelperAppService.h"
+#include "nsCExternalHandlerService.h"
 
 #include "nsCRT.h"
 #include "nsCOMPtr.h"
@@ -198,8 +183,9 @@ extern "C" {
 #include "nsIURI.h"
 #include "nsIMIMEInfo.h"
 #include "nsIChannel.h"
+#include "nsIFTPChannel.h"
 #include "nsILocalFile.h"
-#include "nsIPref.h"
+#include "nsIPrefService.h"
 #include "nsIDOMWindow.h"
 #include "nsIDOMWindowInternal.h"
 
@@ -217,8 +203,6 @@ void helper_chooser_ok_clicked_cb (GtkButton *button,
 				   HelperAppChooserDialog *dialog);
 gint helper_chooser_cancel_clicked_cb (GtkButton *button,
 				       HelperAppChooserDialog *dialog);
-void helper_chooser_browse_clicked_cb (GtkButton *button,
-				       HelperAppChooserDialog *dialog);
 void helper_chooser_auto_default_toggled (GtkToggleButton *button,
 					  HelperAppChooserDialog *dialog);
 void helper_chooser_radio_list_toggled (GtkToggleButton *button,
@@ -227,6 +211,8 @@ void helper_chooser_radio_other_toggled (GtkToggleButton *button,
 					 HelperAppChooserDialog *dialog);
 gint progress_dialog_cancel_cb (GtkButton *button,
 				GDownloadProgressListener *aListener);
+void progress_dialog_pause_cb (GtkButton *button,
+			       GDownloadProgressListener *aListener);
 }
 
 /**
@@ -243,19 +229,21 @@ struct MimeAskActionDialog
 
 struct HelperAppChooserDialog
 {
-	HelperAppChooserDialog (void);
+	HelperAppChooserDialog (void) : mOtherHelperApp(FALSE) {};
 	~HelperAppChooserDialog () {};
 
 	GList *mHelperApps;
 	GContentHandler *mContentHandler;
 	GladeXML *mGXML;
-	gboolean mAutoDefaultChecked;
 	gboolean mOtherHelperApp;
 
 	GtkWidget *mDialogWidget;
-	GtkWidget *mBrowseButton;
+	GtkWidget *mHelperGnomeEntry;
 	GtkWidget *mHelperEntry;
 	GtkWidget *mOptionMenu;
+	GtkWidget *mCheckUseTerminal;
+	GtkWidget *mCheckUseUrl;
+	GtkWidget *mCheckAlwaysUse;
 };
 
 class GContentHandler : public nsIHelperAppLauncherDialog
@@ -274,14 +262,15 @@ class GContentHandler : public nsIHelperAppLauncherDialog
 
 	NS_IMETHODIMP GetLauncher (nsIHelperAppLauncher * *_retval);
 	NS_IMETHODIMP GetContext (nsISupports * *_retval);
-	NS_IMETHODIMP SetHelperApp(GnomeVFSMimeApplication *mHelperApp);
+	NS_IMETHODIMP SetHelperApp(GnomeVFSMimeApplication *mHelperApp,
+				   PRBool alwaysUse);
 
   private:
 	/* additional members */
 	NS_IMETHODIMP Init (void);
+	NS_IMETHODIMP ChooseHelperApp (void);
 	NS_IMETHODIMP ProcessMimeInfo (void);
 	NS_IMETHODIMP MIMEAskAction (void);
-	NS_IMETHODIMP ChooseHelperApp (void);
 	
 	nsCOMPtr<nsIHelperAppLauncher> mLauncher;
 	nsCOMPtr<nsISupports> mContext;
@@ -290,7 +279,7 @@ class GContentHandler : public nsIHelperAppLauncherDialog
 	PRInt64 mTimeDownloadStarted;
 	nsCOMPtr<nsIFile> mTempFile;
 
-	MimeAction mMimeAction;
+	MimeItem *mMimeItem;
 
 	char *mUrl;
 	char *mMimeType;
@@ -300,7 +289,6 @@ class GContentHandler : public nsIHelperAppLauncherDialog
 
 	nsCOMPtr<nsIWebProgressListener> mListener;
 	
-	GnomeVFSMimeApplication *mHelperApp;
 };
 
 class GDownloadProgressListener : public nsIWebProgressListener,
@@ -317,6 +305,7 @@ class GDownloadProgressListener : public nsIWebProgressListener,
 	virtual ~GDownloadProgressListener ();
 
 	NS_IMETHODIMP CancelHelperProgress (void);
+	NS_IMETHODIMP TogglePause (void);
 
  private:
 
@@ -325,6 +314,7 @@ class GDownloadProgressListener : public nsIWebProgressListener,
 	nsCOMPtr<nsIHelperAppLauncher> mLauncher;
 	nsCOMPtr<nsISupports> mContext;
 	nsCOMPtr<GContentHandler> mHandler;
+	nsCOMPtr<nsIRequest> mRequest;
 	PRBool mUsedByHelper;
 	
 	nsCOMPtr<nsIURI> mUri;
@@ -341,6 +331,10 @@ class GDownloadProgressListener : public nsIWebProgressListener,
 	PRInt32 mRateChanges;
 	PRInt32 mRateChangeLimit;
 
+	PRBool mCheckedCanPause;
+	PRBool mCanPause;
+	PRBool mIsPaused;
+
 	GtkWidget *mProgressDialog;
 	GtkWidget *mProgressBar;
 	GtkWidget *mLocation;
@@ -348,6 +342,7 @@ class GDownloadProgressListener : public nsIWebProgressListener,
 	GtkWidget *mStatus;
 	GtkWidget *mTimeElapsed;
 	GtkWidget *mTimeRemaining;
+	GtkWidget *mPauseButton;
 };
 
 
@@ -363,10 +358,9 @@ GContentHandler::GContentHandler()
 	/* member initializers and constructor code */
 	mUri = nsnull;
 	mMimeType = nsnull;
-	mMimeAction = MIME_UNKNOWN;
+	mMimeItem = nsnull;
 	mDownloadCanceled = PR_FALSE;
 	mHelperProgress = PR_FALSE;
-	mHelperApp = nsnull;
 }
 
 GContentHandler::~GContentHandler()
@@ -374,19 +368,7 @@ GContentHandler::~GContentHandler()
 	/* destructor code */
 	g_free (mUri);
 	g_free (mMimeType);
-	if (mHelperApp)
-	{
-		if (nsCRT::strcmp(mHelperApp->id,"OtHeR") == 0)
-		{
-			g_free (mHelperApp->command);
-			g_free (mHelperApp->id);
-			delete mHelperApp;
-		}
-		else
-		{
-			gnome_vfs_mime_application_free (mHelperApp);
-		};
-	};
+	mime_item_free (mMimeItem);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -403,7 +385,7 @@ NS_IMETHODIMP GContentHandler::Show(nsIHelperAppLauncher *aLauncher,
 	mContext = aContext;
 	rv = Init ();
 	
-	switch (mMimeAction)
+	switch (mMimeItem->action)
 	{
 	 case MIME_UNKNOWN:
 		mime_set_action (mMimeType, MIME_ASK_ACTION);
@@ -412,18 +394,24 @@ NS_IMETHODIMP GContentHandler::Show(nsIHelperAppLauncher *aLauncher,
 		break;
 	 case MIME_IGNORE:
 		mLauncher->Cancel ();
+		delete mMimeItem;
 		break;
 	 case MIME_SAVE_TO_DISK:
-		mLauncher->SaveToDisk (nsnull,PR_FALSE);
-		break;
-	 case MIME_SAVE_WITH_GTM:
-		mLauncher->Cancel ();
-		LaunchExternalDownloader ();
+	 	if (gnome_config_get_int (CONF_DOWNLOADING_DOWNLOADER)
+	 	    == DOWNLOADER_BUILTIN)
+	 	{
+			mLauncher->SaveToDisk (nsnull,PR_FALSE);
+		}
+		else
+		{
+			mLauncher->Cancel ();
+			LaunchExternalDownloader ();
+		}
 	 	break;
 	 case MIME_RUN_PROGRAM:
 		FindHelperApp ();
 	 	break;
-	 case MIME_OVERRIDE_MOZILLA:
+	 default:
 		/* shouldn't get here */
 		g_warning ("Unexpected MIME action");
 		break;
@@ -445,26 +433,26 @@ NS_IMETHODIMP GContentHandler::
 	nsCOMPtr<nsIDOMWindowInternal> aWindowInternal = 
 					do_QueryInterface (aWindowContext);
 	
-
-	nsCOMPtr<nsIPref> prefs = do_GetService (NS_PREF_CONTRACTID);
-
+	nsCOMPtr<nsIPrefService> prefService = 
+				do_GetService (NS_PREFSERVICE_CONTRACTID);
+	nsCOMPtr<nsIPrefBranch> pref;
+	prefService->GetBranch ("browser.download.", getter_AddRefs (pref));
 	nsCOMPtr<nsILocalFile> saveDir;
-	rv = prefs->GetFileXPref ("browser.download.dir",
-				  getter_AddRefs(saveDir));
+	rv = pref->GetComplexValue ("dir", NS_GET_IID(nsILocalFile),
+				    getter_AddRefs(saveDir));
 
 	if (NS_FAILED(rv))
 	{
-		char *dirName = g_strdup (gnome_config_get_string 
-				("/galeon/Downloading/download_dir"));
+		char *dirName = gnome_config_get_string (CONF_DOWNLOADING_DIR);
 
 		if (!dirName)
 		{
 			dirName = g_strdup (g_get_home_dir());
-		};
+		}
 		saveDir = do_CreateInstance (NS_LOCAL_FILE_CONTRACTID);
 		saveDir->InitWithPath (dirName);
 		g_free (dirName);
-	};
+	}
 
 	nsCOMPtr<nsIFilePicker> aFilePicker =
 				do_CreateInstance (G_FILEPICKER_CONTRACTID);
@@ -479,7 +467,7 @@ NS_IMETHODIMP GContentHandler::
 	nsCOMPtr <nsILocalFile> aFile (do_CreateInstance(NS_LOCAL_FILE_CONTRACTID));
 
 	PRInt16 okToSave;
-	if (gnome_config_get_bool("/galeon/Downloading/ask_for_download_dir"))
+	if (gnome_config_get_bool (CONF_DOWNLOADING_ASK_DIR))
 	{
 		aFilePicker->Show (&okToSave);
 
@@ -510,8 +498,8 @@ NS_IMETHODIMP GContentHandler::
 		rv = aFile->GetParent (getter_AddRefs(aDirectory));
 		nsCOMPtr<nsILocalFile> aLocalDirectory 
 					(do_QueryInterface(aDirectory));
-		rv = prefs->SetFileXPref ("browser.download.dir",
-					  aLocalDirectory);
+		rv = pref->SetComplexValue ("dir", NS_GET_IID(nsILocalFile),
+					    aLocalDirectory);
 		NS_IF_ADDREF (*_retval = aFile);
 		return NS_OK;
 	};
@@ -540,91 +528,64 @@ NS_IMETHODIMP GContentHandler::
 
 NS_IMETHODIMP GContentHandler::LaunchExternalDownloader (void)
 {
-	save_url (mUrl);
+	downloader_save_url (mUrl);
 	return NS_OK;
 }
 
+
 NS_IMETHODIMP GContentHandler::FindHelperApp (void)
 {
-	nsresult rv;
-
-/*
- 	GnomeVFSMimeActionType actionType = 
- 		gnome_vfs_mime_get_default_action_type (mMimeType);
-
-	GnomeVFSMimeApplication *helper = 
-		gnome_vfs_mime_get_default_application (mMimeType);
-*/
-// 	if ((actionType & GNOME_VFS_MIME_ACTION_TYPE_APPLICATION) &&
-// 	    helper)
- 	{
- 		rv = ChooseHelperApp ();
- 	}
-/*	else if (!(actionType & GNOME_VFS_MIME_ACTION_TYPE_APPLICATION) ||
-		 !helper)
-	{
-		char *msgStr = g_strdup_printf (_("There are no known helper "
-						"applications for MIME type: "
-						"%s."),mMimeType);
-
-		GtkWidget *msgDialog = 
-			gnome_message_box_new (msgStr,
-					       GNOME_MESSAGE_BOX_INFO,
-					       GNOME_STOCK_BUTTON_OK,
-					       NULL);
-					       
-		nsCOMPtr<nsIDOMWindow> parent = do_QueryInterface (mContext);
-		GtkWidget *aParentWidget = mozilla_find_gtk_parent (parent);
-
-		if (aParentWidget)
-		{
-			gnome_dialog_set_parent (GNOME_DIALOG(msgDialog),
-						 GTK_WINDOW(aParentWidget));
-		}
-		else
-		{
-			gtk_widget_show (msgDialog);
-			window_set_layer (msgDialog);
-		};
-		
-		gnome_dialog_run_and_close (GNOME_DIALOG (msgDialog));
-		g_free (msgStr);
-		mLauncher->Cancel ();
-
-	};*/
-	return NS_OK;
+	if (mMimeItem->alwaysUse)
+		return ShowHelperProgressDialog ();
+	else
+		return ChooseHelperApp ();
 }
 
 NS_IMETHODIMP GContentHandler::LaunchHelperApp (void)
 {
-	GnomeVFSMimeApplication *helper = 
-		gnome_vfs_mime_get_default_application (mMimeType);
-	
-	if (helper)
+	if (mMimeItem)
 	{
+#if MOZILLA_VERSION > VERSION3(0,9,2)
+		nsresult rv;
+		nsCOMPtr<nsIExternalHelperAppService> helperService =
+			do_GetService (NS_EXTERNALHELPERAPPSERVICE_CONTRACTID);
+
+		nsCOMPtr<nsPIExternalAppLauncher> appLauncher =
+			do_QueryInterface (helperService, &rv);
+		if (NS_SUCCEEDED(rv))
+		{
+			appLauncher->DeleteTemporaryFileOnExit(mTempFile);
+		}
+#endif
 		char *aFileName, *document(NULL);
 		mTempFile->GetPath(&aFileName);
 
-		if (helper->expects_uris == 
-			GNOME_VFS_MIME_APPLICATION_ARGUMENT_TYPE_PATHS)
-		{
-			document = aFileName;
-		}
-		else
+		if (mMimeItem->urlHelper)
 		{
 			document = mUrl;
 		}
-		if (helper->requires_terminal == FALSE)
+		else
 		{
-			char *argv[2] = { mHelperApp->command, document };
-			gnome_execute_async ("/tmp", 2, argv);
+			document = aFileName;
+		}
+		char *full_command = g_strconcat (mMimeItem->helperApp,
+						  " ", document, NULL);
+
+		if (mMimeItem->terminalHelper == FALSE)
+		{
+			char **argv = g_strsplit (full_command, " ", 100);
+			int i;
+			for (i = 0; argv[i] != NULL; i++);
+			gnome_execute_async ("/tmp", i, argv);
+			g_strfreev (argv);
 		}
 		else
 		{
-			char *argv[4] = { "gnome-terminal", "-e",
-					  mHelperApp->command, document };
-			gnome_execute_async ("/tmp", 4, argv);
+			char *argv[3] = { "gnome-terminal", "-e",
+					  full_command };
+			gnome_execute_async ("/tmp", 3, argv);
 		}
+		g_free (full_command);
 		g_free (aFileName);
 	}
 	else
@@ -653,12 +614,29 @@ NS_IMETHODIMP GContentHandler::GetContext (nsISupports * *_retval)
 	return NS_OK;
 }
 
-NS_IMETHODIMP GContentHandler::SetHelperApp(GnomeVFSMimeApplication *aHelperApp)
+NS_IMETHODIMP GContentHandler::SetHelperApp(GnomeVFSMimeApplication *aHelperApp,
+					    PRBool alwaysUse)
 {
-	if (nsCRT::strcmp(aHelperApp->id,"OtHeR") == 0)
-		mHelperApp = aHelperApp;
+	mMimeItem->helperApp = g_strdup (aHelperApp->command);
+	mMimeItem->alwaysUse = alwaysUse ? TRUE : FALSE;
+	mMimeItem->terminalHelper = aHelperApp->requires_terminal;
+	if (aHelperApp->expects_uris ==
+				GNOME_VFS_MIME_APPLICATION_ARGUMENT_TYPE_PATHS)
+	{
+		mMimeItem->urlHelper = FALSE;
+	}
 	else
-		mHelperApp = gnome_vfs_mime_application_copy (aHelperApp);
+	{
+		mMimeItem->urlHelper = TRUE;
+	}
+
+	if (nsCRT::strcmp(aHelperApp->id,"OtHeR") == 0)
+	{
+		g_free (aHelperApp->command);
+		g_free (aHelperApp->id);
+		delete aHelperApp;
+	}
+
 	return NS_OK;
 }
 
@@ -692,39 +670,45 @@ NS_IMETHODIMP GContentHandler::Init (void)
 
 NS_IMETHODIMP GContentHandler::ProcessMimeInfo (void)
 {
+	/* FIXME: hack for 
+           http://www.xref-tech.com/xrefactory/ftp/xref-i86pcLinux.tar.gz?Cache
+	*/
+	if (mMimeType == NULL)
+	{
+		mMimeType = g_strdup ("unknown/unknown");
+	}
+
 	/* FIXME this it to work around a mozilla bug, when closing popups
 	   sometimes mime types handled internally by mozilla are passed
 	   to the content handler. Force the correct behavior */
-	if
-	(!strcasecmp(mMimeType, "text/html")   ||
-	 !strcasecmp(mMimeType, "text/plain")  ||
-	 !strcasecmp(mMimeType, "application/vnd.mozilla.xul+xml")    ||
-	 !strcasecmp(mMimeType, "text/rdf")    ||
-	 !strcasecmp(mMimeType, "text/xml")    ||
-	 !strcasecmp(mMimeType, "application/xml")    ||
-	 !strcasecmp(mMimeType, "application/xhtml+xml")    ||
-	 !strcasecmp(mMimeType, "text/css")    ||
-	 !strcasecmp(mMimeType, "image/gif")   ||
-	 !strcasecmp(mMimeType, "image/jpeg")  ||
-	 !strcasecmp(mMimeType, "image/png")   ||
-	 !strcasecmp(mMimeType, "application/http-index-format"))
+	if (!strcasecmp(mMimeType, "text/html")   ||
+	    !strcasecmp(mMimeType, "text/plain")  ||
+	    !strcasecmp(mMimeType, "application/vnd.mozilla.xul+xml")    ||
+	    !strcasecmp(mMimeType, "text/rdf")    ||
+	    !strcasecmp(mMimeType, "text/xml")    ||
+	    !strcasecmp(mMimeType, "application/xml")    ||
+	    !strcasecmp(mMimeType, "application/xhtml+xml")    ||
+	    !strcasecmp(mMimeType, "text/css")    ||
+	    !strcasecmp(mMimeType, "image/gif")   ||
+	    !strcasecmp(mMimeType, "image/jpeg")  ||
+	    !strcasecmp(mMimeType, "image/png")   ||
+	    !strcasecmp(mMimeType, "application/http-index-format"))
 	{
-		mMimeAction = MIME_IGNORE;
+		mMimeItem = new MimeItem;
+		mMimeItem->action = MIME_IGNORE;
 		g_warning ("wrong mime type detection !");
 		return NS_OK;
-	};
+	}
 
-	for (GList *mime_type = g_list_first(mime_types) ;
-	     mime_type!=NULL ;
-	     mime_type = g_list_next(mime_type))
+	mMimeItem = mime_item_get (mMimeType);	
+
+	if (!mMimeItem)
 	{
-		gchar *type = ((MimeItem *)mime_type->data)->name;
-		if (nsCRT::strcasecmp(mMimeType, type) == 0)
-		{
-			mMimeAction = ((MimeItem *)mime_type->data)->action;
-			break;
-		};
-	};
+		mime_item_create (mMimeType, MIME_UNKNOWN, _("None"),
+				  FALSE, FALSE, FALSE);
+		mMimeItem = mime_item_get (mMimeType);
+	}
+
 	return NS_OK;
 }
 
@@ -744,14 +728,10 @@ NS_IMETHODIMP GContentHandler::MIMEAskAction (void)
 	nsCOMPtr<nsIDOMWindow> parent = do_QueryInterface (mContext);
 	GtkWidget *aParentWidget = mozilla_find_gtk_parent (parent);
 
-	if (aParentWidget)
-		gnome_dialog_set_parent (GNOME_DIALOG(dialogWidget),
-					 GTK_WINDOW(aParentWidget));
-	window_set_layer (dialogWidget);
+	dialog_set_parent (dialogWidget, aParentWidget);
 
 	label = glade_xml_get_widget (dialog->gxml, "mime_ask_action_message");
-	msg = g_strdup_printf (_("The url %s has MIME type %s.\n"
-				 "What do you want to do with it?"),
+	msg = g_strdup_printf (_("The URL %s has MIME type %s"),
 			       mUrl, mMimeType);
 	gtk_label_set_text (GTK_LABEL (label), msg);
 	g_free (msg);
@@ -770,18 +750,22 @@ NS_IMETHODIMP GContentHandler::ChooseHelperApp (void)
 
 	dialog->mOptionMenu = glade_xml_get_widget (dialog->mGXML,
 						    "option_menu");
-	dialog->mBrowseButton = glade_xml_get_widget (dialog->mGXML,
-						      "browse_button");
+	dialog->mHelperGnomeEntry = 
+		glade_xml_get_widget (dialog->mGXML, 
+				      "other_helper_gnomefileentry");
 	dialog->mHelperEntry = glade_xml_get_widget (dialog->mGXML,
 						     "other_helper_entry");
+	dialog->mCheckUseTerminal = glade_xml_get_widget (dialog->mGXML,
+							  "check_use_terminal");
+	dialog->mCheckUseUrl = glade_xml_get_widget (dialog->mGXML,
+						     "check_use_url");
+	dialog->mCheckAlwaysUse = glade_xml_get_widget (dialog->mGXML,
+							"check_make_default");
 
 	nsCOMPtr<nsIDOMWindow> parent = do_QueryInterface (mContext);
 	GtkWidget *aParentWidget = mozilla_find_gtk_parent (parent);
 
-	if (aParentWidget)
-		gnome_dialog_set_parent (GNOME_DIALOG(dialog->mDialogWidget),
-					 GTK_WINDOW(aParentWidget));
-	window_set_layer (dialog->mDialogWidget);
+	dialog_set_parent (dialog->mDialogWidget, aParentWidget);
 
 	GtkWidget *menu = gtk_menu_new ();
 	for (GList *helperApp = g_list_first(dialog->mHelperApps) ;
@@ -792,12 +776,27 @@ NS_IMETHODIMP GContentHandler::ChooseHelperApp (void)
 			((GnomeVFSMimeApplication *)(helperApp->data))->name);
 		gtk_menu_append (GTK_MENU(menu),menuItem);
 		gtk_widget_show (menuItem);
-	};
+	}
 
 	gtk_option_menu_remove_menu (GTK_OPTION_MENU(dialog->mOptionMenu));
 	gtk_option_menu_set_menu (GTK_OPTION_MENU(dialog->mOptionMenu),
 				  menu);
 
+	gtk_entry_set_text (GTK_ENTRY(dialog->mHelperEntry),
+			    mMimeItem->helperApp);
+	gtk_toggle_button_set_active (
+			GTK_TOGGLE_BUTTON(dialog->mCheckUseTerminal),
+			mMimeItem->terminalHelper);
+	gtk_toggle_button_set_active (
+			GTK_TOGGLE_BUTTON(dialog->mCheckUseUrl),
+			mMimeItem->urlHelper);
+
+	gtk_widget_set_sensitive (dialog->mOptionMenu, TRUE);
+	gtk_widget_set_sensitive (dialog->mHelperGnomeEntry, FALSE);
+	gtk_widget_set_sensitive (dialog->mHelperEntry, FALSE);
+	gtk_widget_set_sensitive (dialog->mCheckUseTerminal, FALSE);
+	gtk_widget_set_sensitive (dialog->mCheckUseUrl, FALSE);
+
 	GtkWidget *label = glade_xml_get_widget (dialog->mGXML, "mime_label");
 	char *msg = g_strdup_printf (_("MIME type: %s"), mMimeType);
 	gtk_label_set_text (GTK_LABEL(label), msg);
@@ -822,7 +821,7 @@ nsresult NS_NewContentHandlerFactory(nsIFactory** aFactory)
 	if (result == NULL)
 	{
 		return NS_ERROR_OUT_OF_MEMORY;
-	};
+	}
     
 	NS_ADDREF(result);
 	*aFactory = result;
@@ -861,7 +860,7 @@ GDownloadProgressListener::
 	GtkWidget *aParentWidget = mozilla_find_gtk_parent (parent);
 
 	GladeXML *gxml = glade_widget_new ("galeon.glade", 
-					   "dl_progress_dialog",
+					   "download_progress_dialog",
 					   &mProgressDialog, this);
 
 	mProgressBar = glade_xml_get_widget (gxml, "progressbar");
@@ -870,14 +869,16 @@ GDownloadProgressListener::
 	mStatus = glade_xml_get_widget (gxml, "status_entry");
 	mTimeElapsed = glade_xml_get_widget (gxml, "elapsed_entry");
 	mTimeRemaining = glade_xml_get_widget (gxml, "remaining_entry");
+	mPauseButton = glade_xml_get_widget (gxml, "pause_button");
+	gtk_object_unref (GTK_OBJECT (gxml));
 
-	if (aParentWidget)
-		gnome_dialog_set_parent (GNOME_DIALOG(mProgressDialog),
-					 GTK_WINDOW(aParentWidget));
+	mCheckedCanPause = PR_FALSE;
+	mCanPause = PR_FALSE;
+	mIsPaused = PR_FALSE;
 
 	PRInt64 now = PR_Now ();
 	mLastUpdate = now;
-#if MOZILLA_VERSION > VERSION3(0,9,1)
+#if MOZILLA_VERSION > VERSION4(0,9,2,1)
 	mStartTime = mTimeDownloadStarted;
 #else
 	mStartTime = now;
@@ -898,8 +899,7 @@ GDownloadProgressListener::
 			    FormatTime(mElapsed/1000000));
 	gtk_label_set_text (GTK_LABEL(mTimeRemaining),FormatTime(0));
 
-	gtk_widget_show (mProgressDialog);
-	window_set_layer (mProgressDialog);
+	dialog_set_parent (mProgressDialog, aParentWidget);
 
 	aLauncher->SetWebProgressListener (this);
 }
@@ -920,11 +920,10 @@ NS_IMETHODIMP GDownloadProgressListener::
 		if (GTK_IS_WIDGET(mProgressDialog))
 			gtk_widget_destroy (mProgressDialog);
 		mLauncher->CloseProgressWindow ();
+		if (mUsedByHelper)
+			mHandler->LaunchHelperApp ();
 	}
 
-	if (mUsedByHelper)
-		mHandler->LaunchHelperApp ();
-
 	return NS_OK;
 }
 
@@ -938,7 +937,27 @@ NS_IMETHODIMP GDownloadProgressListener::
 					 PRInt32 aMaxTotalProgress)
 {
 	char *text, *rateStr, *totalStr;
-	
+
+	if (!mCheckedCanPause)
+	{
+		mCheckedCanPause = PR_TRUE;
+
+		nsresult rv;
+		nsCOMPtr<nsIFTPChannel> aChannel = do_QueryInterface (aRequest,
+								      &rv);
+		if (NS_SUCCEEDED(rv))
+		{
+			mCanPause = PR_TRUE;
+			gtk_widget_show (mPauseButton);
+		}
+		else
+		{
+			mCanPause = PR_FALSE;
+			gtk_widget_hide (mPauseButton);
+		}
+	}
+	mRequest = aRequest;
+
 	PRInt64 now = PR_Now ();
 
 	if ((now - mLastUpdate < mInterval) && 
@@ -1002,8 +1021,15 @@ NS_IMETHODIMP GDownloadProgressListener::
 		rateStr = g_strdup ("??.??");
 	}
 
-	text = g_strdup_printf (_("%dK of %sK bytes at %sK bytes/sec"),
-				currentKBytes, totalStr, rateStr);
+	if (mIsPaused)
+	{
+		text = g_strdup (_("Download Paused"));
+	}
+	else
+	{
+		text = g_strdup_printf (_("%dK of %sK bytes at %sK bytes/sec"),
+					currentKBytes, totalStr, rateStr);
+	}
 	gtk_label_set_text (GTK_LABEL(mStatus),text);
 	g_free (text);
 	g_free (totalStr);
@@ -1053,9 +1079,42 @@ NS_IMETHODIMP GDownloadProgressListener::
 NS_IMETHODIMP GDownloadProgressListener::CancelHelperProgress (void)
 {
 	mUsedByHelper = PR_FALSE;
+	if (mIsPaused)
+		TogglePause ();
 	return mLauncher->Cancel ();
 }
 
+NS_IMETHODIMP GDownloadProgressListener::TogglePause (void)
+{
+	if (!mCheckedCanPause) return NS_ERROR_FAILURE;
+
+	nsresult rv;
+	if (mIsPaused)
+	{
+		rv = mRequest->Resume ();
+		if (NS_SUCCEEDED(rv))
+		{
+			gtk_label_set_text (GTK_LABEL(GTK_BIN(mPauseButton)
+					    ->child),
+					    _("Pause"));
+		}
+	}
+	else
+	{
+		rv = mRequest->Suspend ();
+		if (NS_SUCCEEDED(rv))
+		{
+			gtk_label_set_text (GTK_LABEL(mStatus),
+					    _("Download Paused"));
+			gtk_label_set_text (GTK_LABEL(GTK_BIN(mPauseButton)
+					    ->child),
+					    _("Resume"));
+		}
+	}
+	mIsPaused = !mIsPaused;
+	return rv;
+}
+
 char *GDownloadProgressListener::FormatTime (PRUint32 aTime)
 {
 	PRUint32 secs = (PRUint32)(aTime+.5);
@@ -1081,8 +1140,6 @@ mime_ask_dialog_ok_clicked_cb (GtkButton *button, MimeAskActionDialog *dialog)
 	MimeAction action;
 	GtkWidget *save_check = glade_xml_get_widget 
 		(dialog->gxml, "mime_action_save");
-	GtkWidget *save_gtm_check = glade_xml_get_widget
-		(dialog->gxml, "mime_action_save_gtm");
 	GtkWidget *run_program_check = 
 		glade_xml_get_widget (dialog->gxml, "mime_action_run_program");
 	GtkWidget *remember_check =
@@ -1093,9 +1150,6 @@ mime_ask_dialog_ok_clicked_cb (GtkButton *button, MimeAskActionDialog *dialog)
 
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (save_check)))
 		action = MIME_SAVE_TO_DISK;
-	else if (gtk_toggle_button_get_active 
-			(GTK_TOGGLE_BUTTON (save_gtm_check)))
-		action = MIME_SAVE_WITH_GTM;
 	else if (gtk_toggle_button_get_active 
 			(GTK_TOGGLE_BUTTON (run_program_check)))
 		action = MIME_RUN_PROGRAM;
@@ -1106,21 +1160,27 @@ mime_ask_dialog_ok_clicked_cb (GtkButton *button, MimeAskActionDialog *dialog)
 	nsCOMPtr<nsIHelperAppLauncher> aLauncher;
 	rv = dialog->mContentHandler->GetLauncher (getter_AddRefs(aLauncher));
 	
-	switch (action) {
-	case MIME_SAVE_TO_DISK:
-		aLauncher->SaveToDisk (nsnull,PR_FALSE);
+	switch (action)
+	{
+	 case MIME_SAVE_TO_DISK:
+	 	if (gnome_config_get_int (CONF_DOWNLOADING_DOWNLOADER)
+	 	    == DOWNLOADER_BUILTIN)
+	 	{
+			aLauncher->SaveToDisk (nsnull,PR_FALSE);
+		}
+		else
+		{
+			aLauncher->Cancel ();
+			dialog->mContentHandler->LaunchExternalDownloader ();
+		}
 		break;
-	case MIME_SAVE_WITH_GTM:
-		aLauncher->Cancel ();
-		dialog->mContentHandler->LaunchExternalDownloader ();
-		break;
-	case MIME_RUN_PROGRAM:
+	 case MIME_RUN_PROGRAM:
 		dialog->mContentHandler->FindHelperApp ();
 		break;
-	case MIME_ASK_ACTION:
-	case MIME_UNKNOWN:
-	case MIME_OVERRIDE_MOZILLA:
-	case MIME_IGNORE:
+	 case MIME_ASK_ACTION:
+	 case MIME_UNKNOWN:
+	 case MIME_IGNORE:
+	 default:
 		/* shouldn't get here */
 		g_warning ("Unexpected MIME action");
 		break;
@@ -1156,14 +1216,6 @@ mime_ask_dialog_cancel_clicked_cb (GtkButton *button,
 	return 0; /* FIXME: philipl, is this the right thing to return? */
 }
 
-//------------------------------------------------------------------------------
-
-HelperAppChooserDialog::HelperAppChooserDialog (void)
-{
-	mAutoDefaultChecked = FALSE;
-	mOtherHelperApp = FALSE;
-}
-
 ////////////////////////////////////////////////////////////////////////////////
 // begin HelperAppChooserDialog callbacks.
 ////////////////////////////////////////////////////////////////////////////////
@@ -1188,12 +1240,18 @@ helper_chooser_ok_clicked_cb (GtkButton *button,
 		helperApp->command = g_strdup (gtk_entry_get_text(
 						GTK_ENTRY(dialog->mHelperEntry)));
 		helperApp->id = g_strdup ("OtHeR");
-		helperApp->requires_terminal = FALSE;
-		helperApp->expects_uris =
-				GNOME_VFS_MIME_APPLICATION_ARGUMENT_TYPE_PATHS;
+		helperApp->requires_terminal = gtk_toggle_button_get_active (
+				GTK_TOGGLE_BUTTON(dialog->mCheckUseTerminal));
+		helperApp->expects_uris = gtk_toggle_button_get_active (
+					GTK_TOGGLE_BUTTON(dialog->mCheckUseUrl)) ?
+					GNOME_VFS_MIME_APPLICATION_ARGUMENT_TYPE_URIS :
+					GNOME_VFS_MIME_APPLICATION_ARGUMENT_TYPE_PATHS;
 	};
 
-	dialog->mContentHandler->SetHelperApp (helperApp);
+	PRBool alwaysUse = gtk_toggle_button_get_active (
+				GTK_TOGGLE_BUTTON(dialog->mCheckAlwaysUse)) ?
+			   PR_TRUE : PR_FALSE;
+	dialog->mContentHandler->SetHelperApp (helperApp, alwaysUse);
 	dialog->mContentHandler->ShowHelperProgressDialog ();
 
 	gnome_vfs_mime_application_list_free (dialog->mHelperApps);
@@ -1222,6 +1280,8 @@ helper_chooser_cancel_clicked_cb (GtkButton *button,
 	return 0; /* FIXME: philipl, is this the right thing to return? */
 }
 
+#if 0 /* philipl: I've changed it over to using a GnomeFileEntry for
+       * this, is this okay? -- MattA */
 void helper_chooser_browse_clicked_cb (GtkButton *button,
 				       HelperAppChooserDialog *dialog)
 {
@@ -1261,12 +1321,7 @@ void helper_chooser_browse_clicked_cb (GtkButton *button,
 		nsMemory::Free (fileName);
 	};
 }
-
-void helper_chooser_auto_default_toggled (GtkToggleButton *button,
-					  HelperAppChooserDialog *dialog)
-{
-	dialog->mAutoDefaultChecked = gtk_toggle_button_get_active (button);
-}
+#endif
 
 void helper_chooser_radio_list_toggled (GtkToggleButton *button,
 					HelperAppChooserDialog *dialog)
@@ -1283,8 +1338,10 @@ void helper_chooser_radio_other_toggled (GtkToggleButton *button,
 	gboolean active = gtk_toggle_button_get_active (button);
 	if (active == TRUE)
 		dialog->mOtherHelperApp = TRUE;
-	gtk_widget_set_sensitive (dialog->mBrowseButton, active);
+	gtk_widget_set_sensitive (dialog->mHelperGnomeEntry, active);
 	gtk_widget_set_sensitive (dialog->mHelperEntry, active);
+	gtk_widget_set_sensitive (dialog->mCheckUseTerminal, active);
+	gtk_widget_set_sensitive (dialog->mCheckUseUrl, active);
 }
 
 //////////////////////////////////////////////////////////////////////////////
@@ -1298,3 +1355,9 @@ progress_dialog_cancel_cb (GtkButton *button,
 	aListener->CancelHelperProgress ();
 	return 0; /* FIXME: philipl, is this the right thing to return? */
 }
+
+void progress_dialog_pause_cb (GtkButton *button,
+			       GDownloadProgressListener *aListener)
+{
+	aListener->TogglePause ();
+}
