@@ -22,6 +22,8 @@
 #include "Galeon.h"
 #include "embed.h"
 #include "bookmarks.h"
+#include "window.h"
+#include "session.h"
 
 #include <libgnome/gnome-defs.h>
 #include <libgnome/gnome-config.h>
@@ -54,6 +56,9 @@ impl_Galeon_Browser_addTempBookmark (impl_POA_Galeon_Browser * servant,
 				     CORBA_char * name,
 				     CORBA_char * url,
 				     CORBA_Environment * ev);
+static CORBA_boolean
+impl_Galeon_Browser_quit(impl_POA_Galeon_Browser * servant,
+			 CORBA_boolean disableServer, CORBA_Environment * ev);
 
 /*** epv structures ***/
 static PortableServer_ServantBase__epv impl_Galeon_Browser_base_epv =
@@ -68,7 +73,8 @@ static POA_Galeon_Browser__epv impl_Galeon_Browser_epv =
 	NULL,		/* _private */
 	(gpointer) & impl_Galeon_Browser_loadurl,
 	(gpointer) & impl_Galeon_Browser_configure,
-	(gpointer) & impl_Galeon_Browser_addTempBookmark
+	(gpointer) & impl_Galeon_Browser_addTempBookmark,
+	(gpointer) & impl_Galeon_Browser_quit,
 };
 
 /*** vepv structures ***/
@@ -106,37 +112,57 @@ impl_Galeon_Browser_loadurl(impl_POA_Galeon_Browser * servant,
 			  Galeon_ViewState viewstate,
 			  CORBA_Environment * ev)
 {
-	CORBA_boolean retval = TRUE;
 	gboolean tabbed_mode;  /* user pref              */
 	gboolean new_win;      /* for embed_create_*()   */
-	GaleonEmbed *ge;
-
-	tabbed_mode = gnome_config_get_bool (CONF_APPEARANCE_TABBED);
+	GaleonEmbed *embed = NULL;
+	GaleonWindow *window;
+	GList *list_item;
 
 #ifdef DEBUG_CORBA	
 	g_print("CORBA: loadurl(%s, %d)\n", url, viewstate);
 #endif
 	
-	/* FIXME: this logic of using the last embed is not ideal - Nate */
-	ge = (all_embeds == NULL ? NULL : g_list_last (all_embeds)->data);
+	tabbed_mode = gnome_config_get_bool (CONF_APPEARANCE_TABBED);
 
-	new_win = (viewstate == Galeon_WINDOW) || (viewstate == Galeon_DEFAULT && !tabbed_mode);
+	/* look through all the windows for an active embed (if we don't
+	 * find one, embed will just be NULL) */
+	for (list_item = all_windows; embed == NULL && list_item != NULL;
+		list_item = g_list_next (list_item))
+	{
+		window = list_item->data;
+		return_val_if_not_window (window, FALSE);
+		embed = window->active_embed;
+	}
+
+	/* check whether we should use a new window */
+	new_win = (viewstate == Galeon_WINDOW) ||
+		  (viewstate == Galeon_DEFAULT && !tabbed_mode);
 	
+	/* handle the different viewstates */
 	switch (viewstate)
 	{
-		case Galeon_NONE: if (ge != NULL) embed_load_url(ge, url);
-				  break;
+		case Galeon_NONE:
+			if (embed != NULL) embed_load_url (embed, url);
+			else return FALSE;
+			break;
 		case Galeon_DEFAULT:
 		case Galeon_WINDOW:
-		case Galeon_TAB:  if (strlen(url))
-					 embed_create_from_url (ge, url, TRUE,
-								new_win);
-		    		  else
-					 embed_create_default (ge, new_win);
-				  break;
+		case Galeon_TAB:
+			if (url && strlen (url))
+			{
+				embed_create_from_url (embed, url, TRUE,
+					new_win);
+			}
+		    	else
+			{
+				embed_create_from_url (embed, 
+					embed_get_default_url (NULL), 
+					TRUE, new_win);
+			}
+			break;
 	}	 
 	
-	return retval;
+	return TRUE;
 }
 
 static CORBA_boolean
@@ -164,3 +190,34 @@ impl_Galeon_Browser_addTempBookmark (impl_POA_Galeon_Browser * servant,
 	
 	return retval;
 }
+
+static CORBA_boolean
+impl_Galeon_Browser_quit (impl_POA_Galeon_Browser * servant,
+			  CORBA_boolean disableServer, CORBA_Environment * ev)
+{
+   CORBA_boolean retval = TRUE;
+   GList *l;
+
+   if (disableServer == TRUE) 
+   {
+	   galeon_server_mode = FALSE;
+   }
+
+   if (all_windows == NULL)
+   {
+   /* make sure that we quit */
+	   if (!galeon_server_mode)
+	   {
+		   session_quit (FALSE);
+	   }
+   }
+   else
+   {
+	   /* close all windows */
+	   l = g_list_copy (all_windows);
+	   g_list_foreach (l, (GFunc) window_close, NULL);
+	   g_list_free (l);
+   }
+
+   return retval;
+}
