@@ -20,6 +20,8 @@
 #include "misc.h"
 #include "window.h"
 #include "mozilla.h"
+#include "dialog.h"
+#include "glade.h"
 #include "PromptService.h"
 
 #include "nsCOMPtr.h"
@@ -35,6 +37,7 @@
 #include <gtk/gtktogglebutton.h>
 #include <libgnomeui/gnome-stock.h>
 #include <libgnome/gnome-i18n.h>
+#include <libgnome/gnome-triggers.h>
 
 /* local function prototypes */
 static void set_title (GtkWidget *dialog, const PRUnichar *title);
@@ -86,33 +89,24 @@ NS_IMETHODIMP CPromptService::Alert (nsIDOMWindow *parent,
 				     const PRUnichar *text)
 {    
 	GtkWidget *label;
-	GtkWidget *title_label;
 	GtkWidget *dialog;
 	GladeXML *gxml;
 
 	/* build the dialog */
-	gxml = glade_widget_new ("prompts.glade", 
-				 "alert_dialog", &dialog, NULL);
-	label       = glade_xml_get_widget (gxml, "label");
-	title_label = glade_xml_get_widget (gxml, "title_label");
+	gxml = glade_widget_new ("prompts.glade", "alert_dialog", 
+				 &dialog, NULL);
+	label = glade_xml_get_widget (gxml, "label");
 	gtk_object_unref (GTK_OBJECT (gxml));
 
 	/* set dynamic attributes */
 	set_title (dialog, dialogTitle);
-	set_label_text (title_label, dialogTitle);
 	set_label_text (label, text);
 
 	GtkWidget *parentWidget = mozilla_find_gtk_parent (parent);
-	if (parentWidget)
-	{
-		gnome_dialog_set_parent (GNOME_DIALOG (dialog),
-					 GTK_WINDOW(parentWidget));
-	}
-	else
-	{
-	        gtk_widget_show(dialog);
-	        window_set_layer(dialog);
-	};
+	dialog_set_parent (dialog, parentWidget);
+
+	/* make a suitable sound */
+	gnome_triggers_vdo ("", "warning", NULL);
 
 	/* run dialog and capture return values */
 	gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
@@ -133,7 +127,6 @@ NS_IMETHODIMP CPromptService::AlertCheck (nsIDOMWindow *parent,
 					  PRBool *checkValue)
 {
 	GtkWidget *label;
-	GtkWidget *title_label;
 	GtkWidget *check_button;
 	GtkWidget *dialog;
 	GladeXML *gxml;
@@ -141,29 +134,21 @@ NS_IMETHODIMP CPromptService::AlertCheck (nsIDOMWindow *parent,
 	/* build the dialog */
 	gxml = glade_widget_new ("prompts.glade", "alert_check_dialog", 
 				 &dialog, NULL);
-	label        = glade_xml_get_widget (gxml, "label");
-	title_label  = glade_xml_get_widget (gxml, "title_label");
+	label = glade_xml_get_widget (gxml, "label");
 	check_button = glade_xml_get_widget (gxml, "check_button");
 	gtk_object_unref (GTK_OBJECT (gxml));
 
 	/* set dynamic attributes */
 	set_title (dialog, dialogTitle);
-	set_label_text (title_label, dialogTitle);
 	set_label_text (label, text);
 	set_check_button_text (check_button, checkMsg);
 	set_check_button (check_button, checkValue);
 
 	GtkWidget *parentWidget = mozilla_find_gtk_parent (parent);
-	if (parentWidget)
-	{
-		gnome_dialog_set_parent (GNOME_DIALOG (dialog),
-					 GTK_WINDOW(parentWidget));
-	}
-	else
-	{
-	        gtk_widget_show(dialog);
-	        window_set_layer(dialog);
-	};
+	dialog_set_parent (dialog, parentWidget);
+
+	/* make a suitable sound */
+	gnome_triggers_vdo ("", "warning", NULL);
 
 	/* run dialog and capture return values */
 	gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
@@ -183,7 +168,6 @@ NS_IMETHODIMP CPromptService::Confirm (nsIDOMWindow *parent,
 				       PRBool *_retval)
 {
 	GtkWidget *label;
-	GtkWidget *title_label;
 	GtkWidget *dialog;
 	GladeXML *gxml;
 	gint ret;
@@ -191,26 +175,18 @@ NS_IMETHODIMP CPromptService::Confirm (nsIDOMWindow *parent,
 	/* build the dialog */
 	gxml = glade_widget_new ("prompts.glade", "confirm_dialog", 
 				 &dialog, NULL);
-	label       = glade_xml_get_widget (gxml, "label");
-	title_label = glade_xml_get_widget (gxml, "title_label");
+	label = glade_xml_get_widget (gxml, "label");
 	gtk_object_unref (GTK_OBJECT (gxml));
 
 	/* set dynamic attributes */
 	set_title (dialog, dialogTitle);
-	set_label_text (title_label, dialogTitle);
 	set_label_text (label, text);
 
 	GtkWidget *parentWidget = mozilla_find_gtk_parent (parent);
-	if (parentWidget)
-	{
-		gnome_dialog_set_parent (GNOME_DIALOG (dialog),
-					 GTK_WINDOW(parentWidget));
-	}
-	else
-	{
-	        gtk_widget_show(dialog);
-	        window_set_layer(dialog);
-	};
+	dialog_set_parent (dialog, parentWidget);
+
+	/* make a suitable sound */
+	gnome_triggers_vdo ("", "generic", NULL);
 
 	/* run dialog and capture return values */
 	ret = gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
@@ -232,49 +208,22 @@ NS_IMETHODIMP CPromptService::ConfirmCheck (nsIDOMWindow *parent,
 					    PRBool *checkValue,
 					    PRBool *_retval)
 {
-	GtkWidget *dialog;
-	GtkWidget *label;
-	GtkWidget *title_label;
-	GtkWidget *check_button;
-	GladeXML *gxml;
-	gint ret;
-
-	/* build and show the dialog */
-	gxml = glade_widget_new ("prompts.glade", "confirm_check_dialog", 
-				 &dialog, NULL);
-	label        = glade_xml_get_widget (gxml, "label");
-	title_label  = glade_xml_get_widget (gxml, "title_label");
-	check_button = glade_xml_get_widget (gxml, "check_button");
-	gtk_object_unref (GTK_OBJECT (gxml));
-	
-	/* set dynamic attributes */
-	set_title (dialog, dialogTitle);
-	set_label_text (title_label, dialogTitle);
-	set_label_text (label, text);
-	set_check_button_text (check_button, checkMsg);
-	set_check_button (check_button, checkValue);
-
+	/* convert dynamic attributes */
 	GtkWidget *parentWidget = mozilla_find_gtk_parent (parent);
-	if (parentWidget)
-	{
-		gnome_dialog_set_parent (GNOME_DIALOG (dialog),
-					 GTK_WINDOW(parentWidget));
-	}
-	else
-	{
-	        gtk_widget_show(dialog);
-	        window_set_layer(dialog);
-	};
+	gchar *cdialogTitle = mozilla_unicode_to_locale (dialogTitle),
+		*ctext = mozilla_unicode_to_locale (text),
+		*ccheckMsg = mozilla_unicode_to_locale (checkMsg);
 
 	/* run dialog and capture return values */
-	ret = gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
-
-	/* handle return values */
-	get_check_button (check_button, checkValue);
-	*_retval = (ret == 0);
+	*_retval = dialog_confirm_check (parentWidget, cdialogTitle, ctext,
+					 ccheckMsg, checkValue);
+	
+	/* free converted strings */
+	g_free (cdialogTitle);
+	g_free (ctext);
+	g_free (ccheckMsg);
 
 	/* done */
-	gtk_widget_destroy (dialog);
 	return NS_OK;
 }
 
@@ -285,10 +234,8 @@ NS_IMETHODIMP CPromptService::ConfirmEx (nsIDOMWindow *parent,
 					 const PRUnichar *dialogTitle,
 					 const PRUnichar *text,
 					 PRUint32 buttonFlags, 
-#if MOZILLA_VERSION > VERSION2(0,9)
 					 const PRUnichar *button0Title,
 					 const PRUnichar *button1Title,
-#endif
 					 const PRUnichar *button2Title,
 					 const PRUnichar *checkMsg, 
 					 PRBool *checkValue,
@@ -296,7 +243,6 @@ NS_IMETHODIMP CPromptService::ConfirmEx (nsIDOMWindow *parent,
 {
 	GtkWidget *dialog;
 	GtkWidget *label;
-	GtkWidget *title_label;
 	GtkWidget *check_button;
 	GladeXML *gxml;
 	gint ret;
@@ -305,14 +251,12 @@ NS_IMETHODIMP CPromptService::ConfirmEx (nsIDOMWindow *parent,
 	/* build and show the dialog */
 	gxml = glade_widget_new ("prompts.glade", "confirmex_dialog", 
 				 &dialog, NULL);
-	label        = glade_xml_get_widget (gxml, "label");
-	title_label  = glade_xml_get_widget (gxml, "title_label");
+	label = glade_xml_get_widget (gxml, "label");
 	check_button = glade_xml_get_widget (gxml, "check_button");
 	gtk_object_unref (GTK_OBJECT (gxml));
 	
 	/* set dynamic attributes */
 	set_title (dialog, dialogTitle);
-	set_label_text (title_label, dialogTitle);
 	set_label_text (label, text);
 	set_check_button_text (check_button, checkMsg);
 	set_check_button (check_button, checkValue);
@@ -346,7 +290,6 @@ NS_IMETHODIMP CPromptService::ConfirmEx (nsIDOMWindow *parent,
 			gnome_dialog_append_button (GNOME_DIALOG(dialog),
 						    _("Revert"));
 			break;
-#if MOZILLA_VERSION > VERSION2(0,9)
 		case BUTTON_TITLE_DONT_SAVE:
 			gnome_dialog_append_button (GNOME_DIALOG(dialog),
 						    _("Don't save"));
@@ -370,7 +313,6 @@ NS_IMETHODIMP CPromptService::ConfirmEx (nsIDOMWindow *parent,
 				g_free (tmpstr);
 			}
 			break;
-#endif
 		default:
 			break;
 		}	
@@ -379,16 +321,10 @@ NS_IMETHODIMP CPromptService::ConfirmEx (nsIDOMWindow *parent,
 	}
 
 	GtkWidget *parentWidget = mozilla_find_gtk_parent (parent);
-	if (parentWidget)
-	{
-		gnome_dialog_set_parent (GNOME_DIALOG (dialog),
-					 GTK_WINDOW(parentWidget));
-	}
-	else
-	{
-	        gtk_widget_show(dialog);
-	        window_set_layer(dialog);
-	};
+	dialog_set_parent (dialog, parentWidget);
+
+	/* make a suitable sound */
+	gnome_triggers_vdo ("", "generic", NULL);
 
 	/* run dialog and capture return values */
 	ret = gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
@@ -416,7 +352,6 @@ NS_IMETHODIMP CPromptService::Prompt (nsIDOMWindow *parent,
 	GtkWidget *dialog;
 	GtkWidget *entry;
 	GtkWidget *label;
-	GtkWidget *title_label;
 	GtkWidget *check_button;
 	GladeXML *gxml;
 	gint ret;
@@ -424,31 +359,29 @@ NS_IMETHODIMP CPromptService::Prompt (nsIDOMWindow *parent,
 	/* build and show the dialog */
 	gxml = glade_widget_new ("prompts.glade", "prompt_dialog", 
 				 &dialog, NULL);
-	entry        = glade_xml_get_widget (gxml, "entry");
-	label        = glade_xml_get_widget (gxml, "label");
-	title_label  = glade_xml_get_widget (gxml, "title_label");
+	entry = glade_xml_get_widget (gxml, "entry");
+	label = glade_xml_get_widget (gxml, "label");
 	check_button = glade_xml_get_widget (gxml, "check_button");
 	gtk_object_unref (GTK_OBJECT (gxml));
 
 	/* set dynamic attributes */
 	set_title (dialog, dialogTitle);
-	set_label_text (title_label, dialogTitle);
 	set_label_text (label, text);
 	set_check_button_text (check_button, checkMsg);
 	set_editable (entry, value);
 	set_check_button (check_button, checkValue);
 
+	/* make an enter keypress activate the "ok" button */
+	gnome_dialog_editable_enters (GNOME_DIALOG (dialog), 
+				      GTK_EDITABLE (entry));
+
 	GtkWidget *parentWidget = mozilla_find_gtk_parent (parent);
-	if (parentWidget)
-	{
-		gnome_dialog_set_parent (GNOME_DIALOG (dialog),
-					 GTK_WINDOW(parentWidget));
-	}
-	else
-	{
-	        gtk_widget_show(dialog);
-	        window_set_layer(dialog);
-	};
+	dialog_set_parent (dialog, parentWidget);
+
+	/* make a suitable sound */
+	/* NB: should be question, but this is missing in many
+	 * of the current gnome sound packages that I've tried... */
+	gnome_triggers_vdo ("", "generic", NULL);
 
 	/* run dialog and capture return values */
 	ret = gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
@@ -478,7 +411,6 @@ NS_IMETHODIMP CPromptService::PromptUsernameAndPassword
 					 PRBool *_retval)
 {
 	GtkWidget *dialog;
-	GtkWidget *title_label;
 	GtkWidget *check_button;
 	GtkWidget *username_entry;
 	GtkWidget *password_entry;
@@ -488,7 +420,6 @@ NS_IMETHODIMP CPromptService::PromptUsernameAndPassword
 	/* build and show the dialog */
 	gxml = glade_widget_new ("prompts.glade", "prompt_user_pass_dialog", 
 				 &dialog, NULL);
-	title_label    = glade_xml_get_widget (gxml, "title_label");
 	check_button   = glade_xml_get_widget (gxml, "check_button");
 	username_entry = glade_xml_get_widget (gxml, "username_entry");
 	password_entry = glade_xml_get_widget (gxml, "password_entry");
@@ -496,30 +427,22 @@ NS_IMETHODIMP CPromptService::PromptUsernameAndPassword
 
 	/* set dynamic attributes */
 	set_title (dialog, dialogTitle);
-	set_label_text (title_label, dialogTitle);
 	set_check_button_text (check_button, checkMsg);
 	set_editable (username_entry, username);
 	set_editable (password_entry, password);
 	set_check_button (check_button, checkValue);
 
 	/* make an enter keypress activate the "ok" button */
+	gnome_dialog_editable_enters (GNOME_DIALOG (dialog),
+				      GTK_EDITABLE (username_entry));
 	gnome_dialog_editable_enters (GNOME_DIALOG (dialog),
 				      GTK_EDITABLE (password_entry));
 
-	/* set the keyboard focus to the username entry */
-	gtk_widget_grab_focus (username_entry);
-
 	GtkWidget *parentWidget = mozilla_find_gtk_parent (parent);
-	if (parentWidget)
-	{
-		gnome_dialog_set_parent (GNOME_DIALOG (dialog),
-					 GTK_WINDOW(parentWidget));
-	}
-	else
-	{
-	        gtk_widget_show(dialog);
-	        window_set_layer(dialog);
-	};
+	dialog_set_parent (dialog, parentWidget);
+
+	/* make a suitable sound */
+	gnome_triggers_vdo ("", "question", NULL);
 
 	/* run dialog and capture return values */
 	ret = gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
@@ -548,7 +471,6 @@ NS_IMETHODIMP CPromptService::PromptPassword (nsIDOMWindow *parent,
 					      PRBool *_retval)
 {
 	GtkWidget *dialog;
-	GtkWidget *title_label;
 	GtkWidget *check_button;
 	GtkWidget *password_entry;
 	GladeXML *gxml;
@@ -557,29 +479,25 @@ NS_IMETHODIMP CPromptService::PromptPassword (nsIDOMWindow *parent,
 	/* build and show the dialog */
 	gxml = glade_widget_new ("prompts.glade", "prompt_pass_dialog", 
 				 &dialog, NULL);
-	title_label    = glade_xml_get_widget (gxml, "title_label");
 	check_button   = glade_xml_get_widget (gxml, "check_button");
 	password_entry = glade_xml_get_widget (gxml, "password_entry");
 	gtk_object_unref (GTK_OBJECT (gxml));
 
 	/* set dynamic attributes */
 	set_title (dialog, dialogTitle);
-	set_label_text (title_label, dialogTitle);
 	set_check_button_text (check_button, checkMsg);
 	set_editable (password_entry, password);
 	set_check_button (check_button, checkValue);
 
+	/* make an enter keypress activate the "ok" button */
+	gnome_dialog_editable_enters (GNOME_DIALOG (dialog),
+				      GTK_EDITABLE (password_entry));
+
 	GtkWidget *parentWidget = mozilla_find_gtk_parent (parent);
-	if (parentWidget)
-	{
-		gnome_dialog_set_parent (GNOME_DIALOG (dialog),
-					 GTK_WINDOW(parentWidget));
-	}
-	else
-	{
-	        gtk_widget_show(dialog);
-	        window_set_layer(dialog);
-	};
+	dialog_set_parent (dialog, parentWidget);
+
+	/* make a suitable sound */
+	gnome_triggers_vdo ("", "question", NULL);
 
 	/* run dialog and capture return values */
 	ret = gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
