@@ -21,9 +21,12 @@
 
 #include "nsError.h"
 
-#define NS_PROMPTSERVICE_CID \
+#define G_PROMPTSERVICE_CID \
    {0xa2112d6a, 0x0e28, 0x421f, {0xb4, 0x6a, 0x25, 0xc0, 0xb3, 0x8, 0xcb, 0xd0}}
 
+#define G_PROMPTSERVICE_CLASSNAME "Galeon's Prompt Service"
+#define G_PROMPTSERVICE_CONTRACTID "@mozilla.org/embedcomp/prompt-service;1"
+
 class nsIFactory;
 
 extern nsresult NS_NewPromptServiceFactory(nsIFactory** aFactory);
