@@ -72,7 +72,7 @@ static gdouble score_of_host (const HistoryHost *server);
 BookmarkItem *autobookmarks_root = NULL;
 
 /**
- * autobookmarks_generate: generate AutoBookmarks category from 
+ * autobookmarks_generate: generate AutoBookmarks folder from 
  * history elements
  */
 void 
@@ -140,8 +140,8 @@ autobookmarks_generate (void)
 	{
 		HistoryHost *host = (HistoryHost *)item->data;
 		HistoryItem *hi = (HistoryItem *)host->dominant_item;
+		gchar *title_locale, *name;
 		BookmarkItem *b;
-		gchar *name;
 
 		/* add it if within bounds */
 		if (count < maximum && hi != NULL)
@@ -150,7 +150,9 @@ autobookmarks_generate (void)
 			/* note that shorten_name takes care of
 			 * duplicating the title string for us, even 
 			 * when it's not changed */
-			name = shorten_name (hi->title, shorten);
+			title_locale = (hi->title_locale != NULL ? 
+					hi->title_locale : hi->title_utf8);
+			name = shorten_name (title_locale, shorten);
 			b = bookmarks_new_bookmark (BM_SITE, TRUE, name, 
 						    hi->url, NULL, NULL, NULL);
 			g_free(name);
@@ -166,32 +168,16 @@ autobookmarks_generate (void)
 	g_list_free(list);
 }
 
-/**
- * compare_hosts: compare the scores of two hosts
- */
-static gint 
-compare_hosts (const HistoryHost *a, const HistoryHost *b)
-{
-	gdouble score_a, score_b;
-
-	/* compute scores */
-	score_a = score_of_host(a);
-	score_b = score_of_host(b);
-
-	/* return compared values */
-	return (score_b > score_a ? 1 : -1);
-}
-
 /**
  * score_of_host: compute the "score" of a host.
  * This value represents how likely the item is to make a good AutoBookmark. 
  */
-static gdouble 
+static inline gdouble 
 score_of_host (const HistoryHost *host)
 {
-	gdouble age, age_scale;
- 	gdouble interval_scale;
-	gdouble score;
+	static gdouble age, age_scale;
+ 	static gdouble interval_scale;
+	static gdouble score;
 
 	/* check */
 	if (host->dominant_item == NULL)
@@ -215,3 +201,20 @@ score_of_host (const HistoryHost *host)
 	/* return the score */
 	return score;	
 }
+
+/**
+ * compare_hosts: compare the scores of two hosts
+ */
+static gint 
+compare_hosts (const HistoryHost *a, const HistoryHost *b)
+{
+	static gdouble score_a, score_b;
+
+	/* compute scores */
+	score_a = score_of_host(a);
+	score_b = score_of_host(b);
+
+	/* return compared values */
+	return (score_b > score_a ? 1 : -1);
+}
+
