@@ -20,9 +20,10 @@
 #include "galeon.h"
 #include "misc.h"
 #include "embed.h"
-#include "auto_completion.h"
+#include "autocompletion.h"
 #include "bookmarks.h"
 #include "window.h"
+#include "favicon.h"
 
 /* GNOME includes */
 #include <stdlib.h>
@@ -38,6 +39,8 @@
 /** The pixmaps for the bookmarks */
 PixmapData *folder_pixmap_data = NULL;
 PixmapData *folder_open_pixmap_data = NULL;
+PixmapData *default_folder_pixmap_data = NULL;
+PixmapData *default_folder_open_pixmap_data = NULL;
 PixmapData *site_pixmap_data = NULL;
 
 /** The root of the bookmark tree */
@@ -62,14 +65,16 @@ GHashTable *favicons_cache = NULL;
 const GtkTargetEntry bookmarks_dnd_targets [] = {
 	{ "GALEON_BOOKMARK", 0, DND_TARGET_GALEON_BOOKMARK },
 	{ "GALEON_URL",      0, DND_TARGET_GALEON_URL      },
-	{ "_NETSCAPE_URL",   0, DND_TARGET_NETSCAPE_URL    },
 	{ "text/uri-list",   0, DND_TARGET_TEXT_URI_LIST   },
+	{ "_NETSCAPE_URL",   0, DND_TARGET_NETSCAPE_URL    },
 	{ "STRING",          0, DND_TARGET_STRING          }
 };
 const gint bookmarks_dnd_targets_num_items = (sizeof (bookmarks_dnd_targets) /
 					      sizeof (GtkTargetEntry));
 
 /* Private prototypes */
+static void bookmarks_backup (void);
+static void bookmarks_save (void);
 static void bookmarks_load_icons (void);
 static BookmarkItem *bookmarks_xml_read_item (xmlNodePtr item);
 static BookmarkItem *bookmarks_xml_read (xmlNodePtr item);
@@ -80,6 +85,10 @@ static void bookmarks_remove_all_alias (BookmarkItem *b);
 static void bookmarks_new_alias_do (BookmarkItem *b, BookmarkItem *bookmark);
 static BookmarkItem *bookmarks_find_by_id (long id,
 					   BookmarkItem *root);
+static BookmarkItem *bookmarks_merge_trees_look_for_equal 
+	(BookmarkItem *folder, BookmarkItem *bookmark);
+static gint bookmarks_compare_name (gconstpointer a, gconstpointer b);
+
 
 /**
  * bookmarks_init: intialise bookmarks for the first time
@@ -97,15 +106,32 @@ bookmarks_init (void)
 void
 bookmarks_exit (void)
 {
+	GList *l = g_list_copy (bookmarks_editors);
+	GList *li;
+
+	bookmarks_backup ();
 	bookmarks_save ();
 
+	/* close all editors */
+	for (li = l; li != NULL; li = li->next) 
+	{
+		bookmarks_editor_hide_dialog (li->data);
+	}
+	g_list_free (l);
+
 	g_free (folder_open_pixmap_data);
 	g_free (folder_pixmap_data);
+	g_free (default_folder_open_pixmap_data);
+	g_free (default_folder_pixmap_data);
 	g_free (site_pixmap_data);
 
 	folder_open_pixmap_data = NULL;
 	folder_pixmap_data = NULL;
+	default_folder_open_pixmap_data = NULL;
+	default_folder_pixmap_data = NULL;
 	site_pixmap_data = NULL;
+
+	bookmarks_remove_recursively (bookmarks_root);
 }
 
 /**
@@ -116,26 +142,64 @@ bookmarks_load (void)
 {
 	gchar *bookfile;
 	
+	default_bookmarks_root = NULL;
+
 	bookfile = g_concat_dir_and_file (g_get_home_dir (), 
 					  "/.galeon/bookmarks.xml");
 	
-	bookmarks_root = bookmarks_load_from (bookfile);
+	bookmarks_set_root (bookmarks_load_from (bookfile));
 	
+
+	g_free (bookfile);
+}
+
+/**
+ * Sets the root of the bookmarks tree, freeing the previous tree and updating
+ * everything
+ */
+void
+bookmarks_set_root (BookmarkItem *new_root)
+{
+	GList *li;
+	/* iterate throught the list of all opened editors */
+	for (li = bookmarks_editors; li != NULL; li = li->next) {
+		BookmarksEditorControls *current_controls = li->data;
+		gtk_clist_freeze (GTK_CLIST (current_controls->ctree));
+		/* delete old bookmarks view */
+		//bookmarks_editor_remove_tree_item 
+		//(current_controls, current_controls->root_bookmark);
+	}
+	
+	/* delete the old bookmarks */
+	bookmarks_remove_recursively (bookmarks_root);
+	
+	bookmarks_root = new_root;
 	if (!bookmarks_root) {
 		bookmarks_root = bookmarks_new_bookmark 
-			(BM_CATEGORY, TRUE, _("Bookmarks"), NULL, NULL, NULL, NULL);
+			(BM_FOLDER, TRUE, _("Bookmarks"), NULL, NULL,
+			 NULL, NULL);
 	}
 	
 	if (default_bookmarks_root == NULL) {
 		default_bookmarks_root = bookmarks_root;
 	}
 
-	/* generate the autobookmarks category */
+	/* generate the autobookmarks folder */
 	autobookmarks_generate();
 	
 	bookmarks_add_completion_recursively (bookmarks_root);
 
-	g_free (bookfile);
+	/* view new bookmarks */
+	for (li = bookmarks_editors; li != NULL; li = li->next) {
+		/* FIXME: this sets the root bookmark of every editor
+		   to the root bookmark. This is incorrect. 
+		   (but not a big deal) */
+		BookmarksEditorControls *current_controls = li->data;
+		bookmarks_editor_set_root (current_controls, bookmarks_root);
+		current_controls->selection = NULL;
+		current_controls->last_pressed = NULL;
+		gtk_clist_thaw (GTK_CLIST (current_controls->ctree));
+	}
 }
 
 /**
@@ -148,6 +212,10 @@ bookmarks_load_icons (void)
 		pixmap_data_new_from_file (SHARE_DIR "/dir.xpm");
 	folder_open_pixmap_data = 
 		pixmap_data_new_from_file (SHARE_DIR "/dir_open.xpm");
+	default_folder_pixmap_data = 
+		pixmap_data_new_from_file (SHARE_DIR "/default.xpm");
+	default_folder_open_pixmap_data = 
+		pixmap_data_new_from_file (SHARE_DIR "/default_open.xpm");
 	site_pixmap_data = 
 		pixmap_data_new_from_file (SHARE_DIR "/i-bookmark.xpm");
 	/* TODO: add icons for alias */
@@ -163,6 +231,14 @@ bookmarks_load_icons (void)
 	{
 		folder_open_pixmap_data = g_new0 (PixmapData, 1);
 	}
+	if (default_folder_pixmap_data == NULL)
+	{
+		default_folder_pixmap_data = g_new0 (PixmapData, 1);
+	}
+	if (default_folder_open_pixmap_data == NULL)
+	{
+		default_folder_open_pixmap_data = g_new0 (PixmapData, 1);
+	}
 	if (site_pixmap_data == NULL)
 	{
 		site_pixmap_data = g_new0 (PixmapData, 1);
@@ -208,13 +284,14 @@ bookmarks_xml_read (xmlNodePtr item)
 	static BookmarkItem *error_bookmark = NULL;
 	BookmarkItem *b, *result;
 	item = item->childs;
+	unresolved_aliases = NULL;
 	
 	if (!error_bookmark) 
 		error_bookmark = bookmarks_new_bookmark 
 			(BM_SITE, TRUE, _("Error"), "about:blank", NULL,
 			 _("An error ocurred loading your bookmarks."), NULL);
 	
-	result = bookmarks_new_bookmark (BM_CATEGORY, TRUE, _("Bookmarks"),
+	result = bookmarks_new_bookmark (BM_FOLDER, TRUE, _("Bookmarks"),
 					 NULL, NULL, NULL, NULL);
 	
 	if (item->next) /* old format */
@@ -252,11 +329,11 @@ bookmarks_xml_read_item (xmlNodePtr item)
 {
 	BookmarkItem *b;
 	BookmarkItem *b2;
-	gchar *ct, *expanded, *ccm, *default_root;
+	gchar *ct, *expanded, *ccm, *default_root, *tbs;
 
-	gchar *name = xmlGetPropISOLatin1 (item, "name");	
-	gchar *notes = xmlGetProp (item, "notes");
-	gchar *nick = xmlGetProp (item, "nick");
+	gchar *name = xmlGetRawPropLocale (item, "name");	
+	gchar *notes = xmlGetRawPropLocale (item, "notes");
+	gchar *nick = xmlGetRawPropLocale (item, "nick");
 	gchar *pixmap_file = xmlGetProp (item, "pixmap");
 	gchar *idstr = xmlGetProp (item, "id");
 	gchar *time_added_str = xmlGetProp (item, "time_added");
@@ -265,16 +342,22 @@ bookmarks_xml_read_item (xmlNodePtr item)
   
 	g_return_val_if_fail(item != NULL, NULL);
 
-	if (!strcmp (item->name, "category")) 
+	if (strcmp (item->name, "folder") == 0 ||
+	    strcmp (item->name, "category") == 0) 
 	{
-		b = bookmarks_new_bookmark (BM_CATEGORY, FALSE, name, NULL,
+		b = bookmarks_new_bookmark (BM_FOLDER, FALSE, name, NULL,
 					    nick, notes, pixmap_file);
 		ct = xmlGetProp (item, "create_toolbar");
+		tbs = xmlGetProp (item, "toolbar_style");
 		ccm = xmlGetProp (item, "create_context_menu");
 		expanded = xmlGetProp (item, "expanded");
 	        default_root = xmlGetProp (item, "default_bookmarks_root");
 		b->create_toolbar = (ct && !strcmp (ct, "TRUE") ? 
 				     TRUE : FALSE);
+		if (tbs)
+			b->toolbar_style = atoi(tbs);
+		if (b->toolbar_style > 2) /* backwards compatiblity FIXME */
+			b->toolbar_style = TOOLBAR_STYLE_HORIZONTAL;
 		b->create_context_menu = (ccm && !strcmp (ccm, "TRUE") ? 
 					  TRUE : FALSE);
 		b->expanded = (expanded && !strcmp (expanded, "TRUE") ? 
@@ -284,6 +367,7 @@ bookmarks_xml_read_item (xmlNodePtr item)
 			default_bookmarks_root = b;
 		}
 	        xmlFree (ct);		
+		xmlFree (tbs);
 		xmlFree (ccm);
 		xmlFree (expanded);
 		xmlFree (default_root);
@@ -308,12 +392,16 @@ bookmarks_xml_read_item (xmlNodePtr item)
 		b = bookmarks_new_bookmark (BM_AUTOBOOKMARKS, FALSE, name, 
 					    NULL, nick, notes, pixmap_file);
 		ct = xmlGetProp (item, "create_toolbar");
+		tbs = xmlGetProp (item, "toolbar_style");
 		ccm = xmlGetProp (item, "create_context_menu");
 		expanded = xmlGetProp (item, "expanded");
 		b->create_toolbar = (ct && !strcmp (ct, "TRUE")) ? TRUE : FALSE;
+		if (tbs)
+			b->toolbar_style = atoi(tbs);
 		b->create_context_menu = (ccm && !strcmp (ccm, "TRUE")) ? TRUE : FALSE;
 		b->expanded = (expanded && !strcmp (expanded, "TRUE")) ? TRUE : FALSE;
 		xmlFree (ct);
+		xmlFree (tbs);
 		xmlFree (ccm);
 		xmlFree (expanded);
 		autobookmarks_root = b;
@@ -373,6 +461,7 @@ bookmarks_updated (void)
 
 	/* force a save */
 	bookmarks_dirty = TRUE;
+	bookmarks_backup ();
 	bookmarks_save ();
 
 	/* update each window */
@@ -396,9 +485,78 @@ bookmarks_updated (void)
 	/* FIXME: update applet... */
 }
 
+/**
+ * bookmarks_backup: backup old bookmarks file
+ */
+/* FIXME: check for errors (permissions...) */
+void
+bookmarks_backup (void)
+{
+	int count = 0;
+
+	/* check if it's not enabled */
+	if (gnome_config_get_bool (CONF_BOOKMARKS_BACKUP_ENABLED) == FALSE)
+	{
+		return;
+	}
+
+	if (!bookmarks_dirty)
+	{
+		return;
+	}
+
+	for (count = gnome_config_get_int (CONF_BOOKMARKS_BACKUP_COUNT); 
+	     count >= 1 ; count--) {
+		gchar *str;
+		gchar *str2;
+		gchar *filename;
+		gchar *filename2;
+		GnomeVFSURI *filename_uri;
+		GnomeVFSURI *filename2_uri;
+		GnomeVFSResult rc;
+
+		str = g_strdup_printf ("%d", count);
+		str2 = g_strdup_printf ("%d", count - 1);
+
+		if (count == 1) {
+			/* FIXME: refactor into bookmarks_filename ... */
+			filename = g_strconcat 
+				(g_get_home_dir (),
+				 "/.galeon/bookmarks.xml", NULL);
+		} else {
+			/* FIXME: refactor into bookmarks_filename ... */
+			filename = g_strconcat 
+				(g_get_home_dir (), 
+				 "/.galeon/bookmarks.xml.", str2, NULL);
+		}
+
+		/* FIXME: refactor into bookmarks_filename ... */
+		filename2 = g_strconcat 
+			(g_get_home_dir (), "/.galeon/bookmarks.xml.", str,
+			 NULL);
+
+		/* give it head :) now before we leave the bookmarks keep at */
+		/* least one backup of last saved file for emergency reasons */
+		filename_uri = gnome_vfs_uri_new (filename);
+		filename2_uri = gnome_vfs_uri_new (filename2);
+
+		rc = gnome_vfs_move_uri (filename_uri, filename2_uri, TRUE);
+
+		gnome_vfs_uri_unref (filename_uri);
+		gnome_vfs_uri_unref (filename2_uri);
+
+		/* free allocated strings */
+		g_free (str);
+		g_free (str2);
+		g_free (filename);
+		g_free (filename2);
+	}
+}
+
 /**
  * bookmarks_save: save all bookmarks to the xml file
  */
+/* FIXME: check for errors (permissions...) */
 void
 bookmarks_save (void)
 {
@@ -412,37 +570,34 @@ bookmarks_save (void)
 	/* FIXME: refactor into bookmarks_filename ... */
 	filename = g_strconcat (g_get_home_dir (), 
 				"/.galeon/bookmarks.xml", NULL);
+	
+	bookmarks_save_as (filename);
 
-	/* save the bookmarks */
-	bookmarks_save_as (bookmarks_root, filename);
+	/* now clean */
 	bookmarks_dirty = FALSE;
 
 	/* free allocated string */
 	g_free (filename);
 }
 
-/**
- * bookmarks_save_as: Save a set of bookmarks
- * @root: the root category to be saved
- * @file: the filename, if already exists will be overwriten
- **/
-/* FIXME: check for errors (permisions...) */
 void 
-bookmarks_save_as (BookmarkItem *root, char *file)
+bookmarks_save_as (const gchar *filename)
 {
 	xmlNodePtr node;
-	xmlDocPtr doc = xmlNewDoc ("1.0");
-	node = xmlNewDocNode (doc, NULL, "bookmarks", NULL);
+	xmlDocPtr doc;
 	
-	bookmarks_save_recursively (doc, node, root);
-
+	/* build an XML document and save it to the filename */
+	doc = xmlNewDoc ("1.0");
+	node = xmlNewDocNode (doc, NULL, "bookmarks", NULL);
 	xmlDocSetRootElement (doc, node);
-	xmlSaveFile (file, doc);
-	xmlFreeDoc (doc);
+	bookmarks_save_recursively (doc, node, bookmarks_root);
+	xmlSaveFile (filename, doc);
+	xmlFreeDoc (doc);     
 }
 
 /**
- * bookmarks_save_recursively: recursively save a bookmarks tree to the xml file 
+ * bookmarks_save_recursively: recursively save a bookmarks tree to 
+ * the xml file 
  */
 static void
 bookmarks_save_recursively (xmlDocPtr doc, xmlNodePtr root, BookmarkItem *b)
@@ -452,6 +607,7 @@ bookmarks_save_recursively (xmlDocPtr doc, xmlNodePtr root, BookmarkItem *b)
 	GList *li;
 	gchar *idstr = g_strdup_printf ("%ld", b->id);
 	GString *s = g_string_new ("");
+	gchar *tmp;
 
 	if (b->alias_of)
 	{
@@ -459,6 +615,12 @@ bookmarks_save_recursively (xmlDocPtr doc, xmlNodePtr root, BookmarkItem *b)
 		xmlSetProp (node, "id", idstr);
 		if (b->create_toolbar)
 			xmlSetProp (node, "create_toolbar", "TRUE");
+		if (b->toolbar_style != TOOLBAR_STYLE_HORIZONTAL)
+		{
+			tmp = g_strdup_printf("%d", b->toolbar_style);
+			xmlSetProp (node, "toolbar_style", tmp);
+			g_free (tmp);
+		}
 		if (b->create_context_menu)
 			xmlSetProp (node, "create_context_menu", "TRUE");
 		if (b->time_added) {
@@ -471,24 +633,30 @@ bookmarks_save_recursively (xmlDocPtr doc, xmlNodePtr root, BookmarkItem *b)
 	{
 		switch (b->type)
 		{
-		case BM_CATEGORY:
-			node = xmlNewDocNode (doc, NULL, "category", NULL);
+		case BM_FOLDER:
+			node = xmlNewDocNode (doc, NULL, "folder", NULL);
 			xmlSetProp (node, "id", idstr);
-			xmlSetPropISOLatin1 (node, "name", b->name);
+			xmlSetRawPropLocale (node, "name", b->name);
 			if (b->create_toolbar)
 				xmlSetProp (node, "create_toolbar", "TRUE");
+			if (b->toolbar_style != TOOLBAR_STYLE_HORIZONTAL)
+			{
+				tmp = g_strdup_printf("%d", b->toolbar_style);
+				xmlSetProp (node, "toolbar_style", tmp);
+				g_free (tmp);
+			}
 			if (b->create_context_menu)
 				xmlSetProp (node, "create_context_menu",  "TRUE");
 			if (b->expanded)
 				xmlSetProp (node, "expanded", "TRUE");
 			if (*b->notes)
-				xmlSetProp (node, "notes", b->notes);
+				xmlSetRawPropLocale (node, "notes", b->notes);
 			if (default_bookmarks_root == b) {
 				xmlSetProp (node, "default_bookmarks_root", 
 					    "TRUE");
 			}
 			if (b->pixmap_file && *b->pixmap_file)
-				xmlSetProp(node, "pixmap", b->pixmap_file);
+				xmlSetProp (node, "pixmap", b->pixmap_file);
 			if (b->time_added) {
 				g_string_sprintf (s, "%d", b->time_added);
 				xmlSetProp (node, "time_added", s->str);
@@ -517,16 +685,16 @@ bookmarks_save_recursively (xmlDocPtr doc, xmlNodePtr root, BookmarkItem *b)
 		case BM_SITE:
 			node = xmlNewDocNode(doc, NULL, "site", NULL);
 			xmlSetProp (node, "id", idstr);
-			xmlSetPropISOLatin1 (node, "name", b->name);
-			xmlSetProp(node, "url", b->url);    
+			xmlSetRawPropLocale (node, "name", b->name);
+			xmlSetProp (node, "url", b->url);    
 			if (*b->nick)
-				xmlSetProp(node, "nick", b->nick);
+				xmlSetRawPropLocale (node, "nick", b->nick);
 			if (*b->notes)
-				xmlSetProp(node, "notes", b->notes);
+				xmlSetRawPropLocale (node, "notes", b->notes);
 			if (b->create_context_menu)
 				xmlSetProp (node, "create_context_menu", "TRUE");
 			if (b->pixmap_file && *b->pixmap_file)
-				xmlSetProp(node, "pixmap", b->pixmap_file);
+				xmlSetProp (node, "pixmap", b->pixmap_file);
 			if (b->time_added) {
 				g_string_sprintf (s, "%d", b->time_added);
 				xmlSetProp (node, "time_added", s->str);
@@ -545,15 +713,21 @@ bookmarks_save_recursively (xmlDocPtr doc, xmlNodePtr root, BookmarkItem *b)
 		case BM_AUTOBOOKMARKS:
 			node = xmlNewDocNode(doc, NULL, "autobookmarks", NULL);
 			xmlSetProp (node, "id", idstr);
-			xmlSetPropISOLatin1 (node, "name", b->name);
+			xmlSetRawPropLocale (node, "name", b->name);
 			if (b->create_toolbar)
 				xmlSetProp (node, "create_toolbar", "TRUE");
+			if (b->toolbar_style != TOOLBAR_STYLE_HORIZONTAL)
+			{
+				tmp = g_strdup_printf("%d", b->toolbar_style);
+				xmlSetProp (node, "toolbar_style", tmp);
+				g_free (tmp);
+			}
 			if (b->create_context_menu)
 				xmlSetProp (node, "create_context_menu", "TRUE");
 			if (b->expanded)
 				xmlSetProp (node, "expanded", "TRUE");
 			if (*b->notes)
-				xmlSetProp(node, "notes", b->notes);
+				xmlSetRawPropLocale (node, "notes", b->notes);
 			if (b->pixmap_file && *b->pixmap_file)
 				xmlSetProp(node, "pixmap", b->pixmap_file);
 			xmlAddChild(root, node);
@@ -572,35 +746,41 @@ void
 bookmarks_remove_recursively (BookmarkItem *b) {
 	GList *position, *li;
 	
+	if (b == NULL) return;
+	
 	if (b->parent) {
 		position = g_list_find (b->parent->list, b);
 		b->parent->list = 
 			g_list_remove_link (b->parent->list, position);
 		bookmarks_update_alias (b->parent);
 	}
-
+	
 	if (!b->alias_of) 
 		/* When removing a bookmark, we remove all its alias
-		   We should check before removing if it has alias and warn the user */
+		   Instead, we should check before removing if it has alias 
+		   and warn the user */
 		bookmarks_remove_all_alias (b);
 	else 
-		/* we are removing an alias, connect the aliased bookmark with it's
-		   next, if any */
+		/* we are removing an alias, connect the aliased bookmark with 
+		   it's next, if any */
 		b->alias_of->alias = b->alias;
 	
-	if ((b->type == BM_CATEGORY || b->type == BM_AUTOBOOKMARKS) && !b->alias_of)
+	if ((b->type == BM_FOLDER || b->type == BM_AUTOBOOKMARKS) && 
+	    ! b->alias_of)
 		for (li = b->list; li != NULL; li = li->next) 
 			bookmarks_remove_recursively (li->data);
 	
-	if (b == default_bookmarks_root) default_bookmarks_root = bookmarks_root;
-
+	if (b == default_bookmarks_root) 
+		default_bookmarks_root = bookmarks_root;
+	bookmarks_editor_remove_tree_items (b);
 	bookmarks_free_bookmark (b);
 }
 
 /**
  * bookmarks_new_bookmark: Creates a new allocated BookmarkItem
  * @type: the type of the bookmark
- * @escape_name: TRUE if the name needs to have _'s escaped, FALSE if they already are
+ * @escape_name: TRUE if the name needs to have _'s escaped, 
+ *                       FALSE if they already are
  * @name: the name, if NULL the url is used
  * @url: the url (ignored if type != BM_SITE)
  * @nick: the nickname for the bookmark, if any
@@ -623,14 +803,14 @@ bookmarks_new_bookmark (BookmarkType type, gboolean escape_name,
 	BookmarkItem *bookmark;
 
 	g_return_val_if_fail (!((type == BM_SITE) && (url == NULL)), NULL);
-	g_return_val_if_fail (!((type == BM_CATEGORY) && (name == NULL)), NULL);
+	g_return_val_if_fail (!((type == BM_FOLDER) && (name == NULL)), NULL);
 
 	bookmark = g_new0 (BookmarkItem, 1);
 	bookmark->type = type;
 
 	switch (bookmark->type) {
 	case BM_AUTOBOOKMARKS:
-	case BM_CATEGORY:
+	case BM_FOLDER:
 		if (escape_name)
 			bookmark->name = escape_uline_accel (name);
 		else
@@ -657,6 +837,7 @@ bookmarks_new_bookmark (BookmarkType type, gboolean escape_name,
 	bookmark->tree_items = NULL;
 	bookmark->parent = NULL;
 	bookmark->create_toolbar = FALSE;
+	bookmark->toolbar_style = TOOLBAR_STYLE_HORIZONTAL;
 	bookmark->create_context_menu = FALSE;
 	if (nick) 
 		bookmark->nick = g_strdup (nick);
@@ -697,6 +878,7 @@ bookmarks_new_alias (BookmarkItem *b)
 	bookmark->tree_items = NULL;
 	bookmark->parent = NULL;
 	bookmark->create_toolbar = FALSE;
+	bookmark->toolbar_style = TOOLBAR_STYLE_HORIZONTAL;
 	bookmark->create_context_menu = FALSE;
 	if (b) bookmarks_new_alias_do (b, bookmark);
 	return bookmark;
@@ -760,6 +942,16 @@ bookmarks_find_real_bookmark (BookmarkItem *b)
 		return bookmarks_find_real_bookmark (b->alias_of);
 }
 
+
+/**
+ * Sets the real bookmark visited date and ensures it is saved.
+ */
+void bookmarks_set_visited (BookmarkItem *b)
+{
+	bookmarks_find_real_bookmark (b)->time_visited = time (NULL);
+	bookmarks_dirty = TRUE;
+}
+
 /**
  * bookmarks_copy_bookmark: copy a bookmark
  * @b: the bookmark
@@ -778,9 +970,11 @@ bookmarks_copy_bookmark (BookmarkItem *b)
 						    b->pixmap_file);
 	
 	new->create_toolbar = b->create_toolbar;
+	new->toolbar_style = b->toolbar_style;
 	new->create_context_menu = b->create_context_menu;
 	new->expanded = b->expanded;
-	/* should these times just copied updated here? should they be updated? */
+	/* should these times just copied here? 
+	   should they be updated? */
 	new->time_added = b->time_added;
 	new->time_modified = b->time_modified;
 	new->time_visited = b->time_visited;
@@ -788,7 +982,7 @@ bookmarks_copy_bookmark (BookmarkItem *b)
 	{
 		new->pixmap_data = b->pixmap_data;
 	}
-	if (b->type == BM_CATEGORY) {
+	if (b->type == BM_FOLDER) {
 		for (l = b->list; l != NULL; l = g_list_next (l)) {
 			BookmarkItem *child = l->data;
 			BookmarkItem *new_child = bookmarks_copy_bookmark (child);
@@ -804,7 +998,7 @@ bookmarks_copy_bookmark (BookmarkItem *b)
  * @b: the bookmark to be freed, if NULL thenb do nothing
  * 
  * Frees the bookmarks and all the strings referenced by it and the list of children 
- * if it's a BM_CATEGORY (but does not free the children). Use it instead of g_free. 
+ * if it's a BM_FOLDER (but does not free the children). Use it instead of g_free. 
  **/
 void
 bookmarks_free_bookmark (BookmarkItem *b)
@@ -817,7 +1011,17 @@ bookmarks_free_bookmark (BookmarkItem *b)
 		if (b->url) g_free (b->url);
 		if (b->nick) g_free (b->nick);
 		if (b->pixmap_file) g_free (b->pixmap_file);
-		if (b->pixmap_data) g_free (b->pixmap_data);
+		if (b->pixmap_data)
+		{
+			if (b->pixmap_data->pixmap)
+				gdk_pixmap_unref (b->pixmap_data->pixmap);
+			if (b->pixmap_data->mask)
+				gdk_bitmap_unref (b->pixmap_data->mask);
+
+			/* scrub and free pixmap */
+			memset (b->pixmap_data, 0, sizeof (PixmapData));
+			g_free (b->pixmap_data);
+		}
 		if (b->notes) g_free (b->notes);
 		if (b->list) g_list_free (b->list);
 	}
@@ -843,7 +1047,8 @@ bookmarks_insert_bookmark (BookmarkItem *b, BookmarkItem *near,
 	}
 	else {
 		if (insert_pos == GTK_CLIST_DRAG_INTO 
-		    && (near->type == BM_CATEGORY || near->type == BM_AUTOBOOKMARKS)
+		    && (near->type == BM_FOLDER 
+			|| near->type == BM_AUTOBOOKMARKS)
 		    && !near->alias_of)
 			parent = near;
 		else
@@ -912,12 +1117,13 @@ bookmarks_move_bookmark (BookmarksEditorControls *controls,
 	
 	if (other_pos) {
 		other = other_pos->data;
-		if (other->type == BM_CATEGORY) {
-			/* expand the category and move b into it */
+		if (other->type == BM_FOLDER) {
+			/* expand the folder and move b into it */
 			parent->list = g_list_remove_link (parent->list, pos);
 			bookmarks_update_alias (parent);
 			gtk_ctree_expand (GTK_CTREE (controls->ctree), 
-					bookmarks_get_tree_item (other, controls));
+					bookmarks_get_tree_item
+					  (other, controls));
 			
 			if (!where) { 
 				/* up */
@@ -929,9 +1135,12 @@ bookmarks_move_bookmark (BookmarksEditorControls *controls,
 				other->list = g_list_prepend (other->list, b);
 			}
 		} else {
-			gint newindex = g_list_position (parent->list, other_pos);
-			b->parent->list = g_list_remove_link (b->parent->list, pos);
-			b->parent->list = g_list_insert (parent->list, b, newindex);
+			gint newindex = g_list_position
+				(parent->list, other_pos);
+			b->parent->list = g_list_remove_link 
+				(b->parent->list, pos);
+			b->parent->list = g_list_insert 
+				(parent->list, b, newindex);
 		}
 	} else {
 		/* move b to its grandparent*/
@@ -950,16 +1159,19 @@ bookmarks_move_bookmark (BookmarksEditorControls *controls,
 			b->parent = grandparent;
 		} else {
 			/* down */
-			GList *next = g_list_find(grandparent->list, parent)->next;
-			grandparent->list = g_list_insert (grandparent->list, b, 
-							   g_list_position 
-							   (grandparent->list, next));
+			GList *next = g_list_find
+				(grandparent->list, parent)->next;
+			grandparent->list = g_list_insert 
+				(grandparent->list, b, 
+				 g_list_position 
+				 (grandparent->list, next));
 			b->parent = grandparent;
 		}
 	}
 	bookmarks_editor_place_tree_items (b);
 	bookmarks_update_alias (b->parent);
-	gtk_ctree_select ( GTK_CTREE (controls->ctree), bookmarks_get_tree_item (b, controls));
+	gtk_ctree_select ( GTK_CTREE (controls->ctree),
+			   bookmarks_get_tree_item (b, controls));
 
 	return 0;
 }
@@ -990,7 +1202,7 @@ bookmarks_string_request_callback (gchar *string, BookmarkItem *b)
 		if (b->url &&
 		    gnome_config_get_bool (CONF_GENERAL_FAVICONS_ENABLED))
 		{
-			get_siteicon (NULL, b->url);
+			favicon_get_siteicon (NULL, b->url);
 		}
 		
 		bookmarks_editor_place_tree_items (b);
@@ -1004,7 +1216,7 @@ bookmarks_string_request_callback (gchar *string, BookmarkItem *b)
  * @name: The name of the bookmark, if NULL or "" the url will be used instead.
  * @url: The url of the bookmark
  * @parent: The parent bookmark or NULL if the bookmark should be added to the 
- * main temp category
+ * main temp folder
  * Return value: The new bookmark
  **/
 BookmarkItem *
@@ -1032,7 +1244,7 @@ add_bookmark_default (BookmarkType type, const gchar *name, const gchar *url,
  * @name: The link text
  * @url: The url of the bookmark
  * @parent: The parent bookmark or NULL if the bookmark should be added to the 
- * main temp category
+ * main temp folder
  * Return value: The new bookmark
  **/
 BookmarkItem *
@@ -1040,7 +1252,8 @@ add_temp_link_bookmark (const gchar *name, const gchar *url, BookmarkItem *paren
 {
 	BookmarkItem *bookmark;
 	gchar *newname;
-	gint shorttext = gnome_config_get_int ("/galeon/TempBookmarks/shorttext=16");
+	gint shorttext = gnome_config_get_int 
+		("/galeon/TempBookmarks/shorttext=16");
 
 	/**
 	 * if shorttext is < 0, use old url only behavior,
@@ -1072,7 +1285,7 @@ bookmarks_list_all_urls (BookmarkItem *bookmark)
 	GSList * result;
 
 	g_return_val_if_fail(bookmark != NULL, NULL);
-	g_return_val_if_fail(bookmark->type == BM_CATEGORY, NULL);
+	g_return_val_if_fail(bookmark->type == BM_FOLDER, NULL);
 
 	result = g_slist_alloc();
 
@@ -1081,7 +1294,7 @@ bookmarks_list_all_urls (BookmarkItem *bookmark)
 		BookmarkItem *b = l->data;
 		/* skip alias */
 		if (b->alias_of) continue;
-		if (b->type == BM_CATEGORY) {
+		if (b->type == BM_FOLDER) {
 			g_slist_concat(result, bookmarks_list_all_urls(b));
 		} else
 			if (b->type == BM_SITE) {
@@ -1092,7 +1305,7 @@ bookmarks_list_all_urls (BookmarkItem *bookmark)
 }
 
 /**
- * get a singly linked list of all the category/folder bookmarks 
+ * get a singly linked list of all the folder/folder bookmarks 
  */
 GSList * 
 bookmarks_list_all_folders (BookmarkItem *bookmark)
@@ -1101,14 +1314,14 @@ bookmarks_list_all_folders (BookmarkItem *bookmark)
 	GSList * result;
 
 	g_return_val_if_fail(bookmark != NULL, NULL);
-	g_return_val_if_fail(bookmark->type == BM_CATEGORY, NULL);
+	g_return_val_if_fail(bookmark->type == BM_FOLDER, NULL);
 
 	result = g_slist_alloc();
 
 	for (l = bookmark->list; l != NULL; l = l->next)
 	{
 		BookmarkItem *b = l->data;
-		if (b->type == BM_CATEGORY) {
+		if (b->type == BM_FOLDER) {
 			g_slist_append(result, b);
 			if (!b->alias_of) 
 				g_slist_concat (result, 
@@ -1122,10 +1335,12 @@ bookmarks_list_all_folders (BookmarkItem *bookmark)
 /**
  * bookmarks_parse_nick: returns the url of a given nick with one arg
  * @text: the text to try to parse
- * @root: the root bookmark for start the search, if NULL the usual tree will be used
+ * @root: the root bookmark for start the search, if NULL the usual tree will
+ * be used
  * 
- * Tries to find a bookmarkitem wich has a nick like the first word of @text and if 
- * found return the URL result of substituting each %s in the BookmarkItem url by the 
+ * Tries to find a bookmarkitem wich has a nick like the first word of @text 
+ * and if found return the URL result of substituting each %s in the 
+ * BookmarkItem url by the 
  * text after the first " " in @text
  * 
  * Return value: the resulting URL or NULL if not matching nick found
@@ -1237,10 +1452,11 @@ bookmarks_substitute_argument (BookmarkItem *b, const gchar *arguments)
 /**
  * bookmarks_find_by_nick: search for a BookmarkItem with a nick
  * @nick: the nick to search for.
- * @r: the root bookmark for start the search, if NULL the usual tree will be used
+ * @r: the root bookmark for start the search, if NULL the usual tree will be
+ * used
  * @wantsmart: if we are trying to find a smart bookmark or not
  * 
- * Search a bookmarks category recursivley for a bookmark with the given nick
+ * Search a bookmarks folder recursivley for a bookmark with the given nick
  * 
  * Return value: the found BookmarkItem or NULL if nothing found
  **/
@@ -1257,7 +1473,7 @@ bookmarks_find_by_nick (const gchar *nick, BookmarkItem *r, gboolean wantsmart)
 		return r;
 	}
 
-	if ((r->type == BM_CATEGORY) && !r->alias_of)
+	if ((r->type == BM_FOLDER) && !r->alias_of)
 	{
 		for (l = r->list; l != NULL; l = g_list_next (l))
 		{
@@ -1270,7 +1486,7 @@ bookmarks_find_by_nick (const gchar *nick, BookmarkItem *r, gboolean wantsmart)
 
 /**
  * bookmarks_save_as: convert a bookmark to a string
- * @bookmark: the bookmark to be saved (can be a category)
+ * @bookmark: the bookmark to be saved (can be a folder)
  *
  * Return value: the bookmarks as a string.
  **/
@@ -1348,7 +1564,7 @@ bookmarks_translate_string (gchar *smart_bm_string)
 			*tstring_ptr++ = '+';
 		else
 		{
-			sprintf(key_code, "%%%X", curr_char);
+			sprintf(key_code, "%%%02X", curr_char);
 			strcat(translated_string, key_code);
 			tstring_ptr += 3;
 		}
@@ -1375,32 +1591,27 @@ bookmarks_folder_open_all_items (GaleonEmbed *embed, BookmarkItem *bi,
 		l = l->next;
 	}
 
-	if (l == NULL)
-		return;
-
 	/* reuse the first tab/window if required */
-	/* FIXME: reusing the first tab/window often causes a crash, for
-	   now a new window/tab is always created  --Josh */
-//	if (reuse)
-//		embed_load_url (embed, child->url);
-//	else
-//		embed = embed_create_from_url (embed, child->url, TRUE);
+	if (reuse && l && child->url)
+	{
+		embed_load_url (embed, child->url);
+		l = l->next;
+	}
 
-	embed = embed_create_from_url (embed, child->url, FALSE,
-				       (new_window || !reuse));
-
-	/* process the remaining bookmarks */
-	l = l->next;
+	/* open all the rest of the items */
 	while (l)
 	{
 		BookmarkItem *child = l->data;
 		if (child->url)
-			embed_create_from_url (embed, child->url, FALSE,
+		{
+			embed_create_from_url (embed, child->url, TRUE,
 					       new_window);
+		}
 		l = l->next;
 	}
 }
 
+
 /* FIXME: Unused? */
 /**
  * bookmarks_add_completion_recursively: recursively add bookmarks to the
@@ -1410,11 +1621,12 @@ static void bookmarks_add_completion_recursively (BookmarkItem *b)
 {
 	GList *item;
   
-	if (b->alias_of) return; /* the real one should have been added already */
+	if (b->alias_of)
+		return; /* the real one should have been added already */
 	
 	switch (b->type)
 	{
-	case BM_CATEGORY:
+	case BM_FOLDER:
 	case BM_AUTOBOOKMARKS:
 		for (item = b->list; item != NULL; item = item->next)
 		{
@@ -1456,7 +1668,7 @@ static BookmarkItem *bookmarks_find_by_id (long id, BookmarkItem *r)
 	
 	/* Don't return bookmarks that may be not yet created aliases */
 	if ((r->id == id) && r->name != NULL) return bookmarks_find_real_bookmark (r);
-	if ((r->type == BM_CATEGORY) && !r->alias_of)
+	if ((r->type == BM_FOLDER) && !r->alias_of)
 		for (l = r->list; l != NULL; l = g_list_next (l)) {
 			b = bookmarks_find_by_id (id, l->data);
 			if (b) return b;
@@ -1487,6 +1699,8 @@ bookmarks_get_siteicon (const gchar *url)
 	if (favicons_cache != NULL)
 	{
 		result = g_hash_table_lookup (favicons_cache, url);
+		/* perhaps we need to add a reference to the GdkPixmap
+		   and the GdkBitmap? */
 		if (result != NULL) return result;
 	}
 	
@@ -1562,7 +1776,8 @@ bookmarks_get_siteicon (const gchar *url)
 	return result;
 }
 
-GtkCTreeNode *bookmarks_get_tree_item (BookmarkItem *b, BookmarksEditorControls *c)
+GtkCTreeNode *bookmarks_get_tree_item (BookmarkItem *b,
+				       BookmarksEditorControls *c)
 {
 	GList *l;
 	for (l = b->tree_items; l; l = l->next) {
@@ -1572,7 +1787,8 @@ GtkCTreeNode *bookmarks_get_tree_item (BookmarkItem *b, BookmarksEditorControls
 	return NULL;
 }
 
-void bookmarks_set_tree_item (BookmarkItem *b, BookmarksEditorControls *c, GtkCTreeNode *n)
+void bookmarks_set_tree_item (BookmarkItem *b, BookmarksEditorControls *c,
+			      GtkCTreeNode *n)
 {
 	GList *l;
 	BookmarkTreeItem *ti;		
@@ -1582,13 +1798,14 @@ void bookmarks_set_tree_item (BookmarkItem *b, BookmarksEditorControls *c, GtkCT
 			if (n)
 				ti->tree_item = n;
 			else  {
-				b->tree_items = g_list_remove_link (b->tree_items, l);
+				b->tree_items = g_list_remove_link 
+					(b->tree_items, l);
 				g_free (ti);
 			}	
 			return;
 		}
 	}
-	/* not found, append */
+	/* not found, add */
 	ti = g_new0 (BookmarkTreeItem, 1);
 	ti->tree_item = n;
 	ti->controls = c;
@@ -1596,7 +1813,7 @@ void bookmarks_set_tree_item (BookmarkItem *b, BookmarksEditorControls *c, GtkCT
 }
 
 /**
- * Returns TRUE is "a" is an ancestor of "b"
+ * Returns TRUE if "a" is an ancestor of "b"
  */
 gboolean
 bookmarks_is_ancestor (BookmarkItem *a, BookmarkItem *b)
@@ -1606,3 +1823,112 @@ bookmarks_is_ancestor (BookmarkItem *a, BookmarkItem *b)
 	return FALSE;
 }
 
+
+/**
+ * Merges two trees of bookmarks, adding bookmarks in root2 to root1
+ * root1 is modified, and root2 is not.
+ * This function is intended to be used when importing bookmarks
+ * The policies are very simple, they are open to discussion but I wthink 
+ * they cover most cases and are asy to implement. They are:
+ *    - for categories:
+ *            - two categories with the same name at the same level are merged
+ *              recursively
+ *            - categories in root2 not present in root1 are copied to root1
+ *    - for items
+ *            - two items are considered equal if they have the same url
+ *            - items not present in root1 are copied
+ *    - for separators
+ *            - they are added only if the previous item was added too, unless
+ *              a whole folder is imported
+ *    - for aliases
+ *            - aliases are copied as if they were normal bookmarks 
+ *              (for simplicity)
+ */
+void
+bookmarks_merge_trees (BookmarkItem *root1, BookmarkItem *root2)
+{
+	GList *li;
+	gboolean last_one_added = FALSE;
+	for (li = root2->list; li != NULL; li = li->next) {
+		BookmarkItem *item = li->data;
+		BookmarkItem *item2 = bookmarks_merge_trees_look_for_equal
+			(root1, item);
+		if (item2 == NULL) {
+			if ((item->type != BM_SEPARATOR) || last_one_added) {
+				BookmarkItem *b = bookmarks_copy_bookmark
+					(item);
+				root1->list = g_list_append (root1->list, b);
+				b->parent = root1;
+				last_one_added = TRUE;
+			}
+		} else if ((item2->type == BM_FOLDER)
+			   || (item2->type == BM_AUTOBOOKMARKS)) {
+			bookmarks_merge_trees (item2, item);
+			last_one_added = FALSE;
+		} else {
+			last_one_added = FALSE;
+		}
+	}	
+}
+
+static BookmarkItem *
+bookmarks_merge_trees_look_for_equal (BookmarkItem *folder, 
+				      BookmarkItem *bookmark)
+{
+	GList *li;
+	if (bookmark->type == BM_SEPARATOR) return NULL;
+	for (li = folder->list; li != NULL; li = li->next) {
+		BookmarkItem *item = li->data;
+		if ((item->type == BM_SITE) 
+		    && (bookmark->type == BM_SITE)) {
+			if (!strcmp (item->url, bookmark->url)) 
+				return item;
+		} else if (((bookmark->type == BM_FOLDER)
+			    || (bookmark->type == BM_AUTOBOOKMARKS)) 
+			   && ((item->type == BM_FOLDER)
+			       || (item->type == BM_AUTOBOOKMARKS))) {
+			if (!strcmp (item->name, bookmark->name)) 
+				return item;
+		}
+	}
+	return NULL;
+}
+
+
+void bookmarks_sort_norec (BookmarkItem *b, gboolean update_edit)
+{
+	if ((b->type == BM_FOLDER) || (b->type == BM_AUTOBOOKMARKS)) {
+		if (!b->alias_of) {
+			b->list = g_list_sort 
+				(b->list, bookmarks_compare_name);
+			bookmarks_update_alias (b);
+		}
+		if (update_edit)
+			bookmarks_editor_place_tree_items (b);
+	}
+}
+
+void bookmarks_sort_rec (BookmarkItem *b, gboolean update_edit)
+{
+	GList *l;
+	bookmarks_sort_norec (b, FALSE);
+	if ((b->type == BM_FOLDER) || (b->type == BM_AUTOBOOKMARKS)) 
+		for (l = b->list; l != NULL; l = l->next) {
+			BookmarkItem *b2 = l->data;
+			if (!b2->alias_of) {
+				bookmarks_sort_rec (b2, FALSE);
+			}
+		}
+	if (update_edit)
+		bookmarks_editor_place_tree_items (b);
+
+}
+
+static gint 
+bookmarks_compare_name (gconstpointer a, gconstpointer b)
+{
+	BookmarkItem *b1 = (BookmarkItem *) a;
+	BookmarkItem *b2 = (BookmarkItem *) b;
+
+	return g_strcasecmp (b1->name, b2->name);
+}
