@@ -20,10 +20,15 @@
 #include "bookmarks.h"
 #include "bookmarks_editor_callbacks.h"
 #include "misc.h"
+#include "glade.h"
 #include "window.h"
 #include "state.h"
+#include "context.h"
+#include "dialog.h"
 
+#include <string.h>
 #include <gtk/gtktoolbar.h>
+#include <gtk/gtkmenuitem.h>
 #include <gtk/gtkdnd.h>
 #include <libgnomeui/gnome-preferences.h>
 #include <libgnome/gnome-config.h>
@@ -31,6 +36,7 @@
 #include <libgnomeui/gnome-pixmap.h>
 #include <libgnomeui/gnome-stock.h>
 #include <libgnomeui/gnome-pixmap-entry.h>
+#include <libgnomeui/gnome-popup-menu.h>
 #include <gdk/gdkkeysyms.h>
 
 extern GtkTargetEntry bookmarks_dnd_targets [];
@@ -39,29 +45,39 @@ extern GtkTargetEntry bookmarks_dnd_targets [];
 GList *bookmarks_editors = NULL;
 
 /* Function prototypes */
-BookmarksEditorControls *
-bookmarks_editor_controls_init (BookmarkItem *root,
-				BookmarksEditorControls_type type);
-void bookmarks_editor_remove_tree_item_null (BookmarkItem *b,
-					  BookmarksEditorControls *controls);
-void bookmarks_editor_add_menubar (BookmarksEditorControls *controls);
+static BookmarksEditorControls *bookmarks_editor_controls_init
+                               (BookmarkItem *root,
+				BookmarksEditorControlsType type);
+static void bookmarks_editor_remove_tree_item_null
+                               (BookmarkItem *b,
+				BookmarksEditorControls *controls);
+static void bookmarks_editor_add_menubar (BookmarksEditorControls *controls);
+static void
+bookmarks_editor_fill_menu (GtkMenuShell *menushell, 
+			    GtkAccelGroup *accel, 
+			    BookmarksEditorControls *controls,
+			    GtkWidget **edit_menuitem,
+			    GtkWidget **set_default_folder_menuitem);
 
-BookmarksEditorControls *
-bookmarks_editor_controls_init (BookmarkItem *root,
-				BookmarksEditorControls_type type)
+
+static BookmarksEditorControls *
+bookmarks_editor_controls_init (BookmarkItem *root, 
+				BookmarksEditorControlsType type)
 {
 	BookmarksEditorControls *editor;
-	GtkWidget *close, *detach;
+	GtkWidget *close, *detach, *show_menu;
 	GladeXML *gxml = NULL;
+	GList *toolbar_items, *tlist;
 		
-	editor = g_new0 (BookmarksEditorControls, 1);	
+	editor = g_new0 (BookmarksEditorControls, 1);
 
+	editor->type = type;
 	editor->root_bookmark = root;
 	editor->clipboard = NULL;
 
-	switch (type) {
+	switch (type)
+	{
 	case STANDARD_BOOKMARKS_EDITOR:
-	case COMPACT_BOOKMARKS_EDITOR:
 		gxml = glade_widget_new ("bookmarks.glade", "bookmarks_editor", 
 					 &(editor->dialog), editor);
 		editor->menubar = glade_xml_get_widget 
@@ -113,6 +129,7 @@ bookmarks_editor_controls_init (BookmarkItem *root,
 		g_assert (editor->time_added_entry != NULL);
 		g_assert (editor->time_modified_entry != NULL);
 		g_assert (editor->time_visited_entry != NULL);
+
 		break;
 	case DOCKED_BOOKMARKS_EDITOR:
 		gxml = glade_widget_new ("bookmarks.glade", "bookmarks_dock", 
@@ -154,10 +171,12 @@ bookmarks_editor_controls_init (BookmarkItem *root,
 			(gxml, "bookmarks_dock_close_button");
 		detach = glade_xml_get_widget 
 			(gxml, "bookmarks_dock_detach_button");
+		show_menu = glade_xml_get_widget 
+			(gxml, "bookmarks_dock_menu_button");
 		gtk_button_set_relief (GTK_BUTTON (close),  GTK_RELIEF_NONE);
 		gtk_button_set_relief (GTK_BUTTON (detach), GTK_RELIEF_NONE);
-		gtk_toolbar_set_style (GTK_TOOLBAR (editor->toolbar),
-				       GTK_TOOLBAR_ICONS);
+		gtk_button_set_relief (GTK_BUTTON (show_menu),
+				       GTK_RELIEF_NONE);
 		break;
 	default:
 		g_error ("Incorrect bookmarks editor dialog type");
@@ -169,6 +188,10 @@ bookmarks_editor_controls_init (BookmarkItem *root,
 	editor->lock_edit_controls = FALSE;
 	editor->window = NULL;
 
+	/* get the widgets we need to work with later*/
+	editor->edit_button = glade_xml_get_widget
+			      (gxml, "bookmarks_editor_edit_button");
+
 	/* unref the GladeXML object, we don't use it anymore */
 	gtk_object_unref (GTK_OBJECT (gxml));
 
@@ -205,15 +228,33 @@ bookmarks_editor_controls_init (BookmarkItem *root,
 			(GNOME_PIXMAP_ENTRY (editor->pixmap_entry), SHARE_DIR);
 	}
 
+	/* setup the context menu callback for the toolbar */
+	toolbar_items = gtk_container_children 
+		(GTK_CONTAINER (editor->toolbar));
+	tlist = toolbar_items;
+	while (tlist != NULL)
+	{
+		gtk_signal_connect 
+			(GTK_OBJECT (tlist->data), "button-press-event",
+			GTK_SIGNAL_FUNC 
+			 (bookmarks_editor_toolbar_button_press_event_cb),
+			editor);
+		tlist = tlist->next;
+	}
+	g_list_free (toolbar_items);
+
+	/* callback to make sure that everything is cleaned before destroying 
+	   the widgets */
+	gtk_signal_connect (GTK_OBJECT (editor->ctree), "destroy",
+			    bookmarks_editor_destroy_cb, editor);
+
 	bookmarks_editor_place_tree_item (editor, root);
 	return editor;
 }
 
 BookmarksEditorControls *
-bookmarks_editor_show_dialog (GaleonWindow *window,
-			      BookmarksEditorControls_type type)
+bookmarks_editor_show_dialog (GaleonWindow *window)
 {
-	GtkToolbarStyle style;
 	gboolean toolbar_relief;
 	gboolean menubar_relief;
 	gboolean toolbar_detachable;
@@ -225,40 +266,26 @@ bookmarks_editor_show_dialog (GaleonWindow *window,
 	menubar_detachable = gnome_preferences_get_menubar_detachable ();
 	toolbar_relief = gnome_preferences_get_toolbar_relief ();
 	menubar_relief = gnome_preferences_get_menubar_relief ();
-	style = gnome_config_get_int ("/galeon/Appearance/toolbar_type=0");
+
+	controls = bookmarks_editor_controls_init
+		(bookmarks_root, STANDARD_BOOKMARKS_EDITOR);
+	bookmarks_editors = g_list_prepend (bookmarks_editors, controls);
 	
-	if (type == STANDARD_BOOKMARKS_EDITOR) 
-	{
-		controls = bookmarks_editor_controls_init
-			(bookmarks_root, STANDARD_BOOKMARKS_EDITOR);
-		bookmarks_editors = g_list_prepend (bookmarks_editors, controls);
-
-		state_load_window_state (controls->dialog, "bookmarks_editor",
-					750, 550);
-		state_load_column_widths (controls->ctree, "bookmarks_editor");
-		state_load_pane_pos (controls->vpane,
-				    "bookmarks_editor_vpane=-1");
-	}
-	
-	if (type == COMPACT_BOOKMARKS_EDITOR)
-	{
-		controls = bookmarks_editor_controls_init 
-			(default_bookmarks_root, COMPACT_BOOKMARKS_EDITOR);
-		bookmarks_editors = g_list_prepend (bookmarks_editors, 
-						    controls);
-
-		gtk_widget_hide (controls->menubar);
-		gtk_widget_hide (controls->edit_frame);
-
-		state_load_window_state (controls->dialog,
-					"compact_bookmarks_editor", -1, -1);
-	}
+	/* 630 is chosen since thats wide enough for the toolbar in
+	 * with all the standard GNOME defaults */
+	state_load_window_state (controls->dialog, "bookmarks_editor",
+				 630, 450);
+	state_load_column_widths (controls->ctree, "bookmarks_editor");
+	state_load_pane_pos (controls->vpane,
+			     "bookmarks_editor_vpane=-1");
 	
 	g_return_val_if_fail (controls != NULL, NULL);
 	
-	gtk_toolbar_set_style (GTK_TOOLBAR (controls->toolbar), style);
-
-	/* setup toolbar */
+	/* setup toolbar*/
+	gtk_toolbar_set_style 
+		(GTK_TOOLBAR (controls->toolbar),
+		gnome_config_get_int 
+			(CONF_APPEARANCE_BOOKMARKS_EDITOR_TOOLBAR_STYLE));
 	gnome_dock_item_set_shadow_type
 		(GNOME_DOCK_ITEM (controls->toolbar_dock), 
 		 toolbar_relief ? GTK_SHADOW_OUT : GTK_SHADOW_NONE);
@@ -278,10 +305,10 @@ bookmarks_editor_show_dialog (GaleonWindow *window,
 		 
 	gtk_widget_show (controls->dialog);
 
-	if (window)
+	if (window != NULL)
 	{
 		controls->window = window;
-		window_set_layer (controls->dialog);
+		dialog_set_parent (controls->dialog, window->WMain);
 	}
 	return controls;
 }
@@ -291,32 +318,60 @@ bookmarks_editor_hide_dialog (BookmarksEditorControls *controls)
 {
 	g_return_if_fail (controls != NULL);
 
+	/* the destroy callback of the ctree calls this function.
+	   Disconnect that to acoid recursion */
+	gtk_signal_disconnect_by_func (GTK_OBJECT (controls->ctree),
+				       bookmarks_editor_destroy_cb,
+				       controls);
 
-	// FIXME
-	if (controls->root_bookmark == bookmarks_root)
-	{
-		state_save_window_state (controls->dialog->window,
-					"bookmarks_editor");
-		state_save_column_widths (controls->ctree, "bookmarks_editor");
-		state_save_pane_pos (controls->vpane,
-				    "bookmarks_editor_vpane");
-	}
-	else if (controls->root_bookmark == default_bookmarks_root)
-	{
-		state_save_window_state (controls->dialog->window,
-					"compact_bookmarks_editor");
-	}
+	/* must sync because the editor really does change bookmarks
+	 * in memory, only way out is to use revert */
+	bookmarks_updated ();
 
-	gtk_widget_hide (controls->dialog);
+	if (controls->type != DOCKED_BOOKMARKS_EDITOR) 
+	{
+		// FIXME
+		// I think that we should not save window state 
+		// at all... - ricardo
+		if (controls->root_bookmark == bookmarks_root)
+		{
+			state_save_window_state (controls->dialog->window,
+						 "bookmarks_editor");
+			state_save_column_widths (controls->ctree, 
+						  "bookmarks_editor");
+			state_save_pane_pos (controls->vpane,
+					     "bookmarks_editor_vpane");
+		}
+		else if (controls->root_bookmark == default_bookmarks_root)
+		{
+			state_save_window_state (controls->dialog->window,
+						 "compact_bookmarks_editor");
+		}
+	}
+	
+	gtk_clist_freeze (GTK_CLIST(controls->ctree));
 	bookmarks_editor_remove_tree_item (controls, 
-			                   controls->root_bookmark);
+					   controls->root_bookmark);
+	gtk_clist_thaw (GTK_CLIST(controls->ctree));
 	bookmarks_editors = g_list_remove (bookmarks_editors, controls);
-	gtk_widget_destroy (controls->dialog);
+
 	if (controls->clipboard) g_free (controls->clipboard);
+
+	if (controls->type == DOCKED_BOOKMARKS_EDITOR) 
+	{
+		window_undock (controls->window);
+	} 
+	else 
+	{
+		gtk_widget_destroy (controls->dialog);
+	}
+
+	/* scrub control structure and free */
+	memset (controls, 0, sizeof (BookmarksEditorControls));
 	g_free (controls);
 }
 
-void
+BookmarksEditorControls *
 bookmarks_editor_show_dock (GaleonWindow *window)
 {
 	BookmarksEditorControls *controls;
@@ -328,9 +383,22 @@ bookmarks_editor_show_dock (GaleonWindow *window)
 					 FALSE);
 	bookmarks_editors = g_list_append (bookmarks_editors, controls);
 	controls->window = window;
+	
+	gtk_toolbar_set_style 
+		(GTK_TOOLBAR (controls->toolbar), 
+		gnome_config_get_int 
+		       (CONF_APPEARANCE_BOOKMARKS_EDITOR_DOCKED_TOOLBAR_STYLE));
+
 	window_dock (window, controls->dialog, 
 		     CONF_STATE_BOOKMARKS_DOCK_WIDTH);
-	(GTK_CHECK_MENU_ITEM (window->view_bookmarks_dock))->active = TRUE;
+
+	/* set dock type */
+	window->dock_type = DOCK_BOOKMARKS;
+	
+	/* update toolbar buttons */
+	window_update_bm_and_hist_buttons (window, TRUE, FALSE);
+
+	return controls;
 }
 				
 /**
@@ -343,7 +411,8 @@ bookmarks_editor_show_dock (GaleonWindow *window)
  * 
  **/
 void
-bookmarks_editor_place_tree_item (BookmarksEditorControls *controls, BookmarkItem *b)
+bookmarks_editor_place_tree_item (BookmarksEditorControls *controls,
+				  BookmarkItem *b)
 {
 	gchar *text[3];
 	gboolean isleaf;
@@ -374,15 +443,23 @@ bookmarks_editor_place_tree_item (BookmarksEditorControls *controls, BookmarkIte
 		pixmap_open = pixmap;
 		mask_open = mask;
 		break;
-	case BM_CATEGORY:
+	case BM_FOLDER:
 	case BM_AUTOBOOKMARKS:
 		text[0] = b->name;
 		text[1] = "";
 		isleaf = FALSE;
-		pixmap = folder_pixmap_data->pixmap; 
-		mask = folder_pixmap_data->mask;
-		pixmap_open = folder_open_pixmap_data->pixmap; 
-		mask_open = folder_open_pixmap_data->mask;
+		if (b == default_bookmarks_root) 
+		{
+			pixmap = default_folder_pixmap_data->pixmap; 
+			mask = default_folder_pixmap_data->mask;
+			pixmap_open = default_folder_open_pixmap_data->pixmap;
+			mask_open = default_folder_open_pixmap_data->mask;
+		} else {
+			pixmap = folder_pixmap_data->pixmap; 
+			mask = folder_pixmap_data->mask;
+			pixmap_open = folder_open_pixmap_data->pixmap;
+			mask_open = folder_open_pixmap_data->mask;
+		}
 		break;
 	default:
 		text[0] = text[1] = "";
@@ -402,8 +479,10 @@ bookmarks_editor_place_tree_item (BookmarksEditorControls *controls, BookmarkIte
 		if (list_node) {
 			GList *sibling_list_node = g_list_next (list_node);
 			if (sibling_list_node) {
-				sibling_node = bookmarks_get_tree_item (((BookmarkItem *) 
-					(sibling_list_node->data)), controls);
+				sibling_node = bookmarks_get_tree_item 
+					(((BookmarkItem *) 
+					  (sibling_list_node->data)),
+					 controls);
 			}
 		}
 	}
@@ -415,11 +494,15 @@ bookmarks_editor_place_tree_item (BookmarksEditorControls *controls, BookmarkIte
 			   for now, just leave it w/o pixmap */
 			pixmap = mask = pixmap_open = mask_open = NULL;
 		}
-		bookmarks_set_tree_item (b, controls, gtk_ctree_insert_node
-			(GTK_CTREE (controls->ctree), parent_node, sibling_node, text, 
-			 2, pixmap, mask, pixmap_open, mask_open, isleaf, b->expanded));
+		bookmarks_set_tree_item 
+			(b, controls, gtk_ctree_insert_node
+			 (GTK_CTREE (controls->ctree), parent_node,
+			  sibling_node, text, 2, pixmap, mask, pixmap_open,
+			  mask_open, isleaf, b->expanded));
 		gtk_ctree_node_set_row_data (GTK_CTREE (controls->ctree), 
-					     bookmarks_get_tree_item (b, controls), b);
+					     bookmarks_get_tree_item 
+					     (b,
+					      controls), b);
 	}
 
 	if (!isleaf && !b->alias_of) /* don't display alias' children */ {
@@ -445,13 +528,24 @@ void
 bookmarks_editor_remove_tree_items (BookmarkItem *b)
 {
 	GList *li;
-	for (li = bookmarks_editors; li; li = li->next) {
+
+	for (li = bookmarks_editors; li; li = li->next) 
+	{
 		BookmarksEditorControls *controls = li->data;
-		if (controls->root_bookmark == b)
-			bookmarks_editor_set_root (controls, bookmarks_root);
+		if (controls->root_bookmark == b) {
+			if (b != bookmarks_root) {
+				bookmarks_editor_set_root 
+					(controls, bookmarks_root);
+			} else {
+				controls->root_bookmark = NULL;
+			}
+		}
 	}
+
 	while (b->tree_items) 
-		bookmarks_editor_remove_tree_item (((BookmarkTreeItem *) (b->tree_items->data))->controls, b);
+		bookmarks_editor_remove_tree_item 
+			(((BookmarkTreeItem *) 
+			  (b->tree_items->data))->controls, b);
 }
 
 void
@@ -459,7 +553,8 @@ bookmarks_editor_update_tree_items (BookmarkItem *b)
 {
 	GList *l;
 	for (l = b->tree_items; l; l = l->next) 
-		bookmarks_editor_update_tree_item (((BookmarkTreeItem *) (l->data))->controls, b);
+		bookmarks_editor_update_tree_item (((BookmarkTreeItem *) 
+						    (l->data))->controls, b);
 }
 
 
@@ -468,8 +563,8 @@ bookmarks_editor_update_tree_items (BookmarkItem *b)
  * @controls: the set of controls
  * @b: the bookmark
  * 
- * Removes bookmarks and it's childs from the bookmark editor ctree and sets the 
- * tree_item fields to NULL. This does not delete the bookmarks
+ * Removes bookmarks and it's childs from the bookmark editor ctree and sets
+ * the tree_item fields to NULL. This does not delete the bookmarks
  * 
  **/
 void
@@ -498,16 +593,17 @@ bookmarks_editor_remove_tree_item (BookmarksEditorControls *controls, BookmarkIt
  * 
  * This is an auxiliar function for bookmarks_editor_remove_tree_item.
  **/
-void
+static void
 bookmarks_editor_remove_tree_item_null (BookmarkItem *b, BookmarksEditorControls *controls)
 {
 	GList *l;
 	bookmarks_set_tree_item (b, controls, NULL);
-	if ((b->type == BM_CATEGORY || b->type == BM_AUTOBOOKMARKS) && 
-	    !b->alias_of) /* alias don't have children */
+	if ((b->type == BM_FOLDER || b->type == BM_AUTOBOOKMARKS) 
+	    && ! b->alias_of) /* alias don't have children */
 	{
 		for (l = b->list; l != NULL; l = g_list_next (l)) 
-			bookmarks_editor_remove_tree_item_null (l->data, controls);
+			bookmarks_editor_remove_tree_item_null
+				(l->data, controls);
 	}
 }
 
@@ -549,13 +645,20 @@ bookmarks_editor_update_tree_item (BookmarksEditorControls *controls, BookmarkIt
 		pixmap = bm_icon->pixmap;
 		mask = bm_icon->mask;;
 		break;
-	case BM_CATEGORY:
+	case BM_FOLDER:
 	case BM_AUTOBOOKMARKS:
 		text[0] = b->name;
 		text[1] = "";
 		isleaf = FALSE;
-		pixmap = folder_pixmap_data->pixmap; 
-		mask = folder_pixmap_data->mask;
+		if (b == default_bookmarks_root)
+		{
+			pixmap = default_folder_pixmap_data->pixmap; 
+			mask = default_folder_pixmap_data->mask;
+		} else 
+		{
+			pixmap = folder_pixmap_data->pixmap; 
+			mask = folder_pixmap_data->mask;
+		}
 		break;
 	default:
 		text[0] = text[1] = "";
@@ -600,7 +703,12 @@ void bookmarks_editor_select_bookmark (BookmarksEditorControls *controls,
 	/* make sure this node is visible in the bookmarks editor */
 	if (bi->parent == NULL)
 	{
-		bookmarks_editor_set_root (controls, bi->parent);
+		bookmarks_editor_set_root (controls, bi);
+		/* thaw the view */
+		gtk_clist_thaw (GTK_CLIST (controls->ctree));
+		/* return now, we can't actually focus the item since it
+		 * is the root, and the root isn't shown */
+		return;
 	}
 
 	/* make sure the selection is shown */
@@ -632,8 +740,68 @@ void bookmarks_editor_select_bookmark (BookmarksEditorControls *controls,
 	gtk_clist_thaw (GTK_CLIST (controls->ctree));
 }
 
-void 
+static void 
 bookmarks_editor_add_menubar (BookmarksEditorControls *controls)
+{
+	GtkAccelGroup *accel = GNOME_APP (controls->dialog)->accel_group;
+
+	bookmarks_editor_fill_menu (GTK_MENU_SHELL (controls->menubar),
+				    accel, controls,
+				    &(controls->edit_menuitem),
+				    &(controls->set_default_folder_menuitem));
+
+	gtk_check_menu_item_set_active 
+				(GTK_CHECK_MENU_ITEM (controls->edit_menuitem),
+				 TRUE);
+	gtk_widget_set_sensitive 
+				(controls->set_default_folder_menuitem, FALSE);
+}
+
+void 
+bookmarks_editor_popup_menu (BookmarksEditorControls *controls)
+{
+	gint ret;
+	GtkWidget *popup = gtk_menu_new ();
+
+	bookmarks_editor_fill_menu (GTK_MENU_SHELL (popup),
+				    NULL, controls,
+				    NULL, NULL);
+
+	ret = gnome_popup_menu_do_popup_modal (popup, NULL, NULL,
+					       NULL, NULL);
+
+	/* Wait for the menu to be destroyed if a menu item is selected */
+	if (ret != -1) 
+		while (gtk_events_pending()) 
+			gtk_main_iteration();
+
+	gtk_widget_unref (popup);
+}
+
+/* 
+ * a nonsense needed to get some of the GNOMEUINFO defined strings translated 
+ * blame xgettext 
+ */
+#if 0
+static char *ugh[] = {
+	/* File */
+		N_("_Open..."),
+		N_("_Revert"),
+		N_("_Save"),
+		N_("_Close"),
+	/* Edit */
+		N_("C_ut"),
+		N_("_Copy"),
+		N_("_Paste"),
+}
+#endif
+
+static void 
+bookmarks_editor_fill_menu (GtkMenuShell *menushell, 
+			    GtkAccelGroup *accel, 
+			    BookmarksEditorControls *controls,
+			    GtkWidget **edit_menuitem,
+			    GtkWidget **set_default_folder_menuitem)
 {
 	GnomeUIInfo file_menu_uiinfo[] =
 	{
@@ -646,8 +814,8 @@ bookmarks_editor_add_menubar (BookmarksEditorControls *controls)
 		  GNOME_APP_PIXMAP_FILENAME,
 		  "../galeon/new_separator_small.xpm",
 		  GDK_e, GDK_CONTROL_MASK, NULL },
-		{ GNOME_APP_UI_ITEM, N_("New _category"), NULL,
-		  bookmarks_editor_new_category_clicked_cb, controls, NULL,
+		{ GNOME_APP_UI_ITEM, N_("New _folder"), NULL,
+		  bookmarks_editor_new_folder_clicked_cb, controls, NULL,
 		  GNOME_APP_PIXMAP_FILENAME, "../galeon/new_folder_small.xpm",
 		  GDK_F, GDK_CONTROL_MASK, NULL },
 		{ GNOME_APP_UI_ITEM, N_("New _alias"), NULL,
@@ -655,15 +823,31 @@ bookmarks_editor_add_menubar (BookmarksEditorControls *controls)
 		  GNOME_APP_PIXMAP_FILENAME, "../galeon/new_alias_small.xpm",
 		  GDK_a, GDK_CONTROL_MASK, NULL },
 		GNOMEUIINFO_SEPARATOR,
+		GNOMEUIINFO_MENU_OPEN_ITEM
+                        (bookmarks_editor_open_clicked_cb, controls),
 		GNOMEUIINFO_MENU_REVERT_ITEM 
 		        (bookmarks_editor_revert_clicked_cb, controls),
 		GNOMEUIINFO_MENU_SAVE_ITEM 
 		        (bookmarks_editor_apply_clicked_cb, controls),
+		GNOMEUIINFO_MENU_SAVE_AS_ITEM
+		        (bookmarks_editor_save_as_clicked_cb, controls),
 		GNOMEUIINFO_SEPARATOR,
 		GNOMEUIINFO_MENU_CLOSE_ITEM 
 		        (bookmarks_editor_accept_clicked_cb, controls),
 		GNOMEUIINFO_END
 	};
+	GnomeUIInfo sort_folder_menu_uiinfo[] = 
+	{
+		{ GNOME_APP_UI_ITEM, N_("_One Level"), NULL,
+		  bookmarks_editor_sort_norec_clicked_cb, controls, NULL,
+		  GNOME_APP_PIXMAP_NONE, NULL,
+		  0, 0, NULL },
+		{ GNOME_APP_UI_ITEM, N_("_Recursive"), NULL,
+		  bookmarks_editor_sort_rec_clicked_cb, controls, NULL,
+		  GNOME_APP_PIXMAP_NONE, NULL,
+		  0, 0, NULL },
+		GNOMEUIINFO_END
+	};
 	GnomeUIInfo edit_menu_uiinfo[] =
 	{
 		GNOMEUIINFO_MENU_CUT_ITEM (bookmarks_editor_cut_cb, controls),
@@ -680,6 +864,10 @@ bookmarks_editor_add_menubar (BookmarksEditorControls *controls)
 		  bookmarks_editor_go_clicked_cb, controls, NULL,
 		  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_JUMP_TO,
 		  GDK_g, GDK_CONTROL_MASK, NULL },
+		{ GNOME_APP_UI_ITEM, N_("_Set as default folder"), NULL,
+		  bookmarks_editor_set_default_folder_cb, controls, NULL,
+		  GNOME_APP_PIXMAP_NONE, NULL,
+		  0, 0, NULL },
 		GNOMEUIINFO_SEPARATOR,
 		{ GNOME_APP_UI_ITEM, N_("Move _up"), NULL,
 		  bookmarks_editor_move_up_clicked_cb, controls, NULL,
@@ -698,6 +886,9 @@ bookmarks_editor_add_menubar (BookmarksEditorControls *controls)
 		  bookmarks_editor_add_alias_root_cb, controls, NULL,
 		  GNOME_APP_PIXMAP_NONE, NULL,
 		  0, 0, NULL },
+		GNOMEUIINFO_SEPARATOR,
+		GNOMEUIINFO_SUBTREE (N_("Sort folder"), 
+				     sort_folder_menu_uiinfo),
 		GNOMEUIINFO_END
 	};
 	GnomeUIInfo view_menu_uiinfo[] =
@@ -724,8 +915,8 @@ bookmarks_editor_add_menubar (BookmarksEditorControls *controls)
 		   0, 0, NULL },
 		*/
 		GNOMEUIINFO_SEPARATOR,
-		{ GNOME_APP_UI_ITEM, N_("Show only _this category"), NULL,
-		  bookmarks_editor_show_only_this_category_cb, controls, NULL,
+		{ GNOME_APP_UI_ITEM, N_("Show only _this folder"), NULL,
+		  bookmarks_editor_show_only_this_folder_cb, controls, NULL,
 		  GNOME_APP_PIXMAP_NONE, NULL,
 		  0, 0, NULL },
 		{ GNOME_APP_UI_ITEM, N_("Show _all bookmarks"), NULL,
@@ -741,6 +932,10 @@ bookmarks_editor_add_menubar (BookmarksEditorControls *controls)
 	};
 	GnomeUIInfo import_menu_uiinfo[] =
 	{
+		{ GNOME_APP_UI_ITEM, N_("Import from _XBEL..."), NULL,
+		  bookmarks_import_xbel_activate_cb, controls, NULL,
+		  GNOME_APP_PIXMAP_NONE, NULL,
+		  0, 0, NULL },
 		{ GNOME_APP_UI_ITEM, N_("Import from _Netscape bookmarks"),
 		  NULL,
 		  bookmarks_import_netscape_activate_cb, NULL, NULL,
@@ -751,62 +946,96 @@ bookmarks_editor_add_menubar (BookmarksEditorControls *controls)
 		  bookmarks_import_mozilla_activate_cb, NULL, NULL,
 		  GNOME_APP_PIXMAP_NONE, NULL,
 		  0, 0, NULL },
-		{ GNOME_APP_UI_ITEM, N_("Import from _other location"), NULL,
-		  bookmarks_import_other_activate_cb, NULL, NULL,
+		{ GNOME_APP_UI_ITEM, N_("Import from _other location..."), 
+		  NULL,
+		  bookmarks_import_other_activate_cb, controls, NULL,
 		  GNOME_APP_PIXMAP_NONE, NULL,
 		  0, 0, NULL },
 		GNOMEUIINFO_SEPARATOR,
-		{ GNOME_APP_UI_ITEM, N_("Export bookmarks in _XBEL"), NULL,
-		  bookmarks_export_xbel_activate_cb, NULL, NULL,
+		{ GNOME_APP_UI_ITEM, N_("Export bookmarks in X_BEL..."), NULL,
+		  bookmarks_export_xbel_activate_cb, controls, NULL,
 		  GNOME_APP_PIXMAP_NONE, NULL,
 		  0, 0, NULL },
-		{ GNOME_APP_UI_ITEM, N_("Export to Net_scape bookmarks"), NULL,
-		  bookmarks_export_netscape_activate_cb, NULL, NULL,
+		{ GNOME_APP_UI_ITEM, N_("Export to Net_scape bookmarks..."), 
+		  NULL,
+		  bookmarks_export_netscape_activate_cb, controls->dialog, 
+		  NULL,
 		  GNOME_APP_PIXMAP_NONE, NULL,
 		  0, 0, NULL },
-		{ GNOME_APP_UI_ITEM, N_("Export to Mo_zilla bookmarks"), NULL,
-		  bookmarks_export_mozilla_activate_cb, NULL, NULL,
+		{ GNOME_APP_UI_ITEM, N_("Export to Mo_zilla bookmarks..."), 
+		  NULL,
+		  bookmarks_export_mozilla_activate_cb, controls->dialog, NULL,
 		  GNOME_APP_PIXMAP_NONE, NULL,
 		  0, 0, NULL },
-		{ GNOME_APP_UI_ITEM, N_("Export to o_ther location"), NULL,
-		  bookmarks_export_other_activate_cb, NULL, NULL,
+		{ GNOME_APP_UI_ITEM, N_("Export to o_ther location..."), NULL,
+		  bookmarks_export_other_activate_cb, controls, NULL,
 		  GNOME_APP_PIXMAP_NONE, NULL,
 		  0, 0, NULL },
 		GNOMEUIINFO_END
 	};
 
-/* TODO
-   GnomeUIInfo help_menu_uiinfo[] =
-   {
-   GNOMEUIINFO_END
-   };
-*/
+	/* TODO
+	   GnomeUIInfo help_menu_uiinfo[] =
+	   {
+	   GNOMEUIINFO_END
+	   };
+	*/
 	GnomeUIInfo menubar_uiinfo[] =
 	{
-		GNOMEUIINFO_MENU_FILE_TREE (file_menu_uiinfo),
-		GNOMEUIINFO_MENU_EDIT_TREE (edit_menu_uiinfo),
-		GNOMEUIINFO_MENU_VIEW_TREE (view_menu_uiinfo),
+		GNOMEUIINFO_SUBTREE (N_("_File"), file_menu_uiinfo),
+		GNOMEUIINFO_SUBTREE (N_("_Edit"), edit_menu_uiinfo),
+		GNOMEUIINFO_SUBTREE (N_("_View"), view_menu_uiinfo),
 		GNOMEUIINFO_SUBTREE (N_("I_mport / Export"), 
 				     import_menu_uiinfo),
-		/*TODO: GNOMEUIINFO_MENU_HELP_TREE (help_menu_uiinfo),*/
+		/*TODO: GNOMEUIINFO_SUBTREE (N_("_Help"), help_menu_uiinfo),*/
 		GNOMEUIINFO_END
 	};
-	GtkAccelGroup *accel;
-
-	accel = GNOME_APP (controls->dialog)->accel_group;
-	gnome_app_fill_menu (GTK_MENU_SHELL (controls->menubar),
+	gnome_app_fill_menu (menushell,
 			     menubar_uiinfo, accel, TRUE, 0);
-	gtk_check_menu_item_set_active 
-		(GTK_CHECK_MENU_ITEM (view_menu_uiinfo[0].widget), TRUE);
+	if (edit_menuitem)
+		*edit_menuitem = 
+			GTK_WIDGET (view_menu_uiinfo[0].widget);
+	if (set_default_folder_menuitem)
+		*set_default_folder_menuitem = 
+			GTK_WIDGET (edit_menu_uiinfo[6].widget);
+}
+
+void
+bookmarks_create_edit_items (GtkMenu *popup, BookmarksEditorControls *controls)
+{
+	context_menu_add_item (popup, _("C_ut"), bookmarks_editor_cut_cb,
+			controls, GNOME_STOCK_MENU_CUT, TRUE);
+	context_menu_add_item (popup, _("_Copy"), bookmarks_editor_copy_cb,
+			controls, GNOME_STOCK_MENU_COPY, TRUE);
+	context_menu_add_item (popup, _("_Paste"), bookmarks_editor_paste_cb,
+			controls, GNOME_STOCK_MENU_PASTE, TRUE);
+	context_menu_add_item (popup, _("_Remove bookmark"),
+			bookmarks_editor_remove_clicked_cb,
+			controls, GNOME_STOCK_MENU_TRASH, TRUE);
 }
 
 void 
 bookmarks_editor_set_root (BookmarksEditorControls *controls, BookmarkItem *b)
 {
+	BookmarkItem *oldroot = controls->root_bookmark;
 	gtk_clist_freeze (GTK_CLIST (controls->ctree));
-	bookmarks_editor_remove_tree_item (controls, controls->root_bookmark);
+	controls->root_bookmark = NULL;
+	if (oldroot != NULL)
+		bookmarks_editor_remove_tree_item_null 
+			(oldroot, controls);
+	gtk_clist_clear (GTK_CLIST (controls->ctree));
 	controls->root_bookmark = b;
 	bookmarks_editor_place_tree_item (controls, controls->root_bookmark);
 	gtk_clist_thaw (GTK_CLIST (controls->ctree));
 }
 
+void
+bookmarks_editors_refresh (void)
+{
+	GList *l;
+	for (l = bookmarks_editors; l; l = l->next) {
+		BookmarksEditorControls *controls = l->data;
+		/* this forces recreating all tree items */
+		bookmarks_editor_set_root (controls, controls->root_bookmark);
+	}
+}
