@@ -21,18 +21,24 @@
 #include "embed.h"
 #include "window.h"
 #include "bookmarks.h"
+#include "favicon.h"
 #include "bookmarks_editor_callbacks.h"
+#include "context.h"
+#include "dialog.h"
+#include "filepicker.h"
 
 #include <string.h>
 #include <time.h>
 #include <libgnome/gnome-config.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnomeui/gnome-popup-menu.h>
+#include <libgnomeui/gnome-preferences.h>
 #include <gtk/gtktogglebutton.h>
 #include <gtk/gtkselection.h>
 #include <gtk/gtkcheckmenuitem.h>
 #include <gtk/gtktext.h>
 #include <gdk/gdkkeysyms.h>
+#include <gtk/gtktoolbar.h>
 
 /* internal helper functions */
 static void fetch_icon_recursively (BookmarkItem *b,
@@ -62,12 +68,15 @@ static void bookmarks_editor_set_notes (BookmarksEditorControls *controls,
 static void bookmarks_editor_set_pixmap (BookmarksEditorControls *controls, 
 					 BookmarkItem *bookmark, 
 					 gboolean active);
-static void bookmarks_editor_set_times (BookmarksEditorControls *controls, 
-					 BookmarkItem *bookmark);
+static void bookmarks_editor_set_times (BookmarksEditorControls *controls,
+					BookmarkItem *bookmark,
+					BookmarkItem *bookmarkreal);
 static void bookmarks_selection_prepare (GList *l,
 					BookmarksEditorControls *controls);
-
-
+static gint bookmarks_row_compare (GList *a, GList *b);
+void
+bookmarks_editor_key_press_event_cb (GtkWidget *widget,GdkEventKey *event,
+				     BookmarksEditorControls *controls);
 void
 bookmarks_editor_ctree_tree_select_row_cb (GtkCTree *ctree, GList *node,
 					   gint column,
@@ -78,6 +87,14 @@ bookmarks_editor_ctree_tree_select_row_cb (GtkCTree *ctree, GList *node,
 	if (!b) return;
 	controls->lock_edit_controls = TRUE;
 
+	if (controls->set_default_folder_menuitem != NULL)
+	{
+		gtk_widget_set_sensitive (controls->set_default_folder_menuitem,
+					  (b != default_bookmarks_root) &&
+					   (b->type == BM_AUTOBOOKMARKS ||
+					    b->type == BM_FOLDER));
+	}
+
 	if (controls->notes_text)
 	{
 		gtk_editable_delete_text (GTK_EDITABLE (controls->notes_text),
@@ -96,7 +113,7 @@ bookmarks_editor_ctree_tree_select_row_cb (GtkCTree *ctree, GList *node,
 		bookmarks_editor_set_pixmap               (controls, b, TRUE);
 		break;
 
-	case BM_CATEGORY:
+	case BM_FOLDER:
 	case BM_AUTOBOOKMARKS:
 		bookmarks_editor_set_name                 (controls, b, TRUE);
 		bookmarks_editor_set_url                  (controls, b, FALSE);
@@ -117,7 +134,8 @@ bookmarks_editor_ctree_tree_select_row_cb (GtkCTree *ctree, GList *node,
 		bookmarks_editor_set_pixmap               (controls, b, FALSE);
 		break;
 	}
-	bookmarks_editor_set_times (controls, b);
+	bookmarks_editor_set_times (controls, b, 
+				    bookmarks_find_real_bookmark (b));
 	controls->lock_edit_controls = FALSE;
 }
 
@@ -136,7 +154,7 @@ bookmarks_editor_ctree_tree_unselect_row_cb (GtkCTree *ctree, GList *node,
 	bookmarks_editor_set_create_context_menu  (controls, NULL, FALSE);
 	bookmarks_editor_set_notes                (controls, NULL, FALSE);
 	bookmarks_editor_set_pixmap               (controls, NULL, FALSE);
-	bookmarks_editor_set_times                (controls, NULL);
+	bookmarks_editor_set_times                (controls, NULL, NULL);
 
 	controls->lock_edit_controls = FALSE;
 }
@@ -216,7 +234,7 @@ bookmarks_editor_edited_cb (GtkEditable *editable,
 				b_real->notes = notes;
 			}
 			break;
-		case BM_CATEGORY:
+		case BM_FOLDER:
 		case BM_AUTOBOOKMARKS:
 			if (controls->name_entry) {
 				if (b_real->name) g_free (b_real->name);
@@ -253,7 +271,7 @@ bookmarks_editor_edited_cb (GtkEditable *editable,
 		}
 
 		b_real->time_modified = b_alias->time_modified = time (NULL);
-		bookmarks_editor_set_times (controls, b_alias);
+		bookmarks_editor_set_times (controls, b_alias, b_real);
 		bookmarks_editor_update_tree_items (b_real);
 		bookmarks_update_alias (b_real);
 		bookmarks_dirty = TRUE;
@@ -273,22 +291,9 @@ void
 bookmarks_editor_revert_clicked_cb (GtkWidget *widget,
 				    BookmarksEditorControls *controls)
 {
-	/* delete old bookmarks view */
-	bookmarks_editor_remove_tree_item (controls, controls->root_bookmark);
-
-	/* delete the old bookmarks */
-	/* FIXME: is this enough to delete them all */
-	bookmarks_remove_recursively (controls->root_bookmark);
 
 	/* load bookmarks from disk */
 	bookmarks_load ();
-	
-	/* view new bookmarks */
-	/* FIXME: any implications for temp bookmarks editor? */
-	controls->root_bookmark = bookmarks_root;
-	controls->selection = NULL;
-	controls->last_pressed = NULL;
-	bookmarks_editor_place_tree_item (controls, controls->root_bookmark);
 
 	/* refresh everywhere else */
 	bookmarks_editor_apply (controls);
@@ -305,7 +310,6 @@ void
 bookmarks_editor_accept_clicked_cb (GtkButton *button,
 				    BookmarksEditorControls *controls)
 {
-	bookmarks_editor_apply (controls);
 	bookmarks_editor_hide_dialog (controls);
 }
 
@@ -347,18 +351,17 @@ bookmarks_editor_remove_clicked_cb (GtkButton *button,
 		if (b->type == BM_AUTOBOOKMARKS)
 			autobookmarks_deleted = TRUE;
 
-		/* TODO: when deleting a bookmark which has aliases, we should wanr the 
-		   user, or make one of the aliases the new real bookmark */
+		/* TODO: when deleting a bookmark which has aliases, we should
+		   warn the user, or make one of the aliases the new real
+		   bookmark */
 		if (b->alias && !b->alias_of) {
 			BookmarkItem *alias;
 			for (alias = b->alias; alias; alias = alias->alias) {
 				if (alias == next_to_select) 
 					next_to_select = NULL;
-				bookmarks_editor_remove_tree_items (alias);
 			}
 		}
-
-		bookmarks_editor_remove_tree_items (b);
+		
 		bookmarks_remove_recursively (b);
 		bookmarks_dirty = TRUE;
         }
@@ -371,7 +374,8 @@ bookmarks_editor_remove_clicked_cb (GtkButton *button,
 	}
 
 	if (next_to_select)
-		gtk_ctree_select (ctree, bookmarks_get_tree_item (next_to_select, controls));
+		gtk_ctree_select (ctree, bookmarks_get_tree_item 
+				  (next_to_select, controls));
 
 	g_list_free (selection_list);
 }
@@ -470,7 +474,7 @@ void fetch_icon_recursively (BookmarkItem *b,
 
 	switch (b->type)
 	{
-	case BM_CATEGORY:
+	case BM_FOLDER:
 	case BM_AUTOBOOKMARKS:
 		for (l = b->list; l != NULL; l = g_list_next (l))
 		{
@@ -480,7 +484,7 @@ void fetch_icon_recursively (BookmarkItem *b,
 		break;
 		
 	case BM_SITE:
-		get_siteicon (NULL, b->url);
+		favicon_get_siteicon (NULL, b->url);
 		bookmarks_dirty = TRUE;
 		break;
 
@@ -493,13 +497,30 @@ void
 bookmarks_editor_edit_toggled_cb (GtkWidget *toggle, 
 				  BookmarksEditorControls *controls) 
 {
-	if (controls->edit_frame) {
+	if (controls->edit_frame != NULL) {
 		gboolean visible = TRUE;
 		if (GTK_IS_TOGGLE_BUTTON (toggle))
-			visible = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON 
-								(toggle));
+		{
+			visible = gtk_toggle_button_get_active 
+					(GTK_TOGGLE_BUTTON (toggle));
+			if (controls->edit_menuitem != NULL)
+			{
+				gtk_check_menu_item_set_active 
+					(GTK_CHECK_MENU_ITEM (
+						controls->edit_menuitem),
+					 visible);
+			}
+		}
 		else if (GTK_IS_CHECK_MENU_ITEM (toggle))
+		{
 			visible = GTK_CHECK_MENU_ITEM (toggle)->active;
+			if (controls->edit_button != NULL)
+			{
+				gtk_toggle_button_set_active 
+				     (GTK_TOGGLE_BUTTON (controls->edit_button),
+				      visible);
+			}
+		}
 		if (visible)
 			gtk_widget_show (controls->edit_frame);
 		else
@@ -514,7 +535,7 @@ bookmarks_editor_add_alias_cb (GtkButton *button,
 	BookmarkItem *orig = controls->selection;
 	BookmarkItem *new = bookmarks_new_alias (orig);
 	BookmarkItem *near = controls->selection;
-	/* if the original is a category, add the new boookmark before to avoid loops */
+	/* if the original is a folder, add the new boookmark before to avoid loops */
 	GtkCListDragPos insert_pos = GTK_CLIST_DRAG_INTO;
 	if (new->type != BM_SITE)
 		insert_pos = GTK_CLIST_DRAG_BEFORE;
@@ -572,11 +593,11 @@ bookmarks_editor_new_item_clicked_cb (GtkButton *button,
 }
 
 void 
-bookmarks_editor_new_category_clicked_cb (GtkButton *button,
+bookmarks_editor_new_folder_clicked_cb (GtkButton *button,
 					  BookmarksEditorControls *controls) 
 {
 	BookmarkItem *new = bookmarks_new_bookmark 
-		(BM_CATEGORY, TRUE, _("New bookmark category"), NULL, NULL, NULL, NULL);
+		(BM_FOLDER, TRUE, _("New bookmark folder"), NULL, NULL, NULL, NULL);
 	BookmarkItem *near = controls->selection;
 	GtkCListDragPos insert_pos = GTK_CLIST_DRAG_INTO;
 
@@ -614,41 +635,41 @@ bookmarks_editor_new_separator_clicked_cb (GtkButton *button,
 	bookmarks_editor_select_bookmark (controls, new);
 }
 
-void
-bookmarks_editor_go_clicked_cb (GtkButton *button, 
-				BookmarksEditorControls *controls)
+static void
+bookmarks_editor_handle_link_clicked (BookmarksEditorControls *controls, 
+				      LinkState state)
 {
-	GList *pos;
-	gchar *url;
-
+	GaleonEmbed *embed = NULL, *newembed = NULL;
 	if (!controls->selection || (controls->selection->type != BM_SITE))
 		return;
 
 	/* Check if the window that is selected is already (still) open */
+	if (controls->window && !g_list_find (all_windows, controls->window))
+		controls->window = NULL;
 	if (controls->window)
-	{
-		pos = g_list_find (all_windows, controls->window);
-		if (!pos)
-		{
-			controls->window = NULL;
-		}
-	}
+		embed = controls->window->active_embed;
 
-	/* create a new window if needed */
-	if (controls->selection)
+	bookmarks_set_visited (controls->selection);
+	if (strstr(controls->selection->url, "%s"))
 	{
-		url = controls->selection->url;
-		controls->selection->time_visited = time (NULL);
-		if (!controls->window)
-		{
-			controls->window = 
-				embed_create_from_url (NULL, url, FALSE, TRUE)->parent_window;
-		}
-		else
-		{
-			window_load_url (controls->window, url);
-		}
+		bookmarks_smart_bm_dialog (embed, &newembed,
+					   controls->selection, state);
 	}
+	else
+	{
+		embed_activate_link (embed, &newembed,
+				     controls->selection->url, state);
+	}
+	/* remember created window if necessary */
+	if (!controls->window && newembed)
+		controls->window = newembed->parent_window;
+}
+
+void
+bookmarks_editor_go_clicked_cb (GtkButton *button, 
+				BookmarksEditorControls *controls)
+{
+	bookmarks_editor_handle_link_clicked (controls, 0);
 }
 
 gboolean
@@ -673,7 +694,7 @@ bookmarks_editor_copy_clicked_cb (GtkButton *button,
 		gnome_popup_menu_do_popup_modal
 			(GTK_WIDGET (destination_menu), menu_position_under_widget, 
 			 GTK_WIDGET (button), NULL, NULL);
-		gtk_widget_destroy (GTK_WIDGET (destination_menu));
+		gtk_object_unref (GTK_OBJECT (destination_menu));
 		bookmarks_dirty = TRUE;
 	}
 }
@@ -724,7 +745,7 @@ bookmarks_editor_bookmark_copy_cb (GtkMenuItem *menuitem, BookmarkItem *dest)
 	BookmarkItem *item = gtk_object_get_user_data (GTK_OBJECT (menuitem));
 	BookmarkItem *new;
 	g_assert (item != NULL);
-	g_assert (dest->type == BM_CATEGORY);
+	g_assert (dest->type == BM_FOLDER);
 	new = bookmarks_copy_bookmark (item);
 	new->parent = dest;
 	dest->list = g_list_append (dest->list, new);
@@ -742,15 +763,68 @@ bookmarks_editor_ctree_button_press_event_cb (GtkWidget *widget,
 	controls->last_pressed = gtk_ctree_node_get_row_data 
 		(GTK_CTREE (widget), gtk_ctree_node_nth (GTK_CTREE (widget), row));
 
-	/* Double click goes to the link, as well as ignore the scrollbutton */
-	if (event->type == GDK_2BUTTON_PRESS && 
-	    !(event->button == 4 || event->button == 5) &&
-	    controls->last_pressed != NULL)
+	if (controls->last_pressed == NULL)
+		return FALSE;
+	
+	if (event->button == 3)
+	{
+		GaleonWindow *window = NULL;
+		BookmarkItem *bi = controls->last_pressed;
+		GtkWidget *popup = NULL;
+		
+		/* Check if the window that is selected is still open */
+		if (controls->window && g_list_find (all_windows,
+							controls->window))
+			window = controls->window;
+		else
+			//FIXME: logic of using the last window is not ideal
+			window = g_list_last (all_windows)->data;
+		return_val_if_not_window (window, FALSE);
+
+		if (bi->type == BM_AUTOBOOKMARKS || bi->type == BM_FOLDER)
+		{
+			popup = gtk_menu_new ();
+			bookmarks_create_edit_items (GTK_MENU (popup),
+						     controls);
+			context_menu_add_seperator (GTK_MENU (popup));
+			bookmarks_menu_create_base_extra_items 
+							(GTK_MENU (popup), bi);
+		}
+		else if (bi->type == BM_SITE)
+		{
+			popup = gtk_menu_new ();
+			bookmarks_create_edit_items (GTK_MENU (popup),
+						     controls);
+			context_menu_add_seperator (GTK_MENU (popup));
+			bookmarks_item_create_base_extra_items 
+							(GTK_MENU (popup), bi);
+		}
+		if (popup)
+		{
+			/* make sure that this entry is selected */
+			gtk_ctree_select (GTK_CTREE (controls->ctree),
+				  bookmarks_get_tree_item (bi, controls));
+			/* warning: ugly code */
+			gtk_object_set_data(GTK_OBJECT(popup), "GaleonWindow",
+					    window);
+			gnome_popup_menu_do_popup_modal (popup, NULL,
+							 NULL, event, NULL);
+			gtk_object_unref (GTK_OBJECT(popup));
+
+			gtk_signal_emit_stop_by_name (GTK_OBJECT(widget),
+					"button-press-event");
+			return TRUE;
+		}
+	}
+	/* double click or middle click opens the window */
+	else if (event->button != 1 || event->type == GDK_2BUTTON_PRESS)
 	{
 		/* make sure that this entry is selected */
 		gtk_ctree_select (GTK_CTREE (controls->ctree),
 				  bookmarks_get_tree_item (controls->last_pressed, controls));
-		bookmarks_editor_go_clicked_cb(NULL, controls);
+		bookmarks_editor_handle_link_clicked (controls, 
+				mouse_state_to_link_state (event->button,
+							event->state));
 	}
 
 	return FALSE;
@@ -761,7 +835,10 @@ bookmarks_editor_ctree_key_press_event_cb (GtkWidget *widget,
 					   GdkEventKey *event,
 					   BookmarksEditorControls *controls)
 {
-	if (event->state == 0)
+	if (event->keyval == GDK_Return)
+		bookmarks_editor_handle_link_clicked (controls,
+			key_state_to_link_state (event->state));
+	else if (event->state == 0)
 	{
 		if (event->keyval == GDK_Delete ||
 		    event->keyval == GDK_KP_Delete)
@@ -969,7 +1046,8 @@ bookmarks_editor_set_pixmap (BookmarksEditorControls *controls,
 }
 
 static void 
-bookmarks_editor_set_times (BookmarksEditorControls *controls, BookmarkItem *bookmark)
+bookmarks_editor_set_times (BookmarksEditorControls *controls,
+			    BookmarkItem *bookmark, BookmarkItem *bookmarkreal)
 {
 	gchar *text;
 	if (controls->time_added_entry) {
@@ -978,24 +1056,24 @@ bookmarks_editor_set_times (BookmarksEditorControls *controls, BookmarkItem *boo
 		g_free (text);
 	}
 	if (controls->time_modified_entry) {
-		text = bookmark ? time_to_string (bookmark->time_modified) : g_strdup ("");
+		text = bookmarkreal ? time_to_string (bookmarkreal->time_modified) : g_strdup ("");
 		gtk_entry_set_text (GTK_ENTRY (controls->time_modified_entry), text);
 		g_free (text);
 	}
 	if (controls->time_visited_entry) {
-		text = bookmark ? time_to_string (bookmark->time_visited) : g_strdup ("");
+		text = bookmarkreal ? time_to_string (bookmarkreal->time_visited) : g_strdup ("");
 		gtk_entry_set_text (GTK_ENTRY (controls->time_visited_entry), text);
 		g_free (text);
 	}
 }
 
 void 
-bookmarks_editor_show_only_this_category_cb (GtkButton *button,
+bookmarks_editor_show_only_this_folder_cb (GtkButton *button,
 				             BookmarksEditorControls *controls) 
 {
 	if (controls->selection) {
 		BookmarkItem *new_root = 
-			((controls->selection->type == BM_CATEGORY)
+			((controls->selection->type == BM_FOLDER)
 			 || (controls->selection->type == BM_AUTOBOOKMARKS)) ?
 			controls->selection :
 			controls->selection->parent;
@@ -1025,7 +1103,7 @@ bookmarks_editor_copy_cb (GtkButton *button,
 {
 	if (controls->selection) {
 		BookmarkItem *dummy = bookmarks_new_bookmark 
-			(BM_CATEGORY, TRUE, "Dummy category", 
+			(BM_FOLDER, TRUE, "Dummy folder", 
 			 NULL, NULL, NULL, NULL);
 		GtkCList *clist = GTK_CLIST(controls->ctree);
 		GList *selection_list = 
@@ -1116,6 +1194,7 @@ bookmarks_editor_selection_received_cb (GtkWidget *widget,
 			insert_pos = GTK_CLIST_DRAG_AFTER;
 		}
 		bookmarks_free_bookmark (b);
+		bookmarks_dirty = TRUE;
 		gtk_clist_thaw (GTK_CLIST(controls->ctree));
 	}
 }
@@ -1123,12 +1202,9 @@ bookmarks_editor_selection_received_cb (GtkWidget *widget,
 void
 bookmarks_dock_close_clicked_cb (GtkWidget *button, 
 				 BookmarksEditorControls *controls)
-{
+{	
 	bookmarks_editor_apply (controls);
-	bookmarks_editor_remove_tree_item (controls, 
-			                   controls->root_bookmark);
-	bookmarks_editors = g_list_remove (bookmarks_editors, controls);
-	window_undock (controls->window);
+	bookmarks_editor_hide_dialog (controls);
 }
 
 void
@@ -1139,7 +1215,7 @@ bookmarks_dock_destroy_cb (GtkWidget *dialog,
 
 	window = controls->window;
 	return_if_not_window (window);
-	(GTK_CHECK_MENU_ITEM (window->view_bookmarks_dock))->active = FALSE;
+
 }
 
 void 
@@ -1151,11 +1227,16 @@ bookmarks_dock_detach_button_clicked_cb (GtkButton *button,
 	window = controls->window;
 	return_if_not_window (window);
 	bookmarks_editor_apply (controls);
+
+	gtk_clist_freeze (GTK_CLIST(controls->ctree));
 	bookmarks_editor_remove_tree_item (controls, 
 			                   controls->root_bookmark);
+	gtk_clist_thaw (GTK_CLIST(controls->ctree));
+
 	bookmarks_editors = g_list_remove (bookmarks_editors, controls);
+	gnome_config_set_bool (CONF_STATE_BOOKMARKS_DOCK_ENABLE, FALSE);
 	window_undock (window);
-	bookmarks_editor_show_dialog (window, STANDARD_BOOKMARKS_EDITOR);
+	bookmarks_editor_show_dialog (window);
 }
 
 void
@@ -1167,6 +1248,301 @@ bookmarks_editor_show_attach_cb (GtkWidget *toggle,
 	window = controls->window;
 	return_if_not_window (window);
 
+	gnome_config_set_bool (CONF_STATE_BOOKMARKS_DOCK_ENABLE, TRUE);
+	if (controls->type != DOCKED_BOOKMARKS_EDITOR) {
+		bookmarks_editor_hide_dialog (controls);
+		bookmarks_editor_show_dock (window);
+	}
+}
+
+void
+bookmarks_editor_key_press_event_cb (GtkWidget *widget,GdkEventKey *event,
+				     BookmarksEditorControls *controls)
+{
+	if (event->keyval == GDK_Escape)
+	{
+		bookmarks_editor_hide_dialog (controls);
+	}
+}
+
+void
+bookmarks_editor_set_default_folder_cb (GtkWidget *control, 
+					BookmarksEditorControls *controls)
+{
+	g_return_if_fail (controls);
+
+	if (controls->selection != NULL && 
+	    (controls->selection->type == BM_FOLDER || 
+	     controls->selection->type == BM_AUTOBOOKMARKS))
+	{
+		bookmarks_set_as_default_activate_cb (NULL, controls->selection);
+	}
+}
+
+void
+bookmarks_editor_browse_clicked_cb (GnomeFileEntry *fileentry,
+				    BookmarksEditorControls *controls)
+{
+	if (fileentry->fsw != NULL)
+	{
+		dialog_set_parent (fileentry->fsw, controls->dialog);
+	}
+}
+
+static void
+bookmarks_editor_toolbar_appearance_menu_cb (GtkRadioMenuItem *menuitem, 
+					     BookmarksEditorControls *controls)
+{
+	GSList *list = g_slist_reverse (g_slist_copy (gtk_radio_menu_item_group (menuitem)));
+	gint index = g_slist_index (list, menuitem);
+	
+	g_slist_free (list);
+
+	gtk_toolbar_set_style (GTK_TOOLBAR (controls->toolbar), index);
+	
+	switch (controls->type)
+	{
+	case DOCKED_BOOKMARKS_EDITOR:
+		gnome_config_set_int 
+			(CONF_APPEARANCE_BOOKMARKS_EDITOR_DOCKED_TOOLBAR_STYLE,
+			 index);
+		break;
+	case STANDARD_BOOKMARKS_EDITOR:
+		gnome_config_set_int 
+			(CONF_APPEARANCE_BOOKMARKS_EDITOR_TOOLBAR_STYLE,
+			 index);
+		break;
+	}
+	gnome_config_sync ();
+	
+	gtk_widget_queue_resize (controls->dialog);
+}
+
+gboolean
+bookmarks_editor_toolbar_button_press_event_cb
+                                          (GtkWidget *tb_event_box,
+					   GdkEventButton *event,
+					   BookmarksEditorControls *controls)
+{
+	GSList *radiogroup = NULL;
+	gint style = 0;
+
+	if (event->button == 3)
+	{
+		GtkMenu *popup = GTK_MENU (gtk_menu_new ());
+	
+		if (gnome_preferences_get_menus_have_tearoff ())
+		{
+			GtkWidget *tearoff = gtk_tearoff_menu_item_new ();
+			gtk_menu_prepend (popup, tearoff);
+			gtk_widget_show (tearoff);
+		}
+
+		switch (controls->type)
+		{
+		case DOCKED_BOOKMARKS_EDITOR:
+			style = gnome_config_get_int 
+				(CONF_APPEARANCE_BOOKMARKS_EDITOR_DOCKED_TOOLBAR_STYLE);
+			break;
+		case STANDARD_BOOKMARKS_EDITOR:
+			style = gnome_config_get_int 
+				(CONF_APPEARANCE_BOOKMARKS_EDITOR_TOOLBAR_STYLE);
+			break;
+		}
+	
+		context_menu_add_radio_item 
+			(popup, _("_Icons only"),
+			 bookmarks_editor_toolbar_appearance_menu_cb, controls,
+			 (style == GTK_TOOLBAR_ICONS), &radiogroup);
+		context_menu_add_radio_item 
+			(popup, _("_Text only"),
+			 bookmarks_editor_toolbar_appearance_menu_cb, controls,
+			 (style == GTK_TOOLBAR_TEXT), &radiogroup);
+		context_menu_add_radio_item 
+			(popup, _("Text _under icons"),
+			 bookmarks_editor_toolbar_appearance_menu_cb, controls,
+			 (style == GTK_TOOLBAR_BOTH), &radiogroup);
+	
+		gtk_signal_emit_stop_by_name (GTK_OBJECT(tb_event_box),
+					      "button-press-event");
+
+		gnome_popup_menu_do_popup_modal (GTK_WIDGET (popup), NULL,
+						 NULL, event, NULL);
+		gtk_object_unref (GTK_OBJECT (popup));
+	}
+	return TRUE;
+}
+
+void
+bookmarks_editor_save_as_clicked_cb (GtkWidget *widget,
+					  BookmarksEditorControls *controls)
+{
+	gchar *file = NULL;
+
+	if (show_file_picker (controls->dialog, _("Save as..."),
+			      g_get_home_dir(), NULL, modeSave,
+			      &file)) 
+		if (dialog_check_overwrite_file (file, NULL)) {
+			bookmarks_save_as (file);
+		}
+	g_free (file);
+}
+
+void
+bookmarks_editor_open_clicked_cb (GtkWidget *widget, 
+				       BookmarksEditorControls *controls)
+{
+	gchar *file = NULL;
+
+	if (show_file_picker (controls->dialog, _("Load from..."),
+			      g_get_home_dir(), NULL, modeOpen,
+			      &file)) {
+		gchar *question = g_strdup_printf
+			(_("Your current set of bookmarks will be replaced "
+			   "with the contents of %s.\n"
+			   "Do you want to continue?"), file);
+		if (dialog_ask_question (question, controls->dialog)) {
+			bookmarks_set_root (bookmarks_load_from (file));
+		}
+		g_free (question);
+	}
+	g_free (file);
+	bookmarks_dirty = TRUE;
+	/* refresh everywhere else */
+	bookmarks_editor_apply (controls);
+}
+
+void
+bookmarks_editor_sort_norec_clicked_cb (GtkWidget *widget, 
+					BookmarksEditorControls *controls)
+{
+	GList *l = bookmarks_get_selections_sorted 
+		(GTK_CLIST (controls->ctree));
+	GList *li;
+	bookmarks_selection_prepare (l, controls);
+	for (li = l; li != NULL; li = li->next) {
+		BookmarkItem *sel = li->data;
+		BookmarkItem *sort_root = 
+			((sel->type == BM_FOLDER)
+			 || (sel->type == BM_AUTOBOOKMARKS)) 
+			? sel 
+			: sel->parent;
+		if (sort_root) bookmarks_sort_norec (sort_root, TRUE);
+	}
+	g_list_free (l);
+}
+
+void
+bookmarks_editor_sort_rec_clicked_cb (GtkWidget *widget, 
+				      BookmarksEditorControls *controls)
+{
+	GList *l = g_list_copy (GTK_CLIST (controls->ctree)->selection);
+	GList *li;
+	bookmarks_selection_prepare (l, controls);
+	for (li = l; li != NULL; li = li->next) {
+		BookmarkItem *sel = li->data;
+		BookmarkItem *sort_root = 
+			((sel->type == BM_FOLDER)
+			 || (sel->type == BM_AUTOBOOKMARKS)) 
+			? sel 
+			: sel->parent;
+		if (sort_root) bookmarks_sort_rec (sort_root, TRUE);
+	}
+	g_list_free (l);
+}
+
+/**
+ * Sorts clist selections by row number and prunes any children in the
+ * list whose ancestor is also in the list.  The returned GList should
+ * be freed. 
+ */
+GList *
+bookmarks_get_selections_sorted (GtkCList *clist)
+{
+	GtkCTree *ctree;
+	GList *selections_sorted;
+	GList *node, *child;
+	gint length, i;
+		
+	ctree = GTK_CTREE(clist);
+	selections_sorted = g_list_copy(clist->selection);
+
+	if (!selections_sorted)
+		return NULL;
+
+	/* check for multiple selections. we want to preserve
+	   the original order of the items, regardless of the
+	   order in which they were selected, so the selection
+	   list is sorted by row number */
+	if (selections_sorted->next)
+	{
+		selections_sorted = g_list_sort(selections_sorted,
+					(GCompareFunc) bookmarks_row_compare);
+		length = g_list_length(selections_sorted);
+		node = selections_sorted;
+
+		/* prune the list to make sure we don't try to move
+		   an item whose ancestor is also in the list */
+		for (i = 0; i < length; i++)
+		{
+			gboolean remove_child = FALSE;
+			GtkCTreeNode *del_node = NULL;
+			
+			child = selections_sorted;
+			while (child)
+			{
+				if (gtk_ctree_is_ancestor(ctree,
+					    GTK_CTREE_NODE(node->data),
+					    GTK_CTREE_NODE(child->data)))
+				{
+					del_node = GTK_CTREE_NODE(child->data);
+					remove_child = TRUE;
+				}
+				child = child->next;
+
+				if (remove_child)
+				{
+					selections_sorted = g_list_remove(
+							  selections_sorted,
+							  del_node);
+					remove_child = FALSE;
+				}
+			}
+			if (!(node = g_list_nth(selections_sorted, i+1)))
+				break;
+		}
+	}
+	return (selections_sorted);
+}
+
+static gint bookmarks_row_compare (GList *a, GList *b)
+{
+	GList *root;
+	gint row_a, row_b;
+	
+	root = g_list_first(a);
+	row_a = g_list_position(root, a);
+	row_b = g_list_position(root, b);
+	
+	if (row_a < row_b)
+		return -1;
+	else if (row_a == row_b)
+		return 0;
+	else
+		return 1;
+}
+
+void
+bookmarks_dock_menu_button_clicked_cb (GtkButton *button,
+				       BookmarksEditorControls *controls)
+{
+	bookmarks_editor_popup_menu (controls);
+}
+
+
+void 
+bookmarks_editor_destroy_cb (GtkWidget *widget, 
+			     BookmarksEditorControls *controls)
+{
 	bookmarks_editor_hide_dialog (controls);
-	bookmarks_editor_show_dock (window);
 }
