@@ -23,9 +23,6 @@
 #include <gtk/gtkselection.h>
 #include <gtk/gtkdnd.h>
 
-/* Internal function used for multiple selection DnD */
-static gint bookmarks_row_compare (GList *a, GList *b);
-
 /* Internal variable used to save last insert position in the ctree */
 static gint insert_pos;
 
@@ -291,7 +288,7 @@ bookmarks_editor_ctree_drag_data_get_cb (GtkWidget *widget,
 					 BookmarksEditorControls *controls)
 {
 	if (controls->last_pressed) {
-		gchar *mem;
+		gchar *mem = NULL;
 		switch (info) {
 		case DND_TARGET_GALEON_BOOKMARK:
 			if (controls->last_pressed) {
@@ -302,7 +299,7 @@ bookmarks_editor_ctree_drag_data_get_cb (GtkWidget *widget,
 			}
 			break;
 		case DND_TARGET_STRING:
-			if (controls->last_pressed->type == BM_CATEGORY) {
+			if (controls->last_pressed->type == BM_FOLDER) {
 				if (controls->last_pressed->name
 				    && (strlen (controls->last_pressed->name) > 0)) {
 					gtk_selection_data_set 
@@ -323,7 +320,7 @@ bookmarks_editor_ctree_drag_data_get_cb (GtkWidget *widget,
 			}
 			break;
 		case DND_TARGET_TEXT_URI_LIST:
-			if (controls->last_pressed->type == BM_CATEGORY) {
+			if (controls->last_pressed->type == BM_FOLDER) {
 				GList *l;
 				gchar *text = g_strdup ("");
 				for (l = controls->last_pressed->list; 
@@ -355,88 +352,11 @@ bookmarks_editor_ctree_drag_data_get_cb (GtkWidget *widget,
 			g_warning ("Unknown DND type");
 			break;
 		}
+		/* gtk_selection_data_set copies the data */
+		if (mem) g_free (mem);
 	} 
 }
 
-/* Sorts clist selections by row number and prunes any children in the
-   list whose ancestor is also in the list.  The returned GList should
-   be freed. */
-GList *
-bookmarks_get_selections_sorted (GtkCList *clist)
-{
-	GtkCTree *ctree;
-	GList *selections_sorted;
-	GList *node, *child;
-	gint length, i;
-		
-	ctree = GTK_CTREE(clist);
-	selections_sorted = g_list_copy(clist->selection);
-
-	if (!selections_sorted)
-		return NULL;
-
-	/* check for multiple selections. we want to preserve
-	   the original order of the items, regardless of the
-	   order in which they were selected, so the selection
-	   list is sorted by row number */
-	if (selections_sorted->next)
-	{
-		selections_sorted = g_list_sort(selections_sorted,
-					(GCompareFunc) bookmarks_row_compare);
-		length = g_list_length(selections_sorted);
-		node = selections_sorted;
-
-		/* prune the list to make sure we don't try to move
-		   an item whose ancestor is also in the list */
-		for (i = 0; i < length; i++)
-		{
-			gboolean remove_child = FALSE;
-			GtkCTreeNode *del_node = NULL;
-			
-			child = selections_sorted;
-			while (child)
-			{
-				if (gtk_ctree_is_ancestor(ctree,
-					    GTK_CTREE_NODE(node->data),
-					    GTK_CTREE_NODE(child->data)))
-				{
-					del_node = GTK_CTREE_NODE(child->data);
-					remove_child = TRUE;
-				}
-				child = child->next;
-
-				if (remove_child)
-				{
-					selections_sorted = g_list_remove(
-							  selections_sorted,
-							  del_node);
-					remove_child = FALSE;
-				}
-			}
-			if (!(node = g_list_nth(selections_sorted, i+1)))
-				break;
-		}
-	}
-	return (selections_sorted);
-}
-
-static gint bookmarks_row_compare (GList *a, GList *b)
-{
-	GList *root;
-	gint row_a, row_b;
-	
-	root = g_list_first(a);
-	row_a = g_list_position(root, a);
-	row_b = g_list_position(root, b);
-	
-	if (row_a < row_b)
-		return -1;
-	else if (row_a == row_b)
-		return 0;
-	else
-		return 1;
-}
-
 void
 bookmarks_editor_ctree_drag_data_received_cb (GtkWidget *widget,
 					      GdkDragContext *context,
@@ -460,10 +380,13 @@ bookmarks_editor_ctree_drag_data_received_cb (GtkWidget *widget,
 	{
 	case DND_TARGET_GALEON_BOOKMARK:
 		b = bookmarks_item_from_string (selection_data->data);
+		/* how do we deal with freeing selection_data->data?
+		   I know its allocated in drag_data_get */
+		/* g_free (selection_data->data); <- does not work */
 		break;
 	case DND_TARGET_TEXT_URI_LIST:
 		b = bookmarks_new_bookmark 
-			(BM_CATEGORY, TRUE, "New category", NULL, NULL, NULL, NULL);
+			(BM_FOLDER, TRUE, "New folder", NULL, NULL, NULL, NULL);
 		s = selection_data->data;
 		while ((t = strstr (s, "\r\n"))) {
 			gchar *url = g_strndup (s, t-s);
@@ -506,7 +429,10 @@ bookmarks_editor_ctree_drag_data_received_cb (GtkWidget *widget,
 		GtkCTreeNode *source_node, *dest_node;
 
 		if (clist->click_cell.row < 0)
+		{
+			bookmarks_free_bookmark (b);
 			return;
+		}
 
 		selections_sorted = bookmarks_get_selections_sorted(clist);
 
@@ -550,7 +476,10 @@ bookmarks_editor_ctree_drag_data_received_cb (GtkWidget *widget,
 			source_node = GTK_CTREE_NODE(selections->data);
 
 			if (!source_node || !dest_node)
+			{
+				
 				return;
+			}
 
 			gtk_ctree_move (ctree, source_node,
 					GTK_CTREE_ROW(dest_node)->parent, 
@@ -558,6 +487,9 @@ bookmarks_editor_ctree_drag_data_received_cb (GtkWidget *widget,
 			selections = selections->next;
 		}
 		g_list_free(selections_sorted);
+		/* we just moved the clist around, and did not acually use this
+		   atleast I think so =) MattM */
+		bookmarks_free_bookmark (b);
 	}
 	else
 	{
