@@ -20,6 +20,8 @@
 #include "misc.h"
 #include "embed.h"
 #include "bookmarks.h"
+#include "context.h"
+#include "dialog.h"
 
 #include <string.h>
 #include <libgnomeui/gnome-preferences.h>
@@ -28,29 +30,27 @@
 #include <libgnome/gnome-i18n.h>
 #include <libgnomeui/gtkpixmapmenuitem.h>
 
-
 static void bookmarks_menu_destroy (GaleonWindow *window);
 static GtkWidget *bookmarks_menu_create_item (BookmarkItem *bm, GtkMenu *menu);
 static void bookmarks_create_file_menu (GtkMenuItem *file_menuitem,
 					BookmarkItem *bookmark);
 static void bookmarks_create_file_submenu (GtkMenuItem *file_menuitem, 
 					   BookmarkItem *bookmark);
-static void bookmarks_menu_create_extra_items (GtkMenu *menu,
-					       BookmarkItem *bookmark,
-					       gint opts_location);
 static void bookmarks_menu_create_recursively_cb (GtkMenuItem *menuitem,
 						  gpointer data);
 static void bookmarks_create_file_menu_cb (GtkMenuItem *mi, BookmarkItem *b);
 GtkMenu *bookmarks_create_copy_menu_recursively (BookmarkItem *source,
 						 BookmarkItem *b);
 static gint bookmarks_smart_bm_dialog_single (GtkWidget *dialog, 
-					      GaleonWindow *window, 
+					      GaleonEmbed *embed,
+					      GaleonEmbed **newembed,
 					      BookmarkItem *bi,
-					      gboolean new_window);
+					      LinkState state);
 static gint bookmarks_smart_bm_dialog_multi (GtkWidget *dialog, 
-					     GaleonWindow *window, 
+					     GaleonEmbed *embed,
+					     GaleonEmbed **newembed,
 					     BookmarkItem *bi,
-					     gboolean new_window);
+					     LinkState state);
 static gboolean bookmark_has_subcategories (BookmarkItem *b);
 
 /**
@@ -59,8 +59,7 @@ static gboolean bookmark_has_subcategories (BookmarkItem *b);
 void
 bookmarks_menu_create (GaleonWindow *window) 
 {
-	GtkMenuItem *file_menuitem;
-	GtkWidget *app;
+	GtkMenuItem *add_menuitem;
 	GtkMenu *menu;
 	
 	return_if_not_window (window);
@@ -71,23 +70,14 @@ bookmarks_menu_create (GaleonWindow *window)
 		return;
 	}
 
-	app = window->WMain;
-	menu = GTK_MENU (gtk_object_get_data (GTK_OBJECT(app), 
-					      "bookmarks_menu"));
-	file_menuitem = GTK_MENU_ITEM (gtk_object_get_data (GTK_OBJECT(app), 
-							    "file_bookmark"));
+	menu = (GtkMenu *) window->bookmarks_menu;
+	add_menuitem = (GtkMenuItem *) window->add_bookmark;
 
-	if (window->bookmarks_tooltips != NULL)
-	{
-		gtk_object_destroy (GTK_OBJECT (window->bookmarks_tooltips));
-	}
-	
 	window->bookmarks_tooltips = gtk_tooltips_new ();
-	bookmarks_create_file_submenu (file_menuitem, bookmarks_root);
-	bookmarks_menu_create_recursively (bookmarks_root, menu, file_menuitem,
-					   GTK_TOOLTIPS(window->bookmarks_tooltips), 
-					   TRUE, 
-					   FALSE);
+	bookmarks_create_file_submenu (add_menuitem, bookmarks_root);
+	bookmarks_menu_create_recursively (bookmarks_root, menu,
+		add_menuitem, GTK_TOOLTIPS(window->bookmarks_tooltips), 
+		TRUE, FALSE);
 }
 
 /**
@@ -103,7 +93,6 @@ bookmarks_menu_recreate (GaleonWindow *window)
 static void
 bookmarks_menu_destroy (GaleonWindow *window)
 {
-	GtkWidget *WMain = GTK_WIDGET (window->WMain);
 	GtkWidget *bookmarks_menu;
 	GtkWidget *separator;
 	GList *li, *l;
@@ -114,10 +103,9 @@ bookmarks_menu_destroy (GaleonWindow *window)
 		return;
 	}
 	
-	separator = gtk_object_get_data (GTK_OBJECT (WMain),
-					 "bookmarks_separator");
-	bookmarks_menu = gtk_object_get_data (GTK_OBJECT (WMain),
-					      "bookmarks_menu");
+	separator = window->bookmarks_separator;
+	bookmarks_menu = window->bookmarks_menu;
+
 	l = gtk_container_children (GTK_CONTAINER (bookmarks_menu));
 	
 	/* first look for the separator */
@@ -137,7 +125,13 @@ bookmarks_menu_destroy (GaleonWindow *window)
 			gtk_widget_destroy (GTK_WIDGET (li->data));
 		}
 	}
-	//FIXME: right???
+
+	/* destroy old tooltip */
+	if (window->bookmarks_tooltips)
+	{
+		gtk_object_destroy (GTK_OBJECT (window->bookmarks_tooltips));
+	}
+	
 	g_list_free (l);
 }
 
@@ -161,24 +155,31 @@ bookmarks_menu_create_recursively (BookmarkItem *bookmark, GtkMenu *menu,
 {
 	GList *l; 
 	GtkWidget *mi;
-	gint opts_location = gnome_config_get_int
+	ExtraItemsLocation opts_location = gnome_config_get_int
 				("/galeon/General/bm_extra_items_location=1");
 
-	g_return_if_fail (bookmark->type == BM_CATEGORY || 
+	g_return_if_fail (bookmark->type == BM_FOLDER || 
 			  bookmark->type == BM_AUTOBOOKMARKS);
 	
 	if (file_menuitem != NULL) 
 		bookmarks_create_file_menu (file_menuitem, bookmark);
 			
-	if (bookmark == bookmarks_root)
+	if (options)
 	{
-		if (opts_location == 0)
+		if (opts_location == EXTRA_ITEM_LOCATION_TOP_SUBMENU)
 		{
-			bookmarks_menu_create_extra_items (menu, bookmark, 0);
+			bookmarks_menu_create_extra_items (menu, bookmark, 
+							   opts_location);
 		}
-		else if (opts_location == 2)
+		else if (bookmark == bookmarks_root &&
+				opts_location == EXTRA_ITEM_LOCATION_CONTEXT)
 		{
 			mi = bookmarks_menu_create_item (bookmark, menu);
+			gtk_signal_connect (GTK_OBJECT(mi),
+					    "button-press-event",
+					    GTK_SIGNAL_FUNC (
+					     bookmarks_button_press_event_cb),
+					    bookmark);
 			gtk_signal_connect (GTK_OBJECT(mi),
 					    "button-release-event",
 					    GTK_SIGNAL_FUNC (
@@ -187,6 +188,7 @@ bookmarks_menu_create_recursively (BookmarkItem *bookmark, GtkMenu *menu,
 			gtk_widget_show (mi);
 			gtk_menu_append (menu, mi);
 			mi = gtk_menu_item_new ();
+			gtk_widget_set_sensitive(mi, FALSE);
 			gtk_widget_show (mi);
 			gtk_menu_append (menu, mi);
 		}
@@ -199,12 +201,16 @@ bookmarks_menu_create_recursively (BookmarkItem *bookmark, GtkMenu *menu,
 
 		switch (b->type)
 		{
-		case BM_CATEGORY:
+		case BM_FOLDER:
 		case BM_AUTOBOOKMARKS:
 			submenu = gtk_menu_new ();
 			gtk_menu_set_title (GTK_MENU (submenu), b->name);
 			mi = bookmarks_menu_create_item (b, menu);
 			gtk_menu_item_set_submenu (GTK_MENU_ITEM(mi), submenu);
+			gtk_signal_connect 
+				(GTK_OBJECT (submenu), "button-press-event", 
+				 GTK_SIGNAL_FUNC (
+					bookmarks_button_press_event_cb), b);
 			gtk_signal_connect 
 				(GTK_OBJECT (submenu), "button-release-event", 
 				 GTK_SIGNAL_FUNC (
@@ -224,6 +230,10 @@ bookmarks_menu_create_recursively (BookmarkItem *bookmark, GtkMenu *menu,
 			break;
 		case BM_SITE:
 			mi = bookmarks_menu_create_item (b, menu);
+			gtk_signal_connect_after
+				(GTK_OBJECT (mi), "button_press_event",
+				 GTK_SIGNAL_FUNC (
+					 bookmarks_button_press_event_cb), b);
 			gtk_signal_connect_after
 				(GTK_OBJECT (mi), "button_release_event", 
 				 GTK_SIGNAL_FUNC (
@@ -238,14 +248,14 @@ bookmarks_menu_create_recursively (BookmarkItem *bookmark, GtkMenu *menu,
 		case BM_SEPARATOR:
 		default:
 			mi = gtk_menu_item_new ();
+			gtk_widget_set_sensitive(mi, FALSE);
 			break;
 		}
 		gtk_widget_show (mi);
 		gtk_menu_append (menu, mi);
 	}
 
-	if ( !((bookmark == bookmarks_root) && (opts_location == 0)) 
-	     && options)
+	if (options && opts_location != EXTRA_ITEM_LOCATION_TOP_SUBMENU)
 		bookmarks_menu_create_extra_items (menu, bookmark,
 						   opts_location);
 
@@ -283,7 +293,7 @@ bookmarks_create_file_menu (GtkMenuItem *file_menuitem, BookmarkItem *bookmark)
 	
 	for (l = bookmark->list; l ; l = l->next) {
 		BookmarkItem *b = l->data;
-		if ((b->type == BM_CATEGORY) || (b->type == BM_AUTOBOOKMARKS)) {
+		if ((b->type == BM_FOLDER) || (b->type == BM_AUTOBOOKMARKS)) {
 			GtkWidget *file = bookmarks_menu_create_item (b,
 							GTK_MENU (file_menu));
 			gtk_widget_show (file);
@@ -302,6 +312,7 @@ bookmarks_create_file_menu (GtkMenuItem *file_menuitem, BookmarkItem *bookmark)
 			}			
 		} else if (b->type == BM_SEPARATOR) {
 			GtkWidget *mi = gtk_menu_item_new ();
+			gtk_widget_set_sensitive(mi, FALSE);
 			gtk_widget_show (GTK_WIDGET (mi));
 			gtk_menu_append (GTK_MENU (file_menu), 
 					 GTK_WIDGET (mi));
@@ -351,7 +362,7 @@ bookmark_has_subcategories (BookmarkItem *b)
 	GList *l;
 	for (l = b->list; l; l = l->next) {
 		BookmarkItem *bi = l->data;
-		if ((bi->type == BM_CATEGORY) || (bi->type == BM_AUTOBOOKMARKS))
+		if ((bi->type == BM_FOLDER) || (bi->type == BM_AUTOBOOKMARKS))
 			return TRUE;
 	}
 	return FALSE;
@@ -362,89 +373,117 @@ bookmark_has_subcategories (BookmarkItem *b)
  * in this folder" menuitems if necessary and places them appropriately in
  * the bookmarks menu.
  **/
-static void
+void
 bookmarks_menu_create_extra_items (GtkMenu *menu, BookmarkItem *bookmark,
-				   gint opts_location)
+				   ExtraItemsLocation opts_location)
 {
-	GtkWidget *add_item, *open_all_item, *create_toolbar_item, *separator;
+	GtkMenu *extra_menu;
+	GtkWidget *extra_menu_item = NULL;
 
 	g_return_if_fail (GTK_IS_MENU (menu));
 
-	if (opts_location == 2)
+	if (opts_location == EXTRA_ITEM_LOCATION_CONTEXT)
 		return;
 
-	add_item = gtk_menu_item_new_with_label ("");
-	label_set_accel_text (_("Add _bookmark to this folder"), 
-			GTK_BIN (add_item)->child, GTK_WIDGET (menu),
-			add_item);
-	gtk_signal_connect (GTK_OBJECT (add_item), "button-release-event",
-			    GTK_SIGNAL_FUNC(bookmarks_button_release_event_cb),
-			    bookmark);
-	gtk_signal_connect (GTK_OBJECT (add_item), "activate",
-			    GTK_SIGNAL_FUNC (bookmarks_activate_cb), bookmark);
-	gtk_object_set_data (GTK_OBJECT (add_item), "add_item",
-			     GINT_TO_POINTER (1));
-
-	open_all_item = gtk_menu_item_new_with_label ("");
-	label_set_accel_text (_("_Open all bookmarks in this folder"),
-			GTK_BIN (open_all_item)->child, GTK_WIDGET (menu),
-			open_all_item);
-	gtk_signal_connect (GTK_OBJECT(open_all_item), "button-release-event",
-			    GTK_SIGNAL_FUNC(bookmarks_button_release_event_cb),
-			    bookmark);
-	gtk_signal_connect (GTK_OBJECT (open_all_item), "activate",
-			    GTK_SIGNAL_FUNC (bookmarks_activate_cb), bookmark);
-	gtk_object_set_data (GTK_OBJECT (open_all_item), "open_all_item",
-			     GINT_TO_POINTER (1));
-
-	create_toolbar_item = gtk_check_menu_item_new_with_label ("");
-	label_set_accel_text (_("_Create toolbar from this folder"),
-			GTK_BIN (create_toolbar_item)->child,
-			GTK_WIDGET (menu), create_toolbar_item);
-	gtk_check_menu_item_set_active 
-				(GTK_CHECK_MENU_ITEM(create_toolbar_item), 
-				 bookmark->create_toolbar);
-	gtk_signal_connect (GTK_OBJECT(create_toolbar_item), 
-			    "button-release-event",
-			    GTK_SIGNAL_FUNC(bookmarks_button_release_event_cb),
-			    bookmark);
-	gtk_signal_connect (GTK_OBJECT (create_toolbar_item), "activate",
-			    GTK_SIGNAL_FUNC (bookmarks_activate_cb), bookmark);
-	gtk_object_set_data (GTK_OBJECT (create_toolbar_item), 
-			     "create_toolbar_item",
-			     GINT_TO_POINTER (1));
-
-	separator = gtk_menu_item_new();
-
-	if (opts_location == 0)
+	if (opts_location == EXTRA_ITEM_LOCATION_TOP_SUBMENU || 
+			opts_location == EXTRA_ITEM_LOCATION_BOTTOM_SUBMENU)
 	{
-		if ( (bookmark == bookmarks_root) &&
-		     gtk_menu_get_attach_widget (menu) )
-		{
-			gtk_menu_append (GTK_MENU (menu), add_item);
-			gtk_menu_append (GTK_MENU (menu), open_all_item);
-			gtk_menu_append (GTK_MENU (menu), create_toolbar_item);
-			gtk_menu_append (GTK_MENU (menu), separator);
-		}
-		else
-		{	
-			gtk_menu_prepend (GTK_MENU (menu), separator);
-			gtk_menu_prepend (GTK_MENU (menu), create_toolbar_item);
-			gtk_menu_prepend (GTK_MENU (menu), open_all_item);
-			gtk_menu_prepend (GTK_MENU (menu), add_item);
+		extra_menu = GTK_MENU (gtk_menu_new ());
+		extra_menu_item = gtk_menu_item_new_with_label ("");
+		label_set_accel_text (_("_Folder actions"), 
+			GTK_BIN (extra_menu_item)->child, GTK_WIDGET (menu),
+			extra_menu_item);
+		gtk_menu_item_set_submenu (GTK_MENU_ITEM (extra_menu_item),
+				GTK_WIDGET (extra_menu));
+
+		if (gnome_preferences_get_menus_have_tearoff ()){
+			GtkWidget *tearoff = gtk_tearoff_menu_item_new ();
+			gtk_menu_prepend (extra_menu, tearoff);
+			gtk_widget_show (tearoff);
 		}
 	}
 	else
+		extra_menu = menu;
+
+	bookmarks_menu_create_base_extra_items (extra_menu, bookmark);
+	context_menu_add_item (extra_menu, _("_Edit..."), 
+			bookmarks_edit_activate_cb, bookmark,
+			GNOME_STOCK_MENU_PROP, TRUE);
+	context_menu_add_item (extra_menu, _("Remove bookmark folder"),
+			bookmarks_remove_activate_cb, bookmark,
+			GNOME_STOCK_MENU_TRASH, 
+			!(bookmark == bookmarks_root));
+
+	if (opts_location == EXTRA_ITEM_LOCATION_TOP_SUBMENU)
 	{
-		gtk_menu_append (GTK_MENU (menu), separator);
-		gtk_menu_append (GTK_MENU (menu), add_item);
-		gtk_menu_append (GTK_MENU (menu), open_all_item);
-		gtk_menu_append (GTK_MENU (menu), create_toolbar_item);
+		gtk_menu_append (menu, GTK_WIDGET (extra_menu_item));
+		context_menu_add_seperator (menu);
+		gtk_widget_show (GTK_WIDGET (extra_menu_item));
 	}
-	gtk_widget_show (add_item);
-	gtk_widget_show (open_all_item);
-	gtk_widget_show (create_toolbar_item);
-	gtk_widget_show (separator);
+	else if (opts_location == EXTRA_ITEM_LOCATION_BOTTOM_SUBMENU)
+	{
+		context_menu_add_seperator (menu);
+		gtk_menu_append (menu, GTK_WIDGET (extra_menu_item));
+		gtk_widget_show (GTK_WIDGET (extra_menu_item));
+	}
+}
+/**
+ * Creates the items that are useful in both the menus and the bm editor 
+ **/
+void
+bookmarks_menu_create_base_extra_items (GtkMenu *menu, BookmarkItem *bookmark)
+{
+	context_menu_add_item (menu,
+			_("Add _bookmark here"),
+			bookmarks_add_activate_cb, bookmark, NULL, TRUE);
+	context_menu_add_item (menu,
+			_("_Open whole folder in tabs"),
+			bookmarks_open_all_activate_cb, bookmark, NULL, TRUE);
+	context_menu_add_item (menu,
+			_("Open whole folder in _windows"),
+			bookmarks_open_all_in_wins_activate_cb, bookmark, 
+			NULL, TRUE);
+	context_menu_add_check_item (menu,
+			_("_Create toolbar"),
+			bookmarks_create_toolbar_activate_cb, bookmark,
+			bookmark->create_toolbar);
+	context_menu_add_item (menu,
+			_("_Set as default folder"),
+			bookmarks_set_as_default_activate_cb, bookmark,
+			NULL, !(default_bookmarks_root == bookmark));
+}
+
+/**
+ * Creates the context menuitems and appends them to menu
+ **/
+void
+bookmarks_item_create_extra_items (GtkMenu *extra_menu, BookmarkItem *bookmark)
+{
+	g_return_if_fail (GTK_IS_MENU (extra_menu));
+
+	bookmarks_item_create_base_extra_items (extra_menu, bookmark);
+	context_menu_add_item (extra_menu, _("_Edit..."), 
+			bookmarks_edit_activate_cb, bookmark,
+			GNOME_STOCK_MENU_PROP, TRUE);
+	context_menu_add_item (extra_menu, _("Remove bookmark"),
+			bookmarks_remove_activate_cb, bookmark,
+			GNOME_STOCK_MENU_TRASH, TRUE);
+}
+/**
+ * Creates the items that are useful in both the menus and the bm editor 
+ **/
+void
+bookmarks_item_create_base_extra_items (GtkMenu *menu, BookmarkItem *bookmark)
+{
+	context_menu_add_item (menu, _("Open in a new window"),
+			bookmarks_open_in_new_window_activate_cb,
+			bookmark, GNOME_STOCK_MENU_NEW, TRUE);
+	context_menu_add_item (menu, _("Open in a new tab"),
+			bookmarks_open_in_new_tab_activate_cb, bookmark,
+			GNOME_STOCK_MENU_NEW, TRUE);
+	context_menu_add_item (menu, _("Copy link location"),
+			bookmarks_copy_link_activate_cb, bookmark,
+			GNOME_STOCK_MENU_COPY, TRUE);
 }
 
 /**
@@ -473,6 +512,7 @@ bookmarks_create_file_submenu (GtkMenuItem *file_menuitem,
 	gtk_menu_append (GTK_MENU (file_menu), w);
 	
 	w = gtk_menu_item_new ();
+	gtk_widget_set_sensitive(w, FALSE);
 	gtk_widget_show (w);
 	gtk_menu_append (GTK_MENU (file_menu), w);
 
@@ -507,6 +547,7 @@ bookmarks_create_copy_menu (BookmarkItem *source)
 		BookmarkItem *item = bookmarks_root;
 		GtkWidget *menu_item = bookmarks_menu_create_item (item, res);
 		GtkWidget *sep = gtk_menu_item_new ();
+		gtk_widget_set_sensitive(sep, FALSE);
 		gtk_widget_show (menu_item);
 		gtk_widget_show (sep);
 		gtk_menu_prepend (res, sep);
@@ -525,11 +566,11 @@ bookmarks_create_copy_menu_recursively (BookmarkItem *source, BookmarkItem *b)
 	GtkMenu *res = GTK_MENU (gtk_menu_new ());
 	GList *l;
 	gboolean has_sub_categories = FALSE;
-	g_assert (b->type == BM_CATEGORY);
+	g_assert (b->type == BM_FOLDER);
 	for (l = b->list; l != NULL; l = g_list_next (l)) {
 		BookmarkItem *item = l->data;
 		GtkWidget *menu_item, *sub_menu_item, *sep;
-		if (item->type == BM_CATEGORY && !(item->alias_of)) {
+		if (item->type == BM_FOLDER && !(item->alias_of)) {
 			GtkMenu *submenu = bookmarks_create_copy_menu_recursively (source, item);
 			has_sub_categories = TRUE;
 			sub_menu_item = bookmarks_menu_create_item (item, res);
@@ -540,6 +581,7 @@ bookmarks_create_copy_menu_recursively (BookmarkItem *source, BookmarkItem *b)
 				menu_item = bookmarks_menu_create_item (item, submenu);
 				gtk_widget_show (menu_item);
 				sep = gtk_menu_item_new ();
+				gtk_widget_set_sensitive(sep, FALSE);
 				gtk_widget_show (sep);
 				gtk_menu_prepend (submenu, sep);
 				gtk_menu_prepend (submenu, menu_item);
@@ -611,10 +653,15 @@ bookmarks_menu_create_item (BookmarkItem *bm, GtkMenu *menu)
 				    FALSE, FALSE, 2);
 	}
 	
-	if (bm->type == BM_CATEGORY || bm->type == BM_AUTOBOOKMARKS)
+	if (bm->type == BM_FOLDER || bm->type == BM_AUTOBOOKMARKS)
 	{
-		pixmap = gtk_pixmap_new (folder_pixmap_data->pixmap,
-					 folder_pixmap_data->mask);
+		if (default_bookmarks_root == bm)
+			pixmap = gtk_pixmap_new 
+				(default_folder_pixmap_data->pixmap,
+				 default_folder_pixmap_data->mask);
+		else
+			pixmap = gtk_pixmap_new (folder_pixmap_data->pixmap,
+						 folder_pixmap_data->mask);
 	}
 	else
 	{
@@ -627,6 +674,10 @@ bookmarks_menu_create_item (BookmarkItem *bm, GtkMenu *menu)
 	gtk_widget_show_all (hb);
 	if (gnome_preferences_get_menus_have_icons ())
 		gtk_widget_show (pixmap);
+	/* TODO: we would like to allow bookmark menu item keystroke binding,
+	 * but for now just disable it since it only acts weird and confusing
+	 * without full support for it. */
+	gtk_widget_lock_accelerators (m);
 	return m;
 }
 
@@ -634,16 +685,14 @@ bookmarks_menu_create_item (BookmarkItem *bm, GtkMenu *menu)
  * bookmarks_smart_bm_dialog: prompt user to complete the URL
  */
 void
-bookmarks_smart_bm_dialog (GaleonWindow *window, BookmarkItem *bi,
-			   gboolean new_window)
+bookmarks_smart_bm_dialog (GaleonEmbed *embed, GaleonEmbed **newembed,
+			   BookmarkItem *bi, LinkState state)
 {	
 	GtkWidget *dialog;
 	gchar *s = bi->url, *strippedname;
 	int num_parameters = 0;
 	int dlg_status = 0;
 
-	return_if_not_window (window);	
-
 	strippedname = strip_uline_accel (bi->name);
 	dialog = gnome_dialog_new (strippedname,
 				   GNOME_STOCK_BUTTON_OK,
@@ -676,13 +725,13 @@ bookmarks_smart_bm_dialog (GaleonWindow *window, BookmarkItem *bi,
 
 	if (num_parameters == 1)
 	{
-		dlg_status = bookmarks_smart_bm_dialog_single (dialog, window,
-							       bi, new_window);
+		dlg_status = bookmarks_smart_bm_dialog_single (dialog, embed,
+							newembed, bi, state);
 	}
 	else if (num_parameters > 1)
 	{
-		dlg_status = bookmarks_smart_bm_dialog_multi (dialog, window,
-							      bi, new_window);
+		dlg_status = bookmarks_smart_bm_dialog_multi (dialog, embed,
+							newembed, bi, state);
 	}
 	if (dlg_status != -1)
 		gnome_dialog_close (GNOME_DIALOG (dialog));
@@ -694,13 +743,13 @@ bookmarks_smart_bm_dialog (GaleonWindow *window, BookmarkItem *bi,
  * Returns: return value of the dialog 
  */
 static gint
-bookmarks_smart_bm_dialog_single (GtkWidget *dialog, GaleonWindow *window, 
-				  BookmarkItem *bi, gboolean new_window)
+bookmarks_smart_bm_dialog_single (GtkWidget *dialog, GaleonEmbed *embed, 
+				  GaleonEmbed **newembed, BookmarkItem *bi,
+				  LinkState state)
 {
 	GtkWidget *entry;
 	gchar *s, *tmp, *result, *text, *translated_text;
 	gint dlg_status;
-	gboolean tabbed_mode;
 
 	entry = gtk_entry_new ();
 	gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (dialog)->vbox), 
@@ -712,6 +761,9 @@ bookmarks_smart_bm_dialog_single (GtkWidget *dialog, GaleonWindow *window,
 				      GTK_EDITABLE(entry));
 	gtk_widget_show_all (dialog);
 
+	/* set parent and level in case of fulscreeen */
+	dialog_set_parent (dialog, embed->parent_window->WMain);
+	
 	if ((dlg_status = gnome_dialog_run (GNOME_DIALOG (dialog))) == 0) {
 		s = strstr (bi->url, "%s");
 		g_return_val_if_fail (s, 0);
@@ -721,12 +773,7 @@ bookmarks_smart_bm_dialog_single (GtkWidget *dialog, GaleonWindow *window,
 		translated_text = bookmarks_translate_string (text);
 		result = g_strconcat (tmp, translated_text, s+2, NULL);
 
-		tabbed_mode = gnome_config_get_bool (CONF_APPEARANCE_TABBED);
-		if (new_window)
-			embed_create_from_url(window->active_embed, result,
-					      TRUE, !tabbed_mode);
-		else
-			embed_load_url (window->active_embed, result);
+		embed_activate_link (embed, newembed, result, state);
 
 		g_free (tmp);
 		g_free (translated_text);
@@ -741,14 +788,14 @@ bookmarks_smart_bm_dialog_single (GtkWidget *dialog, GaleonWindow *window,
  * Returns: return value of the dialog
  */
 static gint
-bookmarks_smart_bm_dialog_multi (GtkWidget *dialog, GaleonWindow *window, 
-				 BookmarkItem *bi, gboolean new_window)
+bookmarks_smart_bm_dialog_multi (GtkWidget *dialog, GaleonEmbed *embed, 
+				 GaleonEmbed **newembed, BookmarkItem *bi,
+				 LinkState state)
 {
 	gchar *start, *point, *tmp, *result;
 	int i, dlg_status;
 	GtkWidget *seperator, *entry, *label;
 	GPtrArray *entries;
-	gboolean tabbed_mode;
 
 	seperator = gtk_hseparator_new();
 	gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (dialog)->vbox), 
@@ -808,6 +855,9 @@ bookmarks_smart_bm_dialog_multi (GtkWidget *dialog, GaleonWindow *window,
 
 	gtk_widget_show_all (dialog);
 
+	/* set parent and level in case of fulscreeen */
+	dialog_set_parent (dialog, embed->parent_window->WMain);
+
 	if ((dlg_status = gnome_dialog_run (GNOME_DIALOG (dialog))) == 0)
 	{
 		/* user selected OK, construct URL and go there */
@@ -836,12 +886,7 @@ bookmarks_smart_bm_dialog_multi (GtkWidget *dialog, GaleonWindow *window,
 			result = tmp;
 		}
 
-		tabbed_mode = gnome_config_get_bool (CONF_APPEARANCE_TABBED);
-		if (new_window)
-			embed_create_from_url(window->active_embed, result,
-					      TRUE, !tabbed_mode);
-		else
-			embed_load_url (window->active_embed, result);
+		embed_activate_link (embed, newembed, result, state);
 
 		g_free(result);
 	}
