@@ -22,6 +22,7 @@
 #include "window.h"
 #include "misc.h"
 #include "find.h"
+#include "context.h"
 
 #include <string.h>
 #include <libgnome/gnome-config.h>
@@ -32,28 +33,22 @@
 #include <gtk/gtkcheckmenuitem.h>
 #include <gtk/gtktoolbar.h>
 #include <gtk/gtkbox.h>
+#include <gdk/gdkkeysyms.h>
 
 extern const GtkTargetEntry bookmarks_dnd_targets[];
 
 /* local function prototypes */
 static void bookmarks_toolbars_create_from_root (GaleonWindow *window, 
 						 BookmarkItem *bi);
-static void  bookmarks_toolbar_create_from_category (GaleonWindow *window,
+static void  bookmarks_toolbar_create_from_folder (GaleonWindow *window,
 						     BookmarkItem *bi);
 static void bookmarks_toolbar_create_from_bookmark (GaleonWindow *window, 
 						    GtkToolbar *tb,
-						    BookmarkItem *b);
-static gint bookmarks_folder_show_menu_cb (GtkWidget *menu);
-static void  bookmarks_nick_entry_activate_cb (GtkEntry *entry, 
-					       BookmarkItem *bi);
-static gboolean bookmarks_toolbar_item_button_release_event_cb 
-                                       (GtkWidget *item, 
-					GdkEventButton *event,
-					BookmarkItem *bm);
-static gboolean bookmarks_toolbar_button_release_event_cb 
-                                       (GtkWidget *item, 
-					GdkEventButton *event,
-					BookmarkItem *bm);
+						    BookmarkItem *b,
+						    ToolbarStyle style);
+gboolean bookmarks_nick_entry_key_press_event_cb (GtkEntry *entry,
+						  GdkEventKey *event,
+						  BookmarkItem *bi);
 static void bookmarks_toolbar_drag_data_received_cb 
                                        (GtkWidget *widget, 
 					GdkDragContext *drag_context,
@@ -61,7 +56,27 @@ static void bookmarks_toolbar_drag_data_received_cb
 					GtkSelectionData *selection_data,
 					guint info, guint time,
 					BookmarkItem *bm);
-
+static gboolean bookmarks_toolbar_button_press_event_cb (GtkWidget *item,
+							GdkEventButton *event,
+							BookmarkItem *bi);
+static gboolean folder_button_press_event_cb (GtkWidget *item, 
+						GdkEventButton *event,
+						BookmarkItem *bi);
+static gboolean folder_button_release_event_cb (GtkWidget *item, 
+						  GdkEventButton *event,
+						  BookmarkItem *bi);
+static gboolean bookmark_button_press_event_cb (GtkWidget *item,
+						GdkEventButton *event,
+						BookmarkItem *bi);
+static gboolean bookmark_button_release_event_cb (GtkWidget *item, 
+						  GdkEventButton *event,
+						  BookmarkItem *bi);
+static gboolean smartbookmark_button_release_event_cb (GtkWidget *item, 
+						       GdkEventButton *event,
+						       BookmarkItem *bi);
+static void bookmarks_toolbar_item_menu_hide_cb (GtkMenuItem *menuitem, 
+						 BookmarkItem *bm);
+						
 /* this magic number is used to hack around a problem with
  * gnome-libs (I think) which causes multiple toolbars with
  * the same number not to display properly -- MattA
@@ -144,7 +159,7 @@ bookmarks_toolbars_create_from_root (GaleonWindow *window, BookmarkItem *bi)
 		/* can't create a toolbar from these */
 		return;
 
-	case BM_CATEGORY:
+	case BM_FOLDER:
 	case BM_AUTOBOOKMARKS:
 		/* continue... */
 		break;
@@ -154,7 +169,7 @@ bookmarks_toolbars_create_from_root (GaleonWindow *window, BookmarkItem *bi)
 	if (bi->create_toolbar)
 	{
 		/* do so */
-		bookmarks_toolbar_create_from_category (window, bi);
+		bookmarks_toolbar_create_from_folder (window, bi);
 	}
 	
 	/* now iterate to find any marked subfolders, 
@@ -169,10 +184,10 @@ bookmarks_toolbars_create_from_root (GaleonWindow *window, BookmarkItem *bi)
 }
 
 /**
- * bookmarks_toolbar_create_from_category: create a toolbar from a folder
+ * bookmarks_toolbar_create_from_folder: create a toolbar from a folder
  */
 static void 
-bookmarks_toolbar_create_from_category (GaleonWindow *window, BookmarkItem *bi)
+bookmarks_toolbar_create_from_folder (GaleonWindow *window, BookmarkItem *bi)
 {
 	gboolean toolbar_relief;
 	gboolean toolbar_detachable;
@@ -200,7 +215,7 @@ bookmarks_toolbar_create_from_category (GaleonWindow *window, BookmarkItem *bi)
 	gnome_app_add_docked (GNOME_APP (window->WMain), toolbar, unique_name,
 			      props, GNOME_DOCK_TOP, magic_number++, 0, 0);
 	gtk_container_set_border_width (GTK_CONTAINER (toolbar->parent),
-					toolbar_relief ? 2 : 0);
+					toolbar_relief ? 2 : 1);
 	gnome_dock_item_set_shadow_type (GNOME_DOCK_ITEM (toolbar->parent), 
 					 toolbar_relief ? 
 					 GTK_SHADOW_OUT : GTK_SHADOW_NONE);
@@ -219,8 +234,9 @@ bookmarks_toolbar_create_from_category (GaleonWindow *window, BookmarkItem *bi)
 		/* build the appropriate element */
 		if (bm->type != BM_SEPARATOR)
 		{
-			bookmarks_toolbar_create_from_bookmark
-				(window, GTK_TOOLBAR (toolbar), bm);
+			bookmarks_toolbar_create_from_bookmark 
+						(window, GTK_TOOLBAR (toolbar), 
+						 bm, bi->toolbar_style);
 		} 
 		else 
 		{
@@ -238,11 +254,10 @@ bookmarks_toolbar_create_from_category (GaleonWindow *window, BookmarkItem *bi)
 			   GDK_ACTION_COPY | GDK_ACTION_MOVE |
 			   GDK_ACTION_LINK);
 
-	/* the click handler */
-	gtk_signal_connect (GTK_OBJECT (toolbar->parent), 
-			    "button-release-event",
+	/* setup the toolbar context menu */
+	gtk_signal_connect (GTK_OBJECT (toolbar->parent), "button-press-event",
 			    GTK_SIGNAL_FUNC 
-			    (bookmarks_toolbar_button_release_event_cb), bi);
+				(bookmarks_toolbar_button_press_event_cb), bi);
 
 	/* show it */
 	gtk_widget_show_all (GTK_WIDGET (toolbar));
@@ -251,58 +266,133 @@ bookmarks_toolbar_create_from_category (GaleonWindow *window, BookmarkItem *bi)
 	window->bookmarks_toolbars = 
 		g_list_append (window->bookmarks_toolbars,
 			       GTK_TOOLBAR (toolbar));
+
 }
 
 /**
  * bookmarks_toolbar_create_from_bookmark:
  */
 static void
-bookmarks_toolbar_create_from_bookmark (GaleonWindow *window, GtkToolbar *tb,
-					BookmarkItem *b)
+bookmarks_toolbar_create_from_bookmark (GaleonWindow *window, 
+					GtkToolbar *tb,
+					BookmarkItem *b, 
+					ToolbarStyle style)
 {
-        GtkWidget *button, *hbox, *pixmap, *entry;
+        GtkWidget *button, *box, *pixmap, *entry;
 	const PixmapData *bm_icon;
 	GtkWidget *dnd_dest;
-	gchar *name;
-	
+	gchar *name = NULL;
+	gchar *tip = NULL;
+	gchar *short_url = NULL;
+
 	/* build a box, and pack the pixmap and label inside */
-	hbox = gtk_hbox_new (FALSE, 2);
-	if (b->pixmap_data == NULL)
+	if (style == TOOLBAR_STYLE_VERTICAL)
+		box = gtk_vbox_new (FALSE, 2);
+	else
+		box = gtk_hbox_new (FALSE, 2);
+	if (style != TOOLBAR_STYLE_TEXT_ONLY)
 	{
-		if (b->type == BM_CATEGORY || b->type == BM_AUTOBOOKMARKS)
+		if (b->pixmap_data == NULL)
 		{
-			pixmap = gtk_pixmap_new (folder_pixmap_data->pixmap,
-						 folder_pixmap_data->mask);
+			if (b->type == BM_FOLDER || 
+			    b->type == BM_AUTOBOOKMARKS)
+			{
+				if (default_bookmarks_root == b)
+					pixmap = gtk_pixmap_new 
+						(default_folder_pixmap_data->pixmap,
+					 	 default_folder_pixmap_data->mask);
+				else
+					pixmap = gtk_pixmap_new 
+						(folder_pixmap_data->pixmap,
+					 	 folder_pixmap_data->mask);
+			}
+			else
+			{
+				bm_icon = bookmarks_get_siteicon (b->url);
+				pixmap = gtk_pixmap_new (bm_icon->pixmap, 
+							 bm_icon->mask);
+			}
+			gtk_box_pack_start (GTK_BOX (box), pixmap, 
+					    FALSE, FALSE, 0);
 		}
 		else
 		{
-			bm_icon = bookmarks_get_siteicon (b->url);
-			pixmap = gtk_pixmap_new (bm_icon->pixmap, 
-						 bm_icon->mask);
+			pixmap = gtk_pixmap_new (b->pixmap_data->pixmap, 
+						 b->pixmap_data->mask);
+			gtk_box_pack_start (GTK_BOX (box), GTK_WIDGET (pixmap),
+					    TRUE, FALSE, 0);
 		}
+	}
+
+	if (b->pixmap_data == NULL || style == TOOLBAR_STYLE_TEXT_ONLY)
+	{
 		name = strip_uline_accel (b->name);
-		gtk_box_pack_start (GTK_BOX (hbox), pixmap, FALSE, FALSE, 0);
-		gtk_box_pack_start (GTK_BOX (hbox), gtk_label_new (name),
+		gtk_box_pack_start (GTK_BOX (box), gtk_label_new (name),
 				    TRUE, TRUE, 0);
 		g_free (name);
 	}
-	else
-	{
-		pixmap = gtk_pixmap_new (b->pixmap_data->pixmap, 
-					 b->pixmap_data->mask);
-		gtk_box_pack_start (GTK_BOX (hbox), GTK_WIDGET (pixmap),
-				    TRUE, FALSE, 0);
-	}
 
 	/* is this a normal (i.e. non-smart) bookmark */
-	if (b->url == NULL || strstr(b->url, "%s") == NULL)
+	if ((b->url == NULL) || strstr(b->url, "%s") == NULL)
 	{
 		/* yes, build a button and pack it */
 		dnd_dest = button = gtk_button_new ();
-		gtk_container_add(GTK_CONTAINER(button), hbox);
-		gtk_toolbar_append_widget(tb, button, b->url, NULL);
+		gtk_container_add(GTK_CONTAINER(button), box);
+
+		if ((b->url != NULL) && strlen (b->url) > BOOKMARKS_TOOLTIP_MAX_LENGTH) {
+			gchar *s = g_strndup (b->url, BOOKMARKS_TOOLTIP_MAX_LENGTH);
+			short_url = g_strconcat (s, "...", NULL);
+			g_free (s);
+		} 
+		else 
+		{
+			short_url = b->url;
+		}
+
+		if (b->notes == NULL || b->notes[0] == '\0')
+		{
+			tip = short_url;
+		}
+		else
+		{
+			tip = g_strdup_printf ("%s\n%s", short_url, b->notes);
+		}
+		
+		
+		gtk_toolbar_append_widget (tb, button, tip, NULL);
+		if (short_url != b->url) g_free (short_url);
+		if (tip != short_url) g_free (tip);
+
 		gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
 		GTK_WIDGET_UNSET_FLAGS (button, GTK_CAN_FOCUS);
+
+		/* the click handlers */
+		if (b->type == BM_SITE)
+		{
+			gtk_signal_connect (GTK_OBJECT (dnd_dest), 
+					    "button-press-event",
+					    GTK_SIGNAL_FUNC
+					    (bookmark_button_press_event_cb),
+					    b);
+			gtk_signal_connect (GTK_OBJECT (dnd_dest), 
+					    "button-release-event",
+					    GTK_SIGNAL_FUNC
+					    (bookmark_button_release_event_cb),
+					    b);
+		}
+		else
+		{
+			gtk_signal_connect (GTK_OBJECT (dnd_dest), 
+					    "button-press-event",
+					    GTK_SIGNAL_FUNC
+					    (folder_button_press_event_cb),
+					    b);
+			gtk_signal_connect (GTK_OBJECT (dnd_dest), 
+					    "button-release-event",
+					    GTK_SIGNAL_FUNC
+					    (folder_button_release_event_cb),
+					    b);
+		}
 	}
 	else
 	{
@@ -313,10 +403,10 @@ bookmarks_toolbar_create_from_bookmark (GaleonWindow *window, GtkToolbar *tb,
 		entry_width = gnome_config_get_int
 			("/galeon/Appearance/smart_bm_entry_width=100");
 		gtk_container_add (GTK_CONTAINER (button), 
-				   GTK_WIDGET (hbox));
+				   GTK_WIDGET (box));
 		gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
 		GTK_WIDGET_UNSET_FLAGS (button, GTK_CAN_FOCUS);
-		gtk_container_set_border_width (GTK_CONTAINER (hbox), 0);
+		gtk_container_set_border_width (GTK_CONTAINER (box), 0);
 		/* otherwise build an entry field and pack it */
 		entry = gtk_entry_new ();
 		gtk_widget_set_usize(entry, entry_width, -2);
@@ -328,8 +418,19 @@ bookmarks_toolbar_create_from_bookmark (GaleonWindow *window, GtkToolbar *tb,
 		gtk_object_set_data (GTK_OBJECT (button), "entry", entry);
 		gtk_object_set_data (GTK_OBJECT (entry), "GaleonWindow", 
 				     window);
-		gtk_signal_connect (GTK_OBJECT (entry), "activate",
-				    bookmarks_nick_entry_activate_cb, b);
+		gtk_signal_connect (GTK_OBJECT (entry), "key-press-event",
+		       (GtkSignalFunc) bookmarks_nick_entry_key_press_event_cb,
+		       b);
+
+		/* the click handlers */
+		gtk_signal_connect (GTK_OBJECT (dnd_dest), 
+				    "button-press-event",
+				    GTK_SIGNAL_FUNC
+				    (bookmark_button_press_event_cb), b);
+		gtk_signal_connect (GTK_OBJECT (dnd_dest), 
+				    "button-release-event",
+				    GTK_SIGNAL_FUNC
+				    (smartbookmark_button_release_event_cb), b);
 	}
 
 	gtk_signal_connect (GTK_OBJECT (dnd_dest), "drag_data_received",
@@ -341,12 +442,6 @@ bookmarks_toolbar_create_from_bookmark (GaleonWindow *window, GtkToolbar *tb,
 			   bookmarks_dnd_targets_num_items,
 			   GDK_ACTION_COPY | GDK_ACTION_MOVE |
 			   GDK_ACTION_LINK);
-			   
-	/* the click handler */
-	gtk_signal_connect (GTK_OBJECT (dnd_dest), "button-release-event",
-			    GTK_SIGNAL_FUNC 
-			    (bookmarks_toolbar_item_button_release_event_cb),
-			    b);				
 }
 
 #if 0
@@ -416,106 +511,85 @@ bookmarks_toolbars_set_visibility (GaleonWindow *window, gboolean visible)
 	}
 }
 
-#define TOOLBAR_ITEM_CONTEXT_MENU_EDIT_POS 0
-#define TOOLBAR_ITEM_CONTEXT_MENU_REMOVE_POS 1
-#define TOOLBAR_ITEM_CONTEXT_MENU_HIDE_TOOLBAR_POS 3
-
 /** 
- * bookmarks_toolbar_item_button_release_event_cb:
- * Handles clicks on bookmarks toolbar items.
- *
- * Button 1: Load bookmark in current window
- * Button 2: Load bookmark in new tab/window
- * Button 3: Show context menu
+ * FIXME: comments
  */
 static gboolean
-bookmarks_toolbar_item_button_release_event_cb (GtkWidget *item, 
-						GdkEventButton *event,
-						BookmarkItem *bi)
+folder_button_press_event_cb (GtkWidget *item, GdkEventButton *event,
+				BookmarkItem *bi)
 {
-	static GnomeUIInfo toolbar_item_context_menu_uiinfo[] =
+	GaleonWindow *window = window_from_widget (item);
+
+	/* create the folder menu */
+	if (event->button == 1)
 	{
-		GNOMEUIINFO_ITEM_STOCK (N_("Edit..."), NULL, NULL,
-				        GNOME_STOCK_MENU_PROP),
-		GNOMEUIINFO_ITEM_STOCK (N_("Remove bookmark"), NULL, NULL,
-				        GNOME_STOCK_MENU_CLOSE),
-		GNOMEUIINFO_SEPARATOR,
-		GNOMEUIINFO_ITEM_STOCK (N_("Hide toolbar"), NULL, NULL,
-					GNOME_STOCK_MENU_BLANK),
-		GNOMEUIINFO_END
-	};
-	GtkWidget *WMain = window_lookup_widget (GTK_WIDGET(item), "WMain");
-	GaleonWindow *window = gtk_object_get_data(GTK_OBJECT(WMain),
-						   "GaleonWindow");
-	GtkWidget *entry;
-	GtkWidget *popup;
-	gint item_action;
-	gboolean autobookmarks_deleted = FALSE;
-	gchar *text;
-	BookmarksEditorControls *controls;
+		GtkWidget *menu = gtk_menu_new();
+		GtkTooltips *tooltips = gtk_tooltips_new ();
+		
+		bookmarks_menu_create_recursively
+			(bi, GTK_MENU (menu), NULL, tooltips, 
+			 TRUE, TRUE);
+		
+		/* attach "GaleonWindow" to the menu so it can be looked up */
+		gtk_object_set_data (GTK_OBJECT (menu), "GaleonWindow",
+				     window);
+		
+		gtk_signal_emit_stop_by_name (GTK_OBJECT (item),
+					      "button-press-event");
+		
+		gnome_popup_menu_do_popup_modal (menu, 
+						 menu_position_under_widget,
+						 item, event, NULL);
+		gtk_widget_unref (GTK_WIDGET(menu));
+
+		if (tooltips)
+		{
+			gtk_object_destroy (GTK_OBJECT (tooltips));
+		}
+	}
+	else if (event->button == 3)
+	{
+		GtkMenu *popup = GTK_MENU (gtk_menu_new ());
+
+		bookmarks_menu_create_extra_items (popup,
+				bi, EXTRA_ITEM_LOCATION_EXCLUSIVE);
+
+		context_menu_add_seperator (popup);
+		context_show_appearance_menu (window, popup, bi, TRUE, TRUE);
+		context_menu_add_seperator (popup);
+
+		context_menu_add_item (popup, _("Hide toolbar"),
+				bookmarks_toolbar_item_menu_hide_cb, bi, 
+				NULL, TRUE);
+
+		/* warning: ugly code */
+		gtk_object_set_data(GTK_OBJECT(popup), "GaleonWindow",
+				    window);
+
+		gtk_signal_emit_stop_by_name (GTK_OBJECT(item),
+				"button-press-event");
+
+		gnome_popup_menu_do_popup_modal (GTK_WIDGET (popup), NULL,
+						 NULL, event, NULL);
+		gtk_widget_unref (GTK_WIDGET (popup));
+	}
+	return TRUE;
+}
+
+/**
+ * FIXME: comments
+ */
+static gboolean
+folder_button_release_event_cb (GtkWidget *item, GdkEventButton *event,
+				  BookmarkItem *bi)
+{
+	GaleonWindow *window = window_from_widget (item);
 	
 	if (item->window != event->window)
 		return FALSE;
 
-	if (event->button == 1)
-	{
-		/* Load the bookmark or create the folder menu */
-		if ((bi->type == BM_SITE) && !GTK_IS_EVENT_BOX(item))
-		{
-			if (strstr (bi->url, "%s") == NULL)
-			{
-				/* normal bookmark */
-				window_load_url(window, bi->url);
-			}
-			else
-			{
-				/* smart bookmark, do search for phrase */
-				entry = gtk_object_get_data (GTK_OBJECT (item),
-							     "entry");
-				text = gtk_entry_get_text (GTK_ENTRY (entry));
-				if (text != NULL && strlen (text) > 0)
-				{
-					find_next (window->active_embed, text);
-				}
-				else
-				{
-					BookmarkItem *b = 
-						bookmarks_find_by_nick (
-								bi->nick,
-								bookmarks_root,
-								FALSE);
-					if (b != NULL && b->url != NULL)
-					{
-						window_load_url (window, 
-								 b->url);
-					}
-				}
-			}
-		}
-		else if ((bi->type == BM_CATEGORY) || (bi->type == BM_AUTOBOOKMARKS))
-		{
-			GtkWidget *menu = gtk_menu_new();
-			GtkTooltips *tooltips = gtk_tooltips_new ();
-
-			bookmarks_menu_create_recursively
-				(bi, GTK_MENU (menu), NULL, tooltips, 
-				 TRUE, TRUE);
-
-			/* attach "WMain" to the menu so it can be looked up */
-			gtk_object_set_data (GTK_OBJECT (menu), "WMain", WMain);
-			gtk_object_set_data (GTK_OBJECT (menu), "tooltips",
-					     tooltips);
-			gtk_object_set_data (GTK_OBJECT (menu), "widget", item);
-
-			/* show the menu in a timeout function so GTK+ will
-			   clean up any pointer grabs before displaying the
-			   popup menu */
-			gtk_timeout_add (50,
-				(GtkFunction) bookmarks_folder_show_menu_cb,
-				menu);
-		}
-	}
-	else if (event->button == 2)
+	/* Load the bookmark(s) */
+	if (event->button == 2)
 	{
 		gboolean tabbed_mode;
 
@@ -524,136 +598,200 @@ bookmarks_toolbar_item_button_release_event_cb (GtkWidget *item,
 			tabbed_mode = !tabbed_mode;
 
 		/* Load the bookmark in a new tab/window */
-		if ((bi->type == BM_SITE) && !GTK_IS_EVENT_BOX(item))
-		{
-			embed_create_from_url (window->active_embed, bi->url,
-					       FALSE, !tabbed_mode);
-
-		}
-		else if (bi->type == BM_AUTOBOOKMARKS || bi->type == BM_CATEGORY)
-		{
-			bookmarks_folder_open_all_items (window->active_embed,
-							 bi, !tabbed_mode,
-							 FALSE);
-		}
-	}
-	else if (event->button == 3)
-	{
-		popup = gnome_popup_menu_new(toolbar_item_context_menu_uiinfo);
-		item_action = gnome_popup_menu_do_popup_modal(popup, NULL,
-							      NULL, NULL, NULL);
-		gtk_widget_destroy(popup);
-		
-		switch (item_action) 
-		{
-		case TOOLBAR_ITEM_CONTEXT_MENU_EDIT_POS:
-			controls = bookmarks_editor_show_dialog 
-				(window, STANDARD_BOOKMARKS_EDITOR);
-			bookmarks_editor_select_bookmark (controls, bi);
-			break;
-		case TOOLBAR_ITEM_CONTEXT_MENU_REMOVE_POS:
-			if (bi->type == BM_AUTOBOOKMARKS)
-			{
-				autobookmarks_deleted = TRUE;
-			}			
-			bookmarks_editor_remove_tree_items (bi);
-			bookmarks_remove_recursively (bi);
-			if (autobookmarks_deleted)
-			{
-				autobookmarks_root = NULL;
-			}
-			bookmarks_updated ();
-			break;
-		case TOOLBAR_ITEM_CONTEXT_MENU_HIDE_TOOLBAR_POS:
-			bi->parent->create_toolbar = FALSE;
-			bookmarks_updated ();
-			break;
-		}
-		return FALSE;
+		bookmarks_folder_open_all_items (window->active_embed,
+						 bi, !tabbed_mode, FALSE);
 	}
 	return TRUE;
 }
 
-#define TOOLBAR_CONTEXT_MENU_EDIT_POS 0
-#define TOOLBAR_CONTEXT_MENU_HIDE_TOOLBAR_POS 2
-
-static gboolean
-bookmarks_toolbar_button_release_event_cb (GtkWidget *item, 
-					   GdkEventButton *event,
-					   BookmarkItem *bm)
-{
-	static GnomeUIInfo toolbar_item_context_menu_uiinfo[] =
-	{
-		GNOMEUIINFO_ITEM_STOCK (N_("Edit..."), NULL, NULL,
-				        GNOME_STOCK_MENU_PROP),
-		GNOMEUIINFO_SEPARATOR,
-		GNOMEUIINFO_ITEM_STOCK (N_("Hide toolbar"), NULL, NULL,
-					GNOME_STOCK_MENU_BLANK),
-		GNOMEUIINFO_END
-	};
-	GtkWidget *popup;
-	gint item_action;
-	BookmarksEditorControls *controls;
-	GtkWidget *WMain = window_lookup_widget (GTK_WIDGET(item), "WMain");
-	GaleonWindow *window = gtk_object_get_data(GTK_OBJECT(WMain),
-						   "GaleonWindow");
-
-	if (event->button == 3)
-	{
-		popup = gnome_popup_menu_new(toolbar_item_context_menu_uiinfo);
-		item_action = gnome_popup_menu_do_popup_modal(popup, NULL,
-							      NULL, NULL, NULL);
-		gtk_widget_destroy(popup);
-		
-		switch (item_action) {
-		case TOOLBAR_CONTEXT_MENU_EDIT_POS:
-			controls = bookmarks_editor_show_dialog 
-				(window, STANDARD_BOOKMARKS_EDITOR);
-			bookmarks_editor_select_bookmark (controls, bm);
-			break;
-		case TOOLBAR_CONTEXT_MENU_HIDE_TOOLBAR_POS:
-			bm->create_toolbar = FALSE;
-			bookmarks_updated ();
-			break;
-		}
-		return FALSE;
-	}
-	return TRUE;
-}
-
-static int
-bookmarks_folder_show_menu_cb (GtkWidget *menu)
-{
-	GtkTooltips *tooltips = gtk_object_get_data (GTK_OBJECT (menu),
-						     "tooltips");
-	GtkWidget *widget = gtk_object_get_data (GTK_OBJECT (menu), "widget");
-
-	g_return_val_if_fail (menu != NULL, FALSE);
-
-	gnome_popup_menu_do_popup_modal (menu, menu_position_under_widget,
-					 widget, NULL, NULL);
-	gtk_widget_destroy (menu);
-	if (tooltips)
-	{
-		gtk_object_destroy (GTK_OBJECT (tooltips));
-	}
-
-	return FALSE;
-}
-
 /**
- * bookmarks_nick_entry_activate_cb: called when the user hits return
- * on an entry field on the toolbar (i.e. one created by a bookmark
+ * FIXME: comments
+ */
+static gboolean
+bookmark_button_press_event_cb (GtkWidget *item, GdkEventButton *event,
+				BookmarkItem *bi)
+{
+	GaleonWindow *window = window_from_widget (item);
+
+	if (event->button == 3)
+	{
+		GtkMenu *popup = GTK_MENU (gtk_menu_new ());
+
+		bookmarks_item_create_extra_items (popup, bi);
+		
+		context_menu_add_seperator (popup);
+		context_show_appearance_menu (window, popup, bi, TRUE, TRUE);
+		context_menu_add_seperator (popup);
+
+		context_menu_add_item (popup, _("Hide toolbar"),
+				bookmarks_toolbar_item_menu_hide_cb, bi,
+				NULL, TRUE);
+
+		/* warning: ugly code */
+		gtk_object_set_data(GTK_OBJECT(popup), "GaleonWindow",
+				    window);
+
+		gtk_signal_emit_stop_by_name (GTK_OBJECT(item),
+				"button-press-event");
+
+		gnome_popup_menu_do_popup_modal (GTK_WIDGET (popup), NULL,
+						 NULL, event, NULL);
+		gtk_widget_unref (GTK_WIDGET (popup));
+	}
+	return TRUE;
+}
+
+/**
+ * FIXME: comments
+ */
+static gboolean
+bookmark_button_release_event_cb (GtkWidget *item, GdkEventButton *event,
+				  BookmarkItem *bi)
+{
+	GaleonWindow *window = window_from_widget (item);
+	
+	if (item->window != event->window)
+		return FALSE;
+
+	/* Load the bookmark(s) */
+	if (!GTK_IS_EVENT_BOX(item))
+	{
+		/* normal bookmark */
+		bookmarks_set_visited (bi);
+		embed_activate_link_mouse (window->active_embed, NULL, 
+					   bi->url, event);
+	}
+	return TRUE;
+}
+
+/**
+ * FIXME: comments
+ */
+static gboolean
+bookmarks_toolbar_button_press_event_cb (GtkWidget *item,
+					 GdkEventButton *event,
+					 BookmarkItem *bi)
+{
+	GaleonWindow *window = window_from_widget (item);
+
+	if (event->button == 3)
+	{
+		GtkMenu *popup = GTK_MENU (gtk_menu_new ());
+
+		bookmarks_menu_create_extra_items (popup,
+				bi, EXTRA_ITEM_LOCATION_EXCLUSIVE);
+
+		context_menu_add_seperator (popup);
+		context_show_appearance_menu (window, popup, bi, FALSE, TRUE);
+
+		/* warning: ugly code */
+		gtk_object_set_data(GTK_OBJECT(popup), "GaleonWindow",
+				    window);
+
+		gtk_signal_emit_stop_by_name (GTK_OBJECT(item),
+				"button-press-event");
+
+		gnome_popup_menu_do_popup_modal (GTK_WIDGET (popup), NULL,
+						 NULL, event, NULL);
+		gtk_widget_unref (GTK_WIDGET (popup));
+	}
+	return TRUE;
+}
+
+/**
+ * FIXME: comments
+ */
+static gboolean
+smartbookmark_button_release_event_cb (GtkWidget *item, GdkEventButton *event,
+				       BookmarkItem *bi)
+{
+	GaleonWindow *window = window_from_widget (item);
+	GaleonEmbed *embed = window->active_embed;
+	gchar *text, *translated_text, *url;
+	GtkWidget *entry;
+	
+	if (item->window != event->window)
+		return FALSE;
+
+	if (GTK_IS_EVENT_BOX (item))
+	{
+		return FALSE;
+	}
+
+	/* smart bookmark, do search for phrase */
+	entry = gtk_object_get_data (GTK_OBJECT (item), "entry");
+	text = gtk_entry_get_text (GTK_ENTRY (entry));
+	if (text != NULL && strlen (text) > 0)
+	{
+		/* translate non-alphanumeric characters+into %{} format */
+		translated_text = bookmarks_translate_string (text);
+		
+                /* get a completed url */
+		url = bookmarks_substitute_argument (bi, translated_text);
+
+		/* if the search was already performed... */
+		if (g_strcasecmp (url, embed->site_location) == 0 &&
+		    event->button == 1)
+		{
+			/* search in the current page for the entered text */
+			find_next (embed, text);
+		}
+		else
+		{
+			/* otherwise activate normal behaviour */
+			bookmarks_set_visited (bi);
+			embed_activate_link_mouse (window->active_embed, NULL,
+						   url, event);
+		}
+		g_free (url);
+	}
+	else
+	{
+		/* if search field is empty, go to a normal bookmark
+		 * with matching nick, if one exists. */
+		BookmarkItem *b = bookmarks_find_by_nick (bi->nick,
+							bookmarks_root, FALSE);
+		if (b != NULL && b->url != NULL)
+		{
+			bookmarks_set_visited (b);
+			embed_activate_link_mouse (window->active_embed, NULL,
+						   b->url, event);
+		}
+
+	}
+
+	return TRUE;
+}
+
+/**
+ * bookmarks_toolbar_menu_hide_cb: called by the hide context menu on the
+ * toolbar items. Hide the toolbar.
+ */
+static void
+bookmarks_toolbar_item_menu_hide_cb (GtkMenuItem *menuitem, 
+				BookmarkItem *bm)
+{
+	bm->parent->create_toolbar = FALSE;
+	bookmarks_updated ();
+}
+
+/**
+ * bookmarks_nick_entry_key_press_event_cb: called when the user hits
+ * return on an entry field on the toolbar (i.e. one created by a bookmark
  * which has %s in the url)
  */
-static void 
-bookmarks_nick_entry_activate_cb (GtkEntry *entry, BookmarkItem *bi)
+gboolean
+bookmarks_nick_entry_key_press_event_cb (GtkEntry *entry, GdkEventKey *event,
+					 BookmarkItem *bi)
 {
 	gchar *text, *url, *translated_text;
 	GaleonWindow *window;
+	if (event->keyval != GDK_Return)
+		return FALSE;
 
 	/* find the window */
-	window = gtk_object_get_data (GTK_OBJECT (entry), "GaleonWindow");
+	window = gtk_object_get_data (GTK_OBJECT (entry),
+				      "GaleonWindow");
 
 	/* get the entry text: DON'T free the returned text! */
 	text = gtk_entry_get_text (entry);
@@ -664,12 +802,15 @@ bookmarks_nick_entry_activate_cb (GtkEntry *entry, BookmarkItem *bi)
 	/* get a completed url */
 	url = bookmarks_substitute_argument (bi, translated_text);
 
-	/* load it in this window */
-	window_load_url (window, url);
+	/* load it in correct location*/
+	bookmarks_set_visited (bi);
+	embed_activate_link_keyboard (window->active_embed, NULL, url,
+				      event->state);
 
 	/* free all allocated strings */
 	g_free (url);
 	g_free (translated_text);
+	return TRUE;
 }
 
 /** 
