@@ -23,8 +23,10 @@
 #include "history.h"
 #include "window.h"
 #include "mozilla.h"
+#include "downloader.h"
 #include "context.h"
 
+#include <string.h>
 #include <gtkmozembed.h>
 #include <gtk/gtkmain.h>
 #include <libgnome/gnome-i18n.h>
@@ -33,6 +35,8 @@
 #include <libgnomeui/gnome-popup-menu.h>
 #include <libgnomeui/gnome-stock.h>
 #include <libgnomeui/gnome-app-helper.h>
+#include <libgnomeui/gtkpixmapmenuitem.h>
+#include <libgnomeui/gnome-preferences.h>
 
 /* defines */
 #define CONTEXT_MENU_ITEM(label, stock_id, action) \
@@ -51,8 +55,7 @@ static char UP_STR[]            = N_("Up");
 static char RELOAD_STR[]        = N_("Reload");
 static char OPEN_NEW_WIN_STR[]  = N_("Open in a new window");
 static char OPEN_NEW_TAB_STR[]  = N_("Open in a new tab");
-static char DOWNLOAD_FILE_EXTERNAL_STR[] = N_("Download link with External Handler");
-static char DOWNLOAD_FILE_BUILT_IN_STR[] = N_("Download link with Built-in Handler");
+static char DOWNLOAD_FILE_STR[] = N_("Download link");
 static char COPY_LINK_LOC_STR[] = N_("Copy link location");
 static char ADD_BM_STR[]        = N_("Add bookmark");
 static char OPEN_IMG_STR[]      = N_("Open image");
@@ -81,8 +84,7 @@ static char FULLSCREEN_VIEW_STR[]    = N_("Full Screen View");
 #define LINK_CONTEXT_MENU \
   CONTEXT_MENU_ITEM(OPEN_NEW_WIN_STR,  GNOME_STOCK_MENU_NEW,   GALEON_CONTEXT_OPENWINDOW), \
   CONTEXT_MENU_ITEM(OPEN_NEW_TAB_STR,  GNOME_STOCK_MENU_NEW,   GALEON_CONTEXT_OPENTAB),    \
-  CONTEXT_MENU_ITEM(DOWNLOAD_FILE_BUILT_IN_STR, GNOME_STOCK_MENU_SAVE,  GALEON_CONTEXT_DL_INT), \
-  CONTEXT_MENU_ITEM(DOWNLOAD_FILE_EXTERNAL_STR, GNOME_STOCK_MENU_SAVE,  GALEON_CONTEXT_DL_EXT), \
+  CONTEXT_MENU_ITEM(DOWNLOAD_FILE_STR, GNOME_STOCK_MENU_SAVE,  GALEON_CONTEXT_DOWNLOAD), \
   CONTEXT_MENU_ITEM(COPY_LINK_LOC_STR, GNOME_STOCK_MENU_COPY,  GALEON_CONTEXT_COPYLOC),    \
   CONTEXT_MENU_ITEM(ADD_BM_STR,        GNOME_STOCK_MENU_BLANK, GALEON_CONTEXT_ADDTEMPBMLINK),
 #define IMAGE_CONTEXT_MENU \
@@ -120,8 +122,7 @@ typedef enum
 	GALEON_CONTEXT_ADDTEMPBM,
 	GALEON_CONTEXT_OPENWINDOW,
 	GALEON_CONTEXT_OPENTAB,
-	GALEON_CONTEXT_DL_EXT,
-	GALEON_CONTEXT_DL_INT,
+	GALEON_CONTEXT_DOWNLOAD,
 	GALEON_CONTEXT_COPYLOC,
 	GALEON_CONTEXT_ADDTEMPBMLINK,
 	GALEON_CONTEXT_OPENIMAGE,
@@ -143,17 +144,19 @@ typedef enum
 
 typedef enum
 {
-	GALEON_CONTEXT_LINK = CONTEXT_LINK | CONTEXT_OTHER,
-	GALEON_CONTEXT_IMAGELINK = CONTEXT_LINK | CONTEXT_IMAGE | CONTEXT_OTHER,
-	GALEON_CONTEXT_DOC = CONTEXT_DOCUMENT | CONTEXT_OTHER,
-	GALEON_CONTEXT_IMAGE = CONTEXT_IMAGE | CONTEXT_DOCUMENT | CONTEXT_OTHER,
-	GALEON_CONTEXT_INPUT = CONTEXT_DOCUMENT | CONTEXT_INPUT | CONTEXT_OTHER
+	GALEON_CONTEXT_LINK = CONTEXT_LINK | CONTEXT_DOCUMENT,
+	GALEON_CONTEXT_IMAGELINK = CONTEXT_LINK | CONTEXT_IMAGE | CONTEXT_DOCUMENT,
+	GALEON_CONTEXT_DOC = CONTEXT_DOCUMENT,
+	GALEON_CONTEXT_IMAGE = CONTEXT_IMAGE | CONTEXT_DOCUMENT,
+	GALEON_CONTEXT_INPUT = CONTEXT_DOCUMENT | CONTEXT_INPUT
 } GaleonContextEvent;
 
 /* local functions */
 static GnomeUIInfo *context_make_menu 	     (WrapperContextInfo *info);
 static guint context_show_context_menu 	     (GaleonEmbed *embed, 
-					      GnomeUIInfo *menu);
+					      GnomeUIInfo *menu,
+					      int button,
+					      guint timestamp);
 static void context_do_action 		     (GaleonContextAction action, 
 					      WrapperContextInfo *info,
 					      GaleonEmbed *embed);
@@ -167,7 +170,8 @@ static void context_menu_add_bookmarks (GtkMenu *menu, GaleonEmbed *embed);
 static BookmarkItem *context_menu_create_fake_bookmarks_root (void);
 static void context_menu_create_fake_bookmarks_root_recursively 
 						(BookmarkItem *b, GList **l);
-static void context_menu_bookmarks_cleanup (GtkObject *o, GaleonWindow *window);
+static void fill_in_image_context_menu (gchar *imagename, 
+	gint pos, GnomeUIInfo *uiinfo);
 
 /**
  * context_set_full_screen_check: searches for a check button and sets it
@@ -207,13 +211,11 @@ static void context_set_navigation_buttons (GaleonEmbed *embed,
 					    GtkWidget *popup)
 {
 	gboolean can_go_back, can_go_forward, can_go_up;
-	gboolean show_history_popup, show_up_button;
+	gboolean show_history_popup;
 	GtkMenu *menu_back, *menu_forward, *menu_up;
 	gint context_action, i;
 	gint go_back_pos = 0, go_forward_pos = 1, go_up_pos = 2;
 	
-	show_up_button = 
-	      gnome_config_get_bool ("/galeon/Appearance/show_up_button=FALSE");
 	show_history_popup = 
 	      gnome_config_get_bool ("/galeon/Mouse/history_popup=FALSE");
 	can_go_back = gtk_moz_embed_can_go_back (GTK_MOZ_EMBED(embed->mozEmbed));
@@ -234,15 +236,15 @@ static void context_set_navigation_buttons (GaleonEmbed *embed,
 		}
 	}
 
-	gtk_widget_set_sensitive(uiinfo[go_back_pos].widget,
-				 can_go_back);
-	gtk_widget_set_sensitive(uiinfo[go_forward_pos].widget,
-		 		 can_go_forward);
+	gtk_widget_set_sensitive(uiinfo[go_back_pos].widget, can_go_back);
+	gtk_widget_set_sensitive(uiinfo[go_forward_pos].widget, can_go_forward);
+	gtk_widget_set_sensitive (uiinfo[go_up_pos].widget, can_go_up);
+
 	if (show_history_popup)
 	{
-		if (can_go_back)
+		if (can_go_back && 
+		    (menu_back = embed_create_back_menu(embed)) != NULL)
 		{
-			menu_back = embed_create_back_menu(embed);
 			gtk_signal_connect(
 				GTK_OBJECT(menu_back),
 				"selection-done",
@@ -253,8 +255,9 @@ static void context_set_navigation_buttons (GaleonEmbed *embed,
 				uiinfo[go_back_pos].widget,
 				NULL);
 		}      
-		if (can_go_forward) {
-			menu_forward = embed_create_forward_menu(embed);
+		if (can_go_forward && 
+		    (menu_forward = embed_create_forward_menu(embed)) != NULL)
+		{
 			gtk_signal_connect(
 				GTK_OBJECT(menu_forward),
 				"selection-done",
@@ -265,8 +268,9 @@ static void context_set_navigation_buttons (GaleonEmbed *embed,
 				uiinfo[go_forward_pos].widget,
 				NULL);
 		}
-		if (can_go_up && show_up_button) {
-			menu_up = embed_create_up_menu (embed);
+		if (can_go_up &&
+		    (menu_up = embed_create_up_menu (embed)) != NULL)
+		{
 			gtk_signal_connect(
 				GTK_OBJECT(menu_up),
 				"selection-done",
@@ -276,12 +280,6 @@ static void context_set_navigation_buttons (GaleonEmbed *embed,
 					 uiinfo[go_up_pos].widget, NULL);
 		}
 	}
-
-	if (show_up_button)
-		gtk_widget_set_sensitive (uiinfo[go_up_pos].widget,
-						 can_go_up);
-	else
-		gtk_widget_hide (uiinfo[go_up_pos].widget);
 }
 
 /**
@@ -313,7 +311,8 @@ static GnomeUIInfo *context_make_menu (WrapperContextInfo *info)
 			NAVIGATION_CONTEXT_MENU
 			GNOMEUIINFO_END
 		};
-		return g_memdup (&imagelink_uiinfo, sizeof(imagelink_uiinfo));
+		fill_in_image_context_menu (info->img, 6, imagelink_uiinfo);
+                return g_memdup (&imagelink_uiinfo, sizeof(imagelink_uiinfo));
 	}
 	case GALEON_CONTEXT_DOC:
 	{
@@ -353,7 +352,8 @@ static GnomeUIInfo *context_make_menu (WrapperContextInfo *info)
 			IMAGE_CONTEXT_MENU
 			GNOMEUIINFO_END
 		};
-		return g_memdup (&image_uiinfo, sizeof(image_uiinfo));
+		fill_in_image_context_menu (info->img, 5, image_uiinfo);
+                return g_memdup (&image_uiinfo, sizeof(image_uiinfo));
 	}
 	case GALEON_CONTEXT_INPUT:
 	{ /* FIXME: maybe make this context have cut/copy/paste etc. */
@@ -376,23 +376,35 @@ static GnomeUIInfo *context_make_menu (WrapperContextInfo *info)
  * context_show_menu: show appropriate context menu based on the target type
  **/
 static guint context_show_context_menu (GaleonEmbed *embed, 
-					GnomeUIInfo *menu)
+					GnomeUIInfo *menu,
+					int button,
+					guint timestamp)
 {
 	guint ret;
 	GtkWidget *popup;
-
+	GdkEventButton *event = NULL;
+	/* Create a fake GdkEventButton struct for gnome_popup_menu_new so that
+	 * we can get standard gtk popup menu behavior.
+	 * button and time are the only fields it needs. */
+	if (button>0)
+	{
+		event = g_new0 (GdkEventButton, 1);
+		event->button = button;
+		event->time = timestamp;
+	}
 	popup = gnome_popup_menu_new (menu);
 	context_set_navigation_buttons (embed, menu, popup);
 	context_set_full_screen_check (embed->parent_window, menu, popup);
 	context_menu_add_bookmarks (GTK_MENU (popup), embed);
 	ret = gnome_popup_menu_do_popup_modal (popup, NULL, NULL,
-					       NULL , NULL);
+					       event , NULL);
 	/* Wait for the menu to be destroyed if a menu item is selected */
 	if (ret != -1) 
 		while (gtk_events_pending()) 
 			gtk_main_iteration();
 
-	gtk_widget_destroy (popup);
+	gtk_widget_unref (popup);
+	g_free (event);
 	return ret;
 }
 
@@ -418,7 +430,7 @@ static void context_do_action (GaleonContextAction action,
 		gtk_moz_embed_go_forward (GTK_MOZ_EMBED(embed->mozEmbed));
 		break;
 	case GALEON_CONTEXT_UP:
-		embed_go_up (embed, 0);
+		embed_go_up (embed, 0, 0);
 		break;
 	case GALEON_CONTEXT_RELOAD:
 		embed_reload (embed);
@@ -439,11 +451,8 @@ static void context_do_action (GaleonContextAction action,
 	case GALEON_CONTEXT_OPENTAB:
 		embed_create_from_url (embed, info->link, FALSE, FALSE);
 		break;
-	case GALEON_CONTEXT_DL_INT:
-		mozilla_save_link (embed, info->link);
-		break;
-	case GALEON_CONTEXT_DL_EXT:
-		save_url (info->link);
+	case GALEON_CONTEXT_DOWNLOAD:
+		downloader_save_link (embed, info->link);
 		break;
 	case GALEON_CONTEXT_COPYLOC:
 		copy_to_clipboard (info->link, embed);
@@ -461,10 +470,10 @@ static void context_do_action (GaleonContextAction action,
 		embed_create_from_url (embed, info->img, FALSE, FALSE);
 		break;
 	case GALEON_CONTEXT_SAVEIMAGE:
-		embed_save_url_default (embed, info->img);
+		embed_save_image (embed, info->img, FALSE);
 		break;
 	case GALEON_CONTEXT_SAVEIMAGEAS:
-		embed_save_url (embed, info->img);
+		embed_save_image (embed, info->img, TRUE);
 		break;
 	case GALEON_CONTEXT_USEIMAGEASBG:
 		embed_set_image_as_background (embed, info->img);
@@ -488,7 +497,12 @@ static void context_do_action (GaleonContextAction action,
 		mozilla_reload (embed);
 		break;
 	case GALEON_CONTEXT_COPYFRAMELOC:
-		copy_to_clipboard (mozilla_get_document_url (embed), embed);
+		url = mozilla_get_document_url (embed);
+		if (url)
+		{
+			copy_to_clipboard (url, embed);
+			g_free (url);
+		}
 		break;
 	case GALEON_CONTEXT_ADDTEMPBMFRAME:
 		url = mozilla_get_document_url (embed);
@@ -512,7 +526,8 @@ static void context_do_action (GaleonContextAction action,
 /**
  * context_show_menu: show appropriate context menu based on the target type
  **/
-void context_show_menu (GaleonEmbed *embed, WrapperContextInfo *info)
+void context_show_menu (GaleonEmbed *embed, WrapperContextInfo *info,
+			int button, guint timestamp)
 {
 	GnomeUIInfo *menu;
 
@@ -521,7 +536,8 @@ void context_show_menu (GaleonEmbed *embed, WrapperContextInfo *info)
 	
 	if (menu != NULL)
 	{
-		guint result = context_show_context_menu (embed, menu);
+		guint result = context_show_context_menu (embed, menu, button,
+							  timestamp);
 		if (result != -1)
 		{
 			GaleonContextAction action = 
@@ -551,15 +567,120 @@ void context_show_bookmark_menu (GaleonEmbed *embed)
 					   NULL, tips, TRUE, TRUE);
 	
 	/* warning: ugly code */
-	gtk_object_set_data (GTK_OBJECT (menu), "WMain", (window->WMain)); 
-	gtk_object_set_data (GTK_OBJECT (window->WMain), "bm_context_menu",
-			     menu);
+	gtk_object_set_data (GTK_OBJECT (menu), "GaleonWindow", window); 
 	gnome_popup_menu_do_popup_modal (menu, NULL, NULL,
 					 NULL , NULL);
+	gtk_widget_unref (menu);
+	gtk_object_destroy (GTK_OBJECT(tips));
+}
 
-	gtk_object_remove_data (GTK_OBJECT (window->WMain), "bm_context_menu");
-	gtk_widget_destroy (menu);
-	gtk_object_destroy (GTK_OBJECT (tips));
+/**
+ * context_show_appearance_menu: creates a submenu with
+ * different toolbar styles
+ */
+void
+context_show_appearance_menu (GaleonWindow *window, GtkMenu *menu, 
+			      BookmarkItem *bi, gboolean is_button, 
+			      gboolean in_submenu)
+{
+	GtkMenu *appearance_menu;
+	GtkWidget *appearance_menu_item;
+	GSList *icons_radiogroup = NULL;
+	GSList *radiogroup = NULL;
+	ToolbarStyle style;
+	ToolbarIcons icons;
+
+	if (in_submenu)
+	{
+		appearance_menu = GTK_MENU (gtk_menu_new ());
+		appearance_menu_item = gtk_menu_item_new_with_label ("");
+		label_set_accel_text (_("_Appearance"),
+				      GTK_BIN (appearance_menu_item)->child, 
+				      GTK_WIDGET (menu),
+				      appearance_menu_item);
+		gtk_menu_item_set_submenu (GTK_MENU_ITEM (appearance_menu_item),
+					   GTK_WIDGET (appearance_menu));
+		gtk_menu_append (menu, GTK_WIDGET (appearance_menu_item));
+		gtk_widget_show (GTK_WIDGET (appearance_menu_item));
+	}
+	else
+	{
+		appearance_menu = menu;
+	}
+
+	if (gnome_preferences_get_menus_have_tearoff ())
+	{
+		GtkWidget *tearoff = gtk_tearoff_menu_item_new ();
+		gtk_menu_prepend (appearance_menu, tearoff);
+		gtk_widget_show (tearoff);
+	}
+
+	if (bi)
+	{
+		/* we're on a bookmark toolbar */
+		
+		if (is_button)
+			bi = bi->parent;
+
+		style = bi->toolbar_style;
+
+		context_menu_add_radio_item 
+			(appearance_menu, _("Text _beside icons"),
+			 context_bm_toolbar_appearance_menu_cb, bi,
+			 (style == TOOLBAR_STYLE_HORIZONTAL), &radiogroup);
+		context_menu_add_radio_item 
+			(appearance_menu, _("Text _under icons"),
+			 context_bm_toolbar_appearance_menu_cb, bi,
+			 (style == TOOLBAR_STYLE_VERTICAL), &radiogroup);
+		context_menu_add_radio_item 
+			(appearance_menu, _("_Text only"),
+			 context_bm_toolbar_appearance_menu_cb, bi,
+			 (style == TOOLBAR_STYLE_TEXT_ONLY), &radiogroup);
+	}
+	else
+	{
+		/* we're on the main toolbar */
+
+		style = gnome_config_get_int (CONF_APPEARANCE_TOOLBAR_STYLE);
+		context_menu_add_radio_item 
+			(appearance_menu, _("Text _beside icons"),
+			 context_main_toolbar_appearance_menu_cb, NULL,
+			 (style == TOOLBAR_STYLE_HORIZONTAL), &radiogroup);
+		context_menu_add_radio_item 
+			(appearance_menu, _("Text _under icons"),
+			 context_main_toolbar_appearance_menu_cb, NULL,
+			 (style == TOOLBAR_STYLE_VERTICAL), &radiogroup);
+		context_menu_add_radio_item 
+			(appearance_menu, _("_Text only"),
+			 context_main_toolbar_appearance_menu_cb, NULL,
+			 (style == TOOLBAR_STYLE_TEXT_ONLY), &radiogroup);
+		context_menu_add_radio_item 
+			(appearance_menu, _("_Icons only"),
+			 context_main_toolbar_appearance_menu_cb, NULL,
+			 (style == TOOLBAR_STYLE_ICONS_ONLY), &radiogroup);
+
+		context_menu_add_seperator (appearance_menu);
+
+		icons = gnome_config_get_int (CONF_APPEARANCE_TOOLBAR_ICONS);
+		context_menu_add_radio_item 
+			(appearance_menu, _("_Default icons"),
+			 context_main_toolbar_icons_menu_cb, NULL,
+			 (icons == TOOLBAR_ICONS_NORMAL), &icons_radiogroup);
+		context_menu_add_radio_item 
+			(appearance_menu, _("_Large icons"),
+			 context_main_toolbar_icons_menu_cb, NULL,
+			 (icons == TOOLBAR_ICONS_LARGE), &icons_radiogroup);
+		context_menu_add_radio_item 
+			(appearance_menu, _("_Nautilus icons"),
+			 context_main_toolbar_icons_menu_cb, NULL,
+			 (icons == TOOLBAR_ICONS_NAUTILUS), &icons_radiogroup);
+
+		context_menu_add_seperator (appearance_menu);
+
+		context_menu_add_item (appearance_menu, _("_Edit toolbar..."),
+				       context_main_toolbar_edit_toolbar_cb,
+				       window, GNOME_STOCK_MENU_PROP, TRUE);
+	}
 }
 
 /**
@@ -570,6 +691,7 @@ static void
 context_menu_add_bookmarks (GtkMenu *menu, GaleonEmbed *embed)
 {
 	GtkTooltips *tips = gtk_tooltips_new ();
+	GtkWidget *new_item;
 	GaleonWindow *window;
 	BookmarkItem *fake_bookmarks_root = 
 		context_menu_create_fake_bookmarks_root ();
@@ -578,7 +700,8 @@ context_menu_add_bookmarks (GtkMenu *menu, GaleonEmbed *embed)
 	window = embed->parent_window;
 	return_if_not_window (window);
 
-	gtk_menu_append (menu, gtk_menu_item_new ());
+	new_item = gtk_menu_item_new ();
+	gtk_menu_append (menu, new_item);
 
 	bookmarks_menu_create_recursively (fake_bookmarks_root, 
 					   menu, NULL, tips, FALSE, TRUE);
@@ -586,34 +709,53 @@ context_menu_add_bookmarks (GtkMenu *menu, GaleonEmbed *embed)
 	bookmarks_free_bookmark (fake_bookmarks_root);
 	
 	/* warning: ugly code */
-	gtk_object_set_data (GTK_OBJECT (menu), "WMain", (window->WMain)); 
-	gtk_object_set_data (GTK_OBJECT (window->WMain), "bm_context_menu",
-			     menu);
-
+	gtk_object_set_data (GTK_OBJECT (menu), "GaleonWindow", window); 
 	gtk_signal_connect_object (GTK_OBJECT (menu), "destroy", 
-				   gtk_object_destroy, GTK_OBJECT (tips));
-	gtk_signal_connect (GTK_OBJECT (menu), "destroy", 
-			    context_menu_bookmarks_cleanup, window);
+				   gtk_object_unref, GTK_OBJECT (tips));
 }
 
 /**
- * Do some cleanups before destroying the context menu
+ * fill_in_image_context_menu: fill in the image part
+ * of the context menu with the image name
  */
+/* FIXME LEAAAAAAKKKK ! */
 static void
-context_menu_bookmarks_cleanup (GtkObject *o, GaleonWindow *window)
+fill_in_image_context_menu (gchar *imagename, gint pos,
+			GnomeUIInfo *uiinfo)
 {
-	gtk_object_remove_data (GTK_OBJECT (window->WMain), "bm_context_menu");
+	gchar *name, *s;
+	gint length;
+
+	g_assert (imagename != NULL);
+	
+	s = g_strdup (g_basename (imagename));
+	length = strlen (s);
+	
+	/* shorten the name if needed*/
+	if (length > 40)
+	{
+		gchar *tmp = g_strdup_printf ("%.29s...%s", s, s + length - 8);
+		g_free (s);
+		s = tmp;
+	}
+	
+	name = escape_uline_accel (s);
+	uiinfo[pos].label = 
+		g_strdup_printf (_("Open image (%s)"), name);
+
+	g_free (s);
+	g_free (name);
 }
 
 /**
- * Create a fake bookmark category that haves all the bookmarks that need to be
+ * Create a fake bookmark folder that haves all the bookmarks that need to be
  * addeed to context menus
  */
 static BookmarkItem *
 context_menu_create_fake_bookmarks_root (void)
 {
 	BookmarkItem *r = bookmarks_new_bookmark 
-		(BM_CATEGORY, TRUE, "fake bookmark", NULL, NULL, NULL, NULL);
+		(BM_FOLDER, TRUE, "fake bookmark", NULL, NULL, NULL, NULL);
 	context_menu_create_fake_bookmarks_root_recursively 
 		(bookmarks_root, &r->list);
 	return r;
@@ -631,3 +773,80 @@ context_menu_create_fake_bookmarks_root_recursively (BookmarkItem *b,
 			context_menu_create_fake_bookmarks_root_recursively
 				(li->data, l);
 }
+
+
+void
+context_menu_add_seperator (GtkMenu *extra_menu)
+{
+	GtkWidget *separator = gtk_menu_item_new();
+	gtk_menu_append (extra_menu, separator);
+	gtk_widget_show (separator);
+}
+
+void
+context_menu_add_item (GtkMenu *extra_menu, const gchar *text,
+		       GtkSignalFunc cb_func, void *cb_data, 
+		       const char * icon, gboolean sensitive)
+{
+	GtkWidget *item;
+	GtkWidget *hb = gtk_hbox_new (FALSE, 0);
+	GtkWidget *label = gtk_label_new ("");
+
+	if (icon)
+	{
+		item = gtk_pixmap_menu_item_new();
+		gtk_pixmap_menu_item_set_pixmap (GTK_PIXMAP_MENU_ITEM (item),
+			gnome_stock_pixmap_widget (GTK_WIDGET (extra_menu),
+				icon));
+	}
+	else
+	{
+		item = gtk_menu_item_new();	
+	}
+
+	label_set_accel_text (text, label,
+			      GTK_WIDGET (extra_menu), item);
+	gtk_box_pack_start (GTK_BOX (hb), label, FALSE, FALSE, 2);
+	gtk_container_add (GTK_CONTAINER(item), hb);
+
+	gtk_widget_set_sensitive (item, sensitive);
+
+	gtk_signal_connect (GTK_OBJECT (item), "activate",
+			GTK_SIGNAL_FUNC (cb_func), cb_data);
+	gtk_menu_append (extra_menu, item);
+	gtk_widget_lock_accelerators (item);
+	gtk_widget_show_all (item);
+}
+
+void
+context_menu_add_check_item (GtkMenu *extra_menu, const gchar *text,
+				GtkSignalFunc cb_func, void *cb_data,
+				gboolean checked)
+{
+	GtkWidget *item = gtk_check_menu_item_new_with_label ("");
+	label_set_accel_text (text, GTK_BIN (item)->child,
+			      GTK_WIDGET (extra_menu), item);
+	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item), checked);
+	gtk_signal_connect (GTK_OBJECT (item), "activate",
+			GTK_SIGNAL_FUNC (cb_func), cb_data);
+	gtk_menu_append (extra_menu, item);
+	gtk_widget_lock_accelerators (item);
+	gtk_widget_show (item);
+}
+
+void
+context_menu_add_radio_item (GtkMenu *extra_menu, const gchar *text,
+				GtkSignalFunc cb_func, void *cb_data,
+				gboolean checked, GSList **group)
+{
+	GtkWidget *item = gtk_radio_menu_item_new_with_label (*group, "");
+	*group = gtk_radio_menu_item_group (GTK_RADIO_MENU_ITEM (item));
+	label_set_accel_text (text, GTK_BIN (item)->child,
+			      GTK_WIDGET (extra_menu), item);
+	GTK_CHECK_MENU_ITEM (item)->active = checked;
+	gtk_signal_connect (GTK_OBJECT (item), "activate",
+			    GTK_SIGNAL_FUNC (cb_func), cb_data);
+	gtk_menu_append (extra_menu, item);
+	gtk_widget_lock_accelerators (item);
+	gtk_widget_show (item);
+}
