@@ -28,6 +28,9 @@
 /* FIXME Remove these */
 #include <gtk/gtktooltips.h>
 #include <gtk/gtkctree.h>
+#include <gtk/gtkcheckmenuitem.h>
+#include <gtk/gtkmenuitem.h>
+#include <gtk/gtkdnd.h>
 
 /* forward declarations of types */
 typedef struct _GaleonWindow GaleonWindow;
@@ -36,6 +39,26 @@ typedef struct _BookmarkItem BookmarkItem;
 typedef struct _HistoryHost  HistoryHost;
 typedef struct _HistoryItem  HistoryItem;
 
+
+/**
+ * FocusType: what caused the switch to a embed object
+ */
+typedef enum
+{
+	FOCUS_ON_CREATE,	/* auto_jump or opened via menu */
+	FOCUS_ON_REQUEST	/* user explicitly switched (clicked on tab) */
+} FocusType;
+
+/**
+ * DockType: type of dock
+ */
+typedef enum
+{
+	DOCK_NONE,
+	DOCK_BOOKMARKS,
+	DOCK_HISTORY
+} DockType;
+
 /**
  * GaleonEmbed: this is the type of a single web view (e.g. a tab):
  * this is effectively Galeon's wrapping of a GtkMozEmbed with extra
@@ -52,14 +75,17 @@ struct _GaleonEmbed
 	GtkWidget *mozEmbed;
 
 	/* tab widgets */
-	GtkWidget *notebook_close_button;
-	GtkWidget *notebook_close_pixmap;
+	GtkWidget *notebook_event_box;           /* overall event box */
+	GtkWidget *notebook_hbox;                /* packing box       */
+	GtkWidget *notebook_label;               /* the label         */
+	GtkWidget *notebook_close_button;        /* the button        */
 
 	/* C++ helper, a "GaleonWrapper" */
         gpointer wrapper;
 
 	/* current site location and page title, if any */
 	gchar *site_location;
+	gchar *site_title_utf8;
 	gchar *site_title;
 
 	gint load_started; /* count of currently active connections */
@@ -83,14 +109,20 @@ struct _GaleonEmbed
 	/* is this browser actually visible */
 	gboolean is_visible : 1;
 
-	/* is this a popup browser ? */
-	gboolean is_popup : 1;
-
 	/* is this embed in the process of closing? */
 	gboolean is_closing : 1;
 
 	/* ignore mime processing of next url */
 	gboolean mime_ignore_once : 1;
+
+	/* has this browser content been viewed yet? */
+	gboolean has_been_viewed : 1;
+
+	/* is this a xul dialog ? */
+	gboolean is_chrome : 1;
+
+	/* how this embed was focused if applicable */
+	FocusType focus_type;
 };
 
 /**
@@ -130,33 +162,51 @@ struct _GaleonWindow
 	const gchar *docked_width_pref;   /* where width of dock is stored */
 
 	/* Toolbar components */
-	GtkWidget *main_toolbar;
-	GtkWidget *spinner; /* the GnomeAnimator spinner */
-	GtkWidget *location_dockitem;
-	GtkWidget *main_dockitem;
-	GtkWidget *BNew;
-	GtkWidget *BBack;
-	GtkWidget *BUp;
-	GtkWidget *BForward;
-	GtkWidget *BRefresh;
-	GtkWidget *BStop;
-	GtkWidget *BStart;
-	GtkWidget *zoom_spin;
-	GtkWidget *drag_event_box;
-	GtkWidget *toolbar_gnomeentry;
-	GtkWidget *toolbar_entry;
-	GtkWidget *go_toolbar;
 	GtkWidget *dock;
-
 	GtkWidget *appbar;
 	GtkWidget *menubar;
+	GtkWidget *main_toolbar;
+	GtkWidget *main_dockitem;
+	GtkWidget *right_toolbar;
+	GtkWidget *location_toolbar;
+	GtkWidget *location_dockitem;
+
+	/* Toolbar buttons etc */
+	GtkWidget *new_button;
+	GtkWidget *back_button;
+	GtkWidget *up_button;
+	GtkWidget *forward_button;
+	GtkWidget *refresh_button;
+	GtkWidget *home_button;
+	GtkWidget *stop_button;
+	GtkWidget *go_button;
+	GtkWidget *find_button;
+	GtkWidget *zoom_spin;
+	GtkWidget *drag_event_box;
+	GtkWidget *drag_pixmap;
+	GtkWidget *location_gnomeentry;
+	GtkWidget *location_entry;
+	GtkWidget *spinner;               /* the GnomeAnimator spinner */
+	GtkWidget *history_button;
+	GtkWidget *bookmarks_button;
+	
+	/* menus */
+	GtkWidget *file_menu;
+	GtkWidget *bookmarks_menu;
+	GtkWidget *go_menu;
+
+	/* menu items */
+	GtkWidget *file_session_history_separator_sup;
+	GtkWidget *file_session_history_separator_inf;
 	GtkWidget *view_toolbar;
 	GtkWidget *view_statusbar;
 	GtkWidget *view_menubar;
 	GtkWidget *view_fullscreen;
-	GtkWidget *view_bookmarks_dock;
-	GtkWidget *view_history_dock;
-	GtkWidget *drag_pixmap;
+	GtkWidget *detach_tab;
+	GtkWidget *next_tab;
+	GtkWidget *prev_tab;
+	GtkWidget *move_tab_left;
+	GtkWidget *move_tab_right;
 	GtkWidget *load_images_always;
 	GtkWidget *load_images_from_current_server_only;
 	GtkWidget *load_images_never;
@@ -168,10 +218,19 @@ struct _GaleonWindow
 	GtkWidget *enable_java;
 	GtkWidget *enable_javascript;
 	GtkWidget *enable_proxy;
+	GtkWidget *autojump;
+	GtkWidget *add_bookmark;
+	GtkWidget *bookmarks_separator;
+	GtkWidget *encoding;
+	/* edit menu menuitems */
+	GtkWidget *edit_cut;
+	GtkWidget *edit_copy;
+	GtkWidget *edit_paste;
 
 	/* dialogs and views */
 	GtkWidget *print_dialog;
 	GtkWidget *find_dialog;
+	GtkWidget *open_url_entry;
 
 	/* list of toolbars */
 	GList *bookmarks_toolbars; 
@@ -181,10 +240,16 @@ struct _GaleonWindow
 	gint x, y, width, height;
 
 	/* visibility */
-	gboolean menuBarOn;
-	gboolean toolBarOn;
-	gboolean locationBarOn;
-	gboolean statusBarOn;
+	gboolean menuBarOn : 1;
+	gboolean toolBarOn : 1;
+	gboolean locationBarOn : 1;
+	gboolean statusBarOn : 1;
+
+	/* is this a popup window ? */
+	gboolean is_popup : 1;
+
+	/* dock type */
+	DockType dock_type;
 
 	/* saved layout */
 	gchar *layout;
@@ -429,11 +494,9 @@ typedef enum
 typedef enum
 {
 	MIME_UNKNOWN,
-	MIME_SAVE_WITH_GTM,
 	MIME_SAVE_TO_DISK,
 	MIME_RUN_PROGRAM,      /* should run the GNOME association */
 	MIME_ASK_ACTION,
-	MIME_OVERRIDE_MOZILLA,
 	MIME_IGNORE
 } MimeAction;
 
@@ -464,6 +527,7 @@ typedef enum
 typedef enum
 {
 	NS_SITE,
+	NS_NOTES,
 	NS_FOLDER,
 	NS_FOLDER_END,
 	NS_SEPARATOR,
@@ -490,16 +554,6 @@ typedef enum
 	ANIMATE_NEVER        = 2
 } AnimateType;
 
-/**
- * TabbedStatus: status of a tab (reflected in its colour)
- */
-typedef enum
-{
-	TAB_NORMAL,
-	TAB_NEW,
-	TAB_LOADING
-} TabbedStatus;
-
 /**
  * ViewState: destination for loading URLs
  */
@@ -511,6 +565,20 @@ typedef enum
 	VIEWSTATE_TAB
 } ViewState;
 
+/**
+ * LinkState: generalized URL activation types
+ */
+typedef enum
+{
+	LINKSTATE_BUTTON_MASK = 7,
+	LINKSTATE_SHIFTED     = (1 << 3),
+	LINKSTATE_CTRLED      = (1 << 4),
+	LINKSTATE_ALTED       = (1 << 5),
+	/* force a certain mode rather than basing off the pref */
+	LINKSTATE_NEWTAB      = 1 + (1 << 9),
+	LINKSTATE_NEWWIN      = 1 + (1 << 10),
+} LinkState;
+
 /**
  * ContextMenuType: various types of context menu
  */
@@ -522,7 +590,6 @@ typedef enum
 	CONTEXT_IMAGE    = (1 << 3),
 	CONTEXT_DOCUMENT = (1 << 4),
 	CONTEXT_INPUT    = (1 << 5),
-	CONTEXT_OTHER    = (1 << 6),
 	CONTEXT_XUL      = (1 << 7),
 } ContextMenuType;
 
@@ -533,6 +600,10 @@ typedef struct
 {
 	gchar *name; 
 	MimeAction action;
+	gchar *helperApp;
+	gboolean terminalHelper;
+	gboolean urlHelper;
+	gboolean alwaysUse;
 } MimeItem;
 
 /**
@@ -542,35 +613,64 @@ typedef struct
 {
 	GtkWidget *dialog;
 	GtkWidget *notebook;
-	MimeItem *selected_mime;
-	gboolean enable_edit_mime;
-	GtkWidget *hpaned;
-	GtkWidget *menu_ctree;
+	GtkWidget *sidebar;
 	GtkWidget *themes_page;
+	GtkWidget *spinner_page;
 	GtkWidget *iconlist;
 	GtkWidget *smart_bm_example_entry;
-	GtkWidget *mime_list;
-	GtkWidget *mime_action_optionmenu;
-	GtkWidget *mime_name_entry;
-	GtkWidget *scrolledwindow9;/*FIXME*/
 	GtkWidget *lang_entry;/*FIXME*/
-	GtkWidget *autodownload_checkbutton;
-	GtkWidget *ask_download_dir_checkbutton;
-	GtkWidget *location_optionmenu;
-	GtkWidget *toolbar_optionmenu;
+	GtkWidget *toolbar_style_optionmenu;
+	GtkWidget *toolbar_share_main_toolbar;
+	GtkWidget *toolbar_use_default_icons;
 	GtkWidget *startpage_entry;
-	GtkWidget *startpage_optionmenu;
-	GtkWidget *new_optionmenu;
-	GtkWidget *bookmarks_optionmenu;
-	GtkWidget *bm_extra_items_optionmenu;
+	GtkWidget *startup_show_homepage;
+	GtkWidget *newpage_show_homepage;
+	GtkWidget *always_save_session_check;
+	GtkWidget *bookmarks_use_page_title;
+	GtkWidget *bookmarks_show_actions_top;
 	GtkWidget *favicons_enabled_check;
 	GtkWidget *autobookmarks_enabled_check;
 	GtkWidget *autobookmarks_count_spin;
 	GtkWidget *autobookmarks_shorten_spin;
+	GtkWidget *bookmarks_backup_enabled_check;
+	GtkWidget *bookmarks_backup_count_spin;
 	GtkWidget *smart_bm_entry_width_spin;
 	GtkWidget *java_checkbutton;
 	GtkWidget *javascript_checkbutton;
+
+	/* toolbar editor */
+	GtkWidget *add_button;
+	GtkWidget *remove_button;
+	GtkWidget *up_button;
+	GtkWidget *down_button;
+	GtkWidget *available_clist;
+	GtkWidget *current_clist;
+
+	/* mime page */
+	MimeItem *selected_mime;
+	gboolean enable_edit_mime;
+	gint mime_last_sort_column;
+	GtkSortType mime_last_sort_type;
+	GtkWidget *mime_list;
+	GtkWidget *mime_save_to_disk;
+	GtkWidget *mime_helper_frame;
+	GtkWidget *mime_name_entry;
+	GtkWidget *mime_helper_entry;
+	GtkWidget *mime_always_use;
+	GtkWidget *mime_terminal;
+	GtkWidget *mime_url_helper;
+
+	/* downloader page */
+	GtkWidget *download_use_builtin;
+	GtkWidget *download_command_entry;
+	GtkWidget *download_command_combo;
+	GtkWidget *download_run_in_terminal;
+	GtkWidget *download_autodownload;
+	GtkWidget *download_ask_dir;
 	GtkWidget *download_dir_entry;
+	GtkWidget *download_ftp_use_builtin;
+
+	/* tabbed browsing page */
 	GtkWidget *tabbed_enable_check;
 	GtkWidget *tabbed_popups_enable_check;
 	GtkWidget *tabbed_autojump_check;
@@ -584,10 +684,10 @@ typedef struct
 	GtkWidget *tabbed_separator_check;
 	GtkWidget *tabbed_vowel_check;
 	GtkWidget *tabbed_prefix_check;
+
+	/* ... */
 	GtkWidget *autocompletion_enabled_checkbutton;
-	GtkWidget *completion_sort_optionmenu;
-	GtkWidget *show_zoom_check;
-	GtkWidget *show_spinner_check;
+	GtkWidget *autocompletion_sort_whole;
         GtkWidget *proxy_use_checkbutton;
 	GtkWidget *http_proxy_entry;
 	GtkWidget *ssl_proxy_entry;
@@ -601,20 +701,14 @@ typedef struct
 	GtkWidget *lang_encode;
 	GtkWidget *serif_font_fpicker;
 	GtkWidget *sansserif_font_fpicker;
+	GtkWidget *cursive_font_fpicker;
+	GtkWidget *fantasy_font_fpicker;
 	GtkWidget *monospace_font_fpicker;
-	GtkWidget *size_font_spin;
+	GtkWidget *minimum_font_size_spin;
 	GtkWidget *default_font_optionmenu;
 	GtkWidget *history_expire_spin;
 	GtkWidget *use_own_fonts_checkbutton;
-	GtkWidget *image_optionmenu;
-	GtkWidget *ftp_optionmenu;
-	GtkWidget *ftp_program_optionmenu;
-	GtkWidget *ftp_command_entry;
-	GtkWidget *show_go_button_check;
-	GtkWidget *show_up_button_check;
-	GtkWidget *show_new_button_check;
-	GtkWidget *show_home_button_check;
-	GtkWidget *show_drag_event_box_check;
+	GtkWidget *images_always_load;
 	GtkWidget *show_toolbars_in_fullscreen_check;
 	GtkWidget *show_menubar_in_fullscreen_check;
 	GtkWidget *show_statusbar_in_fullscreen_check;
@@ -642,7 +736,7 @@ typedef struct
 	GtkWidget *history_popup_checkbutton;
 	GtkWidget *middle_button_optionmenu;
 	GtkWidget *right_button_optionmenu;
-	GtkWidget *cookie_behavior_optionmenu;
+	GtkWidget *cookies_always_accept;
 	GtkWidget *cookie_manager_button;
 	GtkWidget *warn_cookie_checkbutton;
 	GtkWidget *passwords_save_checkbutton;
@@ -661,7 +755,6 @@ typedef struct
 	GtkWidget *crash_behaviour_optionmenu;
 	GtkWidget *links_drag_checkbutton;
 	GtkWidget *cache_compare_optionmenu;
-	GtkWidget *nautilus_icons_checkbutton;
 } PreferencesDialog;
 
 /**
@@ -693,7 +786,8 @@ struct _HistoryHost
 {
 	/* NB: these must stay the same as HistoryItem */
 	gchar *url; /* NOTE: unused! */
-	gchar *title;
+	gchar *title_locale;
+	gchar *title_utf8;
 	GTime first;
 	GTime last;
 	gint visits;
@@ -716,7 +810,8 @@ struct _HistoryItem
 {
 	/* NB: these must stay the same as HistoryItem */
 	gchar *url;
-	gchar *title;
+	gchar *title_locale;
+	gchar *title_utf8;
 	GTime first;
 	GTime last;
 	gint visits;
@@ -725,10 +820,66 @@ struct _HistoryItem
 	HistoryHost *host;
 };
 
+/**
+ * ToolbarItemType: various toolbar item types
+ */
+typedef enum
+{
+	TOOLBAR_ITEM_NEW,
+	TOOLBAR_ITEM_BACK,
+	TOOLBAR_ITEM_UP,
+	TOOLBAR_ITEM_FORWARD,
+	TOOLBAR_ITEM_REFRESH,
+	TOOLBAR_ITEM_START,
+	TOOLBAR_ITEM_STOP,
+	TOOLBAR_ITEM_GO,
+	TOOLBAR_ITEM_ZOOM_SPIN,
+	TOOLBAR_ITEM_DRAG_HANDLE,
+	TOOLBAR_ITEM_LOCATION_ENTRY,
+	TOOLBAR_ITEM_SPINNER,
+	TOOLBAR_ITEM_SEPARATOR,
+	TOOLBAR_ITEM_FIND,
+	TOOLBAR_ITEM_BOOKMARKS,
+	TOOLBAR_ITEM_HISTORY,
+	TOOLBAR_ITEM_PRINT,
+
+	/* must be last */
+	TOOLBAR_ITEM_COUNT
+} ToolbarItemType;	
+
+/**
+ * ToolbarItem: a toolbar item
+ */
+typedef struct
+{
+	const gchar *label;
+	const gchar *tooltip;
+	const gchar *stock_icon;
+	const gchar *nautilus_icon;
+	gboolean priority; /* i.e. show the text in horizontal mode */
+	gboolean togglebutton;
+	gint order;        /* order value for default insertion place */
+	void (*clicked_cb) (GtkButton *button, 
+			    GaleonWindow *window);
+	gint (*button_press_event_cb) (GtkButton *button, 
+				       GdkEventButton *event,
+				       GaleonWindow *window);
+	gint (*button_release_event_cb) (GtkButton *button, 
+				         GdkEventButton *event,
+				         GaleonWindow *window);
+	void (*drag_data_received_cb) (GtkWidget *widget, 
+				       GdkDragContext *drag_context,
+				       gint x, gint y,
+				       GtkSelectionData *selection_data,
+				       guint info, guint time,
+				       GaleonWindow *window);
+} ToolbarItem;
+
 /**
  * PixmapData: a single pixmap (stored on the X server)
  */
-typedef struct {
+typedef struct
+{
 	GdkPixmap *pixmap;
 	GdkBitmap *mask;
 } PixmapData;
@@ -738,12 +889,33 @@ typedef struct {
  */ 
 typedef enum
 {
-	BM_CATEGORY,
+	BM_FOLDER,
 	BM_SITE,
 	BM_SEPARATOR,
 	BM_AUTOBOOKMARKS
 } BookmarkType;
 
+/**
+ * ToolbarStyle: various toolbar styles
+ */
+typedef enum
+{
+	TOOLBAR_STYLE_HORIZONTAL,
+	TOOLBAR_STYLE_VERTICAL,
+	TOOLBAR_STYLE_TEXT_ONLY,
+	TOOLBAR_STYLE_ICONS_ONLY,       /* not used for bookmark toolbars */
+} ToolbarStyle;
+
+/**
+ * ToolbarIcons: the different main toolbar types
+ */
+typedef enum
+{
+	TOOLBAR_ICONS_NORMAL,
+	TOOLBAR_ICONS_LARGE,
+	TOOLBAR_ICONS_NAUTILUS
+} ToolbarIcons;
+
 /**
  * Forward declaration
  */
@@ -782,7 +954,7 @@ struct _BookmarkItem
 	/* this is a list of BookmarkTreeItem */
 	GList *tree_items;
 
-	/* parent category (NULL for root bookmark) */
+	/* parent folder (NULL for root bookmark) */
 	BookmarkItem *parent; 
 
 	/* if TRUE and this is a folder, create a toolbar from it */
@@ -810,13 +982,26 @@ struct _BookmarkItem
 	GTime time_modified;
 	GTime time_visited;
 
+	/* toolbar style */
+	ToolbarStyle toolbar_style;
 };
 
+/**
+ * BookmarksEditorControlsType: types of bookmark editor dialogs
+ */
+typedef enum
+{
+	STANDARD_BOOKMARKS_EDITOR,
+	DOCKED_BOOKMARKS_EDITOR
+} BookmarksEditorControlsType;
+
 /**
  * BookmarksEditorControls: representation of a bookmarks editor.
  */
 typedef struct
 {
+	BookmarksEditorControlsType type;
+
 	/* the root folder being edited here */
 	BookmarkItem *root_bookmark;
 
@@ -846,6 +1031,11 @@ typedef struct
 	gboolean lock_edit_controls; /* Don't update bookmark when editing 
 				      * fields. Set to TRUE to avoid recursion
 				      */
+	/* widgets we need to update at various stages*/
+	GtkWidget *edit_menuitem;
+	GtkWidget *edit_button;
+
+	GtkWidget *set_default_folder_menuitem;
 	
 	GaleonWindow *window;
 
@@ -857,31 +1047,35 @@ typedef struct
 /**
  * Pair of controls / tree_item for a bookmark
  */
-struct _BookmarkTreeItem {
+struct _BookmarkTreeItem
+{
 	BookmarksEditorControls *controls;
 	GtkCTreeNode *tree_item; 	
 };
 
 /**
- * BookmarksEditorControls_type types of bookmark editor dialogs
+ * The "folder extra items" actions
  */
-typedef enum {
-	STANDARD_BOOKMARKS_EDITOR,
-	DOCKED_BOOKMARKS_EDITOR,
-	COMPACT_BOOKMARKS_EDITOR,
-} BookmarksEditorControls_type;		
+typedef enum
+{
+	EXTRA_ITEM_LOCATION_EXCLUSIVE = -1,
+	EXTRA_ITEM_LOCATION_TOP_SUBMENU = 0,
+	EXTRA_ITEM_LOCATION_BOTTOM_SUBMENU,
+	EXTRA_ITEM_LOCATION_CONTEXT,
+} ExtraItemsLocation;
 
 /**
- * BookmarksDnDTargetType: various types that can be dropped
+ * DnDType: various types that can be dragged and dropped
  */
-enum BookmarksDnDTargetType
+typedef enum
 {
 	DND_TARGET_GALEON_BOOKMARK, 
 	DND_TARGET_GALEON_URL,
 	DND_TARGET_NETSCAPE_URL, 
 	DND_TARGET_TEXT_URI_LIST,
-	DND_TARGET_STRING
-};
+	DND_TARGET_STRING,
+	DND_TARGET_GALEON_EMBED
+} DnDType;
 
 /**
  * CookieBase: FIXME?
@@ -891,6 +1085,7 @@ typedef struct
 	gchar *type;
 	gchar *domain;
 	gboolean remove;
+	gboolean block;
 } CookieBase;
 
 /**
@@ -909,7 +1104,6 @@ typedef struct
 /**
  * Password: a password manager entry
  */
-
 typedef struct
 {
 	gchar *host;
@@ -927,7 +1121,7 @@ typedef enum
 } PasswordType;
 
 /**
- * CookiePage: For selecting which page in the Cookie manager is active
+ * PDMPage: For selecting which page in the persistent data manager is active
  */
 typedef enum
 {
@@ -936,7 +1130,7 @@ typedef enum
 	IMAGESITES = 2,
 	PASSWORDS = 3,
 	PASSWORDSITES = 4
-} CookiePage;
+} PDMPage;
 
 /**
  * WheelModKey: modifiers that can be used in conjuction with the mouse wheel
@@ -949,6 +1143,40 @@ typedef enum
 	WHEEL_MOD_SHIFT
 } WheelModKey;
 
+/**
+ * Downloader: the choice between the builtin (mozilla) downloader, the
+ * external GTM downloader (via CORBA), or using some external command.
+ */
+typedef enum
+{
+	DOWNLOADER_BUILTIN  = 0,
+	DOWNLOADER_GTM      = 1,
+	DOWNLOADER_EXTERNAL = 2
+} Downloader;
+
+/**
+ * PreferencesPageId: different pages of the prefs dialog
+ */
+typedef enum
+{
+	PREFS_PAGE_ID_BROWSING_GENERAL = 1,
+	PREFS_PAGE_ID_BROWSING_BOOKMARKS,
+	PREFS_PAGE_ID_BROWSING_HISTORY,
+	PREFS_PAGE_ID_USER_INTERFACE_TABS,
+	PREFS_PAGE_ID_USER_INTERFACE_WINDOWS,
+	PREFS_PAGE_ID_USER_INTERFACE_TOOLBARS,
+	PREFS_PAGE_ID_USER_INTERFACE_MOUSE,
+	PREFS_PAGE_ID_HANDLERS_DOWNLOADING,
+	PREFS_PAGE_ID_HANDLERS_PROGRAMS,
+	PREFS_PAGE_ID_HANDLERS_MIME_TYPES,
+	PREFS_PAGE_ID_RENDERING_FONTS_COLORS,
+	PREFS_PAGE_ID_RENDERING_LANGUAGES,
+	PREFS_PAGE_ID_ADVANCED_NETWORK,
+	PREFS_PAGE_ID_ADVANCED_FILTERING,
+	PREFS_PAGE_ID_ADVANCED_PERSISTENT_DATA,
+	PREFS_PAGE_ID_ADVANCED_CRASH_RECOVERY
+} PreferencesPageId;
+
 /**
  * FileCallbackFunc: the type of functions used in file callbacks
  */
