@@ -21,9 +21,11 @@
 #include "history.h"
 #include "window.h"
 #include "bookmarks.h"
-#include "auto_completion.h"
+#include "autocompletion.h"
 #include "misc.h"
 #include "state.h"
+#include "glade.h"
+#include "dialog.h"
 
 #include <string.h>
 #include <gtk/gtkoptionmenu.h>
@@ -41,8 +43,6 @@
 #include <libgnomevfs/gnome-vfs.h>
 #include <libgnomevfs/gnome-vfs-mime.h>
 
-extern const GtkTargetEntry url_drag_types [];
-
 /**
  * The history hash table. A hash table is used because you have to make 
  * an url lookup in history each time you try to add or update a record
@@ -96,8 +96,8 @@ static GtkWidget *history_create_recent_menu_item (HistoryItem *hi, gint num,
 static void history_add_recent_menu (GaleonWindow *window, HistoryItem *hi);
 static void history_make_host_strings (HistoryHost *hh, gchar **text);
 static void history_make_item_strings (HistoryItem *hi, gchar **text);
-static HistoryItem *history_add_item (gchar *url, gchar *title, GTime first, 
-				      GTime last, gint visits);
+static HistoryItem *history_add_item (gchar *url, gchar *title_utf8, 
+				      GTime first, GTime last, gint visits);
 static void history_look_for_newer (gpointer key, gpointer value, 
 				    gpointer user_data);
 static gint calculate_age (GTime last);
@@ -108,6 +108,7 @@ static void history_show_dialog_add_item (gpointer key, gpointer value,
 static HistoryHost *history_get_host (const gchar *url);
 static HistoryHost *history_add_host (const gchar *host);
 static void history_fill_ctree (HistoryView *hv);
+static void history_destroy_view_structure (HistoryView *hv);
 static void history_update_item_views (HistoryItem *hi);
 static void history_update_item_view (HistoryView *hv, HistoryItem *hi);
 static void history_host_scrub_node (gpointer key, gpointer value,
@@ -162,28 +163,27 @@ history_exit (void)
 	/* save the history */
 	history_save ();
 
+	/* destroy the views */
+	g_list_foreach (history_views, (GFunc)(history_destroy_view_structure),
+			NULL);
+	g_list_free (history_views);
+	history_views = NULL;
+
 	/* destroy hosts hash table */
 	g_hash_table_freeze (history_hosts);
-	g_hash_table_foreach_remove (history_hosts, 
-				     (GHRFunc) history_item_free_cb,
-				     NULL);
+	g_hash_table_foreach_remove (history_hosts, (GHRFunc)
+				     history_item_free_cb, NULL);
 	g_hash_table_thaw (history_hosts);
 	g_hash_table_destroy (history_hosts);
 	history_hosts = NULL;
 
 	/* destroy history hash table */
 	g_hash_table_freeze (history);
-	g_hash_table_foreach_remove (history, 
-				     (GHRFunc) history_item_free_cb, 
+	g_hash_table_foreach_remove (history, (GHRFunc)history_item_free_cb,
 				     NULL);
 	g_hash_table_thaw (history);
 	g_hash_table_destroy (history);
 	history = NULL;
-
-	/* destroy the views */
-	/* history_views is NULL when clearin the history and closing galeon */
-	if (history_views)
-		g_list_foreach (history_views, (GFunc)history_destroy_view, NULL);
 }
 
 /**
@@ -196,15 +196,13 @@ history_clear (void)
 
 	/* destroy hosts hash table */
 	g_hash_table_freeze (history_hosts);
-	g_hash_table_foreach_remove (history_hosts, 
-				     (GHRFunc) history_item_free_cb,
-				     NULL);
+	g_hash_table_foreach_remove (history_hosts, (GHRFunc) 
+				     history_item_free_cb, NULL);
 	g_hash_table_thaw (history_hosts);
 
 	/* destroy history hash table */
 	g_hash_table_freeze (history);
-	g_hash_table_foreach_remove (history, 
-				     (GHRFunc) history_item_free_cb, 
+	g_hash_table_foreach_remove (history, (GHRFunc)history_item_free_cb, 
 				     NULL);
 	g_hash_table_thaw (history);
 
@@ -235,12 +233,11 @@ history_show_dialog (GaleonWindow *window)
 	/* build and show the history dialog */
 	hv = history_build_view (window, "history_dialog");
 
-	state_load_window_state (hv->view, "history", -1, -1);
+	state_load_window_state (hv->view, "history", 530, 450);
 	state_load_column_widths (hv->ctree, "history");
 
 	/* show the widget */
-	gtk_widget_show (hv->view);
-	window_set_layer (hv->view);
+	dialog_set_parent (hv->view, window->WMain);
 }
 
 /**
@@ -276,14 +273,17 @@ history_show_dock (GaleonWindow *window)
 	/* add as a dock */
 	window_dock (window, hv->view, CONF_STATE_HISTORY_DOCK_WIDTH);
 
-	/* hack because we don't want to emit toggled signal */
-	(GTK_CHECK_MENU_ITEM (window->view_history_dock))->active = TRUE;
-
 	/* load dock column widths */
 	state_load_column_widths (hv->ctree, "history_dock");	
 
 	/* show the widget */
 	gtk_widget_show (hv->view);
+
+	/* set dock type */
+	window->dock_type = DOCK_HISTORY;
+
+	/* update toolbar buttons */
+	window_update_bm_and_hist_buttons (window, FALSE, TRUE);	
 }
 
 /**
@@ -338,6 +338,18 @@ history_destroy_view (HistoryView *hv)
 	/* remove from list of views */
 	history_views = g_list_remove (history_views, hv);
 
+	/* destory structure */
+	history_destroy_view_structure (hv);
+}
+
+/**
+ * history_destroy_view_structure: destroy a history view structure without
+ * removing it from the global list (for destroying all of them we need to
+ * retain the list structure).
+ */
+static void
+history_destroy_view_structure (HistoryView *hv)
+{
 	/* free allocated storage */
 	g_hash_table_destroy (hv->host_to_node);
 	g_hash_table_destroy (hv->item_to_node);
@@ -371,8 +383,8 @@ history_visited (const gchar *url)
 	/* check arguments */
 	g_return_if_fail (url != NULL);
 
-	/* don't add blank pages or portal into history */
-	if (!strcmp (url, "about:blank") || !strcmp (url, MYPORTAL_URL))
+	/* don't add blank pages into history */
+	if (!strcmp (url, "about:blank"))
 	{
 		return;
 	}
@@ -415,7 +427,9 @@ history_visited (const gchar *url)
  * history_visited: called to update the title of a page in the history
  */
 void
-history_set_page_title (const gchar *url, const gchar *title)
+history_set_page_title (const gchar *url, 
+			gchar *title_locale,
+			gchar *title_utf8)
 {
 	HistoryItem *hi;
 
@@ -431,16 +445,36 @@ history_set_page_title (const gchar *url, const gchar *title)
 	}
 
 	/* check supplied title */
-	g_return_if_fail (hi->title != NULL);
-	if (strcmp (title, hi->title) == 0)
+	g_return_if_fail (title_locale != NULL);
+	g_return_if_fail (title_utf8 != NULL);
+	if (strcmp (title_utf8, hi->title_utf8) == 0)
 	{
 		/* identical */
+		g_free (title_locale);
+		g_free (title_utf8);
 		return;
 	}
       
-        /* set title */	
-	g_free (hi->title);
-	hi->title = g_strdup (title);
+	/* set utf8 title */
+	g_free (hi->title_utf8);
+	hi->title_utf8 = title_utf8;
+
+        /* free old locale title */
+	if (hi->title_locale != NULL)
+	{
+		g_free (hi->title_locale);
+	}
+
+	/* store new one if different */
+	if (strcmp (title_utf8, title_locale) != 0)
+	{
+		hi->title_locale = title_locale;
+	}
+	else
+	{
+		hi->title_locale = NULL;
+		g_free (title_locale);
+	}
 
 	/* update all the views */
 	history_update_item_views (hi);
@@ -646,25 +680,26 @@ history_add_host (const gchar *name)
 	host = g_new0 (HistoryHost, 1);
 
 	/* fill in structure */
-	host->title   = g_strdup (name);
-	host->url     = g_strdup (name);
-	host->node    = NULL;
-	host->sibling = NULL;
-	host->zoom    = 0;
-	host->first   = 0;
-	host->last    = 0;
-	host->visits  = 0;
+	host->title_locale  = g_strdup (name);
+	host->title_utf8    = g_strdup (name);
+	host->url           = g_strdup (name);
+	host->node          = NULL;
+	host->sibling       = NULL;
+	host->zoom          = 0;
+	host->first         = 0;
+	host->last          = 0;
+	host->visits        = 0;
 	host->dominant_item = NULL;
 
 	/* add to the hosts hashtable */
-	g_hash_table_insert (history_hosts, host->title, host);
+	g_hash_table_insert (history_hosts, host->url, host);
 
 	/* return completed structure */
 	return host;
 }
 
 /*
- * history_get_host: get the host category of the url
+ * history_get_host: get the host folder of the url
  */
 static HistoryHost *
 history_get_host (const gchar *url)
@@ -679,7 +714,7 @@ history_get_host (const gchar *url)
  	/* check if it's a local file */
  	if (!g_strncasecmp (url, "file://", 7))
  	{
- 		/* if so, put it in a category named "Local files" */
+ 		/* if so, put it in a folder named "Local files" */
  		name = _("Local files");
   	}
 	else
@@ -714,7 +749,7 @@ history_get_host (const gchar *url)
 		gnome_vfs_uri_unref (vfs_uri);
 	}
 
-	/* return the appropriate category */
+	/* return the appropriate folder */
 	return host;
 }
 
@@ -798,10 +833,13 @@ history_show_dialog_add_item (gpointer key, gpointer value, gpointer user_data)
 	GtkCTreeNode *node, *parent_node;
 	GtkCTreeNode *real_sibling;
 	HistoryHost *host = NULL;
+	gchar *title_locale;
 
 	/* check if this url passes the filter */
+	title_locale = (hi->title_locale != NULL ? 
+			hi->title_locale : hi->title_utf8);
 	if (!(history_filter_text (hv, hi->url) ||
-	      history_filter_text (hv, hi->title)) ||
+	      history_filter_text (hv, title_locale)) ||
 	    !(history_filter_date (hv, hi->last)))
 	{
 		return;
@@ -842,7 +880,7 @@ history_show_dialog_add_item (gpointer key, gpointer value, gpointer user_data)
 			/* where to gather children */
 			host->sibling = NULL;
 
-			/* next category goes alongside this one */
+			/* next folder goes alongside this one */
 			*sibling = host->node;
 		}
 
@@ -892,8 +930,8 @@ static void
 history_make_host_strings (HistoryHost *hh, gchar **text)
 {
 	/* print into strings */
-	text[0] = hh->dominant_item->title;
-	text[1] = hh->title;
+	text[0] = hh->url;
+	text[1] = hh->url;
 	store_time_in_string (hh->last, text[2]);
 	store_time_in_string (hh->first, text[3]);
 	sprintf (text[4], "%d", hh->visits);
@@ -906,7 +944,8 @@ static void
 history_make_item_strings (HistoryItem *hi, gchar **text)
 {
 	/* print into strings */
-	text[0] = hi->title;
+	text[0] = (hi->title_locale != NULL ? 
+		   hi->title_locale : hi->title_utf8);		
 	text[1] = hi->url;
 	store_time_in_string (hi->last, text[2]);
 	store_time_in_string (hi->first, text[3]);
@@ -933,8 +972,7 @@ history_clear_recent_menu (void)
 		GtkWidget *menu;
 		GList *children;
 
-		menu = gtk_object_get_data (GTK_OBJECT (window->WMain),
-					    "go_menu");
+		menu = window->go_menu;
 		children = gtk_container_children (GTK_CONTAINER(menu));
 		children = g_list_nth (children, go_recent_pos);
 		
@@ -954,11 +992,12 @@ history_clear_recent_menu (void)
  * Adds a new history item to the hashtable and to the history dialog
  */
 static HistoryItem *
-history_add_item (gchar *url, gchar *title, GTime first, 
+history_add_item (gchar *url, gchar *title_utf8, GTime first, 
 		  GTime last, gint visits)
 {
 	HistoryItem *hi;
 	HistoryHost *host;
+	gchar *title_locale;
 
 	/* allocate */
 	hi = g_new0 (HistoryItem, 1);
@@ -967,12 +1006,26 @@ history_add_item (gchar *url, gchar *title, GTime first,
 	host = history_get_host (url);
 
 	/* fill in the fields */
-	hi->host   = host;
-	hi->url    = url;
-	hi->title  = title;
-	hi->first  = first;
-	hi->last   = last;
-	hi->visits = visits;
+	hi->host       = host;
+	hi->url        = url;
+	hi->title_utf8 = title_utf8;
+	hi->first      = first;
+	hi->last       = last;
+	hi->visits     = visits;
+
+	
+	/* convert to locale if different */
+	title_locale = utf8_to_locale (title_utf8);
+	if (strcmp (title_locale, title_utf8) != 0)
+	{
+		/* keep this */
+		hi->title_locale = title_locale;
+	}
+	else
+	{
+		/* not needed */
+		g_free (title_locale);
+	}
 
 	/* update the host */
 	host->visits += visits;
@@ -1019,7 +1072,7 @@ history_get_last_url (void)
 	else
 	{
                 /* we should return some "default last page" */
-                return "http://galeon.sourceforge.net";
+                return GALEON_HOMEPAGE_URL;
         }
 }
 
@@ -1060,7 +1113,7 @@ calculate_age (GTime last)
 static void
 history_add_recent_menu (GaleonWindow *window, HistoryItem *hi)
 {
-	GtkWidget *menu = gtk_object_get_data (GTK_OBJECT (window->WMain), "go_menu"); /*FIXME */
+	GtkWidget *menu;
 	GtkWidget *menuitem;
 	GList *l;
 	GList *li;
@@ -1075,6 +1128,8 @@ history_add_recent_menu (GaleonWindow *window, HistoryItem *hi)
 		return;
 	}
 	
+ 	menu = window->go_menu;
+
 	if (gnome_preferences_get_menus_have_tearoff ())
 	{
 		go_recent_pos++;
@@ -1089,8 +1144,6 @@ history_add_recent_menu (GaleonWindow *window, HistoryItem *hi)
 		{
 			mhi = gtk_object_get_user_data (GTK_OBJECT (li->data));
 			g_assert (mhi != NULL);
-			gtk_container_remove (GTK_CONTAINER (menu),
-					      GTK_WIDGET (li->data));
 			gtk_widget_destroy (GTK_WIDGET (li->data));
 			if (mhi != hi)
 			{
@@ -1123,8 +1176,12 @@ history_create_recent_menu_item (HistoryItem *hi, gint num, GtkWidget *menu)
 	GtkWidget *pixmap = gtk_pixmap_new (pixmap_data->pixmap,
 		pixmap_data->mask);
 	GtkWidget *menuitem = gtk_pixmap_menu_item_new ();
-	GtkWidget *l = new_num_accel_label (num, hi->title, FALSE, menu,
-		menuitem);
+	gchar *title_locale;
+	GtkWidget *l;
+
+	title_locale = (hi->title_locale != NULL ? 
+			hi->title_locale : hi->title_utf8);
+	l = new_num_accel_label (num, title_locale, FALSE, menu, menuitem);
 	
 	gtk_box_pack_start (GTK_BOX (hb), l, FALSE, FALSE, 2);
 	gtk_container_add (GTK_CONTAINER(menuitem), hb);
@@ -1135,9 +1192,6 @@ history_create_recent_menu_item (HistoryItem *hi, gint num, GtkWidget *menu)
 	if (gnome_preferences_get_menus_have_icons ())
 		gtk_widget_show (pixmap);
 	
-	gtk_widget_ref (menuitem); /* so we can gtk_container_remove, and then
-				      gtk_widget_destroy */
-	
 	gtk_object_set_user_data (GTK_OBJECT (menuitem), hi);
 	gtk_signal_connect (GTK_OBJECT(menuitem), "activate",
 			    GTK_SIGNAL_FUNC (window_menu_recent_activate_cb),
@@ -1231,7 +1285,11 @@ history_item_free_cb (gpointer key, gpointer value, gpointer user_data)
 
 	/* free the item */
 	g_free (hi->url);
-	g_free (hi->title);
+	if (hi->title_locale != NULL)
+	{
+		g_free (hi->title_locale);
+	}
+	g_free (hi->title_utf8);
 	g_free (hi);
 
 	return TRUE;
@@ -1480,7 +1538,7 @@ static void
 history_parse_item_element (const xmlChar **attrs)
 {
 	const xmlChar **attr;
-	gchar *title = NULL;
+	gchar *title_utf8 = NULL;
 	gchar *url = NULL;
 	GTime first = 0;
 	GTime last = 0;
@@ -1491,7 +1549,7 @@ history_parse_item_element (const xmlChar **attrs)
 	{
 		if (xmlStrcmp(attr[SAX_KEY], "title") == 0)
 		{
-			title = decode_entity (attr[SAX_VALUE]);
+			title_utf8 = decode_entity (attr[SAX_VALUE]);
 		}
 		else if (xmlStrcmp(attr[SAX_KEY], "url") == 0)
 		{
@@ -1521,7 +1579,12 @@ history_parse_item_element (const xmlChar **attrs)
 	if (calculate_age (last) <= expire_days)
 	{
 		/* add to the history */
-		history_add_item (url, title, first, last, visits);
+		history_add_item (url, title_utf8, first, last, visits);
+	}
+	else
+	{
+		if (url) g_free (url);
+		if (title_utf8) g_free (title_utf8);			
 	}
 }
 
@@ -1566,7 +1629,7 @@ history_save_host (gpointer key, gpointer value, gpointer user_data)
 	if (hh->zoom != 0)
 	{
 		fputs ("  <host name=\"", fp);
-		fputs (hh->title, fp);
+		fputs (hh->url, fp);
 		fputs ("\" zoom=\"", fp);
 		fprintf (fp, "%d", hh->zoom);
 		fputs ("\"/>\n", fp);
@@ -1583,7 +1646,7 @@ history_save_item (gpointer key, gpointer value, gpointer user_data)
 	FILE *fp = (FILE *) user_data;
 
 	fputs ("  <item title=\"", fp);
-	eputs (hi->title, fp);
+	eputs (hi->title_utf8, fp);
 	fputs ("\" url=\"", fp);
 	eputs (hi->url, fp);
 	fputs ("\" first_time=\"", fp);
@@ -1607,6 +1670,12 @@ eputs (guchar *text, FILE *fp)
 	length = strlen (text);
 	for (i = 0; i < length; i++)
 	{
+                /**
+		 * This method is not good. In XML spec, &#xxx refers to
+		 * a specific character in the ISO/IEC 10646, not UTF-8.
+		 * [ yes, but this is for attributes which need encoding
+		 *   whatever the charset is being used -- MattA ]
+		 */
 		if (text[i] >= 127)
 		{
 			fwrite (text + last, i - last, 1, fp);
@@ -1614,7 +1683,6 @@ eputs (guchar *text, FILE *fp)
 			fprintf (fp, "&#%03d;", text[i]);
 			continue;
 		}
-
 		switch (text[i])
 		{
 		case '&':
@@ -1653,20 +1721,32 @@ eputs (guchar *text, FILE *fp)
 	fwrite (text + last, i - last, 1, fp);
 }
 
+
+/**
+ * decode_entity: translate &amp; et al and return a new expanded string 
+ */
 static gchar *
 decode_entity (const unsigned char *encoded)
 {
 	gchar *buffer;
 	gint i, j, length;
-
+	
 	if (encoded == NULL)
 	{
 		return NULL;
 	}
 
 	buffer = g_strdup (encoded);
-	length = strlen (encoded);
-	for (i = 0, j = 0; i < length; i++)
+
+	/* Optimize for case where no escape codes are found, otherwise
+	 * use the first found code as the starting index */
+	if ((i = (char *) strstr (buffer, "&") - buffer) < 0)
+	{
+		return buffer;
+	}
+	
+	length = strlen (buffer);
+	for (j = i; i < length; i++)
 	{
 		if (encoded[i] == '&')
 		{
@@ -1711,6 +1791,7 @@ decode_entity (const unsigned char *encoded)
 	return buffer;
 }
 
+
 static void
 history_drag_data_get_cb (GtkWidget *widget, GdkDragContext *context,
 			  GtkSelectionData *selection_data, guint info,
