@@ -26,6 +26,7 @@
 #include "misc.h"
 #include "bookmarks.h"
 #include "window.h"
+#include "dialog.h"
 
 #include <gtkmozembed.h>
 #include <glade/glade.h>
@@ -33,9 +34,13 @@
 /* GNOME includes */
 #include <gconf/gconf.h>
 #include <gconf/gconf-client.h>
+#include <libgnome/gnome-util.h>
+#include <libgnome/gnome-config.h>
+#include <libgnome/gnome-i18n.h>
+#include <libgnomeui/gnome-init.h>
+#include <libgnomeui/gnome-geometry.h>
 #include <libgnomevfs/gnome-vfs.h>
 #include <libgnomevfs/gnome-vfs-mime.h>
-#include <libgnorba/gnorba.h>
 #include <liboaf/liboaf.h>
 #ifdef ENABLE_APPLET
 #include <applet-widget.h>
@@ -50,9 +55,11 @@ static gboolean  new_view_on_running_shell (gint n_urls, gchar **url);
 static void new_shell (CORBA_ORB orb);
 static gboolean galeon_init (int argc, char *argv[]);
 static void create_default_browser (void);
-static ViewState get_viewstate (void);
+static inline ViewState get_viewstate (void);
 static gint read_mozilla_version (void);
 static gint check_mozilla_version (void);
+static GdkFilterReturn theme_changed_cb (GdkXEvent *xevent, GdkEvent *event, 
+					 gpointer data);
 
 /* import from Galeon-impl.c */
 Galeon_Browser impl_Galeon_Browser__create (PortableServer_POA poa, 
@@ -71,6 +78,8 @@ static gboolean open_in_new_window = FALSE;  /* force open in a new window?  */
 static gboolean open_fullscreen    = FALSE;  /* open galeon in full screen ? */
 static gchar *geometry_string      = NULL;   /* the geometry string          */
 static gchar *bookmark_url         = NULL;   /* the temp bookmark to add     */
+static gboolean close_option       = FALSE;  /* --close                      */
+static gboolean quit_option        = FALSE;  /* --quit                       */
 
 /* mozilla version detected at runtime */
 int mozilla_version=0;
@@ -108,6 +117,12 @@ static struct poptOption popt_options[] =
 	  0, N_("Create the initial window with the given geometry.\n"
 		"see X(1) for the GEOMETRY format"),
 	  N_("GEOMETRY")},
+	{ "close", 'c', POPT_ARG_NONE, &close_option, 0,
+	  N_("Close all Galeon windows"),
+	  NULL },
+	{ "quit", 'q', POPT_ARG_NONE, &quit_option, 0,
+	  N_("Same as --close, but exits server mode too"),
+	  NULL },
 
 	/* terminator, must be last */
 	{ NULL, 0, 0, NULL, 0, NULL, NULL }
@@ -164,9 +179,6 @@ main (int argc, char *argv[])
 					    popt_options, 0, &context);
 	}
 
-	if (check_mozilla_version ())
-		return 1;
-
 	/* load arguments that aren't regular options (urls to load) */
 	n_urls = translate_url_arguments (context, &url);
 
@@ -178,14 +190,21 @@ main (int argc, char *argv[])
 	{
 		/* done */
 		return 0;
+	} 
+
+	if (close_option || quit_option)
+	{
+		g_message (_("Nothing to close: "
+			     "no opened Galeon windows found."));
+		return 0;
 	}
 
 	/* add a temp bookmark and exit if needed */
 	if (bookmark_url != NULL)
 	{
 		bookmarks_load ();
+		/* this will save the bookmarks too -- Matthew */
 		add_bookmark_default (BM_SITE, NULL, bookmark_url, NULL);
-		bookmarks_save ();
 		return 0;
 	}
 	
@@ -228,7 +247,9 @@ main (int argc, char *argv[])
 	return 0;
 }
 
-
+/**
+ * read_mozilla_version: extract mozilla version from mozilla prefs file
+ */
 static gint
 read_mozilla_version (void)
 {
@@ -287,6 +308,11 @@ read_mozilla_version (void)
 	return ver;
 }
 
+/**
+ * check_mozilla_version: verify that the runtime mozilla is the same version
+ * as the compile time.  If not print a warning and display a confirmation
+ * dialog.
+ */
 static gint
 check_mozilla_version (void)
 {
@@ -294,11 +320,58 @@ check_mozilla_version (void)
 
 	if (MOZILLA_VERSION != mozilla_version)
 	{
-		g_warning ("compiled MOZILLA_VERSION ("VERSIONFMT") != detected mozilla_version ("VERSIONFMT")",
-			VERSIONFMTARGS (MOZILLA_VERSION), VERSIONFMTARGS (mozilla_version));
-		//return -1;
-	}
+		gchar *questionStr, *rememberStr, *verMatchStr;
+		gboolean continuev=0, rememberv=0;
+		g_warning ("compiled MOZILLA_VERSION ("VERSIONFMT") !="
+			" detected mozilla_version ("VERSIONFMT")",
+			VERSIONFMTARGS (MOZILLA_VERSION),
+			VERSIONFMTARGS (mozilla_version));
 
+		verMatchStr = g_strdup_printf ("%xv%x", MOZILLA_VERSION,
+						mozilla_version);
+		rememberStr = gnome_config_get_string 
+				("/galeon/General/moz_mismatch_remember=");
+
+		if (rememberStr != NULL)
+		{
+			int ignore = strcmp (rememberStr, verMatchStr);
+			g_free (rememberStr);
+			if (ignore == 0)
+			{
+				g_free (verMatchStr);
+				return gnome_config_get_int
+				    ("/galeon/General/moz_mismatch_action=1");
+			}
+		}
+
+		questionStr = g_strdup_printf (_(
+"The version of mozilla galeon was compiled against ("VERSIONFMT") "
+"does not match the detected version ("VERSIONFMT").\n\n"
+"You may experience segfaults and other oddities, please test "
+"with a matching version of mozilla before reporting any bugs.\n\n"
+"Continue anyway?"),
+			VERSIONFMTARGS (MOZILLA_VERSION),
+			VERSIONFMTARGS (mozilla_version));
+
+
+		continuev = dialog_confirm_check (NULL, _("Version mismatch!"),
+					questionStr, _("Don't ask me again"),
+					&rememberv);
+	
+		g_free (questionStr);
+		if (rememberv)
+		{
+			gnome_config_set_string
+				("/galeon/General/moz_mismatch_remember",
+				verMatchStr);
+			gnome_config_set_int
+				("/galeon/General/moz_mismatch_action",
+				 !continuev);
+			gnome_config_sync ();
+		}
+		g_free (verMatchStr);
+		return !continuev;
+	}
 	return 0;
 }
 
@@ -368,13 +441,6 @@ corba_init (int *argc, char *argv[])
 	/* catch exceptions */
 	CORBA_exception_init (&corba_env);
 
-	/* initialise CORBA (not for panel mode, this does its own init) */
-	if (!galeon_panel_mode)
-	{
-		gnorba_CORBA_init (argc, argv, GNORBA_INIT_SERVER_FUNC, 
-				   &corba_env);
-	}
-
 	/* initialise OAF */
 	orb = oaf_init (*argc, argv);
 
@@ -417,6 +483,12 @@ new_view_on_running_shell (gint n_urls, gchar **url)
 		return TRUE;
 	}
 
+	if (close_option || quit_option)
+	{
+		Galeon_Browser_quit (client, quit_option, &corba_env);
+		return TRUE;
+	}
+
 	/* otherwise we'll be opening windows, so warn */
 	g_message (_("Galeon already running, using existing process"));
 
@@ -481,7 +553,7 @@ new_shell (CORBA_ORB orb)
 	}
 
 	/* register server with oaf */
-	reg_id = oaf_make_registration_id ("OAFIID:Galeon:20010802",
+	reg_id = oaf_make_registration_id ("OAFIID:GNOME:galeon:20010802",
 					   g_getenv ("DISPLAY"));
 	oaf_active_server_register (reg_id, client);
 	g_free (reg_id);
@@ -500,6 +572,8 @@ galeon_init (int argc, char *argv[])
 	gboolean new_user;
 	gchar *profile;
 	extern gboolean pushed_startup;
+	gboolean moz_version_checked = FALSE;
+	GdkAtom message_type;
 
 	/* initialise GNOME VFS */
 	vfs_success = gnome_vfs_init ();
@@ -513,14 +587,21 @@ galeon_init (int argc, char *argv[])
 	/* initialise GLADE */
 	glade_gnome_init ();
 
+	/* ensure mozilla matches (and initialise mozilla_version var) */
+	if (g_getenv ("MOZILLA_FIVE_HOME") != NULL)
+	{
+		if (check_mozilla_version ())
+		{
+			exit (1);
+		}
+		moz_version_checked = TRUE;
+	}
+
 	/* initialise Galeon itself */
+	initialise_colours ();
 	preferences_load ();
 	history_init ();
 	bookmarks_init ();
-	initialise_colours ();
-
-	/* check if this is the first time this user has run Galeon */
-	new_user = newuser_check ();
 
 	/* init mozilla home */
 	gtk_moz_embed_set_comp_path (g_getenv ("MOZILLA_FIVE_HOME"));
@@ -535,6 +616,18 @@ galeon_init (int argc, char *argv[])
 	{
 		embed_startup_init ();
 	}
+	/* if we couldn't check the first time, try now since initializing moz
+	 * will set the MOZILLA_FIVE_HOME in some configurations */
+	if (!moz_version_checked)
+	{
+		if (check_mozilla_version ())
+		{
+			exit (1);
+		}
+	}
+
+	/* check if this is the first time this user has run Galeon */
+	new_user = newuser_check ();
 
 	/* resume a crashed session, if any */
 	session_recovery = session_autoresume ();
@@ -547,6 +640,15 @@ galeon_init (int argc, char *argv[])
 					GNOME_RESTART_IF_RUNNING);
 	gtk_signal_connect (GTK_OBJECT (session_client), "save_yourself",
 			    (GtkSignalFunc)client_save_yourself_cb, NULL);
+		
+	/* update tab styles when theme changed */
+	message_type = gdk_atom_intern ("_GTK_READ_RCFILES", FALSE);
+	gdk_add_client_message_filter (message_type, (GdkFilterFunc)
+				       theme_changed_cb, NULL);
+
+	/* FIXME: catch GDK errors which happen on theme changes
+	 * (until we work out what causes this) */
+	gdk_error_trap_push ();
 
 	/* if this flag doesn't get cleared, then Galeon crashed */
 	gnome_config_set_int ("/galeon/General/crashed", TRUE);
@@ -565,7 +667,7 @@ create_default_browser (void)
 {
 	GaleonEmbed *embed;
 	GaleonWindow *window;
-	gint xpos = -1, ypos;
+	gint xpos, ypos;
 	gint width, height;
 	gchar *url;
 	
@@ -594,26 +696,16 @@ create_default_browser (void)
 	else if (gnome_parse_geometry (geometry_string, &xpos, &ypos,
 				       &width, &height))
 	{
-		gtk_window_set_default_size 
-			(GTK_WINDOW (window->WMain), width, height);
+		/* setup size and show it */
+		gtk_window_set_default_size (GTK_WINDOW (window->WMain), 
+					     width, height);
+		gtk_widget_set_uposition (GTK_WIDGET (window->WMain),
+					  xpos, ypos);
 		window->set_size = TRUE;
-		gtk_widget_show (GTK_WIDGET (window->WMain));
 	}
-	
-	/* reposition */
-	if (xpos > 0 && ypos > 0)
-	{
-		gdk_window_move (window->WMain->window, xpos, ypos);
-	}
-	
-	/* make sure it's visible */
-	while (gtk_events_pending ())
-	{
-		gtk_main_iteration ();
-	}
-	
+
 	/* build the default embed inside it */
-	embed = embed_create_in_window (window, FALSE);
+	embed = embed_create_in_window (window);
 	embed_set_visibility (embed, TRUE);
 	url = embed_get_default_url (NULL);
 	embed_load_url (embed, url);
@@ -630,16 +722,38 @@ create_default_browser (void)
  * You're evil ;-)
  *   -- MattA
  * 
+ * VIEWSTATE_NONE:    Open up in existing window if there is one
  * VIEWSTATE_DEFAULT: Use default (setting) viewstate
  * VIEWSTATE_WINDOW:  Open up in window
  * VIEWSTATE_TAB:     Open up in tab
- * VIEWSTATE_NONE:    Open up in existing window if there is one
  */
-static ViewState
+static inline ViewState
 get_viewstate (void)
 {
-	return (2 * (!open_in_existing & (open_in_new_tab ^ 
-					  open_in_new_window)) + 
+	return (((!open_in_existing & (open_in_new_tab ^ 
+					  open_in_new_window)) << 1) + 
 		(!open_in_new_window & !open_in_existing));
 }
- 
+
+/**
+ * theme_changed_cb: called when a Gtk+ theme change is detected, update the
+ * tab label styles
+ */
+static GdkFilterReturn
+theme_changed_cb (GdkXEvent *xevent, GdkEvent *event, gpointer data)
+{
+	GList *e;
+	
+	/* reload colour styles */
+	initialise_colours ();
+
+	/* set in all tabs */
+	for (e = all_embeds; e != NULL; e = g_list_next (e))
+	{
+		embed_set_notebook_label_status ((GaleonEmbed *)(e->data));
+	}
+	
+	/* process other GDK events */
+	return GDK_FILTER_CONTINUE;
+}
+
