@@ -27,12 +27,16 @@
 #include "print.h"
 #include "find.h"
 #include "mozilla.h"
+#include "session.h"
 #include "xlink.h"
+#include "dialog.h"
+#include "filepicker.h"
 
 #include <string.h>
 #include <gtkmozembed.h>
 #include <gtk/gtkspinbutton.h>
 #include <libgnomeui/gnome-about.h>
+#include <libgnome/gnome-util.h>
 #include <libgnome/gnome-help.h>
 #include <libgnome/gnome-config.h>
 #include <libgnome/gnome-i18n.h>
@@ -46,7 +50,7 @@ window_menu_recent_activate_cb (GtkMenuItem *menuitem, gpointer user_data);
 void
 window_menu_prefs_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
 {
-	preferences_open ();
+	preferences_open (window);
 }
 
 /**
@@ -58,7 +62,18 @@ window_menu_bookmarks_edit_activate_cb (GtkMenuItem *menuitem,
 {
 	return_if_not_window (window);
 	return_if_not_embed (window->active_embed);
-	bookmarks_editor_show_dialog (window, STANDARD_BOOKMARKS_EDITOR);
+
+	if (gnome_config_get_bool (CONF_STATE_BOOKMARKS_DOCK_ENABLE))
+	{
+		if (window->dock_type != DOCK_BOOKMARKS)
+		{
+			bookmarks_editor_show_dock (window);
+		}
+	}
+	else
+	{
+		bookmarks_editor_show_dialog (window);
+	}
 }
 
 /**
@@ -80,8 +95,13 @@ window_menu_edit_find_focus(GaleonWindow *window)
 	GtkWidget *widget;
 	GList *tb_entry;
 
-	if (GTK_WIDGET_HAS_FOCUS ( window -> toolbar_entry ) )
-		return(window -> toolbar_entry);
+	if (window->location_entry != NULL)
+	{
+		if (GTK_WIDGET_HAS_FOCUS (window->location_entry))
+		{
+			return (window->location_entry);
+		}
+	}
 
 	/* Search through the toolbars, then down each toolbar
 	** looking for a focused widget
@@ -118,118 +138,50 @@ window_menu_edit_find_focus(GaleonWindow *window)
 	return(NULL);
 }
 
-/*
- * Called before the Cut menu item is exposed. We allow it to be sensitive
- * if we are currently in an updateable field and there is something selected
- */
-static gboolean
-window_menu_edit_cut_event(GtkWidget *widget, GdkEvent *event, gpointer data)
+gboolean window_menu_edit_expose_event_cb (GtkWidget *editmenu, 
+					   GdkEvent *event,
+					   GaleonWindow *window)
 {
-	GaleonWindow *window=data;
+	gboolean can_cut, can_copy, can_paste;
+	GtkWidget *focus_widget;
+	
+	can_cut = FALSE;
+	can_copy = FALSE;
+	can_paste = FALSE;
 
-	if (event->type == GDK_EXPOSE) {
-		gboolean sensitive=FALSE;
-		GtkWidget *focus_widget;
+	focus_widget = window_menu_edit_find_focus (window);
+	if (focus_widget != NULL && GTK_IS_EDITABLE (focus_widget))
+	{
+		GtkEditable *editable = GTK_EDITABLE(focus_widget);
 
-		focus_widget=window_menu_edit_find_focus(window);
-		if (GTK_IS_EDITABLE(focus_widget)) {
-			GtkEditable *editable = GTK_EDITABLE(focus_widget);
-
-			if (editable->has_selection && editable->editable) {
-				sensitive=TRUE;
-			}
+		if (editable->has_selection && editable->editable)
+		{
+			can_cut = TRUE;
 		}
-		gtk_widget_set_sensitive(widget, sensitive);
-	}
-
-	return(FALSE);
-}
-
-/*
- * Called before the Copy menu item is exposed. We allow it to be sensitive
- * if there is something selected
- */
-static gboolean
-window_menu_edit_copy_event(GtkWidget *widget, GdkEvent *event, gpointer data)
-{
-	GaleonWindow *window=data;
-
-	if (event->type == GDK_EXPOSE) {
-		gboolean sensitive=FALSE;
-		GtkWidget *focus_widget;
-
-		focus_widget=window_menu_edit_find_focus(window);
-		if (GTK_IS_EDITABLE(focus_widget)) {
-			GtkEditable *editable = GTK_EDITABLE(focus_widget);
-
-			if (editable->has_selection) {
-				sensitive=TRUE;
-			}
+		if (editable->has_selection)
+		{
+			can_copy = TRUE;
 		}
-		gtk_widget_set_sensitive(widget, sensitive);
-	}
-
-	return(FALSE);
-}
-
-/*
- * Called before the Paste menu item is exposed. We allow it to be sensitive
- * if the current field is updateable and there is something in the clipboard.
- * Note the call to galeon_x_clipboard_present().
- */
-static gboolean
-window_menu_edit_paste_event(GtkWidget *widget, GdkEvent *event, gpointer data)
-{
-	GaleonWindow *window=data;
-
-	if (event->type == GDK_EXPOSE) {
-		gboolean sensitive=FALSE;
-		GtkWidget *focus_widget;
-
-		focus_widget=window_menu_edit_find_focus(window);
-		if (GTK_IS_EDITABLE(focus_widget)) {
-			GtkEditable *editable = GTK_EDITABLE(focus_widget);
-			if (editable->editable && galeon_x_clipboard_present()) {
-				sensitive=TRUE;
-			}
+		if (editable->editable && galeon_x_clipboard_present()) 
+		{
+			can_paste = TRUE;
 		}
-		gtk_widget_set_sensitive(widget, sensitive);
 	}
-
-	return(FALSE);
-}
-
-/*
- * Add event handlers for the expose event onto the cut/copy/paste items
- * on the edit menu
- */
-void
-window_menu_edit_add_events(GaleonWindow *window)
-{
-	GnomeUIInfo *editmenu = edit_menu_uiinfo;
-	gint count;
-
-	for (count=0; editmenu[count].type != GNOME_APP_UI_ENDOFINFO; count++) {
-		gpointer cb = editmenu[count].moreinfo;
-
-		if (cb == window_menu_edit_cut_cb) {
-			gtk_signal_connect ( GTK_OBJECT (editmenu[count].widget),
-				"expose_event",
-				GTK_SIGNAL_FUNC (window_menu_edit_cut_event),
-				window);
-		} else if (cb == window_menu_edit_copy_cb) {
-			gtk_signal_connect ( GTK_OBJECT (editmenu[count].widget),
-				"expose_event",
-				GTK_SIGNAL_FUNC (window_menu_edit_copy_event),
-				window);
-		} else if (cb == window_menu_edit_paste_cb) {	
-			gtk_signal_connect ( GTK_OBJECT (editmenu[count].widget),
-				"expose_event",
-				GTK_SIGNAL_FUNC (window_menu_edit_paste_event),
-				window);
+	else
+	{	
+		if (window->active_embed != NULL) 
+		{
+			can_cut = mozilla_can_cut_selection (window->active_embed);
+			can_copy = mozilla_can_copy_selection (window->active_embed);
+			can_paste = mozilla_can_paste (window->active_embed);
 		}
 	}
 
+	gtk_widget_set_sensitive (window->edit_cut, can_cut);
+	gtk_widget_set_sensitive (window->edit_copy, can_copy);
+	gtk_widget_set_sensitive (window->edit_paste, can_paste);
+
+	return FALSE;
 }
 
 /**
@@ -242,7 +194,10 @@ window_menu_edit_cut_cb (GtkMenuItem *menuitem, GaleonWindow *window)
 	GtkWidget *widget;
 
 	if ((widget=window_menu_edit_find_focus(window)) == NULL)
+	{
+		mozilla_cut_selection (window->active_embed);
 		return;
+	}
 
 	editable = GTK_EDITABLE (widget);
 
@@ -259,7 +214,10 @@ window_menu_edit_copy_cb (GtkMenuItem *menuitem, GaleonWindow *window)
 	GtkWidget *widget;
 
 	if ((widget=window_menu_edit_find_focus(window)) == NULL)
+	{
+		mozilla_copy_selection (window->active_embed);
 		return;
+	}
 
 	editable = GTK_EDITABLE (widget);
 
@@ -276,7 +234,10 @@ window_menu_edit_paste_cb (GtkMenuItem *menuitem, GaleonWindow *window)
 	GtkWidget *widget;
 
 	if ((widget=window_menu_edit_find_focus(window)) == NULL)
+	{
+		mozilla_paste (window->active_embed);
 		return;
+	}
 
 	editable = GTK_EDITABLE (widget);
 
@@ -289,9 +250,7 @@ window_menu_edit_paste_cb (GtkMenuItem *menuitem, GaleonWindow *window)
 void 
 window_menu_recent_activate_cb (GtkMenuItem *menuitem, gpointer user_data)
 {
-        GtkWidget *WMain = window_lookup_widget (GTK_WIDGET(menuitem), "WMain");
-        GaleonWindow *window = (GaleonWindow *) gtk_object_get_data
-		(GTK_OBJECT (WMain), "GaleonWindow");
+        GaleonWindow *window = window_from_widget (GTK_WIDGET (menuitem));
         HistoryItem *hi = gtk_object_get_user_data (GTK_OBJECT (menuitem));
 
 	return_if_not_window (window);
@@ -305,8 +264,7 @@ void
 window_spinner_button_clicked_cb(GtkButton *button, GaleonWindow *window)
 {
 	return_if_not_window (window);
-	/* FIXME: don't hardcode this */
-	window_load_url (window, "http://galeon.sourceforge.net");
+	window_load_url (window, GALEON_HOMEPAGE_URL);
 }
 
 /**
@@ -331,14 +289,35 @@ void window_menu_history_activate_cb (GtkMenuItem *menuitem,
 				      GaleonWindow *window)
 {
 	return_if_not_window (window);
-	history_show_dialog (window);
+	
+	if (gnome_config_get_bool (CONF_STATE_HISTORY_DOCK_ENABLE))
+	{
+		if (window->dock_type != DOCK_HISTORY)
+		{
+			history_show_dock (window);
+		}
+	}
+	else
+	{
+		history_show_dialog (window);
+	}
 }
 
 /**
- * window_menu_about_activate_cb: show about dialog 
+ * window_run_first_time_druid_activate_cb: reruns the fisrt time druid
+ */
+void window_run_first_time_druid_activate_cb (GtkMenuItem *menuitem,
+					      GaleonWindow *window)
+{
+	newuser_show_druid ();
+}
+
+/**
+ * window_menu_help_about_activate_cb: show about dialog 
  */
 void
-window_menu_about_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
+window_menu_help_about_activate_cb (GtkMenuItem *menuitem, 
+				    GaleonWindow *window)
 {
 	static gchar *authors[] = {
 		"Marco Pesenti Gritti <mpeseng@tin.it>",
@@ -356,14 +335,14 @@ window_menu_about_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
 		"Matthew Mueller <donut@azstarnet.com>",
 		" ", /* dont delete the space */
 		N_("Graphic arts: "),
-	       /* if your encoding allows it, use eacute (U00E9)
-		* for the two 'e's of Frederic */
+		/* if your encoding allows it, use eacute (U00E9)
+		 * for the two 'e's of Frederic */
 		N_("Toussaint Frederic (fred@linuxgraphic.org)"),
 		NULL
 	};
-
+	
 	static GtkWidget * about;
-
+	
 	return_if_not_window (window);
 
 	if (!about)
@@ -371,13 +350,15 @@ window_menu_about_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
 		gint i, num_authors = sizeof(authors) / sizeof(gchar *);
 		gchar *translated_names[num_authors];
 		gchar *bottom_line;
-
+		gchar *file;
+		
 		if (strcmp (_("Translation:"), "Translation:") == 0) {
-			bottom_line = g_strdup("A GNOME browser based on mozilla");
+			bottom_line = g_strdup
+				("A GNOME browser based on mozilla");
 		} else {
 			bottom_line = g_strconcat (
-					_("A GNOME browser based on mozilla"),
-					"\n \n",
+				_("A GNOME browser based on mozilla"),
+				"\n \n",
 			/* Translators should localize the following string
 			 * which will be displayed at the bottom of the about
 			 * box to give credit to the translator(s).
@@ -387,37 +368,63 @@ window_menu_about_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
 			 * too long; vertical space is limited in the about
 			 * dialog. 
 			 * If you dont translate it nothing will be added */
-					_("Translation:"), NULL);
+				_("Translation:"), NULL);
 		}
 		
 		for (i = 0; i < num_authors ; i++)
 			translated_names[i] = _(authors[i]);
 		
-		about = gnome_about_new (_(PACKAGE), VERSION,
-				 _("Copyright (C) 2000-2001 Marco Pesenti Gritti"),
+		file = user_file ("about_logo.png", FALSE);
+		about = gnome_about_new 
+			(_(PACKAGE), VERSION,
+			 _("Copyright (C) 2000-2001 Marco Pesenti Gritti"),
 				 (const gchar **)translated_names, 
-				 bottom_line,
-				 SHARE_DIR"/logo.png");
-		g_free(bottom_line);
+				 bottom_line, file);
+		g_free (bottom_line);
+		g_free (file);
 
 		gtk_signal_connect (GTK_OBJECT (about), "destroy",
 			    GTK_SIGNAL_FUNC(gtk_widget_destroyed), &about);
 		gtk_widget_show (about);
 	}
 
-	window_set_layer (about);
+	dialog_set_parent (about, window->WMain);
 }
 
 /**
- * window_menu_manual_activate: show the main Galeon manual page
+ * window_menu_help_manual_activate: show the main Galeon manual page
  */
 void
-window_menu_manual_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
+window_menu_help_manual_activate_cb (GtkMenuItem *menuitem, 
+				     GaleonWindow *window)
 {
 	return_if_not_window (window);
 	gnome_help_goto (NULL, "ghelp:galeon");
 }
 
+/**
+ * window_menu_help_homepage_activate: go to the Galeon homepage
+ */
+void
+window_menu_help_homepage_activate_cb (GtkMenuItem *menuitem, 
+				       GaleonWindow *window)
+{
+	return_if_not_window (window);
+	window_load_url (window, GALEON_HOMEPAGE_URL);
+}
+
+
+/**
+ * window_menu_help_bugreport_activate: go to the Galeon bug report page
+ */
+void
+window_menu_help_bugreport_activate_cb (GtkMenuItem *menuitem, 
+					GaleonWindow *window)
+{
+	return_if_not_window (window);
+	window_load_url (window, GALEON_BUGREPORT_URL);
+}
+
 /** 
  * window_menu_new_window_activate_cb: open a new window
  */
@@ -425,9 +432,15 @@ void
 window_menu_new_window_activate_cb (GtkMenuItem *menuitem,
 				    GaleonWindow *window)
 {
+	GaleonEmbed *embed;
 	return_if_not_window (window);
 	return_if_not_embed (window->active_embed);
-	embed_create_default (window->active_embed, TRUE);
+	embed = embed_create_default (window->active_embed, TRUE);
+	if (embed->parent_window->location_entry != NULL)
+	{
+		gtk_widget_grab_focus 
+			(GTK_WIDGET (embed->parent_window->location_entry));
+	}
 }
 
 /** 
@@ -437,9 +450,15 @@ void
 window_menu_new_tab_activate_cb (GtkMenuItem *menuitem,
 				 GaleonWindow *window)
 {
+	GaleonEmbed *embed;
 	return_if_not_window (window);
 	return_if_not_embed (window->active_embed);
-	embed_create_default (window->active_embed, FALSE);
+	embed = embed_create_default (window->active_embed, FALSE);
+	if (embed->parent_window->location_entry != NULL)
+	{
+		gtk_widget_grab_focus 
+			(GTK_WIDGET (embed->parent_window->location_entry));
+	}
 }
 
 /** 
@@ -485,7 +504,8 @@ window_menu_exit_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
 	   activated from within the mozembed_key_press callback, and if the
 	   widget is destroyed before returning from the callback, very bad
 	   things happen.  --Josh */
-	gtk_timeout_add (50, (GtkFunction) galeon_quit, window);
+	gtk_timeout_add (50, (GtkFunction) session_quit_cb,
+			 GINT_TO_POINTER (FALSE));
 }
 
 /** 
@@ -496,7 +516,7 @@ window_menu_exit_with_session_activate_cb (GtkMenuItem *menuitem,
 					   GaleonWindow *window)
 {
 	return_if_not_window (window);
-	galeon_quit_with_session ();
+	session_quit (TRUE);
 }
 
 /** 
@@ -540,7 +560,7 @@ window_menu_up_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
 {
 	return_if_not_window (window);
 	return_if_not_embed (window->active_embed);
-	embed_go_up (window->active_embed, 0);
+	embed_go_up (window->active_embed, 0, 0);
 }
 
 /** 
@@ -551,7 +571,20 @@ window_menu_forward_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
 {
 	return_if_not_window (window);
 	return_if_not_embed (window->active_embed);
-	gtk_moz_embed_go_forward(GTK_MOZ_EMBED(window->active_embed->mozEmbed));
+	gtk_moz_embed_go_forward 
+		(GTK_MOZ_EMBED (window->active_embed->mozEmbed));
+}
+
+/** 
+ * window_menu_detach_tab_activate_cb: detach current tab into a new window
+ */
+void
+window_menu_detach_tab_activate_cb (GtkMenuItem *menuitem, 
+				    GaleonWindow *window)
+{
+	return_if_not_window (window);
+	return_if_not_embed (window->active_embed);
+	embed_move_tab (window->active_embed, NULL);
 }
 
 /** 
@@ -576,6 +609,30 @@ window_menu_next_tab_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
 	gtk_notebook_next_page (GTK_NOTEBOOK (window->notebook));
 }
 
+/** 
+ * window_menu_move_tab_left_cb: 
+ */
+void
+window_menu_shift_tab_left_activate_cb (GtkMenuItem *menuitem,
+					GaleonWindow *window)
+{
+	return_if_not_window (window);
+	return_if_not_embed (window->active_embed);
+	embed_shift_tab (window->active_embed, -1);
+}
+
+/** 
+ * window_menu_move_tab_right_cb:
+ */
+void
+window_menu_shift_tab_right_activate_cb (GtkMenuItem *menuitem,
+					 GaleonWindow *window)
+{
+	return_if_not_window (window);
+	return_if_not_embed (window->active_embed);
+	embed_shift_tab (window->active_embed, 1);
+}
+
 /** 
  * window_menu_stop_activate_cb: stop menu activated
  */
@@ -606,7 +663,7 @@ window_menu_home_page_activate_cb (GtkMenuItem *menuitem,
 				    GaleonWindow *window)
 {
 	return_if_not_window (window);
-	window_go_home (window, FALSE, FALSE);
+	window_go_home (window, 0);
 }
 
 void
@@ -615,7 +672,8 @@ window_menu_add_bookmark_activate_cb (GtkMenuItem *menuitem,
 {
 	return_if_not_window (window);
 	return_if_not_embed (window->active_embed);
-	embed_add_bookmark_default (window->active_embed);
+	add_bookmark_default (BM_SITE, window->active_embed->site_title, 
+			      window->active_embed->site_location, NULL);
 }
 
 /** 
@@ -625,6 +683,10 @@ void window_menu_toolbar_activate_cb (GtkCheckMenuItem *menuitem,
 				      GaleonWindow *window)
 {
 	return_if_not_window (window);
+
+	/* FIXME handle this properly when the menubar does not exists */
+	g_return_if_fail (menuitem);
+
 	if (menuitem->active) 
 	{		
 		window_toolbar_show (window);
@@ -642,6 +704,10 @@ void window_menu_menubar_activate_cb (GtkCheckMenuItem *menuitem,
 				      GaleonWindow *window)
 {
 	return_if_not_window (window);
+
+	/* FIXME handle this properly when the menubar does not exists */
+	g_return_if_fail (menuitem);
+
 	if (menuitem->active) 
 	{
 		window_menubar_show (window);
@@ -659,6 +725,10 @@ void window_menu_statusbar_activate_cb (GtkCheckMenuItem *menuitem,
 					GaleonWindow *window)
 {
 	return_if_not_window (window);
+
+	/* FIXME handle this properly when the menubar does not exists */
+	g_return_if_fail (menuitem);
+
 	if (menuitem->active) 
 	{
 		window_statusbar_show (window);
@@ -669,40 +739,6 @@ void window_menu_statusbar_activate_cb (GtkCheckMenuItem *menuitem,
 	}
 }
 
-/** 
- * window_menu_bookmarks_dock_activate_cb: hide/show the bookmarks dock
- */
-void window_menu_bookmarks_dock_activate_cb (GtkCheckMenuItem *menuitem,
-					     GaleonWindow *window)
-{
-	return_if_not_window (window);
-	if (menuitem->active)
-	{
-		bookmarks_editor_show_dock (window);
-	}
-	else
-	{
-		window_undock (window);
-	}
-}
-
-/** 
- * window_menu_history_dock_activate_cb: hide/show the history dock
- */
-void window_menu_history_dock_activate_cb (GtkCheckMenuItem *menuitem,
-					   GaleonWindow *window)
-{
-	return_if_not_window (window);
-	if (menuitem->active)
-	{
-		history_show_dock (window);
-	}
-	else
-	{
-		window_undock (window);
-	}
-}
-
 /** 
  * window_menu_fullscreen_activate_cb: set fullscreen mode
  *
@@ -711,6 +747,10 @@ void window_menu_fullscreen_activate_cb (GtkCheckMenuItem *menuitem,
 					 GaleonWindow *window)
 {
 	return_if_not_window (window);
+
+	/* FIXME handle this properly when the menubar does not exists */
+	g_return_if_fail (menuitem);
+
 	window_set_fullscreen_mode (window, menuitem->active);
 }
 
@@ -739,6 +779,14 @@ window_menu_enable_proxy_activate_cb (GtkCheckMenuItem *menuitem,
 }
 
 
+void
+window_menu_autojump_activate_cb (GtkCheckMenuItem *menuitem,
+					   GaleonWindow *window)
+{
+	return_if_not_window (window);
+	preferences_set_autojump(menuitem->active);
+}
+
 void
 window_menu_own_fonts_activate_cb (GtkCheckMenuItem *menuitem,
 				   GaleonWindow *window)
@@ -810,11 +858,11 @@ window_allow_images_from_activate_cb (GtkMenuItem *menuitem,
 }
 
 void
-window_view_stored_sites_activate_cb (GtkMenuItem *menuItem,
+window_view_image_sites_activate_cb (GtkMenuItem *menuItem,
 				      GaleonWindow *window)
 {
 	return_if_not_window (window);
-	cookies_show_page (IMAGESITES);
+	pdm_show_page (window, IMAGESITES);
 }
 
 void
@@ -851,11 +899,19 @@ window_menu_animate_never_activate_cb (GtkCheckMenuItem *menuitem,
 }
 
 void
-window_cookie_manager_launch_cb (GtkMenuItem *menuitem,
+window_view_cookies_activate_cb (GtkMenuItem *menuitem,
 				 GaleonWindow *window)
 {
 	return_if_not_window (window);
-	cookies_show_manager ();
+	pdm_show_page (window, COOKIES);
+}
+
+void
+window_view_cookie_sites_activate_cb (GtkMenuItem *menuitem,
+				      GaleonWindow *window)
+{
+	return_if_not_window (window);
+	pdm_show_page (window, COOKIESITES);
 }
 
 void
@@ -920,26 +976,102 @@ window_menu_encoding_activate_cb (GtkWidget *menuitem,
 	embed_reload (window->active_embed);
 }
 
+/*
+ * window_menu_encoding_state_changed_cb: 
+ * handle the first use of the encoding_menu to create the submenus
+ */
+void 
+window_menu_encoding_state_changed_cb (GtkMenuItem *encoding_menu,
+				       GtkStateType state, 
+				       GaleonWindow *window)
+{
+	/* return immediately if this is not an entry to the widget */
+	if (state != GTK_STATE_NORMAL)
+	{
+		return;
+	}
+
+	/* get the charset titles and names from mozilla, if still not done */
+	if (charsets == NULL)
+	{
+		mozilla_get_charsets (&charsets, &sorted_charset_titles);
+	}
+
+	/* create the submenu if we already havent havent*/
+	if (!window->encodings_menu_set)
+	{
+		/* kill the dummy */
+		gtk_widget_destroy (encoding_menu->submenu);
+		encoding_menu->submenu = NULL;
+		/* set the menu */
+		window_create_charset_submenus (window, sorted_charset_titles);
+		window->encodings_menu_set = TRUE;
+	}
+}
+
 void
 window_view_passwords_activate_cb (GtkMenuItem *menuItem,
 			           GaleonWindow *window)
 {
 	return_if_not_window (window);
-	cookies_show_page (PASSWORDS);
+	pdm_show_page (window, PASSWORDS);
 }
 void
 window_view_password_sites_activate_cb (GtkMenuItem *menuItem,
 				        GaleonWindow *window)
 {
 	return_if_not_window (window);
-	cookies_show_page (PASSWORDSITES);
+	pdm_show_page (window, PASSWORDSITES);
 }
 
-void window_menu_offline_mode_activate_cb (GtkCheckMenuItem *menuitem,
-					   GaleonWindow *window)
+void
+window_menu_offline_mode_activate_cb (GtkCheckMenuItem *menuitem,
+				      GaleonWindow *window)
 {
 	return_if_not_window (window);
 	return_if_not_embed (window->active_embed);
 
 	mozilla_set_offline(window->active_embed, menuitem->active);
 }
+
+void
+window_menu_save_session_as_activate_cb (GtkWidget *menuitem,
+					 GaleonWindow *window)
+{
+	gchar *file = NULL;
+	/* FIXME: add a pref for this? Marco could kill me... */
+	gchar *sessions_dir = g_concat_dir_and_file (g_get_home_dir (), 
+						     "/.galeon/sessions");
+	if (show_file_picker (window->WMain, _("Save session as..."),
+			      sessions_dir, NULL, modeSave,
+			      &file))
+	{
+		if (dialog_check_overwrite_file (file, NULL)) 
+		{
+			session_save_to (file);
+			session_history_add (file);
+		}
+		g_free (file);
+	}
+	g_free (sessions_dir);
+}
+
+void
+window_menu_open_session_activate_cb (GtkWidget *menuitem,
+				      GaleonWindow *window)
+{
+	gchar *file = NULL;
+	/* FIXME: add a pref for this? Marco could kill me... */
+	gchar *sessions_dir = g_concat_dir_and_file (g_get_home_dir (), 
+						     "/.galeon/sessions");
+	if (show_file_picker (window->WMain, _("Open session..."),
+			      sessions_dir, NULL, modeOpen,
+			      &file)) 
+	{
+		session_load_from (file);
+		session_history_add (file);
+	}
+	
+	g_free (file);
+	g_free (sessions_dir);
+}
