@@ -24,6 +24,26 @@
 #include <libgnome/gnome-i18n.h>
 #include <gdk/gdkkeysyms.h>
 
+/* 
+ * a nonsense needed to get some of the GNOMEUINFO defined strings translated 
+ * blame xgettext 
+ */
+#if 0
+static char *ugh[] = {
+	/* File */
+		N_("_Open..."), N_("Open a file"),
+		N_("E_xit"), N_("Exit the program"),
+	/* Edit */
+		N_("C_ut"), N_("Cut the selection"),
+		N_("_Copy"), N_("Copy the selection"),
+		N_("_Paste"), N_("Paste the clipboard"),
+	/* Settings */
+		N_("_Preferences..."), N_("Configure the application"),
+	/* Help */
+		N_("_About..."), N_("About this application")
+}
+#endif
+
 /* The menubar GnomeUIInfo structs */ 
 GnomeUIInfo file_menu_uiinfo[] =
 {
@@ -40,19 +60,34 @@ GnomeUIInfo file_menu_uiinfo[] =
 	  window_menu_open_url_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_OPEN,
 	  GDK_o, GDK_CONTROL_MASK, NULL },
-	GNOMEUIINFO_MENU_SAVE_AS_ITEM (window_menu_save_as_activate_cb, NULL),
+	{ GNOME_APP_UI_ITEM, N_("Save _As..."),
+	  N_("Save the current page with a different name"),
+	  window_menu_save_as_activate_cb, NULL, NULL,
+	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_SAVE_AS,
+	  GDK_S, GDK_CONTROL_MASK, NULL },
 	GNOMEUIINFO_SEPARATOR,
 	{ GNOME_APP_UI_ITEM, N_("_Print..."), NULL,
 	  window_menu_print_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PRINT,
-	  0, 0, NULL },
+	  GDK_P, GDK_CONTROL_MASK, NULL },
 	GNOMEUIINFO_SEPARATOR,
 	{ GNOME_APP_UI_TOGGLEITEM, N_("Work O_ffline"), NULL,
 	  window_menu_offline_mode_activate_cb, NULL, NULL,
-	  GNOME_APP_PIXMAP_NONE, NULL,
-	  GDK_T, GDK_CONTROL_MASK, NULL },
+	  GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL },
 	GNOMEUIINFO_SEPARATOR,
-	GNOMEUIINFO_MENU_CLOSE_ITEM (window_menu_close_activate_cb, NULL),
+	{ GNOME_APP_UI_ITEM, N_("Sa_ve session as..."),
+	  N_("Save the location of each open window to a file"),
+	  window_menu_save_session_as_activate_cb, NULL, NULL,
+	  GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL },
+	{ GNOME_APP_UI_ITEM, N_("_Open session..."),
+	  N_("Restore a previous browsing session from a file"),
+	  window_menu_open_session_activate_cb, NULL, NULL,
+	  GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL },
+	GNOMEUIINFO_SEPARATOR,
+	{ GNOME_APP_UI_ITEM, N_("_Close"), N_("Close the current browser"),
+	  window_menu_close_activate_cb, NULL, NULL,
+	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_CLOSE,
+	  GDK_w, GDK_CONTROL_MASK, NULL },
 	{ GNOME_APP_UI_ITEM, N_("Exit with _session"),
 	  N_("Exit the program, saving the location of each open window"),
 	  window_menu_exit_with_session_activate_cb, NULL, NULL,
@@ -69,17 +104,34 @@ GnomeUIInfo edit_menu_uiinfo[] =
 
 	GNOMEUIINFO_SEPARATOR,
 
-	GNOMEUIINFO_MENU_FIND_ITEM (window_menu_find_activate_cb, NULL),
+	{ GNOME_APP_UI_ITEM, N_("_Find..."),
+	  N_("Search for a word or phrase in the current page"),
+	  window_menu_find_activate_cb, NULL, NULL,
+	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_SEARCH,
+	  GDK_F, GDK_CONTROL_MASK, NULL },
+	GNOMEUIINFO_END
+};
+
+GnomeUIInfo zoom_menu_uiinfo[] =
+{
+	{ GNOME_APP_UI_ITEM, N_("_In"), NULL,
+	  window_menu_zoom_in_activate_cb, NULL, NULL,
+ 	  GNOME_APP_PIXMAP_NONE, NULL,
+	  GDK_equal, GDK_CONTROL_MASK, NULL },
+	  /* xgettext:no-c-format */
+	{ GNOME_APP_UI_ITEM, N_("_100%"), NULL,
+	  window_menu_zoom_100_activate_cb, NULL, NULL,
+ 	  GNOME_APP_PIXMAP_NONE, NULL,
+	  GDK_0, GDK_CONTROL_MASK, NULL },
+	{ GNOME_APP_UI_ITEM, N_("_Out"), NULL,
+	  window_menu_zoom_out_activate_cb, NULL, NULL,
+	  GNOME_APP_PIXMAP_NONE, NULL,
+	  GDK_minus, GDK_CONTROL_MASK, NULL },
 	GNOMEUIINFO_END
 };
 
 GnomeUIInfo view_menu_uiinfo[] =
 {
-	{ GNOME_APP_UI_ITEM, N_("_Page source"), NULL,
-	  window_menu_view_source_activate_cb, NULL, NULL,
-	  GNOME_APP_PIXMAP_NONE, NULL,
-	  GDK_S, GDK_CONTROL_MASK, NULL },
-	GNOMEUIINFO_SEPARATOR,
 	{ GNOME_APP_UI_TOGGLEITEM, N_("_Menubar"), NULL,
 	  window_menu_menubar_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
@@ -91,52 +143,52 @@ GnomeUIInfo view_menu_uiinfo[] =
 	{ GNOME_APP_UI_TOGGLEITEM, N_("_Statusbar"), NULL,
 	  window_menu_statusbar_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
-	  GDK_U, GDK_CONTROL_MASK, NULL },
-	{ GNOME_APP_UI_TOGGLEITEM, N_("_Bookmarks Dock"), NULL,
-	  window_menu_bookmarks_dock_activate_cb, NULL, NULL,
-	  GNOME_APP_PIXMAP_NONE, NULL,
-	  GDK_B, GDK_CONTROL_MASK | GDK_SHIFT_MASK, NULL },
-	{ GNOME_APP_UI_TOGGLEITEM, N_("_History Dock"), NULL,
-	  window_menu_history_dock_activate_cb, NULL, NULL,
-	  GNOME_APP_PIXMAP_NONE, NULL,
-	  GDK_H, GDK_CONTROL_MASK | GDK_SHIFT_MASK, NULL },
+	  GDK_S, GDK_CONTROL_MASK | GDK_SHIFT_MASK, NULL },
 	{ GNOME_APP_UI_TOGGLEITEM, N_("_Full screen"), NULL,
 	  window_menu_fullscreen_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  GDK_F11, 0, NULL },
 	GNOMEUIINFO_SEPARATOR,
-	{ GNOME_APP_UI_ITEM, N_("Zoom _in"), NULL,
-	  window_menu_zoom_in_activate_cb, NULL, NULL,
- 	  GNOME_APP_PIXMAP_NONE, NULL,
-	  GDK_equal, GDK_CONTROL_MASK, NULL },
-	{ GNOME_APP_UI_ITEM, N_("Zoom 100%"), NULL,
-	  window_menu_zoom_100_activate_cb, NULL, NULL,
- 	  GNOME_APP_PIXMAP_NONE, NULL,
-	  GDK_0, GDK_CONTROL_MASK, NULL },
-	{ GNOME_APP_UI_ITEM, N_("Zoom _out"), NULL,
-	  window_menu_zoom_out_activate_cb, NULL, NULL,
+	{ GNOME_APP_UI_SUBTREE, N_("_Zoom"), NULL,
+	  zoom_menu_uiinfo, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
-	  GDK_minus, GDK_CONTROL_MASK, NULL },
-	GNOMEUIINFO_SEPARATOR,
-	{ GNOME_APP_UI_ITEM, N_("Pre_vious tab"), NULL,
-	  window_menu_prev_tab_activate_cb, NULL, NULL,
-	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_UNDO,
-	  GDK_comma, GDK_CONTROL_MASK, NULL },
-	{ GNOME_APP_UI_ITEM, N_("Next ta_b"), NULL,
-	  window_menu_next_tab_activate_cb, NULL, NULL,
-	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_REDO,
-	  GDK_period, GDK_CONTROL_MASK, NULL },
-	GNOMEUIINFO_SEPARATOR,
-	{ GNOME_APP_UI_ITEM, N_("_History..."),
-	  N_("Log of visited pages"),
-	  window_menu_history_activate_cb, NULL, NULL,
-	  GNOME_APP_PIXMAP_NONE, NULL,
-	  GDK_H, GDK_CONTROL_MASK, NULL },
-	GNOMEUIINFO_SEPARATOR,
+	  0, 0, NULL },
 	{ GNOME_APP_UI_ITEM, N_("_Encoding"), NULL,
 	  NULL, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  0, 0, NULL },
+	GNOMEUIINFO_SEPARATOR,
+	{ GNOME_APP_UI_ITEM, N_("_Page source"), NULL,
+	  window_menu_view_source_activate_cb, NULL, NULL,
+	  GNOME_APP_PIXMAP_NONE, NULL,
+	  GDK_U, GDK_CONTROL_MASK, NULL },
+	GNOMEUIINFO_END
+};
+
+GnomeUIInfo tab_menu_uiinfo[] =
+{
+	{ GNOME_APP_UI_ITEM, N_("_Detach tab"), NULL,
+	  window_menu_detach_tab_activate_cb, NULL, NULL,
+	  GNOME_APP_PIXMAP_NONE, NULL,
+	  GDK_I, GDK_CONTROL_MASK, NULL },
+	GNOMEUIINFO_SEPARATOR,
+	{ GNOME_APP_UI_ITEM, N_("_Previous tab"), NULL,
+	  window_menu_prev_tab_activate_cb, NULL, NULL,
+	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_UNDO,
+	  GDK_comma, GDK_CONTROL_MASK, NULL },
+	{ GNOME_APP_UI_ITEM, N_("_Next tab"), NULL,
+	  window_menu_next_tab_activate_cb, NULL, NULL,
+	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_REDO,
+	  GDK_period, GDK_CONTROL_MASK, NULL },
+	GNOMEUIINFO_SEPARATOR,
+	{ GNOME_APP_UI_ITEM, N_("Move tab _left / up"), NULL,
+	  window_menu_shift_tab_left_activate_cb, NULL, NULL,
+	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_BACK, GDK_less,
+	  GDK_CONTROL_MASK | GDK_SHIFT_MASK, NULL },
+	{ GNOME_APP_UI_ITEM, N_("Move tab _right / down"), NULL,
+	  window_menu_shift_tab_right_activate_cb, NULL, NULL,
+	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_FORWARD, GDK_greater,
+	  GDK_CONTROL_MASK | GDK_SHIFT_MASK, NULL },
 	GNOMEUIINFO_END
 };
 
@@ -170,8 +222,8 @@ GnomeUIInfo load_images_menu_uiinfo[] =
 	  window_block_images_from_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  0, 0, NULL },
-	{ GNOME_APP_UI_ITEM, N_("_View stored sites..."), NULL,
-	  window_view_stored_sites_activate_cb, NULL, NULL,
+	{ GNOME_APP_UI_ITEM, N_("_View image sites..."), NULL,
+	  window_view_image_sites_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  0, 0, NULL },
 	GNOMEUIINFO_END
@@ -203,8 +255,12 @@ GnomeUIInfo animate_images_menu_uiinfo[] =
 
 GnomeUIInfo cookies_menu_uiinfo[] =
 {
-	{ GNOME_APP_UI_ITEM, N_("Cookie _Manager..."), NULL,
-	  window_cookie_manager_launch_cb, NULL, NULL,
+	{ GNOME_APP_UI_ITEM, N_("View _cookies..."), NULL,
+	  window_view_cookies_activate_cb, NULL, NULL,
+	  GNOME_APP_PIXMAP_NONE, NULL,
+	  0, 0, NULL },
+	{ GNOME_APP_UI_ITEM, N_("View cookie _sites..."), NULL,
+	  window_view_cookie_sites_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  0, 0, NULL },
 	GNOMEUIINFO_SEPARATOR,
@@ -243,14 +299,6 @@ GnomeUIInfo settings_menu_uiinfo[] =
 	  animate_images_menu_uiinfo, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  0, 0, NULL },
-	{ GNOME_APP_UI_SUBTREE, N_("_Cookies"), NULL,
-	  cookies_menu_uiinfo, NULL, NULL,
-	  GNOME_APP_PIXMAP_NONE, NULL,
-	  0, 0, NULL },
-	{ GNOME_APP_UI_SUBTREE, N_("_Passwords"), NULL,
-	  passwords_menu_uiinfo, NULL, NULL,
-	  GNOME_APP_PIXMAP_NONE, NULL,
-	  0, 0, NULL },
 	GNOMEUIINFO_SEPARATOR,
 	{ GNOME_APP_UI_TOGGLEITEM, N_("Use own _fonts"),
 	  N_("Use own fonts instead of document provided fonts"),
@@ -273,6 +321,10 @@ GnomeUIInfo settings_menu_uiinfo[] =
         { GNOME_APP_UI_TOGGLEITEM, N_("Enable pro_xy"), NULL,
           window_menu_enable_proxy_activate_cb, NULL, NULL,
           GNOME_APP_PIXMAP_NONE, NULL,
+	  0, 0, NULL },
+	{ GNOME_APP_UI_TOGGLEITEM, N_("Auto-jump to new tabs"), NULL,
+	  window_menu_autojump_activate_cb, NULL, NULL,
+	  GNOME_APP_PIXMAP_NONE, NULL,
           0, 0, NULL },
 	GNOMEUIINFO_SEPARATOR,
 	GNOMEUIINFO_MENU_PREFERENCES_ITEM (window_menu_prefs_activate_cb, NULL),
@@ -292,7 +344,7 @@ GnomeUIInfo bookmarks_menu_uiinfo[] =
 	{ GNOME_APP_UI_ITEM, N_("Add _bookmark"), NULL,
 	  window_menu_add_bookmark_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
-	  GDK_A, GDK_CONTROL_MASK, NULL },
+	  GDK_D, GDK_CONTROL_MASK, NULL },
 	GNOMEUIINFO_SEPARATOR,
 	GNOMEUIINFO_END
 };
@@ -326,32 +378,61 @@ GnomeUIInfo go_menu_uiinfo[] =
 	GNOMEUIINFO_SEPARATOR,
 	GNOMEUIINFO_END
 };
-	
+
+GnomeUIInfo tools_menu_uiinfo[] =
+{
+	{ GNOME_APP_UI_SUBTREE, N_("_Cookies"), NULL,
+	  cookies_menu_uiinfo, NULL, NULL,
+	  GNOME_APP_PIXMAP_NONE, NULL,
+	  0, 0, NULL },
+	{ GNOME_APP_UI_SUBTREE, N_("_Passwords"), NULL,
+	  passwords_menu_uiinfo, NULL, NULL,
+	  GNOME_APP_PIXMAP_NONE, NULL,
+	  0, 0, NULL },
+	GNOMEUIINFO_SEPARATOR,
+	{ GNOME_APP_UI_ITEM, N_("_History..."),
+	  N_("Log of visited pages"),
+	  window_menu_history_activate_cb, NULL, NULL,
+	  GNOME_APP_PIXMAP_NONE, NULL,
+	  GDK_H, GDK_CONTROL_MASK, NULL },
+	GNOMEUIINFO_SEPARATOR, 
+	{ GNOME_APP_UI_ITEM, N_("_Rerun first time druid..."),
+	  N_("Lets run the first time druid again"),
+	  window_run_first_time_druid_activate_cb, NULL, NULL,
+	  GNOME_APP_PIXMAP_NONE, NULL,
+	  0, 0, NULL },
+	GNOMEUIINFO_END
+};	
+
 GnomeUIInfo help_menu_uiinfo[] =
 {
 	{ GNOME_APP_UI_ITEM, N_("_Galeon manual"), NULL,
-	  window_menu_manual_activate_cb, NULL, NULL,
+	  window_menu_help_manual_activate_cb, NULL, NULL,
+	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_BOOK_BLUE,
+	  0, 0, NULL },
+	{ GNOME_APP_UI_ITEM, N_("Galeon _homepage"), NULL,
+	  window_menu_help_homepage_activate_cb, NULL, NULL,
+	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_HOME,
+	  0, 0, NULL },
+	{ GNOME_APP_UI_ITEM, N_("_Report a bug"), NULL,
+	  window_menu_help_bugreport_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  0, 0, NULL },
-	GNOMEUIINFO_MENU_ABOUT_ITEM (window_menu_about_activate_cb, NULL),
+	GNOMEUIINFO_MENU_ABOUT_ITEM (window_menu_help_about_activate_cb, NULL),
 	GNOMEUIINFO_END
 };
 
 GnomeUIInfo menubar_uiinfo[] =
 {
-	GNOMEUIINFO_MENU_FILE_TREE (file_menu_uiinfo),
-	GNOMEUIINFO_MENU_EDIT_TREE (edit_menu_uiinfo),
-	GNOMEUIINFO_MENU_VIEW_TREE (view_menu_uiinfo),
-	GNOMEUIINFO_MENU_SETTINGS_TREE (settings_menu_uiinfo),
-	{ GNOME_APP_UI_SUBTREE, N_("B_ookmarks"), NULL,
-	  bookmarks_menu_uiinfo, NULL, NULL,
-	  GNOME_APP_PIXMAP_NONE, NULL,
-	  0, 0, NULL },
-	{ GNOME_APP_UI_SUBTREE, N_("_Go"), NULL,
-	  go_menu_uiinfo, NULL, NULL,
-	  GNOME_APP_PIXMAP_NONE, NULL,
-	  0, 0, NULL },
-	GNOMEUIINFO_MENU_HELP_TREE (help_menu_uiinfo),
+	GNOMEUIINFO_SUBTREE (N_("_File"), file_menu_uiinfo),
+	GNOMEUIINFO_SUBTREE (N_("_Edit"), edit_menu_uiinfo),
+	GNOMEUIINFO_SUBTREE (N_("_View"), view_menu_uiinfo),
+	GNOMEUIINFO_SUBTREE (N_("_Tab"), tab_menu_uiinfo),
+	GNOMEUIINFO_SUBTREE (N_("_Settings"), settings_menu_uiinfo),
+	GNOMEUIINFO_SUBTREE (N_("_Bookmarks"), bookmarks_menu_uiinfo),
+	GNOMEUIINFO_SUBTREE (N_("_Go"), go_menu_uiinfo),
+	GNOMEUIINFO_SUBTREE (N_("T_ools"), tools_menu_uiinfo),
+	GNOMEUIINFO_SUBTREE (N_("_Help"), help_menu_uiinfo),
 	GNOMEUIINFO_END
 };
 
@@ -359,7 +440,9 @@ GnomeUIInfo *all_menus[] =
 {
 	file_menu_uiinfo,
 	edit_menu_uiinfo,
+	zoom_menu_uiinfo,
 	view_menu_uiinfo,
+	tab_menu_uiinfo,
 	load_images_always_uiinfo,
 	load_images_menu_uiinfo,
 	animate_always_uiinfo,
@@ -369,6 +452,7 @@ GnomeUIInfo *all_menus[] =
 	settings_menu_uiinfo,
 	bookmarks_menu_uiinfo,
 	go_menu_uiinfo,
+	tools_menu_uiinfo,
 	help_menu_uiinfo
 };
 
@@ -376,7 +460,9 @@ gint menus_num_items[] =
 {
 	sizeof(file_menu_uiinfo)           / sizeof(GnomeUIInfo),
 	sizeof(edit_menu_uiinfo)           / sizeof(GnomeUIInfo),
+	sizeof(zoom_menu_uiinfo)           / sizeof(GnomeUIInfo),
 	sizeof(view_menu_uiinfo)           / sizeof(GnomeUIInfo),
+	sizeof(tab_menu_uiinfo)            / sizeof(GnomeUIInfo),
 	sizeof(load_images_always_uiinfo)  / sizeof(GnomeUIInfo),
 	sizeof(load_images_menu_uiinfo)    / sizeof(GnomeUIInfo),
 	sizeof(animate_always_uiinfo)      / sizeof(GnomeUIInfo),
@@ -386,6 +472,7 @@ gint menus_num_items[] =
 	sizeof(settings_menu_uiinfo)       / sizeof(GnomeUIInfo),
 	sizeof(bookmarks_menu_uiinfo)      / sizeof(GnomeUIInfo),
 	sizeof(go_menu_uiinfo)             / sizeof(GnomeUIInfo),
+	sizeof(tools_menu_uiinfo)          / sizeof(GnomeUIInfo),
 	sizeof(help_menu_uiinfo)           / sizeof(GnomeUIInfo)
 };
 
