@@ -29,10 +29,14 @@ void window_menu_zoom_out_activate_cb (GtkCheckMenuItem *menuitem,
                                        GaleonWindow *window);
 void window_menu_bookmarks_edit_activate_cb (GtkMenuItem *menuitem,
                                              GaleonWindow *window);
-void window_menu_about_activate_cb (GtkMenuItem *menuitem,
-                                    GaleonWindow *window);
-void window_menu_manual_activate_cb (GtkMenuItem *menuitem,
-                                     GaleonWindow *window);
+void window_menu_help_about_activate_cb (GtkMenuItem *menuitem,
+					 GaleonWindow *window);
+void window_menu_help_manual_activate_cb (GtkMenuItem *menuitem,
+					  GaleonWindow *window);
+void window_menu_help_homepage_activate_cb (GtkMenuItem *menuitem,
+					    GaleonWindow *window);
+void window_menu_help_bugreport_activate_cb (GtkMenuItem *menuitem,
+					     GaleonWindow *window);
 void window_menu_new_window_activate_cb (GtkMenuItem *menuitem,
                                          GaleonWindow *window);
 void window_menu_new_tab_activate_cb (GtkMenuItem *menuitem,
@@ -67,10 +71,6 @@ void window_menu_statusbar_activate_cb (GtkCheckMenuItem *menuitem,
                                         GaleonWindow *window);
 void window_menu_menubar_activate_cb (GtkCheckMenuItem *menuitem,
                                       GaleonWindow *window);
-void window_menu_bookmarks_dock_activate_cb (GtkCheckMenuItem *menuitem,
-					     GaleonWindow *window);
-void window_menu_history_dock_activate_cb (GtkCheckMenuItem *menuitem,
-					   GaleonWindow *window);
 void window_menu_fullscreen_activate_cb (GtkCheckMenuItem *menuitem,
                                          GaleonWindow *window);
 void window_menu_own_colors_activate_cb (GtkCheckMenuItem *menuitem,
@@ -85,6 +85,8 @@ void window_menu_enable_proxy_activate_cb (GtkCheckMenuItem *menuitem,
                                            GaleonWindow *window);
 void window_menu_enable_java_activate_cb (GtkCheckMenuItem *menuitem,
                                           GaleonWindow *window);
+void window_menu_autojump_activate_cb (GtkCheckMenuItem *menuitem,
+					  GaleonWindow *window);
 void window_menu_images_never_activate_cb (GtkCheckMenuItem *menuitem,
                                            GaleonWindow *window);
 void window_menu_images_current_server_activate_cb (GtkCheckMenuItem *menuitem,
@@ -103,10 +105,16 @@ void window_menu_up_activate_cb (GtkMenuItem *menuitem,
 				 GaleonWindow *window);
 void window_menu_forward_activate_cb (GtkMenuItem *menuitem,
                                       GaleonWindow *window);
+void window_menu_detach_tab_activate_cb (GtkMenuItem *menuitem,
+					 GaleonWindow *window);
 void window_menu_prev_tab_activate_cb (GtkMenuItem *menuitem,
 				       GaleonWindow *window);
 void window_menu_next_tab_activate_cb (GtkMenuItem *menuitem,
 				       GaleonWindow *window);
+void window_menu_shift_tab_left_activate_cb (GtkMenuItem *menuitem,
+					     GaleonWindow *window);
+void window_menu_shift_tab_right_activate_cb (GtkMenuItem *menuitem,
+					      GaleonWindow *window);
 void window_menu_stop_activate_cb (GtkMenuItem *menuitem,
                                    GaleonWindow *window);
 void window_menu_reload_activate_cb (GtkMenuItem *menuitem,
@@ -115,7 +123,9 @@ void window_menu_home_page_activate_cb (GtkMenuItem *menuitem,
                                         GaleonWindow *window);   
 void window_menu_find_activate_cb (GtkMenuItem *menuitem,
                                    GaleonWindow *window);
-void window_menu_edit_add_events (GaleonWindow *window);
+gboolean window_menu_edit_expose_event_cb (GtkWidget *editmenu, 
+					   GdkEvent *event,
+					   GaleonWindow *window);
 void window_menu_edit_cut_cb (GtkMenuItem *menuitem,
                               GaleonWindow *window);
 void window_menu_edit_copy_cb (GtkMenuItem *menuitem,
@@ -128,16 +138,20 @@ void window_menu_view_source_activate_cb (GtkMenuItem *menuitem,
                                           GaleonWindow *window);
 void window_menu_history_activate_cb (GtkMenuItem *menuitem,
                                       GaleonWindow *window);
+void window_run_first_time_druid_activate_cb (GtkMenuItem *menuitem,
+					      GaleonWindow *window);
 void window_menu_open_url_activate_cb (GtkMenuItem *menuitem,
                                        GaleonWindow *window);
 void window_block_images_from_activate_cb (GtkMenuItem *menuitem,
                                            GaleonWindow *window);
 void window_allow_images_from_activate_cb (GtkMenuItem *menuitem,
                                            GaleonWindow *window); 
-void window_view_stored_sites_activate_cb (GtkMenuItem *menuItem,
-                                           GaleonWindow *window);
-void window_cookie_manager_launch_cb (GtkMenuItem *menuitem,   
+void window_view_image_sites_activate_cb (GtkMenuItem *menuItem,
+                                          GaleonWindow *window);
+void window_view_cookies_activate_cb (GtkMenuItem *menuitem,   
                                       GaleonWindow *window);   
+void window_view_cookie_sites_activate_cb (GtkMenuItem *menuitem,   
+                                           GaleonWindow *window);   
 void window_block_cookies_from_activate_cb (GtkMenuItem *menuitem,
                                             GaleonWindow *window);
 void window_allow_cookies_from_activate_cb (GtkMenuItem *menuitem,
@@ -148,6 +162,18 @@ void window_view_password_sites_activate_cb (GtkMenuItem *menuItem,
                                              GaleonWindow *window);
 void window_menu_offline_mode_activate_cb (GtkCheckMenuItem *menuitem,
 					   GaleonWindow *window);
+void window_menu_bookmark_activate_cb  (GtkWidget *menuitem, 
+					gpointer user_data);
+void window_menu_encoding_activate_cb (GtkWidget *menuitem,
+				       GaleonWindow *window);
+void window_menu_encoding_state_changed_cb (GtkMenuItem *encoding_menu,
+					    GtkStateType state, 
+					    GaleonWindow *window);
+void window_menu_save_session_as_activate_cb (GtkWidget *menuitem,
+					      GaleonWindow *window);
+void window_menu_open_session_activate_cb  (GtkWidget *menuitem,
+					    GaleonWindow *window);
+
 /* from window_callbacks.c */
 gboolean window_delete_cb (GtkWidget *widget, GdkEventAny *event,
                            GaleonWindow *window);
@@ -155,7 +181,9 @@ gboolean window_delete_cb (GtkWidget *widget, GdkEventAny *event,
 /* from menubar.c */
 extern GnomeUIInfo file_menu_uiinfo[];
 extern GnomeUIInfo edit_menu_uiinfo[];
+extern GnomeUIInfo zoom_menu_uiinfo[];
 extern GnomeUIInfo view_menu_uiinfo[];
+extern GnomeUIInfo tab_menu_uiinfo[];
 extern GnomeUIInfo load_images_always_uiinfo[];
 extern GnomeUIInfo load_images_menu_uiinfo[];
 extern GnomeUIInfo animate_always_uiinfo[];
@@ -165,6 +193,7 @@ extern GnomeUIInfo passwords_menu_uiinfo[];
 extern GnomeUIInfo settings_menu_uiinfo[];
 extern GnomeUIInfo bookmarks_menu_uiinfo[];
 extern GnomeUIInfo go_menu_uiinfo[];
+extern GnomeUIInfo tools_menu_uiinfo[];
 extern GnomeUIInfo help_menu_uiinfo[];
 extern GnomeUIInfo menubar_uiinfo[];
 extern GnomeUIInfo *all_menus[];
