@@ -27,15 +27,18 @@
 #include "session.h"
 #include "bookmarks.h"
 #include "mozilla.h"
-#include "file-operations.h"
-#include "filepicker.h"
 
 /* system includes */
 #include <errno.h>
 #include <dirent.h>
+#include <limits.h>
 
 #include <stdlib.h>
 #include <string.h>
+#include <iconv.h>
+#ifdef HAVE_CODESET
+#include <langinfo.h>
+#endif
 #include <gtk/gtkmain.h>
 #include <gtk/gtkrc.h>
 #include <gtk/gtkcheckmenuitem.h>
@@ -47,92 +50,22 @@
 #include <libgnome/gnome-util.h>
 #include <libgnome/gnome-exec.h>
 #include <libgnome/gnome-config.h>
+#include <gdk-pixbuf/gdk-pixbuf.h>
 
 /* GNOME includes */
 #include <libgnomevfs/gnome-vfs.h>
 #include <libgnomevfs/gnome-vfs-mime.h>
-#include <gdk-pixbuf/gdk-pixbuf.h>
-#include <gmodule.h>
 
-/* external function prototypes imported from libxml1 */
-extern int isolat1ToUTF8(unsigned char* out, int outlen,
-			 const unsigned char* in, int inlen);
-extern int UTF8Toisolat1(unsigned char* out, int outlen,
-			 const unsigned char* in, int inlen);
-
-/** Styles for coloured text */
-GtkStyle *red_text_style;
-GtkStyle *yellow_text_style;
-GtkStyle *green_text_style;
-GtkStyle *blue_text_style;
-
-/* list of known protocols */
-const char *known_protocols[] =
-{
-	"http",
-	"https",
-	"file",
-	"about",
-	"chrome",
-	"resource",
-	"javascript",
-	"jar",
-	"view-source",
-       	NULL /* terminator: must be last! */
-};
+/** Styles for tab labels */
+GtkStyle *active_loading_text_style;
+GtkStyle *inactive_loading_text_style;
+GtkStyle *inactive_new_text_style;
+GtkStyle *inactive_text_style;
 
 /* Local prototypes */
-void save_url_with_dir (const gchar *url, const gchar *directory);
-void save_url_without_dir (const gchar *url);
-static void save_url_with_gtm (const gchar* url, const gchar *directory);
-static void save_url_with_command_line (const gchar *url, const gchar *command,
-					const gchar *directory);
-static void unrecognised_protocol_cb(gint reply, gpointer url);
 static gboolean is_punctuation (gchar c);
-static void favicon_download_completed_cb (gpointer data);
-static void glade_signal_connect_func (const gchar *cb_name, GtkObject *obj, 
-				       const gchar *signal_name, 
-				       const gchar *signal_data,
-				       GtkObject *conn_obj, 
-				       gboolean conn_after,
-				       gpointer user_data);
 static gchar* get_first_directory (const gchar *start_dir);
-
-/**
- * glade_lookup_widget
- */
-GtkWidget *
-glade_lookup_widget (GtkWidget *widget, const gchar *widget_name)
-{
-	GtkWidget *found_widget, *parent;
-	GladeXML *xml = NULL;
-
-	while (xml == NULL)
-	{
-		/* the following line is to allow to override the
-		 * GladeXML object or to set it to an object which was
-		 * not created by libglade. Useful for popup menus */
-		xml = gtk_object_get_data (GTK_OBJECT(widget), "widget_tree");
-		if (!xml) xml = glade_get_widget_tree(widget);
-
-		if (GTK_IS_MENU(widget))
-			parent = gtk_menu_get_attach_widget (GTK_MENU(widget));
-		else
-			parent = widget->parent;
-
-		if (parent == NULL) 
-			break;
-
-		widget = parent;
-	}
-
-	found_widget = glade_xml_get_widget(xml, widget_name);
-
-	if (!found_widget)
-		g_warning ("Widget not found: %s", widget_name);
- 
-	return found_widget;
-}
+static void pixbuf_threshold_alpha (GdkPixbuf *pixbuf);
 
 /**
  * user_file: returns the pathname of galeon shared files (e.g., galeon.glade)
@@ -189,7 +122,7 @@ user_file (gchar *fname, gboolean critical)
                         /* don't warn for the install default */
 			if (i != 4)
 			{
-				g_warning ("Using %s (usually OK)\n", file);
+				g_message ("Using %s (usually OK)\n", file);
 			}
 		}
 		else
@@ -216,355 +149,6 @@ user_file (gchar *fname, gboolean critical)
 	return file;
 }
 
-/**
- * glade_widget_new: build a new widget of the provided name, with all
- * signals attached and data set to the provided parameter.
- */
-GladeXML *
-glade_widget_new (gchar *file, const gchar *widget_name, 
-		  GtkWidget **root, gpointer data)
-{
-	GladeXML *gxml;
-	gchar *glade_file;
-
-	glade_file = user_file (file, TRUE);
-	g_return_val_if_fail (glade_file != NULL, NULL);
-
-	/* build the widget */
-	/* note that libglade automatically caches the parsed file,
-	 * so we don't need to worry about the efficiency of this */
-	gxml = glade_xml_new (glade_file, widget_name);
-	g_free (glade_file);
-	g_return_val_if_fail (gxml != NULL, NULL);
-
-	/* lookup the root widget if requested */
-	if (root != NULL)
-	{
-		*root = glade_xml_get_widget (gxml, widget_name);
-	}
-
-	/* connect signals and data */
-	glade_xml_signal_autoconnect_full
-		(gxml, glade_signal_connect_func, data);
-
-	/* return xml document for subsequent widget lookups */
-	return gxml;
-}
-
-/*
- * glade_signal_connect_func: used by glade_xml_signal_autoconnect_full
- */
-static void
-glade_signal_connect_func (const gchar *cb_name, GtkObject *obj, 
-			   const gchar *signal_name, const gchar *signal_data,
-			   GtkObject *conn_obj, gboolean conn_after,
-			   gpointer user_data)
-{
-	/** Module with all the symbols of the program */
-	static GModule *mod_self = NULL;
-	gpointer handler_func;
-
- 	/* initialize gmodule */
-	if (mod_self == NULL)
-	{
-		mod_self = g_module_open (NULL, 0);
-		g_assert (mod_self != NULL);
-	}
-
-	/*g_print( "glade_signal_connect_func: cb_name = '%s', signal_name = '%s', signal_data = '%s'\n",
-	  cb_name, signal_name, signal_data ); */
-	
-	if (g_module_symbol (mod_self, cb_name, &handler_func))
-	{
-		/* found callback */
-		if (conn_obj)
-		{
-			if (conn_after)
-			{
-				gtk_signal_connect_object_after
-					(obj, signal_name, 
-					 handler_func, conn_obj);
-			}
-			else
-			{
-				gtk_signal_connect_object
-					(obj, signal_name, 
-					 handler_func, conn_obj);
-			}
-		}
-		else
-		{
-			/* no conn_obj; use standard connect */
-			gpointer data = NULL;
-			
-			data = user_data;
-			
-			if (conn_after)
-			{
-				gtk_signal_connect_after
-					(obj, signal_name, 
-					 handler_func, data);
-			}
-			else
-			{
-				gtk_signal_connect
-					(obj, signal_name, 
-					 handler_func, data);
-			}
-		}
-	}
-	else
-	{
-		g_warning("callback function not found: %s", cb_name);
-	}
-}
-
-/**
- * handle_foreign_protocols: work out what protocol is specified
- * by a URL, return TRUE if we find a handler and execute it,
- * otherwise return FALSE to tell mozilla to try to load it
- */
-gboolean
-handle_foreign_protocols (const char *url)
-{
-	gint i, length;
-	gchar *protocol;
-	gchar *key, *result;
-	gchar *url_copy;
-
-	/* ignore any spaces preceding the url */
-	while (*url == ' ' && *url != '\0')
-		url++;
-
-	/* find the colon */
-	length = strlen(url);
-	for (i = 0; i < length && url[i] != ':' && url[i] != '.'; i++)
-		;
-	
-	/* did we run over? */
-	if (i == length)
-	{
-		/* don't know what to do here so leave it to the lizard */
-		return FALSE;
-	}
-
-	/* did we find a period? */
-	if (url[i] == '.')
-	{
-		/* must be a site designation, not a protocol */
-		return FALSE;
-	}
-
-	/* okay, duplicate this */
-	protocol = g_strndup(url, i);
-
-        /* see if it's definitely a known protocol */
-        for (i = 0; known_protocols[i] != NULL; i++)
-        {
-                if (strcasecmp (protocol, known_protocols[i]) == 0)
-		{
-			/* fine, give it to mozilla */
-			g_free(protocol);
-                        return FALSE;
-		}
-        }
-
-	/* if it's FTP, check if the user has setup to use mozilla */
-	if (strcasecmp (protocol, "ftp") == 0 &&
-	    gnome_config_get_int ("/galeon/Handlers/ftp=0") == 0)
-	{
-			/* give it to mozilla */
-			g_free(protocol);
-                        return FALSE;
-	}
-
-	/* we don't know the protocol, check to see what GNOME has */
-	/* unfortunately there's not an easy function to do this... */
-
-	/* build the config key */
-	key = g_strconcat("/Gnome/URL Handlers/", protocol, "-show", NULL);
-	g_free(protocol);
-
-	/* find it */
-	result = gnome_config_get_string(key);
-	g_free(key);
-	if (result != NULL && strlen(result) != 0)
-	{
-		/* we have a GNOME handler for this, pass it through
-		 * and tell mozilla not to try showing it */
-		g_free(result);
-		gnome_url_show(url);
-		return TRUE;
-	}
-
-	/* free */
-	if (result)
-		g_free(result);
-
-	/* no luck, so offer the user the option of trying the
-	 * default handler -- we don't do this automatically in
-	 * case the default handler is erroneously set to galeon */
-	result = gnome_config_get_string("/Gnome/URL Handlers/default-show");
-
-	/* check there is a default */
-	if (result == NULL || strlen(result) == 0)
-	{
-		/* throw the error */
-		gnome_error_dialog(_("Galeon cannot handle this protocol,\n"
-				     "and no GNOME default handler is set"));
-
-		/* free */
-		if (result) g_free(result);
-
-		/* don't let mozilla try blindly */
-		return TRUE;
-	}
- 
-	/* free */
-	g_free(result);
-
-	/* offer the choice */
-	url_copy = g_strdup(url);
-	gnome_question_dialog(_("The protocol specified is not recognised.\n\n"
-				"Would you like to try the GNOME default?"),
-			      unrecognised_protocol_cb, (gpointer)url_copy);
-
-	/* keep mozilla out of it */
-	return TRUE;
-}
-
-/**
- * first_time_cb: called if the user decides to import existing
- * netscape bookmarks or not
- */
-static void
-unrecognised_protocol_cb(gint reply, gpointer url)
-{
-        /* use default GNOME URL handler if asked to */
-        if (reply == GNOME_YES)
-	{
-		gnome_url_show((gchar *)url);
-	}
-
-	/* free the copied url */
-	g_free(url);
-}
-
-/**
- * save_url
- */
-void
-save_url (const gchar *url)
-{
-	if (gnome_config_get_bool ("/galeon/Downloading/"
-				   "ask_for_download_dir=true"))
-	{
-		gchar *dirName, *retPath = NULL;
-		gboolean ret;
-		dirName = g_strdup (gnome_config_get_string 
-				   	("/galeon/Downloading/download_dir"));
-		if (!dirName)
-			dirName = g_strdup (g_get_home_dir());
-		ret = show_file_picker (NULL, N_("Choose destination folder"),
-				        dirName, NULL, modeGetFolder, &retPath);
-		if (ret == TRUE)
-		{
-			save_url_with_dir (url, retPath);
-		}
-		g_free (dirName);
-		g_free (retPath);
-	}
-	else
-	{
-		save_url_without_dir (url);
-	}
-}
-
-void
-save_url_with_dir (const gchar *url, const char *directory)
-{
-	gint use_program;
-	gchar *command;
-
-	use_program = gnome_config_get_int (CONF_DOWNLOADING_FTP_PROGRAM);
-	command = gnome_config_get_string (CONF_DOWNLOADING_FTP_COMMAND);
- 
-	if (use_program && command != NULL)
-	{
-		if (strlen (command) > 0)
-		{
-			save_url_with_command_line (url, command, directory);
-			return;
-		}
-		g_free (command);
-	}
-
-	/* default */
-	save_url_with_gtm (url, directory);
-}
-
-void
-save_url_without_dir (const gchar *url)
-{
-	gint use_program;
-	gchar *command, *dir;
-
-	use_program = gnome_config_get_int (CONF_DOWNLOADING_FTP_PROGRAM);
-	command = gnome_config_get_string (CONF_DOWNLOADING_FTP_COMMAND);
-	dir = gnome_config_get_string ("/galeon/Downloading/download_dir");
- 
-	if (use_program && command != NULL)
-	{
-		if (strlen (command) > 0)
-		{
-			save_url_with_command_line (url, command, dir);
-			g_free (command);
-			g_free (dir);
-			return;
-		}
-		g_free (command);
-	}
-
-	/* default */
-	save_url_with_gtm (url, dir);
-	g_free (dir);
-}
-
-static void 
-save_url_with_command_line (const gchar *url, const gchar *command,
-			    const gchar *directory)
-{
-	char *full_command = g_strdup_printf(command, url);
-	int i;
-
-	gchar **argv = g_strsplit (full_command, " ", 100);
-
-	for (i=0;argv[i]!=NULL;i++);
-
-	gnome_execute_async (directory, i, argv);			 
-
-	g_free (full_command);
-	g_strfreev (argv);
-}
-
-/**
- * save_url_with_gtm
- */
-static void
-save_url_with_gtm (const gchar *url, const gchar *directory)
-{  
-	gboolean disable_auto_download;
-
-	if (gnome_config_get_bool ("/galeon/Downloading/auto_download=true"))
-		disable_auto_download = FALSE;
-	else
-		disable_auto_download = TRUE;
-
-	gtm_add_url (strdup(url), (char *)directory, FALSE,
-		     disable_auto_download);
-		
-}
-
 /**
  * Frees an array of strings
  */ 
@@ -572,9 +156,13 @@ void
 free_string_array (char *array[], int size)
 {
 	int i;
-	for (i = 0; i < size; i++)
-		g_free (array[i]);
-	g_free (array);
+	if (array)
+	{
+		for (i = 0; i < size; i++)
+			if (array[i])
+				g_free (array[i]);
+		g_free (array);
+	}
 }
 
 /**
@@ -598,7 +186,7 @@ new_num_accel_str (gint num, gchar *text, gboolean lettersok)
  * Sets a labels text with shortcut key handling in the standard _<key> accel way
  */
 void
-label_set_accel_text (gchar *text, GtkWidget *label, GtkWidget *menu, 
+label_set_accel_text (const gchar *text, GtkWidget *label, GtkWidget *menu, 
 		      GtkWidget *item)
 {
 	gint tmp_key = gtk_label_parse_uline (GTK_LABEL (label), text);
@@ -739,90 +327,6 @@ copy_to_clipboard (gchar *text, GaleonEmbed *embed)
 	}
 } 
 
-static void
-galeon_close_all_windows (void)
-{
-	GList *wl, *next;
-
-	/* force ourselves out of server mode -- this means that closing
-	 * the last window really will quit out of galeon */
-	galeon_server_mode = FALSE;
-
-	/* close all windows: this in turn will close all embeds */
-	/* at the closing of the last window galeon_exit will be called */
-	for (wl = all_windows; wl != NULL;)
-	{
-		next = wl->next;
-		window_close ((GaleonWindow *)(wl->data));
-		wl = next;
-	}
-}
-
-void galeon_quit (GaleonWindow *window)
-{
-	/* location bar history is saved automatically when the widget is
-	   destroyed. we don't want to save the text currently in the entry */
-	window_clear_url_entry (window);
-
-	/* save window size if we are not in fullscreen and if 
-	 * the window is not a popup */
-	if (!GTK_CHECK_MENU_ITEM (window->view_fullscreen)->active)
-	{	
-		gnome_config_set_int("/galeon/Appearance/winwidth", 
-				     window->WMain->allocation.width);
-		gnome_config_set_int("/galeon/Appearance/winheight", 
-				     window->WMain->allocation.height);
-	}
-
-	/* close all windows, this will also quit */
-	galeon_close_all_windows ();
-}
-
-void
-galeon_quit_with_session (void)
-{
-	gchar *filename;
-
-	/* make the session filename string */
-	filename = g_strconcat (g_get_home_dir (),
-				"/.galeon/session_saved.xml", NULL);
-
-	/* save it out */
-	session_save(filename);
-
-	/* set config marker */
-	gnome_config_set_int ("/galeon/General/session_saved", 1);
-	gnome_config_sync ();
-
-	/* close all windows and exit */
-	galeon_close_all_windows ();
-
-}
-
-void galeon_exit (void)
-{
-	gnome_config_set_int("/galeon/General/crashed", FALSE);
-
-	/* shut down galeon subsystems */
-	preferences_save ();
-	bookmarks_exit ();
-	history_exit ();
-	mozilla_save_prefs ();
-
-	/* check... */
-	g_assert (g_list_length (all_embeds) == 0);
-	g_assert (g_list_length (all_windows) == 0);
-
-	/* shut down GNOME subsystmes */
-	gnome_vfs_shutdown ();
-
-	/* absolutely no more mozembeds */
-	/* FIXME This cause galeon not exiting properly
-	  gtk_moz_embed_pop_startup ();
-	*/
-	gtk_main_quit();
-}
-
 static gboolean
 is_punctuation (gchar c)
 {
@@ -1021,38 +525,43 @@ shorten_name(const gchar *input_name, gint target_length)
 void
 initialise_colours (void)
 {
-	GdkColor red, yellow, green, blue;
 	GtkWidget *label;
 	GtkStyle  *rcstyle;
 
 	/* create a dummy label so we can fetch the associated rc style */
-	label = gtk_label_new("");
-	rcstyle = gtk_rc_get_style(label);
-	gtk_widget_destroy(label);
+	label = gtk_label_new ("");
+	rcstyle = gtk_rc_get_style (label);
+	gtk_widget_destroy (label);
 
 	/* this is needed for some (broken?) themes */
 	if (rcstyle == NULL)
 	{
 		rcstyle = gtk_style_new ();
 	}
-	
-	/* build colour structures */
-	gdk_color_parse ("#ff0000", &red);
-	gdk_color_parse ("#ffff00", &yellow);
-	gdk_color_parse ("#00ff00", &green);
-	gdk_color_parse ("#0000ff", &blue);
-
+	else
+	{
+		gtk_style_ref (rcstyle);
+	}
+		
 	/* make styles */
-	red_text_style = gtk_style_copy(rcstyle);
-	yellow_text_style = gtk_style_copy(rcstyle);
-	green_text_style = gtk_style_copy(rcstyle);
-	blue_text_style = gtk_style_copy(rcstyle);
+	active_loading_text_style = gtk_style_copy (rcstyle);
+	inactive_loading_text_style = gtk_style_copy (rcstyle);
+	inactive_new_text_style = gtk_style_copy (rcstyle);
+	inactive_text_style = gtk_style_copy (rcstyle);
 
-	/* fill in colours */
-	red_text_style->fg[0] = red;
-	yellow_text_style->fg[0] = yellow;
-	green_text_style->fg[0] = green;
-	blue_text_style->fg[0] = blue;
+	/* load colours */
+	gdk_color_parse ("#ff0000", &(active_loading_text_style->fg[0]));
+
+	inactive_loading_text_style->fg[0] = rcstyle->fg[4];
+	inactive_loading_text_style->fg[0].red = 65535;
+
+	inactive_new_text_style->fg[0] = rcstyle->fg[4];
+	inactive_new_text_style->fg[0].blue = 65535;
+
+	inactive_text_style->fg[0] = rcstyle->fg[4];
+
+	/* free */
+	gtk_style_unref (rcstyle);
 }
 
 /**
@@ -1158,47 +667,157 @@ gchar* get_mozilla_prefs_directory ()
 void
 launch_external_viewer (const gchar *filename)
 {
-	gchar *argv[2] = { NULL, (gchar *)filename };
-	gchar *prog, *msg;
+	gchar *prog, *command;
 	int pid;
 
 	g_return_if_fail(filename != NULL);
 
+	/* mark the file to be deleted upon galeon exit. */
+	mozilla_delete_temp_file_on_exit (filename);
+
 	/* get the program to execute */
 	prog = gnome_config_get_string (CONF_HANDLERS_EXTERNAL_SOURCE_VIEWER);
-	argv[0] = prog;
+	command = g_strconcat (prog," ", filename, NULL);
 
 	/* execute the viewer */
-	pid = gnome_execute_async (NULL, 2, argv);
-
-	/* check */
-	if (pid == -1)
-	{
-		msg = g_strdup_printf (_("Error executing `%s': \n%s"),
-				       prog, g_strerror (errno));
-		gnome_error_dialog (msg);
-		g_free (msg);
-	}
+	pid = gnome_execute_shell (NULL, command);
 
 	/* FIXME: need to detect child termination and remove the temporary
 	 * file -- this used to be done with "sleep", but that's a horrible
 	 * solution -- MattA */
 
 	/* free allocated string */
+	g_free (command);
 	g_free (prog);
 }
 
+
+static gchar *get_locale_charset (void)
+{
+#ifdef HAVE_CODESET
+		return nl_langinfo (CODESET);
+#else
+		gchar *charset = g_getenv ("CHARSET");
+		if (charset == NULL)
+		{
+			return "US-ASCII";
+		}
+		return charset;
+#endif
+}
+
 /**
- * xmlSetPropISOLatin1: set a property in an XML file which has value
- * encoded as ISOLatin1. This works around the strictness of libxml1.
+ * locale_to_utf8
+ */
+gchar *locale_to_utf8 (const gchar *locstr)
+{
+	gchar *utfstr;
+	size_t loclen, utflen;
+	gchar *fsave, *tsave;
+	static char *charset = NULL;
+	size_t count;
+	static iconv_t cd = (iconv_t)(-1);
+	
+	if (charset == NULL)
+	{
+		charset = get_locale_charset ();
+	}
+
+	if (cd == (iconv_t)(-1))
+	{
+		cd = iconv_open ("UTF-8", charset);
+		if (cd == (iconv_t)(-1))
+		{
+			g_warning ("iconv_open: %s", strerror (errno));
+			cd = iconv_open ("UTF-8", "US-ASCII");
+			g_assert (cd != (iconv_t)(-1));
+		}
+	}
+
+	if (locstr == NULL)
+	{
+		return NULL;
+	}
+
+	loclen = strlen (locstr);
+	utflen = MB_LEN_MAX * loclen;
+	utfstr = g_new0 (gchar, utflen + 1);
+	fsave = (gchar *)locstr;
+	tsave = utfstr;
+	count = iconv (cd, &fsave, &loclen, &tsave, &utflen);
+	if (count == -1)
+	{
+		g_warning ("locale_to_utf8: %s", strerror (errno));
+		g_free (utfstr);
+		return g_strdup (locstr);
+	}
+	return utfstr;
+}
+
+/**
+ * utf8_to_locale
+ */
+gchar *utf8_to_locale (const gchar *utfstr)
+{
+	gchar *locstr;
+	size_t utflen, loclen;
+	gchar *fsave, *tsave;
+	static char *charset = NULL;
+	size_t count;
+	static iconv_t cd = (iconv_t)(-1);
+
+	if (charset == NULL)
+	{
+		charset = get_locale_charset ();
+	}
+
+	if (cd == (iconv_t)(-1))
+	{
+		cd = iconv_open (charset, "UTF-8");
+		if (cd == (iconv_t)(-1))
+		{
+			g_warning ("iconv_open: %s", strerror (errno));
+			cd = iconv_open ("US-ASCII", "UTF-8");
+			g_assert (cd != (iconv_t)(-1));
+		}
+	}
+
+	if (utfstr == NULL)
+	{
+		return NULL;
+	}
+
+	utflen = strlen (utfstr);
+	loclen = MB_LEN_MAX * utflen;
+	locstr = g_new0 (gchar, loclen + 1);
+	fsave = (gchar *)utfstr;
+	tsave = locstr;
+	count = iconv (cd, &fsave, &utflen, &tsave, &loclen);
+	if (count == -1)
+	{
+		/* warning removed since this can happen quite innocently
+		 * and isn't really a problem -- which just can't display
+		 * the characters properly in the current locale. No need
+		 * to panic though -- MattA
+		g_warning ("utf8_to_locale: %s", strerror (errno));
+ 		 */
+		free (locstr);
+		return g_strdup (utfstr);
+	}
+	return locstr;
+}
+
+/**
+ * xmlSetPropLocale: set a property in an XML file which has value
+ * encoded as Locale. This works around the strictness of libxml1.
  */
 xmlAttrPtr
-xmlSetPropISOLatin1 (xmlNodePtr node, 
-		     const xmlChar *name, 
-		     const xmlChar *value)
+xmlSetPropLocale (xmlNodePtr node, 
+		  const xmlChar *name, 
+		  const xmlChar *value)
 {
-	xmlChar buffer[4096];
-	gint count;
+	xmlChar *buffer;
+	xmlAttrPtr ret;
 
 	/* check for nullification */
 	if (value == NULL)
@@ -1207,24 +826,28 @@ xmlSetPropISOLatin1 (xmlNodePtr node,
 	}
 
 	/* do the conversion */
-	count = isolat1ToUTF8 (buffer, 4096, value, strlen (value));
-	buffer[count] = '\0';
+	buffer = locale_to_utf8 (value);
 	
 	/* store in the node */
-	return xmlSetProp (node, name, buffer);
+	ret = xmlSetProp (node, name, buffer);
+
+	/* done with this */
+	g_free (buffer);
+
+	/* result */
+	return ret;
 }
 
 /**
- * xmlGetPropISOLatin1: get a property in an XML file which has value
- * encoded as UTF8 but was originally ISOLatin1. This works around the
+ * xmlGetPropLocale: get a property in an XML file which has value
+ * encoded as UTF8 but was originally local charset. This works around the
  * strictness of libxml1.
  */
 xmlChar *
-xmlGetPropISOLatin1 (xmlNodePtr node, const xmlChar *name)
+xmlGetPropLocale (xmlNodePtr node, const xmlChar *name)
 {
 	xmlChar *value;
-	xmlChar buffer[4096]; /* FIXME? */
-	gint count;
+	xmlChar *buffer;
 
 	/* get the value */
 	value = xmlGetProp (node, name);
@@ -1234,96 +857,184 @@ xmlGetPropISOLatin1 (xmlNodePtr node, const xmlChar *name)
 	}
 
 	/* do the conversion */
-	count = UTF8Toisolat1 (buffer, 4096, value, strlen (value));
+	buffer = utf8_to_locale (value);
 
-	/* check and do the best we can */
-	if (count >= 0)
-	{
-		/* terminate string */
-		buffer[count] = '\0';
-		xmlFree (value);
-		
-		/* copy and return */
-		return g_strdup (buffer);
-	}
-	else
-	{
-		/* return the undecoded result */
-		return value;
-	}
+	/* done with this */
+	xmlFree (value);
+
+	/* converted result */
+	return buffer;
 }
 
-char *messages [5] = {NULL, N_("Looking for a bookmarks icon"), NULL};
-
 /**
- * get_siteicon: Parse an http URL ans get its favicon if there is one
+ * xmlSetRawPropLocale:
  */
-void
-get_siteicon (GaleonEmbed *embed, gchar *url)
+xmlAttrPtr
+xmlSetRawPropLocale (xmlNodePtr node, 
+		  const xmlChar *name, 
+		  const xmlChar *value)
 {
-	GnomeVFSURI *vfs_uri;
-	gchar *favicon_url, *favicon_path;
-	GList *source_uris = NULL, *target_uris = NULL;
+	xmlChar *buffer;
+	xmlAttrPtr ret;
 
-	g_return_if_fail (url != NULL);
-	
-	vfs_uri = gnome_vfs_uri_new (url);
-	g_return_if_fail (vfs_uri != NULL);
-	
-	if (embed == NULL || !mozilla_get_favicon_location (embed, &favicon_url))
+	/* check for nullification */
+	if (value == NULL)
 	{
-		GnomeVFSURI *vfs_favicon_uri;
-		
-		while (gnome_vfs_uri_has_parent (vfs_uri))
-		{
-			GnomeVFSURI *vfs_parent_uri;
-			vfs_parent_uri = gnome_vfs_uri_get_parent (vfs_uri);
-			gnome_vfs_uri_unref (vfs_uri);
-			vfs_uri = vfs_parent_uri;
-		} 
-		vfs_favicon_uri = gnome_vfs_uri_append_file_name (vfs_uri, 
-								 "favicon.ico");
-		favicon_url = gnome_vfs_uri_to_string (vfs_favicon_uri, 0);
-		gnome_vfs_uri_unref (vfs_favicon_uri);
-	}
-	else if (favicon_url == NULL)
-	{
-		gnome_vfs_uri_unref (vfs_uri);
-		return;
+		return xmlSetRawProp (node, name, NULL);
 	}
 
-	gnome_vfs_uri_unref (vfs_uri);
+	/* do the conversion */
+	buffer = locale_to_utf8 (value);
+	
+	/* store in the node */
+	ret = xmlSetRawProp (node, name, buffer);
 
-	favicon_path = favicon_filename (favicon_url);
+	/* done with this */
+	g_free (buffer);
 
-	source_uris = g_list_append(source_uris, favicon_url);
-        target_uris = g_list_append(target_uris, favicon_path);
-
-        file_operations_copy_move (source_uris, target_uris,
-                                   FALSE,
-                                   messages, 
-				   favicon_download_completed_cb,
-				   favicon_path);
-	g_free (favicon_url);
+	/* result */
+	return ret;
 }
 
-static void 
-favicon_download_completed_cb (gpointer data)
+/**
+ * xmlGetRawPropLocale:
+ */
+xmlChar *
+xmlGetRawPropLocale (xmlNodePtr node, const xmlChar *name)
 {
-	GdkPixbuf *test;
-	char *favicon_path = (char*)data;
-	
-	test = gdk_pixbuf_new_from_file (favicon_path);
-	if (test == NULL)
+	xmlChar *value;
+	xmlChar *buffer;
+
+	/* get the value */
+	value = xmlGetRawProp (node, name);
+	if (value == NULL)
 	{
-		gnome_vfs_unlink (favicon_path);
-	}
-	else
-	{
-		gdk_pixbuf_unref (test);
+		return NULL;
 	}
 
-	g_free(data);
+	/* do the conversion */
+	buffer = utf8_to_locale (value);
+
+	/* done with this */
+	xmlFree (value);
+
+	/* result */
+	return buffer;
+}
+
+/**
+ * xmlSetRawProp: set a property in an XML file which has value encoded.
+ * This works around the newline-losingness and broken entitiy handling 
+ * of libxml1. (see bug #56487)
+ */
+xmlAttrPtr
+xmlSetRawProp (xmlNodePtr node, const xmlChar *name, const xmlChar *value)
+{
+	xmlAttrPtr attr;
+	gchar *escaped = escape_xml_prop (value);
+	attr = xmlSetProp (node, name, escaped);
+	g_free (escaped);
+	return attr;
+}
+/**
+ * xmlGetRawProp: get a property in an XML file which has been encoded
+ * This works around the newline-losingness and broken entitiy handling 
+ * of libxml1. (see bug #56487)
+ */
+xmlChar *
+xmlGetRawProp (xmlNodePtr node, const xmlChar *name)
+{
+	xmlChar *value;
+	gchar *unescaped;
+	value = xmlGetProp (node, name);
+	if (value == NULL)
+	{
+		return NULL;
+	}
+	unescaped = unescape_hexed_string (value);
+	xmlFree (value);
+	return unescaped;
+}
+/**
+ * Escape chars in string such that xmlGetProp doesn't screw it up
+ * caller is responsible for freeing the returned string
+ */
+gchar *
+escape_xml_prop (const gchar *text)
+{
+	GString *out = g_string_new (NULL);
+	guchar c;
+	while ((c = *text))
+	{
+		if (c<0x20 || c>0x7F || c=='&' || c=='<' || c=='>' || c=='%' ||
+				c=='"') //Should anything else be escaped?
+			g_string_sprintfa (out, "%%%02X", c);
+		else
+			g_string_append_c (out, c);
+		++text;
+	}
+	text = out->str;
+	g_string_free (out, FALSE); /* don't free char data, caller must */
+	return (gchar *)text;
+}
+
+gchar *
+escape_path (const gchar *text)
+{
+	GString *out = g_string_new (NULL);
+	guchar c;
+	while ((c = *text))
+	{
+		if (c=='/' ||  c==' ' || c=='?' || c=='#' || c=='%') //Should anything else be escaped?
+			g_string_sprintfa (out, "%%%02X", c);
+		else
+			g_string_append_c (out, c);
+		++text;
+	}
+	text = out->str;
+	g_string_free (out, FALSE); /* don't free char data, caller must */
+	return (gchar *)text;
+}
+
+gchar *
+unescape_hexed_string (const gchar *text)
+{
+	GString *out = g_string_new (NULL);
+	const gchar *u, *cur = text;
+	gchar decodebuf[3];
+	decodebuf[2] = '\0';
+	while ((u = strchr (cur, '%')))
+	{
+		decodebuf[0]=u[1];
+		decodebuf[1]=u[2];
+		g_string_sprintfa (out, "%.*s%c", u-cur, cur, 
+					(char)strtol(decodebuf, NULL, 16));
+		cur = u + 3;
+	}
+	if (cur && *cur)
+	{
+		/* attach rest of string */
+		g_string_append (out, cur);
+	}
+
+	u = out->str;
+	g_string_free (out, FALSE); /* don't free char data, caller must */
+	return (gchar *)u;
+}
+
+LinkState key_state_to_link_state (guint evstate)
+{
+	return ((evstate & GDK_CONTROL_MASK) ? 1 : 0) |
+		((evstate & GDK_SHIFT_MASK) ? LINKSTATE_SHIFTED : 0);
+
+}
+
+LinkState mouse_state_to_link_state (guint button, guint evstate)
+{
+	return ((button - 1) |
+		((evstate & GDK_SHIFT_MASK)   ? LINKSTATE_SHIFTED : 0) |
+		((evstate & GDK_CONTROL_MASK) ? LINKSTATE_CTRLED  : 0) |
+		((evstate & GDK_MOD1_MASK)    ? LINKSTATE_ALTED   : 0));
 }
 
 PixmapData *
@@ -1345,6 +1056,15 @@ pixmap_data_new_from_file (const gchar *file)
 		/* callers should handle this */
 		return NULL;
 	}
+
+	/* threshold out alpha if possible */
+	if (gdk_pixbuf_get_has_alpha       (pixbuf) &&
+	    gdk_pixbuf_get_bits_per_sample (pixbuf) == 8 &&
+	    gdk_pixbuf_get_colorspace      (pixbuf) == GDK_COLORSPACE_RGB &&
+	    gdk_pixbuf_get_n_channels      (pixbuf) == 4)
+	{
+		pixbuf_threshold_alpha (pixbuf);
+	}
 	
 	/* allocate space for a new pixmap */
 	pixmap_data = g_new0 (PixmapData, 1);
@@ -1360,36 +1080,67 @@ pixmap_data_new_from_file (const gchar *file)
 	return pixmap_data; 
 }
 
-gchar *
-favicon_filename (gchar *favicon_url)
+/**
+ * pixbuf_threshold_alpha: this function does the job of thresolding alpha
+ * in a pixbuf, by compositing with the default theme background colour,
+ * and marking alpha < 0.125 as alpha = 0 -- this looks nice enough against
+ * most flat, gradient and pixmap themes
+ */
+static void
+pixbuf_threshold_alpha (GdkPixbuf *pixbuf)
 {
-	GnomeVFSURI *vfs_uri = gnome_vfs_uri_new (favicon_url);
-	gchar  *result;
+	gint width, height, rowstride;
+	guchar r, g, b, a;
+	guchar tr, tg, tb;
+	guchar *buffer;
+	gint x, y, i;
+	GdkColor bg;
 	
-	/* this is the RIGHT thing to do, but it breaks for the link rel icons
-	gchar *slashpos, *filename;
-	filename = gnome_vfs_uri_to_string (vfs_uri, 
-					    GNOME_VFS_URI_HIDE_USER_NAME |
-					    GNOME_VFS_URI_HIDE_PASSWORD |
-					    GNOME_VFS_URI_HIDE_TOPLEVEL_METHOD);
-
-	while ((slashpos = strstr (filename, "/")) != NULL) *slashpos = '_';
-
-	result = g_strconcat (g_get_home_dir (), "/.galeon/favicons/",
-			      filename, NULL);
-	g_free (filename);
-	*/
+	/* get pixbuf properties */
+	width  = gdk_pixbuf_get_width (pixbuf);
+	height = gdk_pixbuf_get_height (pixbuf);
+	buffer = gdk_pixbuf_get_pixels (pixbuf);
+	rowstride = gdk_pixbuf_get_rowstride (pixbuf);
 	
-	/* some braindamaged handling until the situation is fixed. FIXME!! 
-	 * this leaves the filename as tld_favicon.ico, which isnt right
-	 * but will work until we decide to store the favicon remote location
-	 * in the bookmarks file */
-	result = g_strconcat (g_get_home_dir (), "/.galeon/favicons/",
-			      gnome_vfs_uri_get_host_name (vfs_uri), "_",
-			      "favicon.ico", NULL);
-	gnome_vfs_uri_unref (vfs_uri);
+	/* get target colour to blend with based on style background */
+	bg = active_loading_text_style->bg[0];
+	tr = bg.red   >> 8;
+	tg = bg.green >> 8;
+	tb = bg.blue  >> 8;
 
-	return result;
+	/* blend whole image */
+	for (i = 0, y = 0; y < height; y++, i += rowstride - (width * 4))
+	{
+		for (x = 0; x < width; x++, i += 4)
+		{
+			/* get pixel */
+			r = buffer[i + 0];
+			g = buffer[i + 1];
+			b = buffer[i + 2];
+			a = buffer[i + 3];
+
+			/* blend pixel */
+			r = ((a * r) + ((255 ^ a) * tr)) >> 8;
+			g = ((a * g) + ((255 ^ a) * tg)) >> 8;
+			b = ((a * b) + ((255 ^ a) * tb)) >> 8;
+
+			/* threshold alpha */
+			if (a > 16)
+			{
+				a = 255;
+			}
+			else
+			{
+				a = 0;
+			}
+			
+			/* writeback pixel */
+			buffer[i + 0] = r;
+			buffer[i + 1] = g;
+			buffer[i + 2] = b;
+			buffer[i + 3] = a;
+		}
+	}
 }
 
 /* NOTE: str must be at least 256 chars long */
@@ -1405,7 +1156,9 @@ store_time_in_string (GTime t, gchar *str)
 		localtime_r ((time_t *)&t, &stm);
 
 		/* format into string */
-		length = strftime (str, 255, _("%-m/%-d/%Y"), &stm);
+		/* this is used whenever a brief date is needed, like
+		 * in the history (for last visited, first time visited) */
+		length = strftime (str, 255, _("%Y-%m-%d"), &stm);
 		str[length] = '\0';
 	}
 	else
@@ -1414,6 +1167,30 @@ store_time_in_string (GTime t, gchar *str)
 	}
 }
 
+void
+store_full_time_in_string (GTime t, gchar *str)
+{
+	struct tm stm;
+	int length;
+
+	if (t > 0)
+	{
+		/* convert to local time */
+		localtime_r ((time_t *)&t, &stm);
+
+		/* format into string */
+		/* this is used when querying overwriting, as in "are you
+		 * sure you want to overwrite file foo, last modified ..." */
+		length = strftime (str, 255, _("%Y-%m-%d %H:%M:%S"), &stm);
+		str[length] = '\0';
+	}
+	else
+	{
+		str[0] = '\0';
+	}
+}
+
+
 gchar *
 time_to_string (GTime t)
 {
@@ -1446,27 +1223,6 @@ xmlGetIntProp (xmlNodePtr node, const gchar *attribute)
 	return value;
 }
 
-void
-lookup_widgets (GladeXML *gxml, WidgetLookup *lookup_table)
-{
-	gint i;
-	GtkWidget *value;
-
-	/* check */
-	g_return_if_fail (gxml != NULL);
-	g_return_if_fail (lookup_table != NULL);
-
-        /* lookup all of the widgets */
-        for (i = 0; lookup_table[i].name != NULL; i++)
-        {
-		g_return_if_fail (lookup_table[i].name != NULL);
-		g_return_if_fail (lookup_table[i].ptr != NULL);
-
-		value = glade_xml_get_widget (gxml, lookup_table[i].name);
-		*(lookup_table[i].ptr) = value;
-	}
-}
-
 gchar *
 misc_parse_nautilus_uri (const gchar *uri)
 {
@@ -1525,6 +1281,70 @@ misc_parse_nautilus_uri (const gchar *uri)
 	return url;
 }
 
+/**
+ * gtk_radio_button_get: get the active member of a radiobutton group 
+ * from one of the buttons in the group. This should be in GTK+!
+ */
+gint
+gtk_radio_button_get (GtkRadioButton *radio_button)
+{
+	GtkToggleButton *toggle_button;
+	gint i, length;
+        GSList *list;
+	
+	/* get group list */
+        list = gtk_radio_button_group (GTK_RADIO_BUTTON (radio_button));
+        length = g_slist_length (list);
+
+	/* iterate over list to find active button */
+	for (i = 0; list != NULL; i++, list = g_slist_next (list))
+	{
+		/* get button and text */
+		toggle_button = GTK_TOGGLE_BUTTON (list->data);
+		if (gtk_toggle_button_get_active (toggle_button))
+		{
+			break;
+		}
+	}
+
+	/* check we didn't run off end */
+	g_assert (list != NULL);
+
+	/* return index (reverse order!) */
+	return (length - 1) - i;
+}
+
+/**
+ * gtk_radio_button_set: set the active member of a radiobutton group
+ * from one of the buttons in the group. This should be in GTK+!
+ */
+void
+gtk_radio_button_set (GtkRadioButton *radio_button, gint index)
+{
+	GtkToggleButton *button;
+	GSList *list;
+	gint length;
+
+	/* get the list */
+        list = gtk_radio_button_group (GTK_RADIO_BUTTON (radio_button));
+
+	/* check out the length */
+        length = g_slist_length (list);
+
+        /* new buttons are *preppended* to the list, so button added as first 
+         * has last position in the list */
+        index = (length - 1) - index;
+
+	/* find the right button */
+        button = GTK_TOGGLE_BUTTON (g_slist_nth_data (list, index));
+
+	/* set it... this will de-activate the others in the group */
+	if (gtk_toggle_button_get_active (button) == FALSE)
+	{
+		gtk_toggle_button_set_active (button, TRUE);
+	}
+}
+
 /*************************************************************************/
 /* tree utility functions stolen from gtkctree code, GTK+ CREDITS        */
 /* these are necessary because the original GtkCTree is a total f*ck-up, */
