@@ -18,6 +18,7 @@
 
 #include "galeon.h"
 #include "mozcallbacks.h"
+#include "mozilla_prefs.h"
 #include "embed.h"
 #include "misc.h"
 #include "window.h"
@@ -26,9 +27,12 @@
 #include "session.h"
 #include "mozilla.h"
 #include "context.h"
+#include "downloader.h"
 
 #include <time.h>
 #include <string.h>
+#include <libgnomeui/gnome-app.h>
+#include <libgnomeui/gnome-app-helper.h>
 #include <libgnome/gnome-config.h>
 #include <libgnome/gnome-triggers.h>
 #include <libgnome/gnome-i18n.h>
@@ -36,6 +40,9 @@
 
 /* Flag to enable/disable mouse motion listener */
 static gboolean mouse_listener_active = FALSE;
+static gboolean 
+handle_key_without_menubar (GaleonWindow *window, 
+			    gulong key, GdkModifierType ac_mods);
 
 typedef struct
 {
@@ -53,7 +60,7 @@ static gint mouse_listener (MouseInfo *minfo);
  */
 void 
 mozembed_new_window_cb (GtkMozEmbed *dummy, GtkMozEmbed **retval, 
-			guint chromemask, GaleonEmbed *embed) 
+			guint chrome_mask, GaleonEmbed *embed) 
 {
 	GaleonEmbed *new_embed;
 	GaleonWindow *window;
@@ -68,45 +75,40 @@ mozembed_new_window_cb (GtkMozEmbed *dummy, GtkMozEmbed **retval,
 	/* check config */
 	open_in_tab = gnome_config_get_bool (CONF_APPEARANCE_TABBED_POPUPS);
 
+	/* override if it's some sort of XUL (must be some sort of dialog) */
+	if ((chrome_mask & GTK_MOZ_EMBED_FLAG_OPENASCHROME) != 0)
+	{
+		open_in_tab = FALSE;
+	}
+
 	/* create a new browser */
-	new_embed = embed_create (embed, !open_in_tab, FALSE, chromemask);
+	new_embed = embed_create (embed, !open_in_tab, FALSE, chrome_mask);
 
 	/* set the popup flag */
-	new_embed->is_popup = TRUE;
-
-	/* turn off tab display if in a new window */
-	if (!open_in_tab)
+	if (new_embed->parent_window != embed->parent_window)
 	{
-		gtk_notebook_set_show_tabs 
-			(GTK_NOTEBOOK (new_embed->parent_window->notebook),
-			 FALSE);
+		new_embed->parent_window->is_popup = TRUE;
 	}
 
 	/* return the new browser to gtkmozembed */
 	*retval = GTK_MOZ_EMBED (new_embed->mozEmbed);
 }
 
-#if MOZILLA_VERSION > VERSION2(0,9)
 void new_window_orphan_cb (GtkMozEmbedSingle *embed,
-			   GtkMozEmbed **retval, guint chromemask,
+			   GtkMozEmbed **retval, guint chrome_mask,
 			   gpointer data)
 {
 	GaleonEmbed *new_embed;
 
 	/* create a new browser */
-	new_embed = embed_create (NULL, FALSE, FALSE, chromemask);
+	new_embed = embed_create (NULL, FALSE, FALSE, chrome_mask);
 
 	/* set the popup flag */
-	new_embed->is_popup = TRUE;
-
-	/* turn off tab display */
-	gtk_notebook_set_show_tabs 
-		(GTK_NOTEBOOK (new_embed->parent_window->notebook), FALSE);
+	new_embed->parent_window->is_popup = TRUE;
 
 	/* return the new browser to gtkmozembed */
 	*retval = GTK_MOZ_EMBED (new_embed->mozEmbed);
 }
-#endif
 
 /**
  * mozembed_visibility_cb: GTKMOZEMBED SIGNAL, emitted when the toplevel
@@ -148,7 +150,11 @@ mozembed_destroy_brsr_cb (GtkMozEmbed *dummy, GaleonEmbed *embed)
 void
 mozembed_location_changed_cb (GtkMozEmbed *dummy, GaleonEmbed *embed)
 {
+	GaleonWindow *window;
+
  	return_if_not_embed (embed);
+	window = embed->parent_window;
+	return_if_not_window (window);
 
 	/* autosave the updated session */
 	session_autosave ();
@@ -163,6 +169,12 @@ mozembed_location_changed_cb (GtkMozEmbed *dummy, GaleonEmbed *embed)
 
 	/* update in gui */
 	embed_update_page_location (embed);
+
+	/* update the buttons view */
+	if (embed == window->active_embed)
+	{
+		window_update_nav_controls (window);
+	}
 }
 
 /**
@@ -174,16 +186,22 @@ mozembed_title_changed_cb (GtkMozEmbed *dummy, GaleonEmbed *embed)
 {
 	return_if_not_embed (embed);
 
-	/* get the new title */
-	if (embed->site_title != NULL)
+	/* free existing titles */
+	g_free (embed->site_title);
+	g_free (embed->site_title_utf8);
+	
+	/* get both versions of new title */
+	embed->site_title = 
+		mozilla_get_document_title (embed, &(embed->site_title_utf8));
+
+	/* reset to untitled if this title is nonsense */
+	if (embed->site_title == NULL || strlen (embed->site_title) == 0)
 	{
 		g_free (embed->site_title);
+		g_free (embed->site_title_utf8);
+		embed->site_title = g_strdup (_("Untitled"));
+		embed->site_title_utf8 = locale_to_utf8 (embed->site_title);
 	}
-#if MOZILLA_VERSION > VERSION2(0,9)
-	embed->site_title = mozilla_get_document_title (embed);
-#else
-	embed->site_title = gtk_moz_embed_get_title (GTK_MOZ_EMBED(embed->mozEmbed));
-#endif
 
 	/* update in gui */
 	embed_update_page_title (embed);
@@ -203,16 +221,17 @@ mozembed_load_started_cb (GtkMozEmbed *dummy, GaleonEmbed *embed)
 	window = embed->parent_window;
 	return_if_not_window (window);
 
+	embed->load_started++;
+
 	/* if we haven't already, start the window spinner, set the
 	 * load_started tag, and clear the info about the progress */
-	if (embed->load_started == 0)
+	if (embed->load_started == 1)
 	{
 		spinner_start (window);
-		embed_set_notebook_label_status (embed, TAB_LOADING);
+		embed_set_notebook_label_status (embed);
 		embed_progress_clear (embed);
 		embed->when_started = time (NULL);
 	}
-	embed->load_started++;
 
 	/* update the buttons view */
 	if (embed == window->active_embed)
@@ -231,6 +250,7 @@ mozembed_load_finished_cb (GtkMozEmbed *dummy, GaleonEmbed *embed)
 {
 	GaleonWindow *window;
 	gint zoom;
+	extern gboolean is_newuser;
 
 	/* check callback args */
 	return_if_not_embed (embed);
@@ -256,8 +276,14 @@ mozembed_load_finished_cb (GtkMozEmbed *dummy, GaleonEmbed *embed)
 			embed->zoom_auto_set = TRUE;
 		}
 
+		/* stop spinner (actually reduces start count) */
 		spinner_stop (window);
-		embed_set_notebook_label_status (embed, TAB_NORMAL);
+
+		/* new content, so only viewed if this is already active */
+		embed->has_been_viewed = embed->is_active;
+
+		/* set tab properly */
+		embed_set_notebook_label_status (embed);
 	}
 
 	/* update the buttons view */
@@ -266,6 +292,14 @@ mozembed_load_finished_cb (GtkMozEmbed *dummy, GaleonEmbed *embed)
 		window_update_nav_controls (window);
 		window_update_status_bar (window);
 	}
+
+	if (is_newuser)
+	{
+		mozilla_prefs_set_fonts_defaults ();
+		/* FIXME this is very slow !!! */
+		mozilla_prefs_set_fonts ();
+		is_newuser = FALSE;
+	}
 }
 
 /**
@@ -455,14 +489,17 @@ mozembed_js_status_cb (GtkMozEmbed *dummy, GaleonEmbed *embed)
  	if (gnome_config_get_bool (CONF_ADVANCED_STATUSBAR_REWRITE))
   	{
 		/* get the javascript message */
-		message = gtk_moz_embed_get_js_status 
-			(GTK_MOZ_EMBED(embed->mozEmbed));
-		
-		/* update the status bar message */
-		if (message && embed->is_active)
+		if (embed->is_active)
 		{
-			window_update_temp_message (window, message);
-			g_free (message);
+			message = gtk_moz_embed_get_js_status 
+				(GTK_MOZ_EMBED(embed->mozEmbed));
+			
+			/* update the status bar message */
+			if (message != NULL)
+			{
+				window_update_temp_message (window, message);
+				g_free (message);
+			}
 		}
 	}
 }
@@ -476,11 +513,6 @@ mozembed_open_uri_cb (GtkMozEmbed *dummy, const char *uri, GaleonEmbed *embed)
 {
 	return_val_if_not_embed (embed, TRUE);
 
-	/* handle foreign protocols, things like "mailto:" */
-	if (handle_foreign_protocols(uri))
-		return TRUE;
-
-	/* we haven't handled it, so let Mozilla try to load it */
 	return FALSE;
 }
 
@@ -497,9 +529,6 @@ mozembed_dom_mouse_down_cb (GtkMozEmbed *dummy, gpointer dom_event,
 
 	return_val_if_not_embed (embed, FALSE);
 
-	if (button_action == 0)
-		return FALSE;
-
 	info = g_new0 (WrapperMouseEventInfo, 1);
 	if (!mozilla_get_mouse_event_info (embed, dom_event, info)) 
 	{
@@ -508,12 +537,9 @@ mozembed_dom_mouse_down_cb (GtkMozEmbed *dummy, gpointer dom_event,
 		return FALSE;
 	}
 
-	/* FIXME remove this */
-	info->button ++;
-
-	if (info->button == 3)
+	if (info->button == 2)
 	{
-		if (button_action == 2)
+		if (button_action == 1)
 		{
 			MouseInfo *minfo;
 			GdkWindow *win = embed->parent_window->WMain->window;
@@ -535,7 +561,8 @@ mozembed_dom_mouse_down_cb (GtkMozEmbed *dummy, gpointer dom_event,
 			return FALSE;
 		}
 		else
-			context_show_menu (embed, &info->ctx);
+			context_show_menu (embed, &info->ctx, 3,
+					   info->timestamp);
 	}
 
 	mozilla_free_context_info_sub(&info->ctx);
@@ -544,6 +571,19 @@ mozembed_dom_mouse_down_cb (GtkMozEmbed *dummy, gpointer dom_event,
 	return FALSE;
 }
 
+static guint
+mozembed_modifier_to_gdk_state (guint modifier)
+{
+	guint state = 0;
+	if (modifier & CTRL_KEY)
+		state |= GDK_CONTROL_MASK;
+	if (modifier & SHIFT_KEY)
+		state |= GDK_SHIFT_MASK;
+	if (modifier & ALT_KEY)
+		state |= GDK_MOD1_MASK;
+	return state;
+}
+
 /**
  * mozembed_dom_mouse_click_cb: GTKMOZEMBED SIGNAL, emitted when user 
  * clicks on the document
@@ -553,7 +593,6 @@ mozembed_dom_mouse_click_cb (GtkMozEmbed *dummy, gpointer dom_event,
 			     GaleonEmbed *embed)
 {
 	GaleonWindow *window;
-	gboolean tabbed_mode;
 	WrapperMouseEventInfo *info;
 	gboolean handled = FALSE;
 
@@ -561,8 +600,6 @@ mozembed_dom_mouse_click_cb (GtkMozEmbed *dummy, gpointer dom_event,
 	window = embed->parent_window;
 	return_val_if_not_window (window, FALSE);
 
-	tabbed_mode = gnome_config_get_bool (CONF_APPEARANCE_TABBED);
-
 	info = g_new0 (WrapperMouseEventInfo,1);
 	if (!mozilla_get_mouse_event_info (embed, dom_event, info)) 
 	{
@@ -583,7 +620,7 @@ mozembed_dom_mouse_click_cb (GtkMozEmbed *dummy, gpointer dom_event,
 	{
 	case 2:
 		if (gnome_config_get_int("/galeon/Mouse/right_button_action")
-		    == 2)
+		    == 1)
 		{
 			/* kill the motion listener, if it exists */
 			if (mouse_listener_active)
@@ -593,27 +630,15 @@ mozembed_dom_mouse_click_cb (GtkMozEmbed *dummy, gpointer dom_event,
 			gtk_moz_embed_go_back (GTK_MOZ_EMBED(embed->mozEmbed));
 		}
 		else
-			context_show_menu (embed, &info->ctx);
+			context_show_menu (embed, &info->ctx, -1, 0);
 
 		handled = TRUE;
 		break;
 	case 1:
-		if ((info->ctx.context & CONTEXT_LINK) && info->ctx.link)
-		{
-			if ((info->modifier & SHIFT_KEY) &&
-			    !((info->modifier & ALT_KEY) || 
-			    (info->modifier & CTRL_KEY)))
-				embed_create_from_url (embed, info->ctx.link,
-						       FALSE, tabbed_mode);
-			else
-				embed_create_from_url (embed, info->ctx.link,
-						       FALSE, !tabbed_mode);
-			handled = TRUE;
-		} 
-		else 
+		if (!(info->ctx.context & CONTEXT_LINK))
 		{
 			if (gnome_config_get_int(
-				"/galeon/Mouse/middle_button_action") == 0)
+				"/galeon/Mouse/middle_button_action=1") == 0)
 				/* popup a bookmarks menu */
 				context_show_bookmark_menu (embed);
 			else {
@@ -622,6 +647,7 @@ mozembed_dom_mouse_click_cb (GtkMozEmbed *dummy, gpointer dom_event,
 						      GDK_SELECTION_TYPE_STRING,
 						      GDK_CURRENT_TIME);
 			}
+			handled = TRUE;
 		}
 		break;
 
@@ -629,24 +655,30 @@ mozembed_dom_mouse_click_cb (GtkMozEmbed *dummy, gpointer dom_event,
 		if ((info->ctx.context & CONTEXT_IMAGE) &&
 		    (info->modifier & CTRL_KEY))
 		{
-			embed_save_url_default (embed, info->ctx.img);
-			break;
-		}	
-		else if ((info->ctx.context & CONTEXT_LINK) && info->ctx.link &&
-						 (info->modifier & SHIFT_KEY))
-		{
-			mozilla_save_link (embed, info->ctx.link);
+			embed_save_image (embed, info->ctx.img, FALSE);
+			handled = TRUE;
 		}
-		else if ((info->ctx.context & CONTEXT_LINK) && info->ctx.link)
-		{
-			handled = handle_foreign_protocols (info->ctx.link);
-		}
-
 		break;
 	default:
 		break;
 	}
 
+	/* if not handled already handle with generic click handler (only if it
+	 * is not a non modified left click, since we need to let moz handle
+	 * that for frames to work properly ) */
+	if (!handled && (info->ctx.context & CONTEXT_LINK) && 
+		!(info->button==0
+		 && mozembed_modifier_to_gdk_state (info->modifier)==0))
+	{
+		LinkState link_state;
+
+		link_state = mouse_state_to_link_state 
+			(info->button + 1, 
+			 mozembed_modifier_to_gdk_state (info->modifier));
+
+		handled = embed_activate_link (embed, NULL, info->ctx.link,
+					       link_state);
+	}
 	mozilla_free_context_info_sub(&info->ctx);
 	g_free (info);
 
@@ -677,16 +709,31 @@ gint mozembed_dom_key_press_cb (GtkMozEmbed *dummy, gpointer dom_event,
 		return FALSE;
 	}
 
+	state = mozembed_modifier_to_gdk_state (info->modifier);
+
 	/* g_print("key = %ld  modifier = %d\n",info->key, info->modifier); */
 
 	/* Keypresses that are not handled here are now passed on to the main
 	   window's acceltable, so you should *not* handle any functions here
 	   that have an accelerator key in the menubar. --Josh */
 
-	if (info->modifier == 0)
+	if ((info->ctx.context & CONTEXT_LINK) && info->key == DOM_VK_RETURN)
+	{
+		handled = embed_activate_link_keyboard (embed, NULL, 
+							info->ctx.link, state);
+	}
+	else if (info->modifier == KEY_CODE)
 	{
 		switch(info->key)
 		{
+		case DOM_VK_BACK_SPACE:
+			if ((info->ctx.context & CONTEXT_INPUT) == 0)
+			{
+				gtk_moz_embed_go_back
+					(GTK_MOZ_EMBED (embed->mozEmbed));
+				handled = TRUE;
+			}
+			break;
 		default:
 			break;
 		}
@@ -704,7 +751,7 @@ gint mozembed_dom_key_press_cb (GtkMozEmbed *dummy, gpointer dom_event,
 		switch(info->key)
 		{
 		case DOM_VK_F10:
-			context_show_menu (embed, &info->ctx);
+			context_show_menu (embed, &info->ctx, -1, 0);
 			handled = TRUE;
 			break;
 		default:
@@ -712,7 +759,7 @@ gint mozembed_dom_key_press_cb (GtkMozEmbed *dummy, gpointer dom_event,
 		}
 	}
 	else if ((info->modifier & KEY_CODE) && ((info->modifier & CTRL_KEY) ||
-						(info->modifier & ALT_KEY)))
+						 (info->modifier & ALT_KEY)))
 	{
 		switch(info->key)
 		{
@@ -766,10 +813,15 @@ gint mozembed_dom_key_press_cb (GtkMozEmbed *dummy, gpointer dom_event,
 		case DOM_VK_G:
 		case DOM_VK_l:
 		case DOM_VK_L:
-			gtk_editable_select_region
-				(GTK_EDITABLE(window->toolbar_entry), 0, -1);
-			gtk_window_set_focus (GTK_WINDOW(window->WMain),
-					      window->toolbar_entry);
+			if (window->toolBarOn && window->location_entry != NULL)
+			{
+				gtk_editable_select_region
+					(GTK_EDITABLE(window->location_entry),
+					 0, -1);
+				gtk_window_set_focus 
+					(GTK_WINDOW(window->WMain),
+					 window->location_entry);
+			}
 			handled = TRUE;
 			break;
 		default:
@@ -844,29 +896,63 @@ gint mozembed_dom_key_press_cb (GtkMozEmbed *dummy, gpointer dom_event,
 		}
 	}
 
-	if (info->modifier & CTRL_KEY)
-		state |= GDK_CONTROL_MASK;
-	if (info->modifier & SHIFT_KEY)
-		state |= GDK_SHIFT_MASK;
-	if (info->modifier & ALT_KEY)
-		state |= GDK_MOD1_MASK;
-
 	/* We haven't specifically handled the keypress, so send it to the
 	   main window in case it is a menu accelerator key */
 	if (!handled)
 	{
-		handled = 
-			gtk_accel_groups_activate (GTK_OBJECT (window->WMain),
-						   info->key, state);
+		if (window->menuBarOn)
+		{
+			handled = gtk_accel_groups_activate 
+				(GTK_OBJECT (window->WMain), info->key, state);
+		}
+		else
+		{
+			handled = handle_key_without_menubar (window, 
+					info->key, (GdkModifierType)state);
+		}
 	}
 
 	mozilla_free_context_info_sub(&info->ctx);
 	g_free (info);
-
+				
 	/* If it wasn't an accelerator, let Mozilla handle it */
 	return handled;
 }
 
+/**
+ * handle_key_without_menubar: parse menubar uiinfo to handle key presses
+ * when the menubar is not created. FIXME accels configurability is broken
+ */
+gboolean
+handle_key_without_menubar (GaleonWindow *window, gulong key, 
+			    GdkModifierType ac_mods)
+{
+	extern GnomeUIInfo *all_menus[];
+	extern gint num_menus;
+	GnomeUIInfo *uiinfo;
+	int i;
+
+	for (i=0; i<num_menus; i++) 
+	{
+		uiinfo = all_menus[i];
+
+		for (; uiinfo->type != GNOME_APP_UI_ENDOFINFO; uiinfo++)
+		{
+			if (uiinfo->accelerator_key != 0 &&
+			    (uiinfo->accelerator_key == key ||
+			     uiinfo->accelerator_key == key - 32) && 
+			    uiinfo->ac_mods == ac_mods && 
+			    uiinfo->moreinfo)
+			{
+				GTK_SIGNAL_FUNC(uiinfo->moreinfo)(NULL, window);
+				return TRUE;
+			}
+		}
+	}
+	return FALSE;
+}
+
+
 /**
  * mozembed_size_to_cb: GTKMOZEMBED SIGNAL, emitted when a  size change 
  * is requested
@@ -881,11 +967,22 @@ mozembed_size_to_cb (GtkMozEmbed *dummy, gint width, gint height,
 	window = embed->parent_window;
 	return_if_not_window (window);
 
-	/* resize the embed */
-	gtk_widget_set_usize (GTK_WIDGET (embed->mozEmbed), width, height);
-
-	/* don't override this by changing the main window size! */
-	window->set_size = TRUE;
+	/* when window hasn't been shown... */
+	if (window->visible_embeds == 0)
+	{
+		/* resize the embed */
+		gtk_widget_set_usize (GTK_WIDGET (embed->mozEmbed), 
+				      width, height);
+		
+		/* don't override this by changing the main window size! */
+		window->set_size = TRUE;
+	}
+	else
+	{
+		/* forcibly resize the window */
+		gtk_widget_set_usize (GTK_WIDGET (window->WMain), 
+				      width, height);
+	}
 }
 
 /**
@@ -918,7 +1015,9 @@ mozembed_destroy_cb (GtkObject *object, GaleonEmbed *embed)
 	if (embed->site_title != NULL)
 	{
 		g_free (embed->site_title);
+		g_free (embed->site_title_utf8);
 		embed->site_title = NULL;
+		embed->site_title_utf8 = NULL;
 	}
 
 	/* destroy C++ wrapper */
@@ -935,14 +1034,11 @@ mozembed_destroy_cb (GtkObject *object, GaleonEmbed *embed)
 
 	/* show tabs if more than one embed or if the user has requested it */
 	n_embeds = g_list_length (window->embed_list);
-	gtk_notebook_set_show_tabs (GTK_NOTEBOOK (window->notebook), 
-		(g_list_length (window->embed_list) > 1) |
-		gnome_config_get_bool (CONF_APPEARANCE_TABBED_ALWAYS_SHOW));
 
 	/* if that's the last embed in parent window, destroy it */
 	if (n_embeds == 0)
 	{
-		window_close (embed->parent_window);
+		window_close (window);
 	}
 
 	/* autosave the updated session */
@@ -988,7 +1084,8 @@ mouse_listener (MouseInfo *minfo)
 
 	if (ABS(x - minfo->original_x) > 1 || ABS(y - minfo->original_y) > 1)
 	{
-		context_show_menu (minfo->embed, &minfo->info->ctx);
+		context_show_menu (minfo->embed, &minfo->info->ctx, 3,
+				   minfo->info->timestamp);
 		mouse_listener_active = FALSE;
 	}
 
