@@ -1,4 +1,4 @@
- /*
+/*
  *  Copyright (C) 2000 Marco Pesenti Gritti
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -19,34 +19,38 @@
 #include "galeon.h"
 #include "mozilla_i18n.h"
 #include "mozilla.h"
-#include "galeon-wrapper.h"
+#include "GaleonWrapper.h"
 
-#include <gtkmozembed_internal.h>
-/* FIXME !!! */
-#include <gnome.h>
 #include <time.h>
+#include <gtkmozembed_internal.h>
+#include <libgnome/gnome-defs.h>
+#include <libgnome/gnome-config.h>
+#include <libgnome/gnome-i18n.h>
 
 #include "nsString.h"
-#include "nsIPref.h"
+#include "nsXPIDLString.h"
+#include "nsIPrefService.h"
 #include "nsICharsetConverterManager.h"
 #include "nsICharsetConverterManager2.h"
 #include "nsIUnicodeEncoder.h"
+#include "nsISupportsPrimitives.h"
 #include "nsIPrintOptions.h"
 #include "nsGfxCIID.h"
 #include "nsICacheService.h"
 #include "nsIPassword.h"
-#include "nsIPasswordManager.h"
+#include "nsIPasswordManager.h" 
 #include "nsIDOMWindow.h"
 #include "nsIEmbeddingSiteWindow.h"
 #include "nsIWindowWatcher.h"
 #include "nsIWebBrowserChrome.h"
+#include "nsIFontList.h"
+#include "nsIFontEnumerator.h"
+#include "nsIExternalHelperAppService.h"
+#include "nsCExternalHandlerService.h"
+#include "nsILocalFile.h"
+#include "nsPIDOMWindow.h"
+#include "nsIDOMWindowInternal.h"
 
-#include "PromptService.h"
-#include "ContentHandler.h"
-#include "ContentOverrideHandler.h"
-#include "FilePicker.h"
-
-static NS_DEFINE_CID(kPrefCID, NS_PREF_CID);
 static NS_DEFINE_CID(kPrintOptionsCID, NS_PRINTOPTIONS_CID);
 
 /* local function prototypes */
@@ -55,31 +59,23 @@ static char  *convert_ns_string_to_c_string (const nsString & ns_string);
 extern "C" gboolean
 mozilla_load_prefs (void)
 {
-#if MOZILLA_VERSION > VERSION2(0,9)
 	nsCOMPtr<nsIPrefService> prefService = 
 				do_GetService (NS_PREFSERVICE_CONTRACTID);
 	g_return_val_if_fail (prefService != nsnull, FALSE);
 
 	nsresult rv = prefService->ReadUserPrefs (nsnull);
 	return NS_SUCCEEDED (rv) ? TRUE : FALSE;
-#else
-	return FALSE;
-#endif
 }
 
 extern "C" gboolean
 mozilla_save_prefs (void)
 {
-#if MOZILLA_VERSION > VERSION2(0,9)
 	nsCOMPtr<nsIPrefService> prefService = 
 				do_GetService (NS_PREFSERVICE_CONTRACTID);
 	g_return_val_if_fail (prefService != nsnull, FALSE);
 
 	nsresult rv = prefService->SavePrefFile (nsnull);
 	return NS_SUCCEEDED (rv) ? TRUE : FALSE;
-#else
-	return FALSE;
-#endif
 }
 
 /**
@@ -88,210 +84,66 @@ mozilla_save_prefs (void)
 extern "C" gboolean
 mozilla_preference_set(const char *preference_name, const char *new_value)
 {
-  g_return_val_if_fail (preference_name != NULL, FALSE);
-  g_return_val_if_fail (new_value != NULL, FALSE);
-  nsCOMPtr<nsIPref> pref = do_CreateInstance(NS_PREF_CONTRACTID);
+	g_return_val_if_fail (preference_name != NULL, FALSE);
+	g_return_val_if_fail (new_value != NULL, FALSE);
+	nsCOMPtr<nsIPrefService> prefService = 
+				do_GetService (NS_PREFSERVICE_CONTRACTID);
+	nsCOMPtr<nsIPrefBranch> pref;
+	prefService->GetBranch ("", getter_AddRefs(pref));
 
-  if (pref)
-    {
-      nsresult rv = pref->SetCharPref (preference_name, new_value);            
-      return NS_SUCCEEDED (rv) ? TRUE : FALSE;
-    }
-  
-  return FALSE;
+	if (pref)
+	{
+		nsresult rv = pref->SetCharPref (preference_name, new_value);            
+		return NS_SUCCEEDED (rv) ? TRUE : FALSE;
+	}
+
+	return FALSE;
 }
 
 /**
  * mozilla_preference_set_boolean: set a boolean mozilla preference
  */
 extern "C" gboolean
-mozilla_preference_set_boolean (const char        *preference_name,
-				gboolean        new_boolean_value)
+mozilla_preference_set_boolean (const char *preference_name,
+				gboolean new_boolean_value)
 {
-  g_return_val_if_fail (preference_name != NULL, FALSE);
+	g_return_val_if_fail (preference_name != NULL, FALSE);
   
-  nsCOMPtr<nsIPref> pref = do_CreateInstance(NS_PREF_CONTRACTID);
+	nsCOMPtr<nsIPrefService> prefService = 
+				do_GetService (NS_PREFSERVICE_CONTRACTID);
+	nsCOMPtr<nsIPrefBranch> pref;
+	prefService->GetBranch ("", getter_AddRefs(pref));
   
-  if (pref)
-    {
-      nsresult rv = pref->SetBoolPref (preference_name,
-				       new_boolean_value ? PR_TRUE : PR_FALSE);
-      
-      return NS_SUCCEEDED (rv) ? TRUE : FALSE;
-    }
-  
-  return FALSE;
+	if (pref)
+	{
+		nsresult rv = pref->SetBoolPref (preference_name,
+				new_boolean_value ? PR_TRUE : PR_FALSE);
+		return NS_SUCCEEDED (rv) ? TRUE : FALSE;
+	}
+
+	return FALSE;
 }
 
 /**
  * mozilla_preference_set_int: set an integer mozilla preference
  */
 extern "C" gboolean
-mozilla_preference_set_int (const char        *preference_name,
-				int        new_int_value)
+mozilla_preference_set_int (const char *preference_name, int new_int_value)
 {
-  g_return_val_if_fail (preference_name != NULL, FALSE);
-  
-  nsCOMPtr<nsIPref> pref = do_CreateInstance(NS_PREF_CONTRACTID);
-  
-  if (pref)
-    {
-      nsresult rv = pref->SetIntPref (preference_name,
-				       new_int_value);
-      
-      return NS_SUCCEEDED (rv) ? TRUE : FALSE;
-    }
-  
-  return FALSE;
-}
+	g_return_val_if_fail (preference_name != NULL, FALSE);
 
-/**
- * mozilla_init_promptService: Initialise PromptService handler.
- */
-extern "C" gboolean 
-mozilla_init_promptService (void)
-{
-	if (!pushed_startup) return FALSE;
+	nsCOMPtr<nsIPrefService> prefService = 
+				do_GetService (NS_PREFSERVICE_CONTRACTID);
+	nsCOMPtr<nsIPrefBranch> pref;
+	prefService->GetBranch ("", getter_AddRefs(pref));
 
-	nsresult rv = NS_OK;
+	if (pref)
+	{
+		nsresult rv = pref->SetIntPref (preference_name, new_int_value);
+		return NS_SUCCEEDED (rv) ? TRUE : FALSE;
+	}
 
-	static NS_DEFINE_CID(kPromptServiceCID, NS_PROMPTSERVICE_CID);
-     
-	nsCOMPtr<nsIFactory> promptFactory;
-	rv = NS_NewPromptServiceFactory(getter_AddRefs(promptFactory));
-	if (NS_FAILED(rv)) return rv;
-	rv = nsComponentManager::
-		RegisterFactory(kPromptServiceCID, "Prompt Service",
-				"@mozilla.org/embedcomp/prompt-service;1",
-				promptFactory, PR_TRUE);  
-	return NS_SUCCEEDED (rv) ? TRUE : FALSE;
-}
-
-/**
- * mozilla_init_ContentHandler: Initialise Content Handler.
- */
-extern "C" gboolean 
-mozilla_init_ContentHandler (void)
-{
-	if (!pushed_startup) return FALSE;
-
-	nsresult rv = NS_OK;
-
-	static NS_DEFINE_CID(kContentHandlerCID, NS_IHELPERAPPLAUNCHERDIALOG_IID);
-     
-	nsCOMPtr<nsIFactory> ContentHandlerFactory;
-	rv = NS_NewContentHandlerFactory (getter_AddRefs(ContentHandlerFactory));
-	if (NS_FAILED(rv)) return rv;
-	rv = nsComponentManager::
-		RegisterFactory(kContentHandlerCID,
-				NS_IHELPERAPPLAUNCHERDLG_CLASSNAME,
-				NS_IHELPERAPPLAUNCHERDLG_CONTRACTID,
-				ContentHandlerFactory, PR_TRUE);  
-	return NS_SUCCEEDED (rv) ? TRUE : FALSE;
-}
-
-/**
- * mozilla_register_contentOverrideHandler: Registers an explicit contentHandler
- * @mime-type: The mime-type to register a contentHandler for.
- */
-extern "C" gboolean mozilla_register_contentOverrideHandler (const char *mime_type)
-{
-        if (!pushed_startup) return FALSE;
-
-        nsresult rv = NS_OK;
-  
-        static NS_DEFINE_CID(kContentOverrideHandlerCID,
-        		     GALEON_CONTENTOVERRIDEHANDLER_CID);
-     
-        nsCOMPtr<nsIFactory> CHFactory;
-        rv = NS_NewContentOverrideHandlerFactory(getter_AddRefs(CHFactory));
-        if (NS_FAILED(rv)) return FALSE;
-      
-        gchar *handlerName = g_strdup_printf ("Content Handler for %s",
-                                              mime_type);
-        gchar *contractID = g_strdup_printf (NS_CONTENT_HANDLER_CONTRACTID_PREFIX
-                                             "%s",mime_type);
-   
-        rv = nsComponentManager::RegisterFactory(kContentOverrideHandlerCID,
-                                                 handlerName,   
-                                                 contractID,
-                                                 CHFactory,
-                                                 PR_TRUE);
-        g_free (handlerName);
-        g_free (contractID);
-
-        return NS_SUCCEEDED (rv) ? TRUE : FALSE;
-}
-
-/**
- * mozilla_register_stored_contentOverrideHandler: Registers contentHandlers
- * for all stored mime-types with the override action type.
- */
-extern "C" gboolean mozilla_register_stored_contentOverrideHandlers (void)
-{  
-
-        gboolean rv = TRUE;
-        for (GList *mime_type = g_list_first(mime_types) ;
-             mime_type!=NULL ;
-             mime_type = g_list_next(mime_type))
-        {
-                if (((MimeItem *)mime_type->data)->action == 
-               		MIME_OVERRIDE_MOZILLA)
-                {
-                        if (!mozilla_register_contentOverrideHandler
-                                (((MimeItem *)mime_type->data)->name))   
-                        {
-                                rv = FALSE;
-                        }
-                }
-        }
-        return rv;
-}
-
-/**
- * mozilla_unregister_stored_contentOverrideHandler: unregisters contentHandlers
- * for all stored mime-types
- */
-extern "C" gboolean mozilla_unregister_stored_contentOverrideHandlers (void)
-{
-        if (!pushed_startup) return FALSE;
-        
-        nsresult rv = NS_OK;
-        static NS_DEFINE_CID(kContentOverrideHandlerCID,
-        		     GALEON_CONTENTOVERRIDEHANDLER_CID);
-                                                 
-        nsCOMPtr<nsIFactory> CHFactory;
-        
-        rv = nsComponentManager::FindFactory (kContentOverrideHandlerCID,
-                                              getter_AddRefs(CHFactory));
-        if (NS_FAILED(rv)) return FALSE;
- 
-        rv = nsComponentManager::UnregisterFactory (kContentOverrideHandlerCID,
-                                                    CHFactory);
- 
-        return NS_SUCCEEDED (rv) ? TRUE : FALSE;
-}
-
-/**
- * mozilla_init_FilePicker: Register FilePicker Factory.
- */
-extern "C" gboolean 
-mozilla_init_FilePicker (void)
-{
-	if (!pushed_startup) return FALSE;
-
-	nsresult rv = NS_OK;
-
-	static NS_DEFINE_CID(kFilePickerCID, NS_IFILEPICKER_IID);
-     
-	nsCOMPtr<nsIFactory> FilePickerFactory;
-	rv = NS_NewFilePickerFactory (getter_AddRefs(FilePickerFactory));
-	if (NS_FAILED(rv)) return rv;
-	rv = nsComponentManager::RegisterFactory(kFilePickerCID,
-						 G_FILEPICKER_CLASSNAME,
-						 G_FILEPICKER_CONTRACTID,
-						 FilePickerFactory, PR_TRUE);  
-	return NS_SUCCEEDED (rv) ? TRUE : FALSE;
+	return FALSE;
 }
 
 /**
@@ -302,9 +154,12 @@ mozilla_init_FilePicker (void)
 extern "C" gpointer 
 mozilla_wrapper_init (GaleonEmbed *embed, gboolean attach_listener)
 {
+	nsresult result;
 	GaleonWrapper *wrapper = new GaleonWrapper ();
 	wrapper->attachListener = attach_listener;
-	wrapper->Init (embed);
+	result = wrapper->Init (embed);
+	if (NS_FAILED(result))
+		g_warning ("Wrapper initialization failed");
 	return wrapper;
 }
 
@@ -344,14 +199,14 @@ extern "C" gboolean mozilla_find (GaleonEmbed *embed,
 }
 
 extern "C" gboolean mozilla_save_url (GaleonEmbed *embed, const char *url, 
-					const char *file_name)
+				      char *file_name, int action)
 {
 	nsresult result;
 	GaleonWrapper *wrapper = (GaleonWrapper *)embed->wrapper;
 
 	g_return_val_if_fail (embed->wrapper != NULL, FALSE); 
 
-	result = wrapper->SaveURI (url, file_name);
+	result = wrapper->SaveURI (url, file_name, action);
 
 	return NS_SUCCEEDED (result) ? TRUE : FALSE;
 }
@@ -454,34 +309,70 @@ mozilla_print (GaleonEmbed *embed, PrintInfo *info)
 }
 
 /**
- * mozilla_session_history: Gets the session history and current position of a 
- * GtkMozWrapper. On return, *titles will be an array of char * wich must be freed.
+ * mozilla_session_history_get_url: Gets the session history and  ..
+ * GtkMozWrapper. On return,  ..
  */
-extern "C" gboolean
-mozilla_session_history (GaleonEmbed *embed, char **titles[], 
-			 int *count, int *index)
+extern "C" char *
+mozilla_session_history_get_url (GaleonEmbed *embed, int index)
 {
 	GaleonWrapper *wrapper = (GaleonWrapper *)embed->wrapper;
 
 	g_return_val_if_fail (embed->wrapper != NULL, FALSE); 
 
-	wrapper->GetSHInfo (count, index);
+	nsresult result;
+	nsXPIDLCString url;
+
+	result = wrapper->GetSHUrlAtIndex(index, getter_Copies(url));
+
+	if (NS_FAILED(result))
+	{
+		return NULL;
+	}
+	
+	return g_strdup(url.get());
+}
+
+extern "C" char *
+mozilla_session_history_get_url_relative (GaleonEmbed *embed, int offset)
+{
+	int count, index;
+	GaleonWrapper *wrapper = (GaleonWrapper *)embed->wrapper;
+	g_return_val_if_fail (embed->wrapper != NULL, FALSE); 
+	wrapper->GetSHInfo (&count, &index);
+	return mozilla_session_history_get_url (embed, index+offset);
+}
+
+/**
+ * mozilla_session_history: Gets the session history and current position of a 
+ * GtkMozWrapper. On return, *titles will be an array of char * wich must be freed.
+ */
+extern "C" gboolean
+mozilla_session_history_get_all_titles (GaleonEmbed *embed, char **titles[], 
+					int *count, int *index)
+{
+	GaleonWrapper *wrapper = (GaleonWrapper *)embed->wrapper;
+
+	g_return_val_if_fail (embed->wrapper != NULL, FALSE); 
+
+	if (wrapper->GetSHInfo (count, index) == NS_ERROR_FAILURE)
+		return FALSE;
 
 	char **t = g_new(char *, *count);
 
 	for (PRInt32 i = 0; i < *count; i++) {
 
 		nsresult result;
-		PRUnichar *unicodeTitle;
+		nsXPIDLString unicodeTitle;
 
-		result = wrapper->GetSHTitleAtIndex(i, &unicodeTitle);
+		result = wrapper->GetSHTitleAtIndex(i,
+						    getter_Copies(unicodeTitle));
 
 		if (NS_FAILED(result))
 		{
 			return NS_OK;
 		}
 		
-		t[i] = mozilla_unicode_to_locale (unicodeTitle);
+		t[i] = mozilla_unicode_to_locale (unicodeTitle.get());
 	}
 	*titles = t;
 
@@ -512,14 +403,12 @@ extern "C" gchar *
 mozilla_get_main_document_url (GaleonEmbed *embed)
 {
 	GaleonWrapper *wrapper = (GaleonWrapper *)embed->wrapper;
-	gchar *url = NULL;
-	nsresult result;
-
 	g_return_val_if_fail (embed->wrapper != NULL, NULL); 
 
-	result = wrapper->GetMainDocumentUrl (&url);
+	nsXPIDLCString url;
+	nsresult rv = wrapper->GetMainDocumentUrl (getter_Copies(url));
 
-	return NS_SUCCEEDED (result) ? url : NULL;
+	return NS_SUCCEEDED (rv) ? g_strdup(url.get()) : NULL;
 }
 
 /**
@@ -530,23 +419,20 @@ extern "C" gchar *
 mozilla_get_document_url (GaleonEmbed *embed)
 {
 	GaleonWrapper *wrapper = (GaleonWrapper *)embed->wrapper;
-	gchar *url = NULL;
-	nsresult result;
-
 	g_return_val_if_fail (embed->wrapper != NULL, NULL); 
 
-	result = wrapper->GetDocumentUrl (&url);
+	nsXPIDLCString url;
+	nsresult rv = wrapper->GetDocumentUrl (getter_Copies(url));
 
-	return NS_SUCCEEDED (result) ? url : NULL;
+	return NS_SUCCEEDED (rv) ? g_strdup(url.get()) : NULL;
 }
 
-#if MOZILLA_VERSION > VERSION2(0,9)
 /**
  * mozilla_get_document_title: get the document title in a locale specific
  * NULL terminated C string, using the private UniCode interface.
  */
 extern "C" gchar * 
-mozilla_get_document_title (GaleonEmbed *embed)
+mozilla_get_document_title (GaleonEmbed *embed, gchar **title_utf8)
 {
 	PRUnichar *unicode_title;
 	gchar *title;
@@ -555,6 +441,12 @@ mozilla_get_document_title (GaleonEmbed *embed)
 	unicode_title = gtk_moz_embed_get_title_unichar 
 		(GTK_MOZ_EMBED(embed->mozEmbed));
 
+	/* do get utf8 version too */
+	if (title_utf8 != NULL)
+	{
+		*title_utf8 = mozilla_unicode_to_utf8 (unicode_title);
+	}
+
 	/* attempt conversion */
 	title = mozilla_unicode_to_locale (unicode_title);
 
@@ -564,7 +456,6 @@ mozilla_get_document_title (GaleonEmbed *embed)
 	/* return it */
 	return title;
 }
-#endif
 
 /**
  * mozilla_get_key_event_info:
@@ -656,10 +547,16 @@ mozilla_list_cookies (void)
 
 		c = g_new0 (Cookie, 1);
 
-		nsCookie->GetHost (&(c->base.domain));
-		nsCookie->GetName (&(c->name));
-		nsCookie->GetValue (&(c->value));
-		nsCookie->GetPath (&(c->path));
+		nsXPIDLCString transfer;
+
+		nsCookie->GetHost (getter_Copies(transfer));
+		c->base.domain = g_strdup (transfer.get());
+		nsCookie->GetName (getter_Copies(transfer));
+		c->name = g_strdup (transfer.get());
+		nsCookie->GetValue (getter_Copies(transfer));
+		c->value = g_strdup (transfer.get());
+		nsCookie->GetPath (getter_Copies(transfer));
+		c->path = g_strdup (transfer.get());
 		
 		PRBool isSecure;
 		nsCookie->GetIsSecure (&isSecure);
@@ -672,8 +569,9 @@ mozilla_list_cookies (void)
 		nsCookie->GetExpires (&dateTime);
 		c->expire = g_strdup_printf ("%s",ctime((time_t*)&dateTime));
 		
-		cookies = g_list_append (cookies, c);
+		cookies = g_list_prepend (cookies, c);
 	}	
+	cookies = g_list_reverse (cookies);
 	return cookies;
 }
 
@@ -718,10 +616,11 @@ mozilla_get_permissions (int type)
 			else 
 				b->type = g_strdup_printf ("Blocked");
 
-			permissions = g_list_append (permissions, b);
+			permissions = g_list_prepend (permissions, b);
 		}
 	}
 
+	permissions = g_list_reverse (permissions);
 	return permissions;
 };
 
@@ -764,7 +663,7 @@ mozilla_set_permission (GaleonEmbed *embed, gboolean permit, int type)
  * @block: whether to block the originating servers in future
  */
 extern "C" gboolean 
-mozilla_remove_cookies (GList *gone, gboolean block)
+mozilla_remove_cookies (GList *gone)
 {
 	nsresult result;
 	nsCOMPtr<nsICookieManager> 
@@ -774,9 +673,10 @@ mozilla_remove_cookies (GList *gone, gboolean block)
 	     cookies = g_list_next(cookies))
 	{
 		Cookie *c = (Cookie *)cookies->data;
+
 		result = cookieManager->Remove (c->base.domain,c->name,
 					 	c->path,
-						block ? PR_TRUE : PR_FALSE);
+						c->base.block ? PR_TRUE : PR_FALSE);
 		if (NS_FAILED(result)) return FALSE;
 	};
 	return TRUE;
@@ -829,6 +729,7 @@ mozilla_get_charsets (GHashTable **charsets, GList **sorted_charset_titles)
 	nsresult rv;
 	PRUint32 cscount;
 	PRUint32 translated_cscount = get_translated_cscount ();
+	char *charset_str, *charset_title_str;
 
 	nsCOMPtr<nsIAtom> docCharsetAtom;
 	nsCOMPtr<nsICharsetConverterManager2> ccm2 =
@@ -836,11 +737,11 @@ mozilla_get_charsets (GHashTable **charsets, GList **sorted_charset_titles)
 	if (!NS_SUCCEEDED(rv)) return;
 
 	nsCOMPtr <nsISupportsArray> cs_list;
-	rv = ccm2->GetDecoderList(getter_AddRefs(cs_list));
+	rv = ccm2->GetDecoderList (getter_AddRefs(cs_list));
 	if (!NS_SUCCEEDED(rv)) return;
 
 	rv = cs_list->Count(&cscount);
-	*charsets = g_hash_table_new(g_str_hash,g_str_equal);
+	*charsets = g_hash_table_new (g_str_hash, g_str_equal);
 	for (PRUint32 i = 0; i < cscount; i++)
 	{
 		nsCOMPtr<nsISupports> cssupports =
@@ -850,24 +751,27 @@ mozilla_get_charsets (GHashTable **charsets, GList **sorted_charset_titles)
 
 		/* charset name */
 		rv = csatom->ToString(charset_ns);
-		char *charset_str = convert_ns_string_to_c_string (charset_ns);
+		charset_str = convert_ns_string_to_c_string (charset_ns);
+		if (charset_str == NULL || strlen (charset_str) == 0)
+		{
+			continue;
+		}
 
 		/* charset readable title */
 		rv = ccm2->GetCharsetTitle2(csatom, &charset_title_ns);
-		char *charset_title_str = convert_ns_string_to_c_string 
-						(charset_title_ns);
-
-		if (strlen(charset_title_str) == 0)
+		charset_title_str = convert_ns_string_to_c_string 
+							(charset_title_ns);
+		if (charset_title_str == NULL || 
+		    strlen (charset_title_str) == 0)
 		{
-			g_free (charset_title_str);
-			charset_title_str = g_strdup(charset_str);
+			charset_title_str = g_strdup (charset_str);
 		}
 		
 		for (PRUint32 j = 0; j < translated_cscount; j++)
 		{
 			if (g_strcasecmp (
-				charset_title_str, 
-				charset_trans_array[j].charset_title) == 0)
+				charset_str, 
+				charset_trans_array[j].charset_name) == 0)
 			{
 				g_free (charset_title_str);
 				charset_title_str = (char *) 
@@ -912,16 +816,11 @@ static char *
 convert_ns_string_to_c_string (const nsString & ns_string)
 {
 	char *c_string;
-	char *ns_c_string = ns_string.ToNewCString ();
+	nsCString ns_c_string;
 
-	if (ns_c_string == NULL)
-	{
-		return NULL;
-	}
+	ns_c_string.AssignWithConversion (ns_string);
 
-	c_string = g_strdup (ns_c_string);
-
-	nsMemory::Free (ns_c_string);
+	c_string = g_strdup (ns_c_string.get());
 
 	return c_string;
 }
@@ -929,12 +828,14 @@ convert_ns_string_to_c_string (const nsString & ns_string)
 extern "C" gboolean
 mozilla_get_favicon_location (GaleonEmbed *embed, gchar **url)
 {
-	nsresult result;
 	GaleonWrapper *wrapper = (GaleonWrapper *)embed->wrapper;
+	g_return_val_if_fail (embed->wrapper != NULL, FALSE); 
+
+	nsXPIDLCString transfer;
+	nsresult rv = wrapper->GetFaviconURL (getter_Copies(transfer));
+	*url = g_strdup (transfer.get());
 	
-	result = wrapper->GetFaviconURL (url);
-	
-	return NS_SUCCEEDED (result);
+	return NS_SUCCEEDED (rv) ? TRUE : FALSE;
 }
 
 /**
@@ -969,17 +870,20 @@ mozilla_list_passwords (PasswordType type)
 
 		Password *p = g_new0 (Password, 1);
 
-		nsPassword->GetHost (&(p->host));
+		nsXPIDLCString transfer;
+		nsPassword->GetHost (getter_Copies(transfer));
+		p->host = g_strdup (transfer.get());
 
 		if (type==PASSWORD)
 		{
-			PRUnichar *unicodeName;
-			nsPassword->GetUser (&unicodeName);
-			p->username = mozilla_unicode_to_locale (unicodeName);
+			nsXPIDLString unicodeName;
+			nsPassword->GetUser (getter_Copies(unicodeName));
+			p->username = mozilla_unicode_to_locale (unicodeName.get());
 		}
 		
-		passwords = g_list_append (passwords, p);
+		passwords = g_list_prepend (passwords, p);
 	}	
+	passwords = g_list_reverse (passwords);
 	return passwords;
 }
 
@@ -1027,11 +931,8 @@ mozilla_unicode_to_utf8 (const PRUnichar *uniStr)
 	PRInt32 sSize,dSize;
 	nsresult result;
 
-#if MOZILLA_VERSION > VERSION2(0,9)
 	const nsString str (uniStr);
-#else
-	const nsLocalString str = nsLiteralString (uniStr);
-#endif
+
 	sSize = str.Length ();
 
 	nsCOMPtr<nsIUnicodeEncoder> 
@@ -1047,7 +948,7 @@ mozilla_unicode_to_utf8 (const PRUnichar *uniStr)
 	   the size of the char* buffer so that bad things(tm)
 	   won't happen. No null termination here. */
 	result = unicodeEncoder->Convert (uniStr, &sSize, utfStr, &dSize);
-					  
+
 	/* Finish ensures that the encoder is left in a clean state
 	   for it's next use. */
 	result = unicodeEncoder->Finish (utfStr, &dSize);
@@ -1076,11 +977,7 @@ mozilla_unicode_to_locale (const PRUnichar *uniStr)
 		return NULL;
 	}
 
-#if MOZILLA_VERSION > VERSION2(0,9)
 	const nsString str (uniStr);
-#else
-	const nsLocalString str = nsLiteralString (uniStr);
-#endif
 	sSize = str.Length ();
 
 	/* allocate a wide string big enough to hold the unicode string,
@@ -1098,10 +995,12 @@ mozilla_unicode_to_locale (const PRUnichar *uniStr)
 	/* check for success */
 	if (count == -1)
 	{
-		/* not sure what we should do here, but it seems safe to
-		 * fallback to encode_to_utf8 */
+		/* let Mozilla do a (lossy) conversion then */
+		nsCString str;
+		str.AssignWithConversion(uniStr);
 		g_free (wide);
-		return mozilla_unicode_to_utf8 (uniStr);
+		return g_strdup (str.get()); /* FIXME strdup needed? */
+					     /* Phil: Oh Yes! indeed. */
 	}
 
 	/* allocate a string big enough and do the actual conversion */
@@ -1227,3 +1126,171 @@ mozilla_set_offline (GaleonEmbed *embed, gboolean offline)
 
 	return NS_SUCCEEDED (result) ? TRUE : FALSE;
 }
+
+extern "C" gboolean
+mozilla_can_cut_selection (GaleonEmbed *embed)
+{
+	GaleonWrapper *wrapper = (GaleonWrapper *)embed->wrapper;
+	gboolean result;
+
+	g_return_val_if_fail (embed->wrapper != NULL, FALSE); 
+
+	wrapper->CanCutSelection (&result);
+
+	return result;
+}
+
+extern "C" gboolean
+mozilla_can_copy_selection (GaleonEmbed *embed)
+{
+	GaleonWrapper *wrapper = (GaleonWrapper *)embed->wrapper;
+	gboolean result;
+
+	g_return_val_if_fail (embed->wrapper != NULL, FALSE); 
+
+	wrapper->CanCopySelection (&result);
+
+	return result;
+}
+
+extern "C" gboolean
+mozilla_can_paste (GaleonEmbed *embed)
+{
+	GaleonWrapper *wrapper = (GaleonWrapper *)embed->wrapper;
+	gboolean result;
+
+	g_return_val_if_fail (embed->wrapper != NULL, FALSE); 
+
+	wrapper->CanPaste (&result);
+
+	return result;
+}
+
+extern "C" void
+mozilla_cut_selection (GaleonEmbed *embed)
+{
+	GaleonWrapper *wrapper = (GaleonWrapper *)embed->wrapper;
+
+	g_return_if_fail (embed->wrapper != NULL); 
+
+	wrapper->CutSelection ();
+}
+
+extern "C" void
+mozilla_copy_selection (GaleonEmbed *embed)
+{
+	GaleonWrapper *wrapper = (GaleonWrapper *)embed->wrapper;
+
+	g_return_if_fail (embed->wrapper != NULL); 
+
+	wrapper->CopySelection ();
+}
+
+extern "C" void
+mozilla_activate (GaleonEmbed *embed)
+{
+	GaleonWrapper *wrapper = (GaleonWrapper *)embed->wrapper;
+
+	g_return_if_fail (embed->wrapper != NULL); 
+
+	wrapper->Activate ();
+}
+
+extern "C" void
+mozilla_deactivate (GaleonEmbed *embed)
+{
+	GaleonWrapper *wrapper = (GaleonWrapper *)embed->wrapper;
+
+	g_return_if_fail (embed->wrapper != NULL); 
+
+	wrapper->Deactivate ();
+}
+
+extern "C" void
+mozilla_paste (GaleonEmbed *embed)
+{
+	GaleonWrapper *wrapper = (GaleonWrapper *)embed->wrapper;
+
+	g_return_if_fail (embed->wrapper != NULL); 
+
+	wrapper->Paste ();
+}
+
+extern "C" gchar *
+mozilla_get_default_font (const char *language, 
+			  const char *type, 
+			  const char *default_font)
+{
+       nsresult result;
+       gchar *ret = NULL;
+       gchar *font_name = NULL;
+       PRBool moreElements;
+ 
+       nsCOMPtr<nsIFontList> 
+               fontList = do_CreateInstance ("@mozilla.org/gfx/fontlist;1");
+       if (!fontList)
+               return NULL;
+ 
+       nsCOMPtr<nsISimpleEnumerator> fontEnumerator;
+       result = fontList->AvailableFonts (mozilla_locale_to_unicode(language),
+                                          mozilla_locale_to_unicode(type),
+                                          getter_AddRefs(fontEnumerator));
+       if (NS_FAILED(result))
+               return NULL;
+ 
+       nsCOMPtr<nsISupportsWString> font;
+ 
+       for (fontEnumerator->HasMoreElements(&moreElements) ;
+            moreElements == PR_TRUE ;
+            fontEnumerator->HasMoreElements(&moreElements))
+       {
+               result = fontEnumerator->GetNext (getter_AddRefs(font));
+               if (NS_FAILED(result))
+                       return NULL;
+ 
+               PRUnichar *fontName; 
+               result = font->ToString (&fontName);
+               font_name = mozilla_unicode_to_locale (fontName);
+               if (NS_FAILED(result))
+                       return NULL;
+ 
+               if (!ret)
+               {
+                       ret = g_strdup (font_name);
+               }
+               else
+               {
+                       if (default_font &&
+			   strstr(font_name, default_font))
+		       {
+                               g_free (ret);
+                               ret = g_strdup (font_name);
+                       }
+               }
+               g_free (font_name);
+               nsMemory::Free (fontName);
+       }
+       
+       return ret;
+}
+
+extern "C" void
+mozilla_delete_temp_file_on_exit (const char *filename)
+{
+#if MOZILLA_VERSION > VERSION3(0,9,2)
+	nsresult rv;
+	nsCOMPtr<nsILocalFile> localFile;
+	rv = NS_NewLocalFile (filename, PR_TRUE, getter_AddRefs(localFile));
+	if (NS_FAILED(rv)) return;
+
+	nsCOMPtr<nsIExternalHelperAppService> helperService =
+			do_GetService (NS_EXTERNALHELPERAPPSERVICE_CONTRACTID);
+
+	nsCOMPtr<nsPIExternalAppLauncher> appLauncher =
+					do_QueryInterface (helperService, &rv);
+	if (NS_SUCCEEDED(rv))
+	{
+		appLauncher->DeleteTemporaryFileOnExit (localFile);
+	}
+#endif
+}
