@@ -44,6 +44,7 @@ void mozilla_prefs_set_autodetect_charset (int charset);
 void mozilla_prefs_set_default_charset (void);
 void mozilla_prefs_set_languages (void);
 void mozilla_prefs_set_proxy (void);
+void mozilla_prefs_set_download (void);
 void mozilla_prefs_set_cache (void);
 void mozilla_prefs_set_fonts (void);
 void mozilla_prefs_set_images (void);
@@ -64,13 +65,14 @@ void
 mozilla_prefs_set (void)
 {
        	mozilla_prefs_set_proxy ();
+       	mozilla_prefs_set_fonts ();
 	mozilla_prefs_set_cache ();
-	mozilla_prefs_set_fonts ();
 	mozilla_prefs_set_images ();
 	mozilla_prefs_set_animate ();
 	mozilla_prefs_set_colors ();
 	mozilla_prefs_set_java ();
 	mozilla_prefs_set_network ();
+	mozilla_prefs_set_download ();
 	mozilla_prefs_set_autodetect_charset (gnome_config_get_int ("/galeon/Advanced/autodetect_charset=0"));
 	mozilla_prefs_set_default_charset ();
 	mozilla_prefs_set_languages ();
@@ -118,10 +120,6 @@ void mozilla_prefs_set_wheel_mouse (void)
 	/* set default search engine */
 	mozilla_preference_set ("keyword.URL",
 				"http://www.google.com/search?q=");
-
-	/* turn off warning since we cant show them properly yet (FIXME) */
-	mozilla_preference_set_boolean ("security.warn_entering_secure",
-					FALSE);
 }
 
 /**
@@ -186,8 +184,16 @@ mozilla_prefs_set_network (void)
 
 	mozilla_preference_set_int ("network.cookie.cookieBehavior", cookieBeh);
 	mozilla_preference_set ("network.http.version", http_version);
+
 	g_free (http_version);
 
+	/* FIXME: dont think this is needed any more -- MattA 9/7/2001 */
+#if 0
+	/* HACK! don't accept gzip coz that causes tar.gz files to be
+	 * automatically decompressed when downloading (with some servers) */
+	mozilla_preference_set ("network.http.accept-encoding", 
+				"deflate,compress,identity");
+#endif
 }
 
 /**
@@ -218,30 +224,152 @@ mozilla_prefs_set_animate (void)
 	mozilla_preference_set ("image.animation_mode", type[animate]);
 } 
 
+const gchar *font_types[] =
+{
+	"serif",
+	"sans-serif",
+	"cursive",
+	"fantasy",
+	"monospace",
+	NULL
+};
+
+const gchar *font_languages[] =
+{
+	"x-western",
+	"ar",
+	"el",
+	"he",
+	"ja",
+	"ko",
+	"th",
+	"tr",
+	"x-baltic",
+	"x-central-euro",
+	"x-cyrillic",
+	"zh-CN",
+	"zh-TW",
+	NULL
+};
+
+const gchar *font_defaults[][5] =
+{
+/* x-western: Western */
+	{"adobe-times", "adobe-helvetica", NULL, NULL,
+	 "adobe-courier"},
+	
+/* ar: Arabic */
+	{NULL, NULL, NULL, NULL, NULL},
+	
+/* el: Greek */
+	{"misc-fixed", "misc-fixed", NULL, NULL, 
+	 "misc-fixed"},
+	
+/* he: Hebrew */  
+	{NULL, NULL, NULL, NULL, NULL},
+	
+/* ja: Japanese */
+	{NULL, NULL, NULL, NULL, NULL},
+	
+/* ko: Korean */
+	{"daewoo-mincho", "daewoo-mincho", NULL, NULL, 
+	 "daewoo-mincho"},
+        
+/* th: Thai */
+	{NULL, NULL, NULL, NULL, NULL},
+	
+/* tr: Turkish */
+	{NULL, NULL, NULL, NULL, NULL},
+	
+/* x-baltic: Baltic */
+	{"b&h-lucidux serif", "b&h-lucidux sans", 
+	 NULL, NULL, "b&h-lucidux mono"},
+	
+/* x-central-euro: Central European */ 
+	{NULL, NULL, NULL, NULL, NULL},
+	
+/* x-cyrillic: Cyrillic */
+	{NULL, NULL, NULL, NULL, NULL},
+	
+/* zh-CN: Simplified Chinese */
+	{"isas-song ti", "isas-song ti", 
+	 NULL, NULL, "isas-song ti"},
+	
+/* zh-TW: Traditional Chinese */
+	{NULL, NULL, NULL, NULL, NULL}
+};
+
+const gchar *pref_key_prefix[] =
+{
+	"/galeon/Appearance/serif_font",
+	"/galeon/Appearance/sansserif_font",
+	"/galeon/Appearance/cursive_font",
+	"/galeon/Appearance/fantasy_font",
+	"/galeon/Appearance/monospace_font",
+	"/galeon/Appearance/minimum_font_size",
+	"/galeon/Appearance/default_font"
+};
+
 /**
- * mozilla_prefs_set: set fonts preferences
+ * mozilla_prefs_set_fonts_defaults: set fonts defaults
+ */
+void 
+mozilla_prefs_set_fonts_defaults (void)
+{
+	int i, k;
+	gchar *font_name;
+	gchar **splitted;
+	gchar key_buf[128];
+	gchar buf[128];
+
+	for (i = 0; font_languages[i]; i++)
+	{
+		for (k = 0; font_types[k]; k++)
+		{
+
+			font_name = 
+				mozilla_get_default_font 
+				(font_languages[i], font_types[k],
+				 font_defaults [i][k]);
+
+			if (!font_name) continue;
+
+			splitted = g_strsplit (font_name,"-",-1);	    
+			g_snprintf (key_buf, 128, "%s_%s",
+				    pref_key_prefix[k], font_languages[i]);
+			g_snprintf (buf, 128, "*-%s-%s-*-*-*-*-*-*-*-*-*-*-%s-%s",
+				    splitted[0], splitted[1], splitted[2],
+				    splitted[3]);
+			gnome_config_set_string (key_buf, buf);
+
+			g_strfreev (splitted);
+			g_free (font_name);
+		}
+	}
+
+	gnome_config_sync ();
+}
+
+/**
+ * mozilla_prefs_set_fonts: set fonts preferences
  */
 void 
 mozilla_prefs_set_fonts (void)
 {
 	gchar *font_name;
 	const gchar *default_font_name = 
-		"adobe-times-medium-r-normal-*-14-*-*-*-p-*-iso8859-1";
-	const gchar *default_font_size = "14";
-	const gchar *default_font_type = "0";
-	const gchar *pref_key_prefix[5] =
-	{
-		"/galeon/Appearance/serif_font",
-		"/galeon/Appearance/sansserif_font",
-		"/galeon/Appearance/monospace_font",
-		"/galeon/Appearance/size_font",
-		"/galeon/Appearance/default_font"
-	};
-	gchar *serif_font, *sansserif_font, *monospace_font;
-	gchar *default_font[2] = {"sans-serif","serif"};
-	gchar **font, buf[128];
-	int size_font,default_font_idx,i;
+		"*-adobe-times-medium-r-normal-*-14-*-*-*-p-*-iso8859-1";
+	const gchar *default_min_font_size = "8";
+	const gchar *default_font_type = "1";
 
+	gchar *serif_font, *sansserif_font, *cursive_font, *fantasy_font,
+		*monospace_font;
+	const gchar *default_font[2] = {"sans-serif","serif"};
+	gchar **font, buf[128];
+	int min_font_size, default_font_idx, i;
+	int default_font_size = 14;
+
+	/* for each encoding */
 	for (i = 0; i < LANG_ENC_NUM; i++)
 	{
 		g_snprintf (buf, 128, "%s_%s=%s", pref_key_prefix[0],
@@ -254,16 +382,25 @@ mozilla_prefs_set_fonts (void)
 		
 		g_snprintf (buf, 128, "%s_%s=%s", pref_key_prefix[2],
 			    lang_encode_item[i], default_font_name);
+		cursive_font = gnome_config_get_string (buf);
+
+		g_snprintf (buf, 128, "%s_%s=%s", pref_key_prefix[3],
+			    lang_encode_item[i], default_font_name);
+		fantasy_font = gnome_config_get_string (buf);
+
+		g_snprintf (buf, 128, "%s_%s=%s", pref_key_prefix[4],
+			    lang_encode_item[i], default_font_name);
 		monospace_font = gnome_config_get_string (buf);
 		
-		g_snprintf (buf, 128, "%s_%s=%s", pref_key_prefix[3],
-			    lang_encode_item[i], default_font_size);
-		size_font = gnome_config_get_int (buf);
+		g_snprintf (buf, 128, "%s_%s=%s", pref_key_prefix[5],
+			    lang_encode_item[i], default_min_font_size);
+		min_font_size = gnome_config_get_int (buf);
 		
-		g_snprintf (buf, 128, "%s=%s", pref_key_prefix[4], 
+		g_snprintf (buf, 128, "%s=%s", pref_key_prefix[6], 
 			    default_font_type);
-		default_font_idx = gnome_config_get_int(buf);
+		default_font_idx = gnome_config_get_int (buf);
 
+		/* set serif font */
 		font = g_strsplit (serif_font,"-",-1);
 		g_free (serif_font);
 		font_name = g_strconcat(font[1],"-",font[2],"-",
@@ -271,9 +408,22 @@ mozilla_prefs_set_fonts (void)
 		g_snprintf (buf, 128, "font.name.serif.%s", 
 			    lang_encode_item[i]);
 		mozilla_preference_set (buf, font_name);
+
+		/* set variable font size */
+		g_snprintf (buf, 128, "font.size.variable.%s", 
+			    lang_encode_item[i]);
+		if (strcmp (font[7], "*") == 0 || atoi (font[7]) == 0)
+		{
+			mozilla_preference_set_int (buf, default_font_size);
+		}
+		else
+		{
+			mozilla_preference_set_int (buf, atoi (font[7]));
+		}
 		g_strfreev (font);
 		g_free (font_name);
 		
+		/* set sans serif font */
 		font = g_strsplit (sansserif_font,"-",-1);
 		g_free (sansserif_font);
 		font_name = g_strconcat(font[1],"-",font[2],"-",font[13],
@@ -284,6 +434,29 @@ mozilla_prefs_set_fonts (void)
 		g_strfreev (font);
 		g_free (font_name);
 		
+		/* set cursive font */
+		font = g_strsplit (cursive_font,"-",-1);
+		g_free (cursive_font);
+		font_name = g_strconcat(font[1],"-",font[2],"-",font[13],
+					"-",font[14], NULL);
+		g_snprintf (buf, 128, "font.name.cursive.%s", 
+			    lang_encode_item[i]);
+		mozilla_preference_set (buf, font_name);
+		g_strfreev (font);
+		g_free (font_name);
+
+		/* set fantasy font */
+		font = g_strsplit (fantasy_font,"-",-1);
+		g_free (fantasy_font);
+		font_name = g_strconcat(font[1],"-",font[2],"-",font[13],
+					"-",font[14], NULL);
+		g_snprintf (buf, 128, "font.name.fantasy.%s", 
+			    lang_encode_item[i]);
+		mozilla_preference_set (buf, font_name);
+		g_strfreev (font);
+		g_free (font_name);
+
+		/* set monospace font */
 		font = g_strsplit (monospace_font,"-",-1);
 		g_free (monospace_font);
 		font_name = g_strconcat(font[1],"-",font[2],"-",font[13],
@@ -291,15 +464,28 @@ mozilla_prefs_set_fonts (void)
 		g_snprintf (buf, 128, "font.name.monospace.%s", 
 			    lang_encode_item[i]);
 		mozilla_preference_set (buf, font_name);
+
+		/* set fixed font size */
 		g_snprintf (buf, 128, "font.size.fixed.%s", 
 			    lang_encode_item[i]);
-		mozilla_preference_set_int (buf, atoi(font[7]));
+		if (strcmp (font[7], "*") == 0 || atoi (font[7]) == 0)
+		{
+			mozilla_preference_set_int (buf, default_font_size);
+		}
+		else
+		{
+			mozilla_preference_set_int (buf, atoi(font[7]));
+		}
 		g_strfreev (font);
 		g_free (font_name);
 		
-		g_snprintf (buf, 128, "font.size.variable.%s", 
+		/* set minimum font sizes */
+		g_snprintf (buf, 128, "font.min-size.variable.%s",
 			    lang_encode_item[i]);
-		mozilla_preference_set_int (buf,size_font);
+		mozilla_preference_set_int (buf, min_font_size);
+		g_snprintf (buf, 128, "font.min-size.fixed.%s",
+			    lang_encode_item[i]);
+		mozilla_preference_set_int (buf, min_font_size);
 	}
 
 	mozilla_preference_set ("font.default", 
@@ -428,31 +614,39 @@ mozilla_prefs_set_proxy (void)
 void 
 mozilla_prefs_set_cache (void)
 {
-	int disk_cache_size = gnome_config_get_int("/galeon/Advanced/disk_cache_size=7680");
-	int mem_cache_size = gnome_config_get_int("/galeon/Advanced/mem_cache_size=1024");
-	int frequency = gnome_config_get_int("/galeon/Advanced/cache_compare=0");
+	gint disk_cache_size;
+	gint mem_cache_size;
+	gint frequency;
 
-	mozilla_preference_set_int ("browser.cache.check_doc_frequency", frequency);
+	/* fetch from Galeon prefs */
+	disk_cache_size = gnome_config_get_int (CONF_ADVANCED_MEMORY_CACHE);
+	mem_cache_size = gnome_config_get_int (CONF_ADVANCED_DISK_CACHE);
+	frequency = gnome_config_get_int (CONF_ADVANCED_CACHE_COMPARE);
 
-	if (disk_cache_size > 0)
-	{
-		mozilla_preference_set_boolean ("browser.cache.disk.enable", TRUE);
-		mozilla_preference_set_int ("browser.cache.disk_cache_size", disk_cache_size);
-	}
-	else
-	{
-		mozilla_preference_set_boolean ("browser.cache.disk.enable", FALSE);
-	}
+	/* set in mozilla -- cache size 0 means disable */
+	mozilla_preference_set_boolean ("browser.cache.disk.enable", 
+					(disk_cache_size > 0));
+	mozilla_preference_set_boolean ("browser.enable.cache",
+					(mem_cache_size > 0));
+	mozilla_preference_set_int ("browser.cache.disk_cache_size", 
+				    disk_cache_size);
+	mozilla_preference_set_int ("browser.cache.memory_cache_size",
+				    mem_cache_size);
+	mozilla_preference_set_int ("browser.cache.check_doc_frequency", 
+				    frequency);
+}
 
-	if (mem_cache_size > 0)
-	{
-		mozilla_preference_set_boolean ("browser.enable.cache", TRUE);
-		mozilla_preference_set_int ("browser.cache.memory_cache_size", mem_cache_size);
-	}
-	else
-	{
-		mozilla_preference_set_boolean ("browser.enable.cache", FALSE);
-	}
+/**
+ * mozilla_prefs_set_download: sets download-related preferences
+ */
+void
+mozilla_prefs_set_download (void)
+{
+	gchar *dir;
+
+	dir = gnome_config_get_string (CONF_DOWNLOADING_DIR);
+	mozilla_preference_set ("browser.download.dir", dir);
+	g_free (dir);
 }
 
 /**
@@ -470,18 +664,14 @@ mozilla_prefs_set_autodetect_charset (int charset)
 void 
 mozilla_prefs_set_default_charset (void)
 {
-	char *charset; 
-	char *charset_code;
-
-	charset = gnome_config_get_string ("/galeon/Advanced/default_charset=Western (ISO-8859-1)");
-	charset_code = g_hash_table_lookup (charsets, charset);
+	char *charset_code = gnome_config_get_string 
+				("/galeon/Advanced/default_charset=ISO-8859-1");
 	
-	if (charset_code)
+	if (charset_code != NULL)
 	{
 		mozilla_preference_set ("intl.charset.default", charset_code);
 	}
-
-	g_free(charset);
+	g_free (charset_code);
 }
 
 /**
@@ -524,7 +714,7 @@ mozilla_prefs_set_languages (void)
 	}
 	g_free (strings);
 
-	if (strings)
+	if (languages)
 	{
 		mozilla_preference_set ("intl.accept_languages", languages);
 		g_free (languages);
