@@ -19,6 +19,7 @@
 #include "galeon.h"
 #include "misc.h"
 #include "embed.h"
+#include "glade.h"
 #include "bookmarks.h"
 
 #include <string.h>
@@ -34,9 +35,10 @@ void new_user_druid_cancel_cb (GtkWidget *widget, gpointer dummy);
 gboolean new_user_druid_delete_event_cb (GtkWidget *widget, GdkEventAny *event,
 					 gpointer dummy);
 void new_user_druid_create_smart_bookmarks_check_button_toggled_cb 
-     (GtkWidget *toggle);
+(GtkWidget *toggle);
 static void add_smart (BookmarkItem *parent, gchar *name, gchar *nick,
 		       gchar *url, gchar *pixmap_file);
+static void add_bookmarklets (BookmarkItem *parent);
 
 /**
  * new_user_druid_finish_cb: called when the user gets to the end of
@@ -48,7 +50,7 @@ new_user_druid_finish_cb (GtkWidget *widget, gpointer dummy)
 	static const gchar *galeon_show = "galeon \"%s\"";
 	GtkWidget *dialog;
 	gint i;
-
+	
 	/* values fetched from widgets */
 	gboolean import_netscape_bookmarks_checked;
 	gboolean import_mozilla_bookmarks_checked;
@@ -58,8 +60,12 @@ new_user_druid_finish_cb (GtkWidget *widget, gpointer dummy)
 	gboolean google_groups_checked;
 	gboolean dogpile_checked;
 	gboolean yahoo_checked;
-	gboolean merriam_websters_checked;
+	gboolean dictionary_checked;
 	gboolean freshmeat_checked;
+	gboolean rpmfind_checked;
+	gboolean debian_packages_checked;
+	gboolean bugzilla_checked;
+	gboolean bookmarklets_checked;
 	gboolean http_checked;
 	gboolean https_checked;
 	gboolean ftp_checked;
@@ -70,40 +76,48 @@ new_user_druid_finish_cb (GtkWidget *widget, gpointer dummy)
 	gint http_proxy_port;
 	gint ssl_proxy_port;
 	gint ftp_proxy_port;
-
+	
 	/* lookup table for check buttons */
 	const struct { gchar *name; gboolean *state; } check_lookups[] = {
-        { "new_user_druid_import_netscape_bookmarks_radio_button",
-	  &import_netscape_bookmarks_checked, },
-        { "new_user_druid_import_mozilla_bookmarks_radio_button",
-	  &import_mozilla_bookmarks_checked, },
-        { "new_user_druid_import_netscape_preferences_check_button",
-	  &import_netscape_preferences_checked, },
-	{ "new_user_druid_create_smart_bookmarks_check_button",
-	  &create_smart_bookmarks_checked, },
-	{ "new_user_druid_google_web_check_button",
-	  &google_web_checked, },
-	{ "new_user_druid_google_groups_check_button",
-	  &google_groups_checked, },
-	{ "new_user_druid_dogpile_check_button",
-	  &dogpile_checked, },
-	{ "new_user_druid_yahoo_check_button",
-	  &yahoo_checked, },
-	{ "new_user_druid_merriam_websters_check_button",
-	  &merriam_websters_checked, },
-	{ "new_user_druid_freshmeat_check_button",
-	  &freshmeat_checked, },
-	{ "new_user_druid_http_check_button",
-	  &http_checked, },
-	{ "new_user_druid_https_check_button",
-	  &https_checked, },
-	{ "new_user_druid_ftp_check_button",
-	  &ftp_checked, },
-	{ "new_user_druid_file_check_button",
-	  &file_checked, },
-	{ NULL, NULL }/* terminator, must be last */
+		{ "new_user_druid_import_netscape_bookmarks_radio_button",
+		  &import_netscape_bookmarks_checked, },
+		{ "new_user_druid_import_mozilla_bookmarks_radio_button",
+		  &import_mozilla_bookmarks_checked, },
+		{ "new_user_druid_import_netscape_preferences_check_button",
+		  &import_netscape_preferences_checked, },
+		{ "new_user_druid_create_smart_bookmarks_check_button",
+		  &create_smart_bookmarks_checked, },
+		{ "new_user_druid_google_web_check_button",
+		  &google_web_checked, },
+		{ "new_user_druid_google_groups_check_button",
+		  &google_groups_checked, },
+		{ "new_user_druid_dogpile_check_button",
+		  &dogpile_checked, },
+		{ "new_user_druid_yahoo_check_button",
+		  &yahoo_checked, },
+		{ "new_user_druid_dictionary_check_button",
+		  &dictionary_checked, },
+		{ "new_user_druid_freshmeat_check_button",
+		  &freshmeat_checked, },
+		{ "new_user_druid_rpmfind_check_button",
+		  &rpmfind_checked, },
+		{ "new_user_druid_debian_packages_check_button",
+		  &debian_packages_checked, },
+		{ "new_user_druid_bugzilla_check_button",
+		  &bugzilla_checked, },
+		{ "new_user_druid_bookmarklets_check_button",
+		  &bookmarklets_checked, },
+		{ "new_user_druid_http_check_button",
+		  &http_checked, },
+		{ "new_user_druid_https_check_button",
+		  &https_checked, },
+		{ "new_user_druid_ftp_check_button",
+		  &ftp_checked, },
+		{ "new_user_druid_file_check_button",
+		  &file_checked, },
+		{ NULL, NULL }/* terminator, must be last */
         };
-
+	
 	/* lookup table for string entry fields */
 	const struct { gchar *name; gchar **value; } edit_lookups[] = {
 		{ "new_user_http_proxy_site", &http_proxy_site },
@@ -111,7 +125,7 @@ new_user_druid_finish_cb (GtkWidget *widget, gpointer dummy)
 		{ "new_user_ftp_proxy_site",  &ftp_proxy_site },
 		{ NULL, NULL }/* terminator, must be last */
 	};
-
+	
 	/* lookup table for spin entrys */
 	const struct { gchar *name; gint *value; } spin_lookups[] = {
 		{ "new_user_http_proxy_port", &http_proxy_port },
@@ -119,74 +133,79 @@ new_user_druid_finish_cb (GtkWidget *widget, gpointer dummy)
 		{ "new_user_ftp_proxy_port",  &ftp_proxy_port },
 		{ NULL, NULL }/* terminator, must be last */
 	};
-
+	
 	/* lookup all the widgets in the list */
 	for (i = 0; check_lookups[i].name != NULL; i++)
 	{
 		GtkWidget *check;
-
+		
 		/* find the checkbutton widget */
 		check = glade_lookup_widget (widget, check_lookups[i].name);
-
+		
 		/* get it's state and store it */
 		*(check_lookups[i].state) = gtk_toggle_button_get_active
 			(GTK_TOGGLE_BUTTON (check));
 	}
-
+	
 	/* lookup all the widgets in the list */
 	for (i = 0; edit_lookups[i].name != NULL; i++)
 	{
 		GtkWidget *edit;
-
+		
 		/* find the entry widget */
 		edit = glade_lookup_widget (widget, edit_lookups[i].name);
-
+		
 		/* get all the text into a new string */
 		*(edit_lookups[i].value) = 
 			gtk_editable_get_chars (GTK_EDITABLE (edit), 0, -1);
 	}
-
+	
 	/* lookup all the widgets in the list */
 	for (i = 0; spin_lookups[i].name != NULL; i++)
 	{
 		GtkWidget *spin;
-
+		
 		/* find the spinbutton widget */
 		spin = glade_lookup_widget (widget, spin_lookups[i].name);
-
+		
 		/* get the value as an int */
 		*(spin_lookups[i].value) = 
 			gtk_spin_button_get_value_as_int 
 			(GTK_SPIN_BUTTON (spin));
 	}
-
+	
 	/* get preferences from netscape */
 	if (import_netscape_preferences_checked)
 	{
 		netscape_import_prefs ();
 	}
-
+	
 	/* create smart bookmarks toolbar */
 	if (create_smart_bookmarks_checked)
 	{
 		BookmarkItem *toolbar;
-
+		
 		/* create the toolbar */
 		toolbar = bookmarks_new_bookmark 
-			(BM_CATEGORY, TRUE, _("Smart Bookmarks Toolbar"),
+			(BM_FOLDER, TRUE, _("Smart Bookmarks Toolbar"),
 			 NULL, NULL, NULL, NULL);
 		toolbar->create_toolbar = TRUE;
-
+		toolbar->toolbar_style = TOOLBAR_STYLE_HORIZONTAL;
+		
 		/* add it into the root folder */
 		bookmarks_root->list = 
 			g_list_append (bookmarks_root->list, toolbar);
 		toolbar->parent = bookmarks_root;
-
+		
 		/* add each smart bookmark checked */
 		if (google_web_checked)
 		{
+			/* URL marked as translatable so you can add the
+			 * appropriate lang= bits to get the results in the
+			 * right language (normally detection is done at
+			 * the google homepage I think -- MattA) */
 			add_smart (toolbar, "Google", "google",
-				   "http://www.google.com/search?q=%s",
+				   _("http://www.google.com/search?q=%s"),
 				   SHARE_DIR "/google.png");
 		}
 		if (google_groups_checked)
@@ -215,7 +234,7 @@ new_user_druid_finish_cb (GtkWidget *widget, gpointer dummy)
 				   "http://search.yahoo.com/bin/search?p=%s",
 				   NULL);
 		}
-		if (merriam_websters_checked)
+		if (dictionary_checked)
 		{
 			/* URL marked as translatable so that non-English
 			 * dictionaries can be used -- don't forget to
@@ -225,13 +244,41 @@ new_user_druid_finish_cb (GtkWidget *widget, gpointer dummy)
 				   _("http://www.m-w.com/cgi-bin/"
 				     "dictionary?va=%s"), NULL);
 		}
+		if (bugzilla_checked)
+		{
+			add_smart (toolbar, "Bug #", "bug",
+				   "http://bugzilla.gnome.org/"
+				   "show_bug.cgi?id=%s", 
+				   SHARE_DIR "/bug.png");
+		}
 		if (freshmeat_checked)
 		{
-			add_smart (toolbar, "fm ][", "fm",
-				   "http://freshmeat.net/search?q=%s", NULL);
+			add_smart (toolbar, "Freshmeat", "fm",
+				   "http://freshmeat.net/search?q=%s",
+				   SHARE_DIR "/freshmeat.png");
+		}
+		if (rpmfind_checked)
+		{
+			add_smart (toolbar, "RPMFind", "rpm",
+				   "http://rpmfind.net/linux/rpm2html/"
+				   "search.php?query=%s",
+				   SHARE_DIR "/rpm.png");
+		}
+		if (debian_packages_checked)
+		{
+			add_smart (toolbar, "Debian", "deb",
+				   "http://packages.debian.org/cgi-bin/"
+				   "search_packages.pl?keywords=%s&"
+				   "searchon=names&subword=1&version=all&"
+				   "release=all", 
+				   SHARE_DIR "/debian.png");
+		}
+		if (bookmarklets_checked)
+		{
+			add_bookmarklets (toolbar);
 		}
 	}
-
+	
 	/* import bookmarks */
 	if (import_netscape_bookmarks_checked)
 	{
@@ -242,13 +289,13 @@ new_user_druid_finish_cb (GtkWidget *widget, gpointer dummy)
 	{
   	        /* FIXME: dont use this callback directly! */
 	        bookmarks_import_mozilla_activate_cb (NULL, NULL);
-
+		
 #ifdef ENABLE_NLS
 		/* FIXME: right thing in the right place? */
 		mozilla_prefs_import_language ();
 #endif
 	}
-
+	
 	/* set URL handlers as requested */
 	gnome_config_push_prefix ("/Gnome/URL Handlers/");
 	if (http_checked)
@@ -268,7 +315,7 @@ new_user_druid_finish_cb (GtkWidget *widget, gpointer dummy)
 		gnome_config_set_string ("file-show", galeon_show);
 	}
 	gnome_config_pop_prefix ();
-
+	
 	/* setup proxies */
 	gnome_config_push_prefix("/galeon/Advanced/");
 	if (http_proxy_site != NULL && strlen (http_proxy_site) > 0)
@@ -286,13 +333,16 @@ new_user_druid_finish_cb (GtkWidget *widget, gpointer dummy)
 	/* kill the dialog */
 	dialog = glade_lookup_widget (widget, "new_user_druid");
 	gtk_widget_destroy (GTK_WIDGET (dialog));
-
+	
 	/* save all the initial settings */
 	gnome_config_sync ();
-	bookmarks_save ();
-
-	/* create an initial window */
-	embed_create_default (NULL, TRUE);
+	bookmarks_updated ();
+	
+	/* create an initial window if needed */
+	if (g_list_length (all_windows) == 0)
+	{
+		embed_create_default (NULL, TRUE);
+	}
 }
 
 /**
@@ -303,13 +353,16 @@ void
 new_user_druid_cancel_cb (GtkWidget *widget, gpointer dummy)
 {
 	GtkWidget *dialog;
-
+	
 	/* kill the dialog */
 	dialog = glade_lookup_widget (widget, "new_user_druid");
 	gtk_widget_destroy (GTK_WIDGET (dialog));
-
-	/* create an initial window */
-	embed_create_default (NULL, TRUE);
+	
+	/* create an initial window if needed */
+	if (g_list_length (all_windows) == 0)
+	{
+		embed_create_default (NULL, TRUE);
+	}
 }
 
 /**
@@ -320,9 +373,12 @@ gboolean
 new_user_druid_delete_event_cb (GtkWidget *widget, GdkEventAny *event,
 				gpointer dummy)
 {
-	/* create an initial window */
-	embed_create_default (NULL, TRUE);
-
+	/* create an initial window if needed */
+	if (g_list_length (all_windows) == 0)
+	{
+		embed_create_default (NULL, TRUE);
+	}
+	
 	/* still destroy the druid window */
 	return FALSE;
 }
@@ -332,45 +388,104 @@ new_user_druid_delete_event_cb (GtkWidget *widget, GdkEventAny *event,
  * of options on and off when the user toggles a master switch.
  */
 void
-new_user_druid_create_smart_bookmarks_check_button_toggled_cb
+new_user_druid_create_smart_bookmarks_check_button_toggled_cb 
 (GtkWidget *toggle)
 {
-	GtkWidget *options_table;
+	GtkWidget *options_frame;
 	gboolean sensitive;
-	GList *l;
 	
 	/* find the new state */
 	sensitive = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (toggle));
-
-	/* find the table containing the list of options */
-	options_table = 
+	
+	/* find the frame containing the list of options */
+	options_frame = 
 		glade_lookup_widget (GTK_WIDGET (toggle), 
 				     "new_user_druid_smart_bookmarks_list");
-
-	/* iterate over it's children (each is an check button) */
-	l = GTK_TABLE (options_table)->children;
-	for (; l != NULL; l = g_list_next (l))
-	{
-		GtkTableChild *child = (GtkTableChild *)(l->data);
-		gtk_widget_set_sensitive (GTK_WIDGET (child->widget),
-					  sensitive);
-	}
+	
+	/* set the frame sensitivity, this will set all children too */
+	gtk_widget_set_sensitive (GTK_WIDGET (options_frame), sensitive);
 }
 
 /**
  * add_smart: add a bookmark to the smart bookmarks toolbar
  */
-static
-void add_smart (BookmarkItem *parent, gchar *name, gchar *nick, 
-		gchar *url, gchar *pixmap_file)
+static void
+add_smart (BookmarkItem *parent, gchar *name, gchar *nick, 
+	   gchar *url, gchar *pixmap_file)
 {
 	BookmarkItem *site;
-
+	
 	/* create the bookmarks */
 	site = bookmarks_new_bookmark (BM_SITE, TRUE, name, url, nick, NULL, 
 				       pixmap_file);
-
+	
 	/* add it to the toolbar */
 	parent->list = g_list_append (parent->list, site);
 	site->parent = parent;
 }
+
+/**
+ * add_bookmarklets: add some bookmarklets
+ */
+static void 
+add_bookmarklets (BookmarkItem *parent)
+{
+	gint i;
+	BookmarkItem *root;
+	BookmarkItem *item;
+	static const struct
+	{ 
+		gchar *name;
+		gchar *url;  
+	} bookmarklets[] = {
+		{ N_("More Bookmarklets"), "http://www.bookmarklets.com/" }, 
+		{ "separator", NULL },
+		{ N_("More Info About..."), "javascript:if(frames.length>0){F=' (Open frame in new window first.)'}else{F=''}Q=document.getSelection();if(!Q){void(Q=prompt('No text selected on page.'+F+'\\n\\nKeywords...?',''))};if(Q)location.href='http://bookmarklets.com/moreinfo.phtml?q='+escape(Q)" },
+		{ "separator", NULL },
+		{ N_("Zoom on First Image..."), "javascript:if(document.images.length<1){alert('No image!')}else{D0iB=document.images[0];sZ2T=prompt('Resize by..','1.5');if(sZ2T){with(document){write('<img src='+D0iB.src+' width='+sZ2T*D0iB.width+' height='+sZ2T*D0iB.height+'>');close()}}}void(null)" },
+		{ N_("Page Color..."), "javascript:if(frames.length<1){void(document.bgColor=prompt('Change to which background color?',''))}else{alert('Page has frames. Use the version of this bookmarklet for frames. (bookmarklets.com)')}" },
+		{ "separator", NULL },
+		{ N_("Scroll Page (very slow)"), "javascript:var wN2sc8Rl;Sa5gNA9k=new Function('clearTimeout(wN2sc8Rl)');document.onkeydown=Sa5gNA9k;Sa5gNA9k();void(wN2sc8Rl=setInterval('if(pageYOffset<document.height-innerHeight){window.scrollBy(0,1)}else{Sa5gNA9k()}',150))" },
+		{ N_("Scroll Page (slow)"), "javascript:var wN2scRl;Sa5gNA9k=new Function('clearTimeout(wN2scRl)');document.onkeydown=Sa5gNA9k;Sa5gNA9k();void(wN2scRl=setInterval('if(pageYOffset<document.height-innerHeight){window.scrollBy(0,1)}else{Sa5gNA9k()}',50))" },
+		{ N_("Scroll Page (fast)"), "javascript:var wN2scRl;Sa5gNA9k=new Function('clearTimeout(wN2scRl)');document.onkeydown=Sa5gNA9k;Sa5gNA9k();void(wN2scRl=setInterval('if(pageYOffset<document.height-innerHeight){window.scrollBy(0,2)}else{Sa5gNA9k()}',50))" },
+		{ N_("Scroll Page (very fast)"), "javascript:var wN2scRl;Sa5gNA9k=new Function('clearTimeout(wN2scRl)');document.onkeydown=Sa5gNA9k;Sa5gNA9k();void(wN2scRl=setInterval('if(pageYOffset<document.height-innerHeight){window.scrollBy(0,10)}else{Sa5gNA9k()}',50))" },
+		{ N_("Scroll Page... (variable)"), "javascript:var wN2scRl;S3g=new Function('clearTimeout(wN2scRl)');document.onkeydown=S3g;a7T=prompt('Rate...','');if(a7T!=null){S3g();void(wN2scRl=setInterval('if(pageYOffset<document.height-innerHeight){scrollBy(0,a7T)}else{S3g()}',50))}else{void(null)}" },
+		{ "separator", NULL },
+		{ N_("Page Freshness?"), "javascript:if(frames.length<1){alert('The server indicates that the page was last modified: ' + window.document.lastModified)}else{alert('Page is framed. Use version of bookmarklet for frames. (bookmarklets.com)')}" },
+		{ N_("Go To Selected URL"), "javascript:if(frames.length>0){alert('Frames!')}else{ulBV3pQy=document.getSelection();if(ulBV3pQy!=''){location.href=ulBV3pQy.replace(/\\042/g,'').replace(/\\047/g,'').replace(/\\s/g,'')}else{alert('Select text on page like:\\n   http://www.domain.com...')}}" },
+		{ N_("Duplicate Page"), "javascript:dT30FfN3b=new Date();void(window.open(location.href,'w'+dT30FfN3b.getTime()))" },
+		{ N_("Set In-Page Bookmark"), "javascript:void(InPageBookmark=pageYOffset)" },
+		{ N_("Go To In-Page Bookmark"), "javascript:if (typeof InPageBookmark=='undefined'){alert('No in-page bookmark. Use Set In-Page Bookmark tool. (bookmarklets.com)')}else{scrollBy(0,InPageBookmark-pageYOffset)}" },
+		{ "separator", NULL },
+		{ N_("Read Cookie for Site"), "javascript:if(document.cookie.length<1){alert('No cookie for this site.')}else{alert('Cookie for this site: '+document.cookie)}" },
+		{ N_("AutoFill Anonymous"), "javascript:function ROIoiW(){var i=0,j,A='anonymous',D,E,F=document.forms;while(i<F.length){E=F[i].elements;for(j=0;j<E.length;j++){D=E[j];if(D.type=='text'){D.value=(D.name.toUpperCase().indexOf('MAIL')!=-1)?A+'@example.com':A}}i++}}ROIoiW();void(null)" },
+		{ NULL, NULL}
+	};
+
+	root = bookmarks_new_bookmark (BM_FOLDER, TRUE, _("Bookmarklets"),
+				       NULL, NULL, NULL, NULL);
+	
+	parent->list = g_list_append (parent->list, root);
+	root->parent = parent;
+	root->create_context_menu = TRUE;
+	
+	for (i = 0; bookmarklets[i].name != NULL; i++)
+	{
+		if (bookmarklets[i].url == NULL)
+		{
+			item = bookmarks_new_bookmark 
+				(BM_SEPARATOR, FALSE, NULL, NULL, NULL, NULL,
+				 NULL);
+		}
+		else
+		{
+			item = bookmarks_new_bookmark
+				(BM_SITE, TRUE, _(bookmarklets[i].name), 
+				 bookmarklets[i].url, NULL, NULL, NULL);
+		}
+
+		item->parent = root;
+		root->list = g_list_append (root->list, item);
+	}
+}
+
