@@ -17,12 +17,14 @@
  */
 
 #include "galeon.h"
+#include "mime.h"
 #include "misc.h"
 #include "spinner.h"
 #include "history.h"
 #include "prefs.h"
 #include "window.h"
 #include "mozilla.h"
+#include "dialog.h"
 
 #include <gtk/gtkcombo.h>
 #include <gtk/gtkoptionmenu.h>
@@ -51,10 +53,6 @@ enum
 };
 
 /* local function prototypes */
-void prefs_ctree_tree_select_row_cb (GtkWidget *ctree, GtkCTreeNode *node,
-				     gint column, PreferencesDialog *pd);
-void prefs_ctree_tree_unselect_row_cb (GtkWidget *ctree, GList *node,
-				       gint column, PreferencesDialog *pd);
 void prefs_ok_button_clicked_cb (GtkButton *button, PreferencesDialog *pd);
 void prefs_apply_button_clicked_cb (GtkButton *button, PreferencesDialog *pd);
 void prefs_cancel_button_clicked_cb (GtkButton *button, PreferencesDialog *pd);
@@ -71,8 +69,6 @@ void prefs_mime_list_select_row_cb (GtkCList *clist, gint row, gint column,
 void prefs_mime_list_unselect_row_cb (GtkCList *clist, gint row, gint column,
 				      GdkEventButton *event, 
 				      PreferencesDialog *pd);
-void prefs_ctree_tree_expand_cb (GtkCTree *ctree, GList *node,
-				 PreferencesDialog *pd);
 void prefs_homepage_my_portal_button_clicked_cb (GtkButton *button,
 						 PreferencesDialog *pd);
 void prefs_homepage_current_button_clicked_cb (GtkButton *button,
@@ -92,7 +88,8 @@ void prefs_clear_memory_cache_button_clicked_cb (GtkButton *button,
 void prefs_clear_history_button_clicked_cb (GtkButton *button,
 					    PreferencesDialog *pd);
 void prefs_clear_history_question_cb (gint reply, gpointer data);
-void prefs_browse_clicked_cb (GnomeFileEntry *fileentry, gpointer data);
+void prefs_browse_clicked_cb (GnomeFileEntry *fileentry, 
+			      PreferencesDialog *pd);
 void prefs_font_picker_clicked_cb (GtkWidget *widget, PreferencesDialog *pd);
 void prefs_color_picker_clicked_cb (GtkWidget *widget, PreferencesDialog *pd);
 void prefs_spinner_select_icon_cb (GnomeIconList *gil, gint num, 
@@ -103,8 +100,38 @@ void preferences_switch_page_cb (GtkWidget *notebook,
 				 GtkNotebookPage *notebook_page,
 				 guint page_num,
 				 PreferencesDialog *pd);
-void prefs_cookie_manager_button_clicked_cb (GtkWidget *button, 
+void prefs_image_sites_button_clicked_cb (GtkWidget *button, 
+					  PreferencesDialog *pd);
+void prefs_cookies_button_clicked_cb (GtkWidget *button, 
+				      PreferencesDialog *pd);
+void prefs_cookie_sites_button_clicked_cb (GtkWidget *button, 
+					   PreferencesDialog *pd);
+void prefs_passwords_button_clicked_cb (GtkWidget *button, 
+					PreferencesDialog *pd);
+void prefs_password_sites_button_clicked_cb (GtkWidget *button, 
 					     PreferencesDialog *pd);
+void prefs_download_use_command_toggled_cb (GtkWidget *button, 
+					    PreferencesDialog *pd);
+void prefs_mime_list_sort_cb (GtkCList *clist, gint column,
+			      PreferencesDialog *pd);
+void prefs_mime_save_to_disk_toggled_cb (GtkWidget *button, 
+					 PreferencesDialog *pd);
+
+/**
+ * prefs_mime_list_sort_cb:
+ */
+void prefs_mime_list_sort_cb (GtkCList *clist, gint column,
+			      PreferencesDialog *pd)
+{
+	if (column == pd->mime_last_sort_column)
+		pd->mime_last_sort_type = !pd->mime_last_sort_type;
+	else
+		pd->mime_last_sort_type = GTK_SORT_ASCENDING;
+	gtk_clist_set_sort_type (clist, pd->mime_last_sort_type);
+	pd->mime_last_sort_column = column;
+	gtk_clist_set_sort_column (clist, column);
+	gtk_clist_sort (clist);
+}
 
 /**
  * prefs_ok_button_clicked_cb: preferences dialog ok button clicked
@@ -179,14 +206,20 @@ prefs_mime_add_button_clicked_cb (GtkButton *button, PreferencesDialog *pd)
 {
 	MimeItem *newitem = g_new0 (MimeItem, 1);
 	GtkWidget *clist = pd->mime_list;
-	gchar *text[3];
+	gchar *text[5];
 	gint row;
 
-	newitem->name = g_strdup (_("New mime item"));
+	newitem->name = g_strdup (N_("New mime item"));
 	newitem->action = MIME_UNKNOWN;
+	newitem->helperApp = g_strdup (N_("None"));
+	newitem->terminalHelper = FALSE;
+	newitem->urlHelper = FALSE;
+	newitem->alwaysUse = FALSE;
 	text[0] = newitem->name;
 	text[1] = N_("Unknown");
-	text[2] = NULL;
+	text[2] = newitem->helperApp;
+	text[3] = N_("False");
+	text[4] = NULL;
 	row = gtk_clist_append (GTK_CLIST (clist), text);
 	gtk_clist_set_row_data (GTK_CLIST (clist), row, newitem);
 	gtk_clist_select_row (GTK_CLIST (clist), row, 0);
@@ -208,7 +241,9 @@ prefs_mime_delete_button_clicked_cb (GtkButton *button, PreferencesDialog *pd)
 	row = gtk_clist_find_row_from_data (GTK_CLIST (pd->mime_list),
 					    pd->selected_mime);
 	gtk_clist_remove (GTK_CLIST (pd->mime_list), row);
+	mime_item_free (pd->selected_mime);
 	pd->selected_mime = NULL;
+	gtk_clist_select_row (GTK_CLIST (pd->mime_list), row, 0);
 }
 
 
@@ -229,29 +264,31 @@ void prefs_mime_list_select_row_cb (GtkCList *clist,
 	switch (pd->selected_mime->action)
 	{
 	case MIME_SAVE_TO_DISK:
-		act_num = 0; /* save */
+		act_num = 0;	/* save */
 		break;
-	case MIME_SAVE_WITH_GTM:
-		act_num = 1; /* save with External Download Handler */
+	case MIME_RUN_PROGRAM:	/* run with helper app */
+		act_num = 1;
 		break;
-	case MIME_ASK_ACTION:
-		act_num = 3; /* Ask */
-		break;
-	case MIME_RUN_PROGRAM: /* run with helper app */
+	case MIME_ASK_ACTION:	/* Ask */
+	case MIME_UNKNOWN:	/* default to ask */
 		act_num = 2;
 		break;
-	case MIME_UNKNOWN:
-		act_num = 3; /* default to ask */
-		break;
-	case MIME_OVERRIDE_MOZILLA: /* Override a mozilla built-in type */
 	case MIME_IGNORE:
 		break;
 	}
 
-	gtk_option_menu_set_history 
-		(GTK_OPTION_MENU (pd->mime_action_optionmenu), act_num);
+	gtk_radio_button_set (GTK_RADIO_BUTTON (pd->mime_save_to_disk),
+			      act_num);
 	gtk_entry_set_text (GTK_ENTRY (pd->mime_name_entry), 
 			    pd->selected_mime->name);
+	gtk_entry_set_text (GTK_ENTRY (pd->mime_helper_entry),
+			    pd->selected_mime->helperApp);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(pd->mime_always_use),
+				      pd->selected_mime->alwaysUse);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(pd->mime_terminal),
+				      pd->selected_mime->terminalHelper);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(pd->mime_url_helper),
+				      pd->selected_mime->urlHelper);
 	pd->enable_edit_mime = TRUE;
 }
 
@@ -266,27 +303,28 @@ prefs_mime_list_unselect_row_cb (GtkCList *clist, gint row, gint column,
 	pd->enable_edit_mime = FALSE;
 
 	/* The default is Ask */
-	gtk_option_menu_set_history 
-		(GTK_OPTION_MENU (pd->mime_action_optionmenu), 2); 
 	gtk_entry_set_text (GTK_ENTRY (pd->mime_name_entry), "");
-
+	gtk_entry_set_text (GTK_ENTRY (pd->mime_helper_entry), "");
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(pd->mime_always_use),
+				      FALSE);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(pd->mime_terminal),
+				      FALSE);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(pd->mime_url_helper),
+				      FALSE);
+				      
 }
 
+/* FIXME! */
 void
 prefs_mime_edit_cb (GtkWidget *w, PreferencesDialog *pd)
 {
-	gchar *text[3];
-	gint row;
-        GtkWidget *action_menu;
-        GList *l;
-        gpointer item;
         gint action_index;
+	gchar *text[4];
+	gint row;
 	
         /* extract the action menu index */
-        action_menu = GTK_OPTION_MENU (pd->mime_action_optionmenu)->menu;
-        l = GTK_MENU_SHELL (action_menu)->children;
-        item = gtk_menu_get_active (GTK_MENU (action_menu));
-        action_index = g_list_index (l, item);
+	action_index = gtk_radio_button_get 
+		(GTK_RADIO_BUTTON (pd->mime_save_to_disk));
 
 	if (pd->selected_mime == NULL || !pd->enable_edit_mime)
 	{
@@ -300,24 +338,34 @@ prefs_mime_edit_cb (GtkWidget *w, PreferencesDialog *pd)
 	pd->selected_mime->name = gtk_editable_get_chars 
 		(GTK_EDITABLE (pd->mime_name_entry), 0, -1);
 
+	if (pd->selected_mime->helperApp)
+	{
+		g_free (pd->selected_mime->helperApp);
+	}
+	pd->selected_mime->helperApp = gtk_editable_get_chars 
+		(GTK_EDITABLE (pd->mime_helper_entry), 0, -1);
+
 	switch (action_index)
 	{
 	case 0:
 		pd->selected_mime->action = MIME_SAVE_TO_DISK;
 		break;
 	case 1:
-		pd->selected_mime->action = MIME_SAVE_WITH_GTM;
-		break;
-	case 2:
 		pd->selected_mime->action = MIME_RUN_PROGRAM;
 		break;
-	case 3:
+	case 2:
 		pd->selected_mime->action = MIME_ASK_ACTION;
 		break;
 	default:
 		pd->selected_mime->action = MIME_UNKNOWN;
 		break;
 	}
+	pd->selected_mime->terminalHelper = gtk_toggle_button_get_active
+				(GTK_TOGGLE_BUTTON(pd->mime_terminal));
+	pd->selected_mime->urlHelper = gtk_toggle_button_get_active
+				(GTK_TOGGLE_BUTTON(pd->mime_url_helper));
+	pd->selected_mime->alwaysUse = gtk_toggle_button_get_active
+				(GTK_TOGGLE_BUTTON(pd->mime_always_use));
 
 	row = gtk_clist_find_row_from_data (GTK_CLIST (pd->mime_list),
 					    pd->selected_mime);
@@ -325,11 +373,8 @@ prefs_mime_edit_cb (GtkWidget *w, PreferencesDialog *pd)
 	text[0] = pd->selected_mime->name;
 	switch (pd->selected_mime->action)
 	{
-	case MIME_SAVE_WITH_GTM:
-		text[1] = N_("Save to disk with External Download Handler");
-		break;
 	case MIME_SAVE_TO_DISK:
-		text[1] = N_("Save to disk with Built-in Download Handler");
+		text[1] = N_("Save to disk");
 		break;
 	case MIME_RUN_PROGRAM:  
 		text[1] = N_("Run with Helper App");
@@ -340,64 +385,19 @@ prefs_mime_edit_cb (GtkWidget *w, PreferencesDialog *pd)
 	case MIME_ASK_ACTION:
 		text[1] = N_("Ask the user");
 		break;
-	case MIME_OVERRIDE_MOZILLA:
 	case MIME_IGNORE:
 		break;
 	}
+	text[2] = pd->selected_mime->helperApp;
+	if (pd->selected_mime->alwaysUse)
+		text[3] = N_("True");
+	else
+		text[3] = N_("False");
 
 	gtk_clist_set_text (GTK_CLIST (pd->mime_list), row, 0, text[0]);
 	gtk_clist_set_text (GTK_CLIST (pd->mime_list), row, 1, text[1]);
-}
-
-/**
- * prefs_ctree_tree_select_row_cb: select a prefs category
- */
-void
-prefs_ctree_tree_select_row_cb (GtkWidget *ctree, GtkCTreeNode *node,
-				gint column, PreferencesDialog *pd)
-{
-	int row = 0;
-	
-	row = (gint) gtk_ctree_node_get_row_data (GTK_CTREE (ctree), 
-						  GTK_CTREE_NODE (node));
-	gtk_notebook_set_page (GTK_NOTEBOOK (pd->notebook), row);
-}
-
-/**
- * prefs_ctree_tree_unselect_row_cb: unselect a prefs category
- */
-void
-prefs_ctree_tree_unselect_row_cb (GtkWidget *ctree, GList*node,
-				  gint column, PreferencesDialog *pd)
-{
-	gtk_notebook_set_page (GTK_NOTEBOOK (pd->notebook), 0);
-}
-
-/**
- * automatically scroll the prefs tree window if clicking on either
- * of the last two items has made the scrollbars appear
- */
-void 
-prefs_ctree_tree_expand_cb (GtkCTree *ctree, GList *node, 
-			    PreferencesDialog *pd)
-{
-	GtkCTreeNode *last_child;
-	GtkAdjustment *vadj;
-	gint newpos;
-
-	last_child = GTK_CTREE_ROW(node)->children;
-	while (GTK_CTREE_ROW(last_child)->sibling != NULL)
-		last_child = GTK_CTREE_ROW(last_child)->sibling;
-
-	if (gtk_ctree_node_is_visible(ctree, last_child) != 
-	    GTK_VISIBILITY_FULL)
-	{
-		vadj = gtk_scrolled_window_get_vadjustment
-			(GTK_SCROLLED_WINDOW(pd->scrolledwindow9));
-		newpos = vadj->upper - vadj->lower - vadj->page_size;
-		gtk_adjustment_set_value(vadj, newpos);
-		gtk_adjustment_value_changed(vadj);
-	}
+	gtk_clist_set_text (GTK_CLIST (pd->mime_list), row, 2, text[2]);
+	gtk_clist_set_text (GTK_CLIST (pd->mime_list), row, 3, text[3]);
 }
 
 /**
@@ -506,11 +506,10 @@ prefs_clear_history_button_clicked_cb (GtkButton *button,
 	dialog = gnome_question_dialog_modal_parented
 		(_("This will delete all items stored in your history.\n"
 		   "Are you sure you want to do this?"),
-		 prefs_clear_history_question_cb,
-		 NULL,
+		 (GnomeReplyCallback)prefs_clear_history_question_cb, pd,
 		 GTK_WINDOW (pd->dialog));
 	
-	window_set_layer (dialog);
+	dialog_set_parent (dialog, pd->dialog);
 	gnome_dialog_run (GNOME_DIALOG (dialog));
 }
 
@@ -527,24 +526,26 @@ prefs_clear_history_question_cb (gint reply, gpointer data)
 }
 
 void
-prefs_browse_clicked_cb (GnomeFileEntry *fileentry, gpointer data)
+prefs_browse_clicked_cb (GnomeFileEntry *fileentry, PreferencesDialog *pd)
 {
 	if (fileentry->fsw != NULL)
 	{
-		window_set_layer (fileentry->fsw);
+		dialog_set_parent (fileentry->fsw, pd->dialog);
 	}
 }
 
 void
 prefs_font_picker_clicked_cb (GtkWidget *widget, PreferencesDialog *pd)
 {
-	window_set_layer (GNOME_FONT_PICKER (widget)->font_dialog);
+	dialog_set_parent (GNOME_FONT_PICKER (widget)->font_dialog,
+			   pd->dialog);
 }
 
 void
 prefs_color_picker_clicked_cb (GtkWidget *widget, PreferencesDialog *pd)
 {
-	window_set_layer (GNOME_COLOR_PICKER (widget)->cs_dialog);
+	dialog_set_parent (GNOME_COLOR_PICKER (widget)->cs_dialog,
+			   pd->dialog);
 }
 
 void 
@@ -588,12 +589,80 @@ preferences_switch_page_cb (GtkWidget *notebook,
 }
 
 /**
- * prefs_cookie_manager_button_clicked_cb: open up cookies manager when 
- * button is clicked.
+ * prefs_image_sites_button_clicked_cb
  */
 void
-prefs_cookie_manager_button_clicked_cb (GtkWidget *button, 
-					PreferencesDialog *pd)
+prefs_image_sites_button_clicked_cb (GtkWidget *button, 
+				     PreferencesDialog *pd)
 {
-	cookies_show_page (COOKIES);
+	pdm_show_page (NULL, IMAGESITES);
+}
+
+/**
+ * prefs_cookies_button_clicked_cb
+ */
+void
+prefs_cookies_button_clicked_cb (GtkWidget *button, 
+				 PreferencesDialog *pd)
+{
+	pdm_show_page (NULL, COOKIES);
+}
+
+/**
+ * prefs_cookie_sites_button_clicked_cb
+ */
+void
+prefs_cookie_sites_button_clicked_cb (GtkWidget *button, 
+				      PreferencesDialog *pd)
+{
+	pdm_show_page (NULL, COOKIESITES);
+}
+
+/**
+ * prefs_passwords_button_clicked_cb
+ */
+void
+prefs_passwords_button_clicked_cb (GtkWidget *button, 
+				   PreferencesDialog *pd)
+{
+	pdm_show_page (NULL, PASSWORDS);
+}
+
+/**
+ * prefs_password_sites_button_clicked_cb
+ */
+void
+prefs_password_sites_button_clicked_cb (GtkWidget *button, 
+				        PreferencesDialog *pd)
+{
+	pdm_show_page (NULL, PASSWORDSITES);
+}
+
+/**
+ * prefs_downloader_use_command_toggled_cb: called when the use command
+ * radiobutton is toggled; we have to set the command entry and terminal
+ * widgets to sensitive or not, appropriately.
+ */
+void
+prefs_download_use_command_toggled_cb (GtkWidget *button, 
+				       PreferencesDialog *pd)
+{
+	gboolean active;
+
+	active = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button));
+	gtk_widget_set_sensitive (pd->download_command_combo,   active);
+	gtk_widget_set_sensitive (pd->download_run_in_terminal, active);
+}
+
+/**
+ * prefs_mime_save_to_disk_toggled_cb: called whenever "save to disk" is
+ * toggled, make the helper chooser frame sensitive or not accordingly
+ */
+void
+prefs_mime_save_to_disk_toggled_cb (GtkWidget *button, PreferencesDialog *pd)
+{
+	gboolean active;
+
+	active = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button));
+	gtk_widget_set_sensitive (pd->mime_helper_frame, !active);
 }
