@@ -18,5 +18,13 @@
 
 void session_autosave(void);
 gboolean session_autoresume(void);
-void session_save(gchar *filename);
-
+void session_save (void);
+void session_add_window (GaleonWindow *window);
+void session_remove_window (GaleonWindow *window);
+void session_quit (gboolean auto_save);
+void session_quit_cb (gpointer data);
+void session_save_to (const gchar *filename);
+void session_load_from (const gchar *filename);
+GList *session_history_get (void);
+void session_history_add (const gchar *filename);
+void session_history_load_index (gint index);
