@@ -20,6 +20,7 @@
 #include "galeon.h"
 #include "spinner.h"
 #include "misc.h"
+#include "window.h"
 
 #include <string.h>
 #include <gtk/gtkbox.h>
@@ -28,6 +29,7 @@
 #include <libgnome/gnome-config.h>
 #include <libgnome/gnome-util.h>
 #include <libgnomeui/gnome-animator.h>
+#include <libgnomeui/gnome-preferences.h>
 #include <libgnomevfs/gnome-vfs.h>
 #include <libgnomevfs/gnome-vfs-mime.h>
 
@@ -37,21 +39,23 @@ static void spinner_fill_iconlist_from_dir (GnomeIconList *icon_list,
 					    gchar *base);
 
 /**
- * spinner_create: create a spinner and add it to an hbox
+ * spinner_create: create a spinner, note this DOESN'T pack it anywhere,
+ * so the user has to take the return value and pack it somewhere
  */
-void
+GtkWidget *
 spinner_create (GaleonWindow *window)
 {
+	gint i, width, height, target_height;
 	GtkWidget *spinner_button;
 	GtkWidget *spinner;
-	int toolbar, i;
+	ToolbarStyle style;
+	ToolbarIcons icons;
+	GdkImlibImage *first;
 	gchar *anim_dir;
 	gchar *filename;
-	GdkImlibImage *first;
-	gint width, height;
 
 	/* check window */
-	return_if_not_window (window);
+	return_val_if_not_window (window, NULL);
 
 	/* initially no spinner */
 	window->spinner = NULL;
@@ -59,7 +63,7 @@ spinner_create (GaleonWindow *window)
 	/* get the directory of the animations from configuration space,
  	 * default to installed share directory */
 	anim_dir = gnome_config_get_string("/galeon/Appearance/spinner_dir="
-					   SHARE_DIR "/spinners/pipeon/");
+					   SHARE_DIR "/spinners/galeon/");
 	if (anim_dir == NULL || strlen(anim_dir) == 0)
 	{
 		anim_dir = g_strdup(SHARE_DIR);
@@ -73,7 +77,7 @@ spinner_create (GaleonWindow *window)
 		g_warning ("unable to load first spinner frame: %s\n",
 			   filename);
 		g_free (filename);
-		return;
+		return NULL;
 	}
 	width = first->rgb_width;
 	height = first->rgb_height;
@@ -82,21 +86,52 @@ spinner_create (GaleonWindow *window)
 
 	/* set the size of the pixmap appropriately to whether 
 	 * text is turned on in the toolbars, so it resizes okay */
-	toolbar = gnome_config_get_int("/galeon/Appearance/toolbar_type=0");
+	style = gnome_config_get_int (CONF_APPEARANCE_TOOLBAR_STYLE);
+	icons = gnome_config_get_int (CONF_APPEARANCE_TOOLBAR_ICONS);
 
 	/* downsize the image if it would bloat the toolbar */
-	if (toolbar == 0 && height > 24)
+	target_height = 0;
+	switch (style)
 	{
-		/* preserve aspect ratio */
-		width = (width * 24) / height;
-		height = 24;
-	}
+	case TOOLBAR_STYLE_TEXT_ONLY:
+		target_height = 22;
+		break;
 
-	if (toolbar == 1 && height > 22)
+	case TOOLBAR_STYLE_ICONS_ONLY:
+	case TOOLBAR_STYLE_HORIZONTAL:
+		switch (icons)
+		{
+		case TOOLBAR_ICONS_NORMAL:
+		case TOOLBAR_ICONS_NAUTILUS:
+			target_height = 24;
+			break;
+
+		case TOOLBAR_ICONS_LARGE:
+			target_height = 36;
+			break;
+		}
+		break;
+
+	case TOOLBAR_STYLE_VERTICAL:
+		switch (icons)
+		{
+		case TOOLBAR_ICONS_NORMAL:
+		case TOOLBAR_ICONS_NAUTILUS:
+			target_height = 40;
+			break;
+
+		case TOOLBAR_ICONS_LARGE:
+			target_height = 52;
+			break;
+		}
+		break;
+	}
+	g_assert (target_height != 0);
+	if (height > target_height)
 	{
 		/* preserve aspect ratio */
-		width = (width * 22) / height;
-		height = 22;
+		width = (width * target_height) / height;
+		height = target_height;
 	}
 
 	/* build a button to hold the spinner */
@@ -136,7 +171,7 @@ spinner_create (GaleonWindow *window)
 			{
 				g_warning("no frames of animation found in "
 					  "directory `%s'", anim_dir);
-				return;
+				return NULL;
 			}
 
 			/* jump out of loading */
@@ -148,22 +183,22 @@ spinner_create (GaleonWindow *window)
 			(GNOME_ANIMATOR(spinner),
 			 filename,       /* filename */
 			 0, 0,           /* x_offset, y_offset */
-			 100,            /* interval */
+			 70,            /* interval */
 			 width, height); /* x_size, y_size */
  
 		/* free allocated filename */
 		g_free(filename);
 	}
 
-	/* add it to the hbox */
-	g_return_if_fail (GTK_IS_WIDGET (window->main_toolbar));
-	gtk_box_pack_end (GTK_BOX (window->main_toolbar->parent), 
-			  GTK_WIDGET (spinner_button), FALSE, FALSE, 0);
-
 	/* attach click signal */
 	gtk_signal_connect (GTK_OBJECT (spinner_button), "clicked",
 			    window_spinner_button_clicked_cb, window);
 
+	/* setup context menu */
+	gtk_signal_connect (GTK_OBJECT (spinner_button), "button-press-event",
+			    GTK_SIGNAL_FUNC
+			    (window_generic_button_press_event_cb), window);
+
 	/* show the spinner */
 	gtk_widget_show (spinner_button);
 	gtk_widget_show (spinner);
@@ -173,6 +208,9 @@ spinner_create (GaleonWindow *window)
 
 	/* set in window */
 	window->spinner = spinner;
+
+	/* return button */
+	return spinner_button;
 }
 
 /**
@@ -270,7 +308,7 @@ spinner_fill_iconlist_from_dir (GnomeIconList *icon_list, gchar *base)
 
 	pref_spinner_path = 
 		gnome_config_get_string("/galeon/Appearance/spinner_dir="
-					SHARE_DIR "/spinners/pipeon/");	
+					SHARE_DIR "/spinners/galeon/");	
 
 	rc = gnome_vfs_directory_list_load
 		(&list, base, (GNOME_VFS_FILE_INFO_GET_MIME_TYPE |
