@@ -20,9 +20,14 @@
 #include "embed.h"
 #include "window.h"
 #include "misc.h"
-#include "auto_completion.h"
+#include "autocompletion.h"
 #include "history.h"
 #include "bookmarks.h"
+#include "mozilla.h"
+#include "context.h"
+#include "toolbar.h"
+#include "find.h"
+#include "session.h"
 
 #include <string.h>
 #include <gtkmozembed.h>
@@ -42,8 +47,7 @@ gint window_zoom_spin_timeout_cb (GaleonWindow *window);
 void window_entry_changed_cb (GtkEditable *editable, gboolean *changed); 
 void window_location_gnomeentry_popwin_cb (GtkWidget *widget, 
 					   GaleonWindow *window);
-static void window_handle_go_button (GaleonWindow *window, gboolean new_embed,
-				     gboolean new_window);
+static void window_handle_go_button (GaleonWindow *window, LinkState state);
 
 GTimer *zoom_timer = NULL;
 #define ZOOM_DELAY 0.20
@@ -74,29 +78,17 @@ window_selection_received_cb (GtkWidget *widget,
 			      GtkSelectionData *selection_data,
 			      guint time, GaleonWindow *window)
 {
-	gboolean tabbed_mode;
 	GdkModifierType modifier;
 	
 	return_if_not_window (window);
 
 	if (selection_data->data)
 	{
-		
 		gdk_window_get_pointer (window->WMain->window,
 			NULL, NULL, &modifier);
-			
-		if (modifier & GDK_SHIFT_MASK) {
-			tabbed_mode = gnome_config_get_bool (CONF_APPEARANCE_TABBED);
-			if (modifier & GDK_CONTROL_MASK) {
-				tabbed_mode = !tabbed_mode;
-			}
-		  	embed_create_from_url (window->active_embed, 
-		  			       selection_data->data,
-		  			       TRUE, !tabbed_mode);
-		}
-		else {
-			window_load_url (window, selection_data->data);
-		}
+
+		embed_activate_link_keyboard (window->active_embed, NULL, 
+					      selection_data->data, modifier);
 	}
 }
 
@@ -106,26 +98,26 @@ window_selection_received_cb (GtkWidget *widget,
 gboolean
 window_delete_cb (GtkWidget *widget, GdkEventAny *event, GaleonWindow *window)
 {
-	extern gboolean fullscreen_active;
-	
-        /* location bar history is saved automatically when the widget is
-	   destroyed. we don't want to save the text currently in the entry */
-	window_clear_url_entry (window);
+	/* this can cause one of two things; if this is one of many windows
+	 * then we want to close this window but keep the others going. If
+	 * this is the last window, we want to behave as though the user
+	 * chose to exit Galeon (and do automatic session saving if the
+	 * pref is set */
 
-	/* save window size if we are not in fullscreen and if 
-	 * the window is not a popup */
-	if (window->active_embed != NULL &&
-	    !window->active_embed->is_popup &&
-	    !fullscreen_active)
-	{	
-		gnome_config_set_int("/galeon/Appearance/winwidth", 
-				     window->WMain->allocation.width);
-		gnome_config_set_int("/galeon/Appearance/winheight", 
-				     window->WMain->allocation.height);
+	/* FIXME: I think this is broken. Should call always window_close, 
+	 * window_close already takes care of this (ricardo) 
+	 */
+	if (g_list_length (all_windows) == 1 && 
+	    !galeon_server_mode && !galeon_panel_mode)
+	{
+		/* exit the session */
+		session_quit (FALSE);
+	}
+	else
+	{
+		/* close the window */
+		window_close (window);
 	}
-
-	/* close the window */
-	window_close (window);
 
 	return TRUE;
 }
@@ -147,11 +139,11 @@ window_back_forward_button_press_cb (GtkButton *button,
 
 	if (event->button == 3)
 	{
-		if (button == GTK_BUTTON (window->BForward))
+		if (GTK_WIDGET (button) == window->forward_button)
 		{
 			menu = embed_create_forward_menu (embed);
 		}
-		else if (button == GTK_BUTTON (window->BBack))
+		else if (GTK_WIDGET (button) == window->back_button)
 		{
 			menu = embed_create_back_menu (embed);
 		}
@@ -159,23 +151,37 @@ window_back_forward_button_press_cb (GtkButton *button,
 		{
 			g_assert_not_reached ();
 		}
-		gnome_popup_menu_do_popup_modal (GTK_WIDGET (menu),
-						 menu_position_under_widget, 
-						 button, NULL, NULL);
-		gtk_widget_destroy (GTK_WIDGET (menu));
-		return FALSE;
+#if 0		/* FIXME: removed this since people objected to it --
+		 * MattA 12/07/01 */
+		context_menu_add_seperator (GTK_MENU (menu));
+		context_show_appearance_menu (window, GTK_MENU (menu), 
+					      NULL, TRUE, TRUE);
+#endif
+		gnome_popup_menu_do_popup_modal (GTK_WIDGET (menu), NULL,
+						 NULL, event, NULL);
+		gtk_widget_unref (GTK_WIDGET(menu));
+		return TRUE;
 	}	
-	return TRUE;
+	return FALSE;
 }
 
 /** 
- * window_home_button_clicked_cb: home button clicked
+ * window_home_button_release_event_cb: home button clicked
  */
-void
-window_home_button_clicked_cb (GtkButton *button, GaleonWindow *window)
+gboolean
+window_home_button_release_event_cb (GtkButton *button, GdkEventButton *event,
+				     GaleonWindow *window)
 {
-	return_if_not_window (window);
-	window_go_home (window, FALSE, FALSE);
+	if (GTK_WIDGET (button)->window != event->window)
+		return FALSE;
+	return_val_if_not_window (window, FALSE);
+	if (event->button != 3)
+	{
+		window_go_home (window, mouse_state_to_link_state 
+						(event->button, event->state));
+		return TRUE;
+	}
+	return FALSE;
 }
 
 /** 
@@ -189,8 +195,6 @@ window_home_button_press_event_cb (GtkButton *button, GdkEventButton *event,
 	GaleonEmbed *embed;
 	GtkWidget *menu;
 	gint action;
-	gboolean tabbed_mode;
-	gboolean shift_modifier;
 
 	/* homepage context menu */
 	/* although there's only one entry I think it's still necessary
@@ -208,29 +212,15 @@ window_home_button_press_event_cb (GtkButton *button, GdkEventButton *event,
 	return_val_if_not_embed (embed, FALSE);
 
 	/* handle button accordingly */	
-	switch (event->button)
+	if (event->button == 3)
 	{
-	case 1:
-		/* nothing -- handled by click callback */
-		return FALSE;
-
-	case 2:
-		/* open home page in new window/tab */
-		tabbed_mode =
-			gnome_config_get_bool (CONF_APPEARANCE_TABBED);
-		shift_modifier =
-			(event->state & GDK_SHIFT_MASK) ? 0 : 1;
-		window_go_home (window, TRUE,
-			tabbed_mode ^ shift_modifier);
-		return TRUE;
-
-	case 3:
 		/* show context menu */
 		menu = gnome_popup_menu_new (menu_uiinfo);
+		context_menu_add_seperator (GTK_MENU (menu));
+		context_show_appearance_menu (window, GTK_MENU (menu), 
+					      NULL, TRUE, TRUE);
 		action = gnome_popup_menu_do_popup_modal 
-			(GTK_WIDGET (menu),
-			 menu_position_under_widget, 
-			 button, NULL, NULL);
+			(GTK_WIDGET (menu), NULL, NULL, event, NULL);
 
 		/* perform set homepage action if chosen */
 		if (action == 0 && embed->site_location != NULL &&
@@ -242,12 +232,10 @@ window_home_button_press_event_cb (GtkButton *button, GdkEventButton *event,
 		}
 
 		/* destroy the popup menu */
-		gtk_widget_destroy (GTK_WIDGET (menu));
+		gtk_widget_unref (menu);
 		return TRUE;
-
-	default:
-		return FALSE;
 	}
+	return FALSE;
 }
 
 /** 
@@ -257,9 +245,6 @@ gboolean
 window_new_button_press_cb (GtkButton *button, GdkEventButton *event,
 			    GaleonWindow *window)
 {
-	gint tabbed_mode;
-	gint shift_modifier;
-	GdkModifierType modifier;
 	GaleonEmbed *embed;
 	GtkWidget *menu;
 	gint action;
@@ -278,14 +263,62 @@ window_new_button_press_cb (GtkButton *button, GdkEventButton *event,
 	embed = window->active_embed;
 	return_val_if_not_embed (embed, FALSE);
 
+	switch (event->button)
+	{
+	case 3:
+		/* show context menu */
+		menu = gnome_popup_menu_new (menu_uiinfo);
+		context_menu_add_seperator (GTK_MENU (menu));
+		context_show_appearance_menu (window, GTK_MENU (menu), 
+					      NULL, TRUE, TRUE);
+		action = gnome_popup_menu_do_popup_modal 
+			(GTK_WIDGET (menu), NULL, NULL, event, NULL);
+		
+		/* do action */
+		if (action != -1)
+		{
+			embed_create_default (embed, action == 0);
+		}
+		gtk_widget_unref (menu);
+		return TRUE;
+
+	default:
+		return FALSE;
+	}
+}
+/** 
+ * window_new_button_release_cb: new browser button clicked
+ */
+gboolean
+window_new_button_release_cb (GtkButton *button, GdkEventButton *event,
+			    GaleonWindow *window)
+{
+	gint tabbed_mode;
+	gint shift_modifier;
+	GaleonEmbed *embed;
+	GaleonEmbed *new_embed;
+
+	if (GTK_WIDGET (button)->window != event->window)
+		return FALSE;
+	return_val_if_not_window (window, FALSE);
+	embed = window->active_embed;
+	return_val_if_not_embed (embed, FALSE);
+
 	tabbed_mode = gnome_config_get_bool (CONF_APPEARANCE_TABBED) ? 1 : 0;
-	gdk_window_get_pointer (window->WMain->window, NULL, NULL, &modifier);
-	shift_modifier = (modifier & GDK_SHIFT_MASK) ? 0 : 1;
+	shift_modifier = (event->state & GDK_SHIFT_MASK) ? 0 : 1;
 	
 	switch (event->button)
 	{
 	case 1:
-		embed_create_default (embed, (tabbed_mode ^ shift_modifier));
+		new_embed = embed_create_default (embed, (tabbed_mode ^ 
+							  shift_modifier));
+		/* FIXME: why this, why here? */
+		window_update_nav_controls (window);
+		if (new_embed->parent_window->location_entry != NULL)
+		{
+			gtk_widget_grab_focus (GTK_WIDGET (
+				new_embed->parent_window->location_entry));
+		}
 		return TRUE;
 		
 	case 2:
@@ -297,22 +330,6 @@ window_new_button_press_cb (GtkButton *button, GdkEventButton *event,
 				       GDK_SELECTION_TYPE_STRING,
 				       GDK_CURRENT_TIME);
 		return FALSE;
-
-	case 3:
-		/* show context menu */
-		menu = gnome_popup_menu_new (menu_uiinfo);
-		action = gnome_popup_menu_do_popup_modal 
-			(GTK_WIDGET (menu),
-			 menu_position_under_widget, 
-			 button, NULL, NULL);
-		
-		/* do action */
-		if (action != -1)
-		{
-			embed_create_default (embed, action == 0);
-		}
-		return TRUE;
-
 	default:
 		return FALSE;
 	}
@@ -350,6 +367,7 @@ window_new_button_drag_data_received_cb (GtkWidget *widget,
 		embed_create_from_url (window->active_embed,
 				       selection_data->data,
 				       TRUE, !tabbed_mode);
+		window_update_nav_controls (window);
 		break;
 
 	default:
@@ -359,14 +377,60 @@ window_new_button_drag_data_received_cb (GtkWidget *widget,
 }
 
 /** 
- * window_back_button_clicked_cb: back button clicked
+ * window_back_forward_button_release_cb: back/forward button clicked
  */
-void
-window_back_button_clicked_cb (GtkButton *button, GaleonWindow *window)
+gboolean
+window_back_forward_button_release_cb (GtkButton *button,
+				     GdkEventButton *event,
+				     GaleonWindow *window)
 {
-	return_if_not_window (window);
-	return_if_not_embed (window->active_embed);
-	gtk_moz_embed_go_back (GTK_MOZ_EMBED(window->active_embed->mozEmbed));
+	char *url = NULL;
+	GaleonEmbed *embed;
+	
+	if (GTK_WIDGET (button)->window != event->window)
+		return FALSE;
+	return_val_if_not_window (window, FALSE);
+	return_val_if_not_embed (window->active_embed, FALSE);
+
+	if (event->button == 3)
+		return FALSE;
+	
+	embed = window->active_embed;
+	if (event->button == 1 && !(event->state & GDK_SHIFT_MASK))
+	{
+		if (GTK_WIDGET (button) == window->forward_button)
+		{
+			gtk_moz_embed_go_forward 
+				(GTK_MOZ_EMBED (embed->mozEmbed));
+		}
+		else if (GTK_WIDGET (button) == window->back_button)
+		{
+			gtk_moz_embed_go_back 
+				(GTK_MOZ_EMBED (embed->mozEmbed));
+		}
+		else
+		{
+			g_assert_not_reached ();
+		}
+		return TRUE;
+	}
+	
+	if (GTK_WIDGET (button) == window->forward_button)
+	{
+		url = mozilla_session_history_get_url_relative (embed, 1);
+	}
+	else if (GTK_WIDGET (button) == window->back_button)
+	{
+		url = mozilla_session_history_get_url_relative (embed, -1);
+	}
+	else
+	{
+		g_assert_not_reached ();
+	}
+	embed_activate_link_mouse (window->active_embed, NULL, url, event);
+	if (url != NULL) free (url);
+	
+	return TRUE;
 }
 
 /** 
@@ -390,6 +454,9 @@ window_up_button_press_event_cb (GtkButton *button, GdkEventButton *event,
 
 	/* make menu */
 	menu = embed_create_up_menu (window->active_embed);
+	context_menu_add_seperator (GTK_MENU (menu));
+	context_show_appearance_menu (window, GTK_MENU (menu), 
+				      NULL, TRUE, TRUE);
 	if (menu == NULL)
 	{
 		return TRUE;
@@ -397,35 +464,31 @@ window_up_button_press_event_cb (GtkButton *button, GdkEventButton *event,
 
 	/* show popup menu */
 	gnome_popup_menu_do_popup_modal (GTK_WIDGET (menu),
-					 NULL, NULL, NULL, NULL);
+					 NULL, NULL, event, NULL);
 
 	/* destroy the popup menu */
-	gtk_widget_destroy (GTK_WIDGET (menu));
+	gtk_object_unref (GTK_OBJECT (menu));
 
 	/* success */
 	return FALSE;
 }
 
-/**
- * window_up_button_clicked_cb: Up button clicked
- */
-void
-window_up_button_clicked_cb (GtkButton *button, GaleonWindow *window)
-{
-	return_if_not_window (window);
-	return_if_not_embed (window->active_embed);
-	embed_go_up (window->active_embed, 0);
-}
-
 /** 
- * window_forward_button_clicked_cb: forward button clicked
-*/
-void
-window_forward_button_clicked_cb (GtkButton *button, GaleonWindow *window)
+ * window_up_button_release_event_cb: up browser button clicked
+ */
+gboolean
+window_up_button_release_event_cb (GtkButton *button, GdkEventButton *event,
+				   GaleonWindow *window)
 {
-	return_if_not_window (window);
-	return_if_not_embed (window->active_embed);
-	gtk_moz_embed_go_forward(GTK_MOZ_EMBED(window->active_embed->mozEmbed));
+	if (GTK_WIDGET (button)->window != event->window)
+		return FALSE;
+	return_val_if_not_window (window, FALSE);
+	return_val_if_not_embed (window->active_embed, FALSE);
+	if (event->button == 3)
+		return FALSE;
+	embed_go_up (window->active_embed, 0, 
+		mouse_state_to_link_state (event->button, event->state));
+	return TRUE;
 }
 
 /** 
@@ -476,10 +539,11 @@ window_refresh_button_press_event_cb (GtkButton *button, GdkEventButton *event,
 	case 3:
 		/* show context menu */
 		menu = gnome_popup_menu_new (menu_uiinfo);
+		context_menu_add_seperator (GTK_MENU (menu));
+		context_show_appearance_menu (window, GTK_MENU (menu), 
+					      NULL, TRUE, TRUE);
 		action = gnome_popup_menu_do_popup_modal 
-			(GTK_WIDGET (menu),
-			 menu_position_under_widget, 
-			 button, NULL, NULL);
+			(GTK_WIDGET (menu), NULL, NULL, event, NULL);
 
 		/* perform set homepage action if chosen */
 		switch (action)
@@ -497,7 +561,7 @@ window_refresh_button_press_event_cb (GtkButton *button, GdkEventButton *event,
 		}
 
 		/* destroy the popup menu */
-		gtk_widget_destroy (GTK_WIDGET (menu));
+		gtk_widget_unref (menu);
 		break;
 	}
 
@@ -514,6 +578,46 @@ window_stop_button_clicked_cb (GtkButton *button, GaleonWindow *window)
 	gtk_moz_embed_stop_load (GTK_MOZ_EMBED(window->active_embed->mozEmbed));
 }
 
+/** 
+ * window_generic_button_press_event_cb: some button clicked, show appearance
+ * menu
+ */
+gboolean
+window_generic_button_press_event_cb (GtkButton *button, GdkEventButton *event,
+				      GaleonWindow *window)
+{
+	GtkWidget *menu;
+
+	/* check args */
+	return_val_if_not_window (window, FALSE);
+
+	/* handle button accordingly */	
+	switch (event->button)
+	{
+	case 1:
+		/* nothing -- handled by click callback */
+		break;
+
+	case 2:
+		/* nothing on middle click */
+		break;
+
+	case 3:
+		/* show context menu */
+		menu = gtk_menu_new ();
+		context_show_appearance_menu (window, GTK_MENU(menu), 
+					      NULL, TRUE, FALSE);
+		gnome_popup_menu_do_popup_modal 
+			(GTK_WIDGET (menu), NULL, NULL, event, NULL);
+
+		/* destroy the popup menu */
+		gtk_widget_unref (menu);
+		break;
+	}
+
+	return TRUE;
+}
+
 /** 
  * window_location_entry_key_press_cb: key pressed in the url entry
  */
@@ -541,29 +645,38 @@ window_location_entry_key_press_cb (GtkWidget *widget, GdkEventKey *event,
 	else
 		keyval = event->keyval;
 
-	if ((event->state & GDK_Control_L || event->state & GDK_Control_R) 
-	    && (keyval == GDK_a || keyval == GDK_b || keyval == GDK_c ||
-	        keyval == GDK_d || keyval == GDK_e || keyval == GDK_f ||
-	        keyval == GDK_h || keyval == GDK_k || keyval == GDK_u ||
-	        keyval == GDK_v || keyval == GDK_w || keyval == GDK_x ))
+	if (((event->state & GDK_Control_L || event->state & GDK_Control_R) &&
+	     (keyval == GDK_a || keyval == GDK_b || keyval == GDK_c ||
+	      keyval == GDK_d || keyval == GDK_e || keyval == GDK_f ||
+	      keyval == GDK_h || keyval == GDK_k || keyval == GDK_u ||
+	      keyval == GDK_v || keyval == GDK_w || keyval == GDK_x )) ||
+	    (event->state == 0 && event->keyval == GDK_BackSpace))
 	{
 		return TRUE;
 	}
-		  
+
 	/* don't grab alt combos, thus you can still access the menus. */
 	if (event->state & GDK_MOD1_MASK)
 		return FALSE;
 
 	/* check to see if key press is in the window's location bar or
 	   the open url dialog */
-	if (widget != window->toolbar_entry)
+	if (widget != window->location_entry)
+	{
 		url_dialog = TRUE;
+	}
 
 	if ((event->keyval == GDK_Return) || (event->keyval == GDK_KP_Enter))
 	{
-		if (!url_dialog)
+		if (url_dialog)
 		{
-			window_handle_go_button (window, FALSE, FALSE);
+			open_url_handle_link (window, 
+				key_state_to_link_state (event->state));
+		}
+		else
+		{
+			window_handle_go_button (window, 
+				key_state_to_link_state (event->state));
 		}
 		return TRUE;
 	}
@@ -662,12 +775,11 @@ window_location_entry_button_press_cb (GtkWidget *entry,
 		                              "button-press-event");
 		return TRUE;
 	}
-	return FALSE;
+	return TRUE;
 }
 
 static void
-window_handle_go_button (GaleonWindow *window, gboolean new_embed,
-			 gboolean new_window)
+window_handle_go_button (GaleonWindow *window, LinkState state)
 {
 	gchar *text;
 	gchar *text2;
@@ -676,56 +788,35 @@ window_handle_go_button (GaleonWindow *window, gboolean new_embed,
 
 	return_if_not_window (window);
 
-	text = gtk_editable_get_chars (GTK_EDITABLE (window->toolbar_entry),
+	if (window->location_entry == NULL) return;
+	if (window->location_gnomeentry == NULL) return;
+
+	text = gtk_editable_get_chars (GTK_EDITABLE (window->location_entry),
 				       0, -1);
 
 	/* make sure the string isn't empty */
 	if (strcmp (text, "") != 0)
 	{
-		/* don't save the text in the entry -
-		 * avoids duplicate entries */
-		window_clear_url_entry (window);
-		ge = GNOME_ENTRY (window->toolbar_gnomeentry);
+		/* forcibly save contents of entry */
+		ge = GNOME_ENTRY (window->location_gnomeentry);
 		gnome_entry_save_history (ge);
-		gtk_entry_set_text (GTK_ENTRY (window->toolbar_entry), text);
+		gnome_config_sync ();
 
 		/* add item to location bars in all other
 		 * open browser windows */
 		for (wl = all_windows; wl != NULL; wl = wl->next)
 		{
-			GtkList *list;
-			GList *items;
-			GaleonWindow *awindow;
-			gchar *label;
-			gboolean dup = FALSE;
+			GaleonWindow *awindow = (GaleonWindow *)(wl->data);
 
-			awindow = (GaleonWindow *) (wl->data);
-
-			/* check if it's the active window or
-			   if the window is without entry */
-			if (wl->data == window ||
-			    (!awindow->toolbar_gnomeentry))
+			/* skip if no locatio entry to fill */
+			if (awindow->location_gnomeentry == NULL)
+			{
 				continue;
-
-			ge = GNOME_ENTRY(awindow->toolbar_gnomeentry);
-
-			g_return_if_fail (ge);
-
-			list = GTK_LIST(ge->combo.list);
-			for (items = list->children; items != NULL;
-			     items = items->next)
-			{
-				gtk_label_get(GTK_LABEL(
-					GTK_BIN(items->data)->child), &label);
-				if (!strcmp(label, text))
-				{
-					dup = TRUE;
-				}
-			}
-			if (!dup)
-			{
-				gnome_entry_prepend_history(ge, TRUE, text);
 			}
+
+			/* reload entry */
+			gnome_entry_load_history 
+				(GNOME_ENTRY (awindow->location_gnomeentry));
 		}
 
 		text2 = bookmarks_parse_nick (text, NULL);
@@ -743,71 +834,29 @@ window_handle_go_button (GaleonWindow *window, gboolean new_embed,
 		text = g_strdup ("about:blank");
 	}
 
-	/* if we need to open in the current embed... */
-	if (!new_embed)
-	{
-		/* update the contents of the location field of the
-		 * GaleonEmbed if it's empty */
-		if (window->active_embed->site_location) 
-		{
-			g_free (window->active_embed->site_location);
-		}
-		window->active_embed->site_location = g_strdup (text);
-		window_load_url (window, text);
-	}
-	/* otherwise, open in a new tab or window */
-	else
-	{
-		embed_create_from_url (window->active_embed, text, 
-				       TRUE, new_window);
-	}
+	embed_activate_link (window->active_embed, NULL, text, state);
 
 	g_free (text);
 }
 
-
 /** 
- * window_go_button_press_cb: go button pressed
+ * window_go_button_release_cb: go button released
  */
 gboolean
-window_go_button_press_cb (GtkButton *button, GdkEventButton *event,
-			   GaleonWindow *window)
+window_go_button_release_cb (GtkButton *button, GdkEventButton *event,
+			     GaleonWindow *window)
 {
-	gboolean tabbed_mode;
-	gboolean shift_modifier;
-
+	if (GTK_WIDGET (button)->window != event->window)
+		return FALSE;
 	return_val_if_not_window (window, FALSE);
 
-	switch (event->button)
-	{
-		case 1:
-			/* nothing -- handled by click callback */
-			break;
-
-		case 2:
-			/* open location in new window/tab */
-			tabbed_mode = gnome_config_get_bool
-				(CONF_APPEARANCE_TABBED);
-			shift_modifier =
-				(event->state & GDK_SHIFT_MASK) ? 0 : 1;
-			window_handle_go_button (window, TRUE,
-						 tabbed_mode ^ shift_modifier);
-			break;
-	}
+	/* open location in correct place */
+	window_handle_go_button (window,
+		mouse_state_to_link_state (event->button, event->state));
 
 	return TRUE;
 }
 
-/** 
- * window_go_button_press_cb: go button clicked
- */
-void
-window_go_button_clicked_cb (GtkButton *button, GaleonWindow *window)
-{
-	return_if_not_window (window);
-	window_handle_go_button (window, FALSE, FALSE);
-}
-
 /**
  * Changes the zoom if enough time time has passed 
  */
@@ -882,6 +931,18 @@ window_zoom_spin_changed_cb (GtkEditable *editable, GaleonWindow *window)
 	g_timeout_add (50, (GSourceFunc) window_zoom_spin_timeout_cb, window);
 }
 
+/** 
+ * window_zoom_spin_button_press_cb: stop context menu from appearing
+ * (context menu here will make galeon go nuts)
+ */
+gboolean
+window_zoom_spin_button_press_cb (GtkButton *button,
+				     GdkEventButton *event,
+				     GaleonWindow *window)
+{
+	return TRUE;
+}
+
 /** 
  * window_drag_pixmap_drag_data_get_cb:
  */
@@ -944,7 +1005,7 @@ window_location_entry_drag_data_received_cb (GtkWidget *widget,
 	case DND_TARGET_GALEON_URL:
 	case DND_TARGET_TEXT_URI_LIST:/* FIXME: won't work right for a list */
 		window_clear_url_entry (window);
-		gtk_editable_insert_text (GTK_EDITABLE (window->toolbar_entry),
+		gtk_editable_insert_text (GTK_EDITABLE (window->location_entry),
 					  url, strlen (url), &i);
 		break;
 	default:
@@ -987,11 +1048,11 @@ window_location_gnomeentry_popwin_cb (GtkWidget *widget, GaleonWindow *window)
 	static gint selection_made;
 
 	g_assert(widget!=NULL);
-	g_assert(window->toolbar_gnomeentry!=NULL);
+	g_assert(window->location_gnomeentry!=NULL);
 
 	if (GTK_WIDGET_VISIBLE(widget))
 	{
-		combo = GTK_COMBO(window->toolbar_gnomeentry);
+		combo = GTK_COMBO(window->location_gnomeentry);
 		list = GTK_LIST(combo->list);
 		container = GTK_CONTAINER(list);
 
@@ -999,7 +1060,8 @@ window_location_gnomeentry_popwin_cb (GtkWidget *widget, GaleonWindow *window)
 		{
 			if (!GTK_LIST(container)->selection)
 			{
-				gtk_window_set_focus(GTK_WINDOW(GTK_COMBO(combo)->popwin), NULL);
+				gtk_window_set_focus (GTK_WINDOW (GTK_COMBO 
+						      (combo)->popwin), NULL);
 				container->focus_child = NULL;
 				list->last_focus_child = NULL;
 			}
@@ -1009,26 +1071,23 @@ window_location_gnomeentry_popwin_cb (GtkWidget *widget, GaleonWindow *window)
 		selection_made = FALSE;
 
 		/* connect a handler to the entry's "changed" signal */
-		gtk_signal_connect(GTK_OBJECT(window->toolbar_entry),
+		gtk_signal_connect(GTK_OBJECT(window->location_entry),
 		                   "changed",
 		                   GTK_SIGNAL_FUNC(window_entry_changed_cb),
 		                   &selection_made);
 	}
 	else
 	{
-		gboolean go_toolbar_hidden = FALSE;
-
-		/* check if the go toolbar is hidden */
-		if (!GTK_WIDGET_VISIBLE (window->go_toolbar))
-			go_toolbar_hidden = TRUE;
-
 		/* connect a handler to the entry's "changed" signal */
-		gtk_signal_disconnect_by_func(GTK_OBJECT(window->toolbar_entry),
-	 	                              GTK_SIGNAL_FUNC(window_entry_changed_cb),
-		                              &selection_made);
+		gtk_signal_disconnect_by_func
+			(GTK_OBJECT (window->location_entry),
+			 GTK_SIGNAL_FUNC (window_entry_changed_cb),
+			 &selection_made);
 
-		if (go_toolbar_hidden && selection_made)
-			window_go_button_clicked_cb(NULL, window);
+		if (window->go_button != NULL && selection_made)
+		{
+			window_handle_go_button (window, 0);
+		}
 	}
 }
 
@@ -1092,8 +1151,109 @@ window_progress_action (GaleonWindow *window)
 	{
 		val = 0;
 	}
-	gtk_progress_set_value (progress, val);
+	gtk_progress_set_value (progress, val/100);
 
 	/* continue to be called */
 	return TRUE;
 }
+
+/**
+ * window_find_button_clicked_cb: Find button on toolar clicked; open find
+ * dialog
+ */
+void
+window_find_button_clicked_cb (GtkButton *button, GaleonWindow *window)
+{
+	find_show_dialog (window);
+}
+
+/**
+ * window_drag_data_received_cb: handle dropped data
+ */
+void
+window_drag_data_received_cb (GtkWidget *widget, GdkDragContext *context,
+			      gint x, gint y, GtkSelectionData *selection_data,
+			      guint info, guint time, GaleonWindow *window)
+{
+	switch (info)
+	{
+		case DND_TARGET_GALEON_EMBED:
+			{
+				GaleonEmbed *embed;
+
+				return_if_not_window (window);
+
+				/* get embed address */
+				memcpy (&embed, selection_data->data,
+					sizeof (GaleonEmbed *));
+				return_if_not_embed (embed);
+
+				embed_move_tab (embed, window->active_embed);
+				break;
+			}
+	}
+}
+
+/**
+ * window_history_button_clicked_cb: history button clicked, show docked
+ * history view unless already shown, in which case hide it.
+ */
+void
+window_history_button_clicked_cb (GtkButton *button, GaleonWindow *window)
+{
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button)))
+	{
+		if (window->dock_type != DOCK_HISTORY)
+		{
+			history_show_dock (window);
+		}
+	}
+	else
+	{
+		if (window->dock_type != DOCK_NONE)
+		{
+			window_undock (window);
+		}
+	}
+}
+
+/**
+ * window_bookmarks_button_clicked_cb: bookmarks button clicked, show docked
+ * bookmarks view unless already shown, in which case hide it.
+ */
+void
+window_bookmarks_button_clicked_cb (GtkButton *button, GaleonWindow *window)
+{
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button)))
+	{
+		if (window->dock_type != DOCK_BOOKMARKS)
+		{
+			bookmarks_editor_show_dock (window);
+		}
+	}
+	else
+	{
+		if (window->dock_type != DOCK_NONE)
+		{
+			window_undock (window);
+		}
+	}
+}
+
+/**
+ * window_print_button_clicked_cb: print button clicked, show dialog
+ */
+void
+window_print_button_clicked_cb (GtkButton *button, GaleonWindow *window)
+{
+	print_show_dialog (window);
+}
+
+void 
+window_session_history_menuitem_activate_cb (GtkMenuItem *menuitem, 
+					     gint index)
+{
+	session_history_load_index (index);
+}
+
+
